// --------------------------------------------------------------- -*- JS -*-
// IBM Confidential
// OCO Source Materials
// 5724-Y00 5724-Y02 5724-Y21
//  Copyright IBM Corp. 2004, 2010. All Rights Reserved.
// The source code for this program is not published or otherwise
// divested of its trade secrets, irrespective of what has
// been deposited with the U.S. Copyright Office.
// --------------------------------------------------------------------------

Ext.namespace('Ilog', 'Ilog.RuleEditor', 'Ilog.RuleEditor.CallbackManagement');

Ilog.RuleEditor.CallbackManagement.RuleEditorClientCallbackServerRequest = function(requestType, requestBody){
    this.RequestType = requestType;
    this.RequestBody = requestBody;
};

Ilog.RuleEditor.CallbackManagement.RefreshRequest = function(timestamp, editorReference, customArgs, ruleText){
    this.RequestType = Ilog.Constants.RuleEditorConstants.CallbackStringRequestTypeRefreshEditor;
    this.TimeStamp = ''+timestamp;
    this.EditorControlReference = editorReference;
    this.CustomArgs = customArgs;
    this.RuleText = ruleText;
};

Ilog.RuleEditor.CallbackManagement.SaveRequest = function(timestamp, editorReference, customArgs, ruleText){
    this.RequestType = Ilog.Constants.RuleEditorConstants.CallbackStringRequestTypeSaveRule;
    this.TimeStamp = ''+timestamp;
    this.EditorControlReference = editorReference;
    this.CustomArgs = customArgs;
    this.RuleText = ruleText;
};
   
Ilog.RuleEditor.CallbackManagement.GetPredictionsRequest = function(timestamp, editorReference, customArgs, ruleText, selectionStart, selectionEnd, useSentenceCompletion){
    this.RequestType = Ilog.Constants.RuleEditorConstants.CallbackStringRequestTypeGetPredictions;
    this.TimeStamp = ''+timestamp;
    this.EditorControlReference = editorReference;
    this.CustomArgs = customArgs;
    this.RuleText = ruleText;
    this.TextRangeStart = selectionStart;
    this.TextRangeEnd = selectionEnd;
    this.SentenceCompletion = useSentenceCompletion;
};

Ilog.RuleEditor.CallbackManagement.GetValueEditorRequest = function(timestamp, editorReference, customArgs, valueEditorKey){
    this.RequestType = Ilog.Constants.RuleEditorConstants.CallbackStringRequestTypeGetValueEditor;
    this.TimeStamp = ''+timestamp;
    this.EditorControlReference = editorReference;
    this.CustomArgs = customArgs;
    this.ValueEditorKey = valueEditorKey;
};

Ilog.RuleEditor.CallbackManagement.GetClientCallbackArgumentForRequest = function(requestType, requestXmlBody){
    var requestObj = new Ilog.RuleEditor.CallbackManagement.RuleEditorClientCallbackServerRequest(requestType, requestXmlBody);
    var requestAsString = Ext.util.JSON.encode(requestObj);
    return requestAsString;
    
};

Ilog.RuleEditor.CallbackManagement.SendEditorRequestWithDirectAjaxCall = function (oXMLHttpRequest, xmlToPost, serviceUrl, requestType){
    try{
        oXMLHttpRequest.open("POST", serviceUrl, true);
        oXMLHttpRequest.setRequestHeader("Content-Type","text/xml");
        
        if( requestType != null && requestType != undefined){
            oXMLHttpRequest.setRequestHeader("RequestType", requestType+"");
        }
        
        oXMLHttpRequest.send(xmlToPost);
    }
    catch (ex) {
        Ilog.Debug.RuleEditorDebugger.PrintDebug('[SendEditorRequestWithDirectAjaxCall] error: '+ex.message);
    }
};
       
Ilog.RuleEditor.CallbackManagement.GetEditorRequestWithDirectAjaxCall = function(serviceUrl, xmlToPost, sucessfulResponseHandler, editorInstance, errorResponseHandler, timestamp, textRange, isPlaceholder, keepTextRangeSelected, valueEditorTextRange) {
    var oXMLHttpRequest	= new XMLHttpRequest;

    // if using GET, need to put the parameters directly in the URL of the open(), remove the content-type text/xml, and do : send(null) instead of sending something  
    oXMLHttpRequest.onreadystatechange	= function() {
	    if ( (XMLHttpRequest.DONE != null && XMLHttpRequest.DONE != undefined && this.readyState == XMLHttpRequest.DONE) || this.readyState == 4 ) {  
	      oXMLHttpRequest.ResponseReceived = true;
		  if( this.status == 200 ){
		    sucessfulResponseHandler(this.responseText, "", editorInstance, timestamp, textRange, isPlaceholder, keepTextRangeSelected, valueEditorTextRange);
		  }
		  else{
		      errorResponseHandler(editorInstance, this, timestamp, textRange);
		  }
	    }
    }
    try{
        return { Request : oXMLHttpRequest,  Xml : xmlToPost, ServiceUrl : serviceUrl, Timestamp : timestamp };
    }
    catch (ex) {
        Ilog.Debug.RuleEditorDebugger.PrintDebug('[GetEditorRequestWithDirectAjaxCall] error: ' + ex.message);
    }
    
    return null;
};
    
// predictions time stamp
Ilog.RuleEditor.CallbackManagement.GetLastPredictionsRequestTimeStampForEditor = function (editorId){
    if( ! Ilog.RuleEditor.CallbackManagement.LastEditorRefreshRequestSent["lasttimestamppreds_"+editorId] ) {
        return null;
    }

    return Ilog.RuleEditor.CallbackManagement.LastEditorRefreshRequestSent["lasttimestamppreds_"+editorId];
};

Ilog.RuleEditor.CallbackManagement.SetLastPredictionsRequestTimeStampForEditor = function (editorId, timestamp){
    Ilog.RuleEditor.CallbackManagement.LastEditorRefreshRequestSent["lasttimestamppreds_"+editorId] = timestamp;
};

// predictions requests
Ilog.RuleEditor.CallbackManagement.GetLastPredictionsRequestSentForEditor = function (editorId){
    if( ! Ilog.RuleEditor.CallbackManagement.LastEditorRefreshRequestSent["lastpredsrequestsent_"+editorId] ) {
        return null;
    }
    
    return Ilog.RuleEditor.CallbackManagement.LastEditorRefreshRequestSent["lastpredsrequestsent_"+editorId];
};

Ilog.RuleEditor.CallbackManagement.SetLastPredictionsRequestSentForEditor = function (editorId, request){
    Ilog.RuleEditor.CallbackManagement.LastEditorRefreshRequestSent["lastpredsrequestsent_"+editorId] = request;
};

// refresh time stamp
Ilog.RuleEditor.CallbackManagement.GetLastRefreshRequestTimeStampForEditor = function (editorId){
    if( ! Ilog.RuleEditor.CallbackManagement.LastEditorRefreshRequestSent["lasttimestamp_"+editorId] ) {
        return null;
    }

    return Ilog.RuleEditor.CallbackManagement.LastEditorRefreshRequestSent["lasttimestamp_"+editorId];
};

Ilog.RuleEditor.CallbackManagement.SetLastRefreshRequestTimeStampForEditor = function (editorId, timestamp){
    Ilog.RuleEditor.CallbackManagement.LastEditorRefreshRequestSent["lasttimestamp_"+editorId] = timestamp;
};
 
 // refresh requests
Ilog.RuleEditor.CallbackManagement.GetLastRefreshRequestSentForEditor = function (editorId){
    if( ! Ilog.RuleEditor.CallbackManagement.LastEditorRefreshRequestSent["lastrequestsent_"+editorId] ) {
        return null;
    }
    
    return Ilog.RuleEditor.CallbackManagement.LastEditorRefreshRequestSent["lastrequestsent_"+editorId];
};

Ilog.RuleEditor.CallbackManagement.SetLastRefreshRequestSentForEditor = function (editorId, request){
    Ilog.RuleEditor.CallbackManagement.LastEditorRefreshRequestSent["lastrequestsent_"+editorId] = request;
};
    
// tooltips time stamp
Ilog.RuleEditor.CallbackManagement.GetLastTooltipRequestTimeStampForEditor = function (editorId){
    if( ! Ilog.RuleEditor.CallbackManagement.LastTooltipRequestSent["lasttooltiptimestamp_"+editorId] ) {
        return null;
    }

    return Ilog.RuleEditor.CallbackManagement.LastTooltipRequestSent["lasttooltiptimestamp_"+editorId];
};

Ilog.RuleEditor.CallbackManagement.SetLastTooltipRequestTimeStampForEditor = function (editorId, timestamp){
    Ilog.RuleEditor.CallbackManagement.LastTooltipRequestSent["lasttooltiptimestamp_"+editorId] = timestamp;
};
    
// tooltips requests
Ilog.RuleEditor.CallbackManagement.GetLastTooltipRequestSentForEditor = function (editorId){
    if( ! Ilog.RuleEditor.CallbackManagement.LastTooltipRequestSent["lastrequestsent_"+editorId] ) {
        return null;
    }
    
    return Ilog.RuleEditor.CallbackManagement.LastTooltipRequestSent["lastrequestsent_"+editorId];
};

Ilog.RuleEditor.CallbackManagement.SetLastTooltipRequestSentForEditor = function (editorId, request){
    Ilog.RuleEditor.CallbackManagement.LastTooltipRequestSent["lastrequestsent_"+editorId] = request;
};
    
// XmlHttpRequest+Xml objects store
// Refresh requests
Ilog.RuleEditor.CallbackManagement.GetLastDirectAjaxRefreshRequest = function (editorId){
    if( ! Ilog.RuleEditor.CallbackManagement.LastEditorRefreshRequestSent["lastdirectajaxrefreshrequest_"+editorId] ) {
        return null;
    }
    
    return Ilog.RuleEditor.CallbackManagement.LastEditorRefreshRequestSent["lastdirectajaxrefreshrequest_"+editorId];
};

Ilog.RuleEditor.CallbackManagement.SetLastDirectAjaxRefreshRequest = function (editorId, objRequest){
    Ilog.RuleEditor.CallbackManagement.LastEditorRefreshRequestSent["lastdirectajaxrefreshrequest_"+editorId] = objRequest;
};

// GetPredictions request
Ilog.RuleEditor.CallbackManagement.GetLastDirectAjaxGetPredictionsRequest = function (editorId){
    if( ! Ilog.RuleEditor.CallbackManagement.LastEditorGetPredictionsRequestSent["lastdirectajaxgetpredictionsrequest_"+editorId] ) {
        return null;
    }
    
    return Ilog.RuleEditor.CallbackManagement.LastEditorGetPredictionsRequestSent["lastdirectajaxgetpredictionsrequest_"+editorId];
};

Ilog.RuleEditor.CallbackManagement.SetLastDirectAjaxGetPredictionsRequest = function (editorId, objRequest){
    Ilog.RuleEditor.CallbackManagement.LastEditorGetPredictionsRequestSent["lastdirectajaxgetpredictionsrequest_"+editorId] = objRequest;
};

// Save request
Ilog.RuleEditor.CallbackManagement.GetLastDirectAjaxSaveRequest = function (editorId){
    if( ! Ilog.RuleEditor.CallbackManagement.LastEditorSaveRequestSent["lastdirectajaxsaverequest_"+editorId] ) {
        return null;
    }
    
    return Ilog.RuleEditor.CallbackManagement.LastEditorSaveRequestSent["lastdirectajaxsaverequest_"+editorId];
};

Ilog.RuleEditor.CallbackManagement.SetLastDirectAjaxSaveRequest = function (editorId, objRequest){
    Ilog.RuleEditor.CallbackManagement.LastEditorSaveRequestSent["lastdirectajaxsaverequest_"+editorId] = objRequest;
}

// NodeDescription request
Ilog.RuleEditor.CallbackManagement.GetLastDirectAjaxNodeDescriptionRequest = function (editorId){
    if( ! Ilog.RuleEditor.CallbackManagement.LastTooltipRequestSent["lastdirectajaxnodedescrequest_"+editorId] ) {
        return null;
    }
    
    return Ilog.RuleEditor.CallbackManagement.LastTooltipRequestSent["lastdirectajaxnodedescrequest_"+editorId];
};

Ilog.RuleEditor.CallbackManagement.SetLastDirectAjaxNodeDescriptionRequest = function (editorId, objRequest){
    Ilog.RuleEditor.CallbackManagement.LastTooltipRequestSent["lastdirectajaxnodedescrequest_"+editorId] = objRequest;
};
    
Ilog.RuleEditor.CallbackManagement.LastEditorRefreshRequestSent = new Object();
Ilog.RuleEditor.CallbackManagement.LastEditorGetPredictionsRequestSent = new Object();
Ilog.RuleEditor.CallbackManagement.LastEditorSaveRequestSent = new Object();
Ilog.RuleEditor.CallbackManagement.LastEditorRefreshRequestSentTimeStamp = new Object();
Ilog.RuleEditor.CallbackManagement.LastTooltipRequestSent = new Object();
Ilog.RuleEditor.CallbackManagement.LastTooltipRequestSentTimeStamp = new Object();
Ilog.RuleEditor.CallbackManagement.Contexts = new Object();

Ilog.RuleEditor.CallbackManagement.OnSaveSuccessfulResponseReceived = function(result, context, editorInstanceReceived){
// Analyzing expected result
   if( result == null || result == '' ) {
    ruleEditorInstance.fireEvent("SaveInError", "");
    return;
   }
           
    var resultObj = null;
    // JSON :
    try{
        resultObj = Ext.util.JSON.decode(result);
    }
    catch (ex) {
        Ilog.Debug.RuleEditorDebugger.PrintDebug('[OnSaveSuccessfulResponseReceived] JSON decoding error: ' + ex.message);
        ruleEditorInstance.fireEvent("SaveInError", ex.message, responseTimeStamp);
        return;
    }
                
    if( ! resultObj.EditorReference ){
        return;
    }

    var editorIdentifier = resultObj.EditorReference;         
    if( editorIdentifier == null || editorIdentifier == '' ) {
        return;
    }

    var ruleEditorInstance = editorInstanceReceived;
    if( ruleEditorInstance == null ){
        if( context ){
            ruleEditorInstance = context;
        }
        else{
            return;
        }
    }
    
    var responseTimeStamp = resultObj.Timestamp;
    var status = resultObj.Status;
    var message = resultObj.Message;

    if( status === 0 ){
       ruleEditorInstance.fireEvent("SaveCompletedSuccessfully", responseTimeStamp);
       
       if( responseTimeStamp == ruleEditorInstance.LastSaveRequestTimestamp){
           ruleEditorInstance.MarkDocumentAsUpToDate();
       }
    }
    else{
        ruleEditorInstance.fireEvent("SaveInError", ""+message, responseTimeStamp);
    }
};
               
// Error from client callback
// or from direct ajax request if used from Java
// occurs when HTTP status is not 200  (server error)
Ilog.RuleEditor.CallbackManagement.OnSaveErrorResponseReceived = function(result, context, ruleEditorInstanceReceived, timestamp){
   // Analyzing expected result
   if( result == null || result == '' ) {
       return;
   }

    // if result is a rule editor, this function is called after a direct ajax call with XHR,
    // and not after a callback handler
   if( result instanceof Ilog.RuleEditor ){ 
        result.fireEvent("SaveInError", "");
        return;               
   }

    // if using the callback handler,
    // the context should contain the editor:
    if( context instanceof Ilog.RuleEditor ){ 
        context.fireEvent("SaveInError", ""); 
        return;               
   }         
};

/// HTTP 200 , response received properly from server
Ilog.RuleEditor.CallbackManagement.OnEditorRefreshSuccessfulResponseReceived = function(result, context, ruleEditorInstanceReceived, timestamp) {
    if (ruleEditorInstanceReceived == null || ruleEditorInstanceReceived == undefined || timestamp == null || timestamp == undefined) {
        if (context != null) {
            ruleEditorInstanceReceived = context;
        }
        else {
            Ilog.Debug.RuleEditorDebugger.PrintDebug('[OnEditorRefreshSuccessfulResponseReceived] Error : no editor');
            return;
        }
    }

    // Analyzing expected result
    // Should get a non-null string with 3 fields delimited
    if (result == null || result == '') {
        Ilog.Debug.RuleEditorDebugger.PrintDebug('[OnEditorRefreshSuccessfulResponseReceived] error : empty result received');
        ruleEditorInstanceReceived.fireEvent('RefreshInError');
        ruleEditorInstanceReceived.OngoingRefreshRequestObject = undefined;
        ruleEditorInstanceReceived.OngoingRefreshRequestTimeStamp = undefined;
        ruleEditorInstanceReceived.OngoingRefreshRequestContext = undefined;
        ruleEditorInstanceReceived.OngoingRefreshRequestDelayedActionIdentifier = undefined;
        return;
    }

    // JSON :
    var resultObj = null;
    try {
        if( result && result.toString().indexOf('$0$') >= 0 ){
            if( ruleEditorInstanceReceived.SelfVariableId ){
                result = result.replace(/$0$/g, ruleEditorInstanceReceived.SelfVariableId);
            }
        }
        
        resultObj = Ext.util.JSON.decode(result);

        if (resultObj == null || resultObj == undefined) {
            Ilog.Debug.RuleEditorDebugger.PrintDebug('[OnEditorRefreshSuccessfulResponseReceived] error : JSON decoding failed : JSON decoding resulted in a null object');
        }
    }
    catch (ex) {
        Ilog.Debug.RuleEditorDebugger.PrintDebug('[OnEditorRefreshSuccessfulResponseReceived] error : JSON decoding failed : ' + ex.message + ' -- ' + result);
        ruleEditorInstanceReceived.fireEvent('RefreshInError');
        ruleEditorInstanceReceived.OngoingRefreshRequestObject = undefined;
        ruleEditorInstanceReceived.OngoingRefreshRequestTimeStamp = undefined;
        ruleEditorInstanceReceived.OngoingRefreshRequestContext = undefined;
        ruleEditorInstanceReceived.OngoingRefreshRequestDelayedActionIdentifier = undefined;
    }

    if (timestamp == null || undefined) {
        if (resultObj.Timestamp) {
            timestamp = resultObj.Timestamp;
        }
    }

    // Discarding "old" request responses
    if (timestamp != ruleEditorInstanceReceived.OngoingRefreshRequestTimeStamp) {
        return;
    }

    if (!resultObj.EditorReference || resultObj.EditorReference == '') {
        ruleEditorInstanceReceived.OngoingRefreshRequestObject = undefined;
        ruleEditorInstanceReceived.OngoingRefreshRequestTimeStamp = undefined;
        ruleEditorInstanceReceived.OngoingRefreshRequestContext = undefined;
        ruleEditorInstanceReceived.OngoingRefreshRequestDelayedActionIdentifier = undefined;
        Ilog.Debug.RuleEditorDebugger.PrintDebug('[OnEditorRefreshSuccessfulResponseReceived] error : EditorReference not found');
        ruleEditorInstanceReceived.fireEvent('RefreshInError');

        return false;
    }

    var editorIdentifier = resultObj.EditorReference;

    var ruleEditorInstance = ruleEditorInstanceReceived;
    if (ruleEditorInstance == null || ruleEditorInstance == undefined) {
        return;
    }

    // First part of result should be a timestamp and match the expected one :
    // if it doesn't match, response is ignored
    var responseTimeStamp = resultObj.Timestamp;

    if (responseTimeStamp != ruleEditorInstance.OngoingRefreshRequestTimeStamp) { // Ilog.RuleEditor.CallbackManagement.GetLastRefreshRequestTimeStampForEditor(editorIdentifier)) {
        return;
    }

    // special error case for asp.net
    if (resultObj.Status != null && resultObj.Status == Ilog.Constants.RuleEditorConstants.ServerReponseStatusCacheMiss) {
        Ilog.Debug.RuleEditorDebugger.PrintDebug('[OnEditorRefreshSuccessfulResponseReceived] Cache miss was detected');
        
        // sending request to asp.net callback,
        // so that cache is filled back in (and request is processed at the same time)
        var timestamp = Ilog.Utilities.GetTimeStamp();
        var editorIdentifier = ruleEditorInstance.editorFieldId;
        var customArgs = new Array();
        if (document.getElementById(ruleEditorInstance.customFieldReference) != null) {
            var ruleGuid = new Array(ruleEditorInstance.customFieldKey, document.getElementById(ruleEditorInstance.customFieldReference).value);
            customArgs.push(ruleGuid);
        }
        var ruleText = ruleEditorInstance.GetPlainText();

        var obj = new Ilog.RuleEditor.CallbackManagement.RefreshRequest(timestamp, editorIdentifier, customArgs, ruleText);

        if ( ruleEditorInstance.SelfVariableId ){
            obj.EditorClientInstanceReference = ruleEditorInstance.SelfVariableId;
        }
        var encoded = Ext.util.JSON.encode(obj);
        window.setTimeout(function() { ruleEditorInstanceReceived.SendEditorRefreshRequestWithAspNetCallback(encoded); }, 15); // setTimeout is used to overcome a MS bug in WebForm_DoCallback js function

        return;
    }

    // special error case for asp.net
    if (resultObj.Status != null && resultObj.Status == Ilog.Constants.RuleEditorConstants.ServerReponseStatusDtCacheMiss) {
        Ilog.Debug.RuleEditorDebugger.PrintDebug('[OnEditorRefreshSuccessfulResponseReceived] DT cache miss was detected');
        ruleEditorInstance.fireEvent("CacheMiss", ruleEditorInstance, ruleEditorInstance.GetEditorState());
        return;
    }

    // Treating error response
    if (resultObj.Status != null && resultObj.Status < 0) {
        Ilog.Debug.RuleEditorDebugger.PrintDebug("[OnEditorRefreshSuccessfulResponseReceived] Server error received : " + resultObj.Message);
        ruleEditorInstanceReceived.fireEvent('RefreshInError');
        ruleEditorInstanceReceived.OngoingRefreshRequestObject = undefined;
        ruleEditorInstanceReceived.OngoingRefreshRequestTimeStamp = undefined;
        ruleEditorInstanceReceived.OngoingRefreshRequestContext = undefined;
        ruleEditorInstanceReceived.OngoingRefreshRequestDelayedActionIdentifier = undefined;
        return;
    }

    ruleEditorInstance.fireEvent("RefreshDoneSuccessfully");
    ruleEditorInstance.TextCurrentlyUpToDate = ruleEditorInstance.GetPlainText();

    // Response is valid, preparing refresh of the editor
    var htmlTextUpdated = resultObj.RuleText;
    var tooltips = resultObj.Tooltips;
    var pbListUpdated = resultObj.ProblemList;

    if (!ruleEditorInstance.RuleTextUpdated) {
        ruleEditorInstance.RuleTextUpdated = new Object();
    }

    if (ruleEditorInstance.RuleTextSent && ruleEditorInstance.RuleTextSent[responseTimeStamp]) {
        var plainText = ruleEditorInstance.RuleTextSent[responseTimeStamp];
        ruleEditorInstance.RuleTextUpdated = new Object();
        ruleEditorInstance.RuleTextUpdated[plainText] = true;
    }

    if (!(resultObj.IgnoreRuleText === true)) {
        ruleEditorInstance.TooltipsToSetOnRefresh = tooltips;
        ruleEditorInstance.HtmlDocumentToDisplayOnRefresh = htmlTextUpdated;
        ruleEditorInstance.UpdateDisplay();
    }

    ruleEditorInstance.SetPbListHtmlContent(pbListUpdated);
    resultObj = undefined;
    htmlTextUpdated = undefined;
    pbListUpdated = undefined;

    var context = ruleEditorInstance.OngoingRefreshRequestContext;
    ruleEditorInstance.OngoingRefreshRequestObject = undefined;
    ruleEditorInstance.OngoingRefreshRequestTimeStamp = undefined;
    ruleEditorInstance.OngoingRefreshRequestDelayedActionIdentifier = undefined;

    // we "jump" to 1st placeholder if a prediction has just been validated
    if (context != null && context.reason == Ilog.Constants.RuleEditorConstants.ContextValidatePrediction && ruleEditorInstance.SentenceCompletion === true) {
        var caretPositionStart = ruleEditorInstance.PredictionsView.ReplacementArea.start;
        var maximumCaretForPlaceholderInPrediction = context.caretPositionAfterPrediction;

        if (ruleEditorInstance.PlaceHolders && ruleEditorInstance.PlaceHolders.length > 0) {
            var placeHolderToSelect = null;
            for (var k = 0; k < ruleEditorInstance.PlaceHolders.length; k++) {
                var currentPlaceHolder = ruleEditorInstance.PlaceHolders[k];

                // Get this placeholder absolute start offset
                var bodyNode;
                if (ruleEditorInstance.contentDocument) {
                    bodyNode = ruleEditorInstance.contentDocument.body;
                }
                else {
                    bodyNode = ruleEditorInstance.editorDocument.body;
                }

                var offset = 0;
                for (var p = 0; p < bodyNode.childNodes.length; p++) {
                    var currentNode = bodyNode.childNodes[p];
                    if (currentNode == currentPlaceHolder) {
                        break;
                    }
                    else if (currentNode.contains && currentNode.contains(currentPlaceHolder)) {
                        break;
                    }
                    else if (currentNode.compareDocumentPosition && (currentPlaceHolder.compareDocumentPosition(currentNode) & 16)) {
                        break;
                    }
                    else {
                        offset += Ilog.Utilities.GetNodeLength(currentNode);
                    }
                }

                if (offset >= caretPositionStart && offset <= maximumCaretForPlaceholderInPrediction) { //&& offset <= maximumCaretForPlaceholderInPrediction ) {
                    placeHolderToSelect = currentPlaceHolder;
                    break;
                }
            }

            if (placeHolderToSelect != null) {
                ruleEditorInstance.SelectPlaceholderNode(placeHolderToSelect);
            }
        }

        ruleEditorInstance.OngoingRefreshRequestContext = undefined;
    }

    resultObj = undefined;
    htmlTextUpdated = undefined;
    pbListUpdated = undefined;
};
    
/// HTTP status != 200 : server error occurred
/// [CALLBACK HANDLER ASPNET]
/// can occur for a request sent after a "cache miss" server response, or when HTTP handler could not respond after a few times
Ilog.RuleEditor.CallbackManagement.OnEditorRefreshErrorResponseReceived = function(result, context){
    var editorInstance = context;
    if (editorInstance != null && editorInstance != undefined) {
        Ilog.Debug.RuleEditorDebugger.PrintDebug("[OnEditorRefreshErrorResponseReceived][CC] error : editorInstance not found");
        editorInstance.fireEvent("RefreshInError"); // for notification
        editorInstance.OngoingRefreshRequestObject = undefined;
        editorInstance.OngoingRefreshRequestTimeStamp = undefined;
        editorInstance.OngoingRefreshRequestContext = undefined;
        editorInstance.OngoingRefreshRequestDelayedActionIdentifier = undefined;
    }
};
    
/// HTTP status != 200 : server error occurred
/// [DIRECT AJAX REQUESTS]
Ilog.RuleEditor.CallbackManagement.OnEditorRefreshDirectAjaxErrorResponseReceived = function(editorInstance, requestObject, timestamp){
    Ilog.Debug.RuleEditorDebugger.PrintDebug('[OnEditorRefreshDirectAjaxErrorResponseReceived] Error received with http status != 200. (' + requestObject.status + ')'); // in this case, requestObject = xmlhttprequest object
    editorInstance.ReportDirectAjaxError();
    
    if( editorInstance != null && timestamp != null ){
        
        if( editorInstance.OngoingRefreshRequestTimeStamp == timestamp ){
            if( editorInstance.IsAspNetCallbackHandlerAvailable === true ){
                editorInstance.SendRefreshRequest();          
            }
            else{
                editorInstance.fireEvent("RefreshInError"); // for notification
                editorInstance.OngoingRefreshRequestObject = undefined;
                editorInstance.OngoingRefreshRequestTimeStamp = undefined;
                editorInstance.OngoingRefreshRequestContext = undefined;
                editorInstance.OngoingRefreshRequestDelayedActionIdentifier = undefined;
            }
        }
    }
};
    
/// HTTP status != 200 : server error occurred
/// [CALLBACK HANDLER ASPNET]
Ilog.RuleEditor.CallbackManagement.OnGetPredictionsErrorResponseReceived = function(result, context) {
    if (context != null && context != undefined) {
        Ilog.Debug.RuleEditorDebugger.PrintDebug('[OnGetPredictionsErrorResponseReceived][CC] '+result);
        context.fireEvent("CompletionInError"); // for notification
    }
};
    
/// HTTP status != 200 : server error occurred
/// [DIRECT AJAX REQUESTS]
Ilog.RuleEditor.CallbackManagement.OnGetPredictionsDirectAjaxErrorResponseReceived = function(editorInstance, requestObject, timestamp, textRange){
    Ilog.Debug.RuleEditorDebugger.PrintDebug('[OnGetPredictionsDirectAjaxErrorResponseReceived] error received with http status != 200. ' + requestObject); // in this case, requestObject = xmlhttprequest object
    editorInstance.ReportDirectAjaxError();
            
    if( editorInstance != null && timestamp != null ){
        if (editorInstance.IsAspNetCallbackHandlerAvailable === true) {
            var ts = Ilog.Utilities.GetTimeStamp();
            editorInstance.LatestGetPredictionsTimeStamp = ts;
            editorInstance.SendGetPredictionsRequest(ts);          
        }
        else if( editorInstance.OngoingPredictionsRequestTimeStamp == timestamp ){
            editorInstance.fireEvent("CompletionInError");
            editorInstance.OngoingPredictionsRequestObject = undefined; 
            editorInstance.OngoingPredictionsRequestTimeStamp = undefined;
            editorInstance.OngoingPredictionsRequestDelayedActionIdentifier = undefined;   
        }
    }
};

Ilog.RuleEditor.CallbackManagement.OnGetPredictionsSuccessfulResponseReceived = function(result, context, ruleEditorInstanceReceived, timestamp, textRange, isPlaceholder, keepTextRangeSelected) {
    if (ruleEditorInstanceReceived == null || ruleEditorInstanceReceived == undefined) {
        if (context != null && context != undefined) {
            ruleEditorInstanceReceived = context;
        }
        else {
            return;
        }
    }

    var scrollPos = { x: 0, y: 0 };
    try {
        scrollPos = { x: ruleEditorInstanceReceived.GetScrollXPosition(), y: ruleEditorInstanceReceived.GetScrollYPosition() };
    }
    catch (ex) {
        Ilog.Debug.RuleEditorDebugger.PrintDebug('[OnGetPredictionsSuccessfulResponseReceived] get scroll position error: ' + ex.message);
    }

    // Should get a non-null string
    if (result == null || result == '') {
        Ilog.Debug.RuleEditorDebugger.PrintDebug('[OnGetPredictionsSuccessfulResponseReceived] error : empty result received');
        ruleEditorInstanceReceived.fireEvent('CompletionInError');
        ruleEditorInstanceReceived.OngoingPredictionsRequestObject = undefined;
        ruleEditorInstanceReceived.OngoingPredictionsRequestTimeStamp = undefined;
        ruleEditorInstanceReceived.OngoingPredictionsRequestDelayedActionIdentifier = undefined;
        return;
    }

    // JSON :            
    var resultObj = null;
    try {
        resultObj = Ext.util.JSON.decode(result);

        if (resultObj == null || resultObj == undefined) {
            Ilog.Debug.RuleEditorDebugger.PrintDebug('[OnGetPredictionsSuccessfulResponseReceived] error : JSON decoding failed : JSON decoding resulted in a null object');
            ruleEditorInstanceReceived.fireEvent('CompletionInError');
            ruleEditorInstanceReceived.OngoingPredictionsRequestObject = undefined;
            ruleEditorInstanceReceived.OngoingPredictionsRequestTimeStamp = undefined;
            ruleEditorInstanceReceived.OngoingPredictionsRequestDelayedActionIdentifier = undefined;
            return false;
        }
    }
    catch (ex) {
        Ilog.Debug.RuleEditorDebugger.PrintDebug('[OnGetPredictionsSuccessfulResponseReceived] error : JSON decoding failed : ' + ex.message + ' -- ' + result);
        ruleEditorInstanceReceived.fireEvent('CompletionInError');
        ruleEditorInstanceReceived.OngoingPredictionsRequestObject = undefined;
        ruleEditorInstanceReceived.OngoingPredictionsRequestTimeStamp = undefined;
        ruleEditorInstanceReceived.OngoingPredictionsRequestDelayedActionIdentifier = undefined;
        return false;
    }

    if (timestamp == null || undefined) {
        if (resultObj.Timestamp) {
            timestamp = resultObj.Timestamp;
        }
    }

    // Discarding "old" request responses
    if (timestamp != ruleEditorInstanceReceived.OngoingPredictionsRequestTimeStamp || timestamp != ruleEditorInstanceReceived.CompletionMenuRequested) {
        return;
    }

    if (resultObj.EditorReference == null || resultObj.EditorReference == '') {
        Ilog.Debug.RuleEditorDebugger.PrintDebug('[OnGetPredictionsSuccessfulResponseReceived] error : EditorReference not found');
        ruleEditorInstanceReceived.fireEvent('CompletionInError');
        ruleEditorInstanceReceived.OngoingPredictionsRequestObject = undefined;
        ruleEditorInstanceReceived.OngoingPredictionsRequestTimeStamp = undefined;
        ruleEditorInstanceReceived.OngoingPredictionsRequestDelayedActionIdentifier = undefined;
        return false;
    }
    else {
        var editorIdentifier = resultObj.EditorReference;

        var ruleEditorInstance = ruleEditorInstanceReceived;
        if (ruleEditorInstance == null) {
            return;
        }

        // special error case for asp.net
        if (resultObj.Status != null && resultObj.Status == Ilog.Constants.RuleEditorConstants.ServerReponseStatusCacheMiss) {
            if (ruleEditorInstance.isEmbeddedInDT === true) {
                ruleEditorInstance.fireEvent("CacheMiss", ruleEditorInstance, ruleEditorInstance.GetEditorState());
            }
            else {
                var timestamp = Ilog.Utilities.GetTimeStamp();
                var replacementRange = ruleEditorInstance.GetTextRange();
                var jsonData = Ilog.RuleEditor.CallbackManagement.GetCallbackArgumentForPredictionsRequest(ruleEditorInstance, timestamp, replacementRange.start, ruleEditorInstance.DtEditionContext, ruleEditorInstance.DtFieldId, ruleEditorInstance.DataCacheEnabled, ruleEditorInstance.DtSessionId, ruleEditorInstance.DtDescriptor); //{start:startIndex, end:currentPosition.end}); // caretPositionForPlaceholder
                window.setTimeout(function() { ruleEditorInstance.SendEditorGetPredictionsRequestWithAspNetCallback(jsonData); }, 15); // setTimeout is used to overcome a MS bug in WebForm_DoCallback js function
            }
            return;
        }

        // special error case for asp.net
        if (resultObj.Status != null && resultObj.Status == Ilog.Constants.RuleEditorConstants.ServerReponseStatusDtCacheMiss) {
            Ilog.Debug.RuleEditorDebugger.PrintDebug('[OnGetPredictionsSuccessfulResponseReceived] DT cache miss was detected');
            var stateToRestore = ruleEditorInstance.GetEditorState();
            stateToRestore.ResendGetPredictions = true;
            ruleEditorInstance.fireEvent("CacheMiss", ruleEditorInstance, stateToRestore);
            return;
        }

        // Treating error response
        if (resultObj.Status != null && resultObj.Status < 0) {
            Ilog.Debug.RuleEditorDebugger.PrintDebug("[OnGetPredictionsSuccessfulResponseReceived] Server error detected : " + resultObj.Message);
            ruleEditorInstanceReceived.fireEvent('CompletionInError');
            ruleEditorInstanceReceived.OngoingPredictionsRequestObject = undefined;
            ruleEditorInstanceReceived.OngoingPredictionsRequestTimeStamp = undefined;
            ruleEditorInstanceReceived.OngoingPredictionsRequestDelayedActionIdentifier = undefined;
            return;
        }

        var responseTimeStamp = resultObj.Timestamp;
        if (responseTimeStamp != Ilog.RuleEditor.CallbackManagement.GetLastPredictionsRequestTimeStampForEditor(editorIdentifier)) {
            return;
        }

        ruleEditorInstance.fireEvent("CompletionDoneSuccessfully");

        var predictionsArray = resultObj.PredictionItems;
        for (var p = 0; p < predictionsArray.length; p++) {
            predictionsArray[p].TextToDisplay = Ext.util.Format.htmlEncode(predictionsArray[p].TextToDisplay);
            predictionsArray[p].ValueToInsert = Ext.util.Format.htmlEncode(predictionsArray[p].ValueToInsert);
        }

        var xy = ruleEditorInstance.GetXYAbsolutePositionOfCaret();
        if (xy == null) {
            return;
        }

        var caretAbsoluteX = xy.x;
        var caretAbsoluteY = xy.y;

        var predictionsView = new Ilog.RuleEditor.PredictionsView(ruleEditorInstance, predictionsArray, caretAbsoluteX, caretAbsoluteY, ruleEditorInstanceReceived.GetTextRange());
        ruleEditorInstanceReceived.predictionsView = predictionsView;

        var textFilter = "";
        var filterOnBeginning = false;
        if ((isPlaceholder === false) && textRange != null) {
            var textRangePreFilter = { start: textRange.start, end: ruleEditorInstanceReceived.GetTextRange().start };
            textFilter = ruleEditorInstanceReceived.GetTextForTextRange(textRangePreFilter);
            filterOnBeginning = true;
        }
        predictionsView.RequestTimestamp = timestamp;
        predictionsView.init(textFilter, filterOnBeginning); // will make the prediction view appear

        if (textRange != null) {
            predictionsView.ReplacementArea = textRange;
            predictionsView.OriginalCaretPosition = ruleEditorInstanceReceived.GetTextRange().start;
        }
        else {
            ruleEditorInstanceReceived.Focus();
            predictionsView.ReplacementArea = ruleEditorInstanceReceived.GetTextRange();
            predictionsView.OriginalCaretPosition = predictionsView.ReplacementArea.start;
        }

        ruleEditorInstanceReceived.Focus();

        // Fix Firefox - DO NOT REMOVE
        // it is used to fix the following bug :
        // when placing caret somewhere inside a node (between the "h" and the "e" of "then", for example), after if we lose focus or change caret position,
        // when trying to place back caret/text range into that node we get an exception if the end of the range/caret is placed after the original position ! 
        // (for example, if we try to place the caret after the "e" or after the "n", or if we try to select the whole word, .....)
        if (Ext.isGecko && !(isPlaceholder === true)) {

            var pos = ruleEditorInstanceReceived.GetTextRange();

            if (keepTextRangeSelected != null && keepTextRangeSelected.start && keepTextRangeSelected.end) {
                pos = keepTextRangeSelected;
            }

            try {
                var startNodeAndOffset = Ilog.Utilities.GetNodeAndRelativeOffsetForAbsoluteOffset(ruleEditorInstanceReceived.contentDocument, textRange.start);
                var endNodeAndOffset = Ilog.Utilities.GetNodeAndRelativeOffsetForAbsoluteOffset(ruleEditorInstanceReceived.contentDocument, textRange.end);

                // The fix consists of simply re-writing the content of the node(s)
                if (startNodeAndOffset.node.parentNode) {
                    startNodeAndOffset.node.parentNode.innerHTML = startNodeAndOffset.node.parentNode.innerHTML;
                }

                if (endNodeAndOffset.node != startNodeAndOffset.node) {
                    if (endNodeAndOffset.node.parentNode) {
                        endNodeAndOffset.node.parentNode.innerHTML = endNodeAndOffset.node.parentNode.innerHTML;
                    }
                }

                ruleEditorInstanceReceived.SelectTextRange({ start: pos.start, end: pos.end });
            }
            catch (ex) {
                Ilog.Debug.RuleEditorDebugger.PrintDebug('[OnGetPredictionsSuccessfulResponseReceived] select range error: ' + ex.message);
            }
        }

        if (keepTextRangeSelected != null && keepTextRangeSelected.start && keepTextRangeSelected.end) {
            predictionsView.ReplacementArea = keepTextRangeSelected;
        }

        if (Ext.isIE && isPlaceholder === true) {
            if (ruleEditorInstance.SelectedPlaceholder) {
                var textRange = ruleEditorInstance.editorDocument.selection.createRange();
                textRange.moveToElementText(ruleEditorInstance.SelectedPlaceholder);
                textRange.select();
            }
            else {
                ruleEditorInstance.SelectTextRange(textRange);
            }
        }

        window.setTimeout(function() {
            ruleEditorInstanceReceived.SetScrollXYPosition(scrollPos.x, scrollPos.y);
        }, 10);

        // IE workaround for EditBar : re-scrolling to quite identical position
        if (Ext.isIE && ruleEditorInstanceReceived.IsMultiline === false) {
            window.setTimeout(function() {
                ruleEditorInstanceReceived.SetScrollXYPosition(scrollPos.x + 1, scrollPos.y);
            }, 15);
        }

        predictionsView.on("ValidatePrediction", ruleEditorInstanceReceived.ValidatePrediction, ruleEditorInstanceReceived);
        predictionsView.on("ValidateValueEditor", ruleEditorInstanceReceived.ValidateValueEditorPrediction, ruleEditorInstanceReceived);
    }
};

Ilog.RuleEditor.CallbackManagement.OnGetValueEditorSuccessfulResponseReceived = function(result, context, ruleEditorInstanceReceived, timestamp, editedValue, valueDomNode, valueType, textrange) {
    if (ruleEditorInstanceReceived == null || ruleEditorInstanceReceived == undefined) {
        if (context != null && context != undefined) {
            ruleEditorInstanceReceived = context;
        }
        else {
            return;
        }
    }
    
    var errorMsg = Ilog.Constants.RuleEditorConstants.ValueEditorError;
    if( valueType ){
        errorMsg = errorMsg.replace(/\{0\}/g, valueType);
    }
                
    // Should get a non-null string
    if (result == null || result == '') {
        try {
            Ext.Msg.show({
               msg: errorMsg,
               buttons: Ext.MessageBox.OK,
               icon: Ext.MessageBox.ERROR
            });
        } catch (ex) {
            Ilog.Debug.RuleEditorDebugger.PrintDebug('[OnGetValueEditorSuccessfulResponseReceived] messagebox#1 error: '+ex.message);
        }
        
        return false;
    }

    // JSON :            
    var resultObj = null;
    try {
        resultObj = Ext.util.JSON.decode(result);
    }
    catch (ex) {
        Ilog.Debug.RuleEditorDebugger.PrintDebug('[OnGetValueEditorSuccessfulResponseReceived] JSON decoding error: ' + ex.message);
        
        try {
            Ext.Msg.show({
               msg: errorMsg,
               buttons: Ext.MessageBox.OK,
               icon: Ext.MessageBox.ERROR
            });
        } catch (ex) {
            Ilog.Debug.RuleEditorDebugger.PrintDebug('[OnGetValueEditorSuccessfulResponseReceived] messagebox#2 error: ' + ex.message);
        }
        return false;
    }

    if (resultObj.ValueEditorKey && resultObj.Script && resultObj.ScriptType && resultObj.EntryPoint) {
        var scriptInjected = Ilog.Utilities.InjectScriptIntoPage(resultObj.ValueEditorKey, resultObj.Script, resultObj.ScriptType);
    }
    else {
        try {
            Ext.Msg.show({
               msg: errorMsg,
               buttons: Ext.MessageBox.OK,
               icon: Ext.MessageBox.ERROR
            });
        } catch (ex) {
            Ilog.Debug.RuleEditorDebugger.PrintDebug('[OnGetValueEditorSuccessfulResponseReceived] messagebox#3 error: ' + ex.message);
        }
        return false;
    }
    
    var displayType = Ilog.Constants.RuleEditorConstants.DisplayModal;
    try{
        if( resultObj.DisplayType != undefined && resultObj.DisplayType != null && resultObj.DisplayType === Ilog.Constants.RuleEditorConstants.DisplayInline ){
            displayType = Ilog.Constants.RuleEditorConstants.DisplayInline;
        }
    }
    catch(ex){}
    
    window.setTimeout(function() {
        ruleEditorInstanceReceived.DisplayValueEditor(resultObj.EntryPoint, displayType, editedValue, valueDomNode, textrange);
    }, 100);
};

// HTTP code != 200
Ilog.RuleEditor.CallbackManagement.OnGetValueEditorDirectAjaxErrorResponseReceived = function(result, context, ruleEditorInstanceReceived, timestamp, editedValue, valueDomNode, valueType){
    var errorMsg = Ilog.Constants.RuleEditorConstants.ValueEditorError;
    if( valueType ){
        errorMsg = errorMsg.replace(/\{0\}/g, valueType);
    }
    // Modal messagebox to let the user know about the error
    Ext.Msg.alert(errorMsg);
};
    
Ilog.RuleEditor.CallbackManagement.GetCallbackArgumentForPredictionsRequest = function(ruleEditorInstance, timestamp, caretPositionForPlaceholder, dtEditionContext, dtArtifactId, dataCacheEnabled, dtSessionId, dtDescriptor, placeholderConcept){
    var plainText = ruleEditorInstance.GetPlainText();
    var sentenceCompletion = 0;
    
    if( ruleEditorInstance.SentenceCompletion ) {
        sentenceCompletion = 1;
    }
    
    if( caretPositionForPlaceholder != null && caretPositionForPlaceholder != undefined ){
        var startSelection = caretPositionForPlaceholder;
        var endSelection = caretPositionForPlaceholder;
    }
    else{
        var pos = ruleEditorInstance.GetTextRange();
        var startSelection = pos.start;
        var endSelection = pos.end;
    }
    
    if( startSelection > plainText.length ){
        startSelection = plainText.length;
    }
    if( endSelection > plainText.length ){
        endSelection = plainText.length;
    }
    
    var customArgs = new Array();
    if (document.getElementById(ruleEditorInstance.customFieldReference) != null) {
        var ruleGuid = new Array(ruleEditorInstance.customFieldKey, document.getElementById(ruleEditorInstance.customFieldReference).value);
        customArgs.push(ruleGuid);
    }
    
    var objRequest = new Ilog.RuleEditor.CallbackManagement.GetPredictionsRequest(timestamp, ruleEditorInstance.editorFieldId, customArgs, plainText, startSelection, endSelection, sentenceCompletion.toString());
    
    if( dtEditionContext != null && dtEditionContext != undefined && dtEditionContext > 0 ){
        objRequest.DtEditionContext = dtEditionContext;
        if( dtArtifactId){
            objRequest.DtFieldId = dtArtifactId;
        }
        if( dtSessionId ){
            objRequest.DtSessionId = dtSessionId;
        }
        if( dtDescriptor ){
            objRequest.DtDescriptor = dtDescriptor;
        }
    }
    
    if( dataCacheEnabled != null && dataCacheEnabled != undefined ){
        objRequest.DataCacheEnabled = dataCacheEnabled;
    }
    
    if( placeholderConcept != null && placeholderConcept != undefined ){
        objRequest.PlaceHolderConcept = placeholderConcept;
    }
        
    var encoded = Ext.util.JSON.encode(objRequest);
    return encoded;
};
    
Ilog.RuleEditor.CallbackManagement.GetCallbackArgumentForValueEditorRequest = function(ruleEditorInstance, timestamp, valueEditorKey){
    var customArgs = new Array();
    if (document.getElementById(ruleEditorInstance.customFieldReference) != null) {
        var ruleGuid = new Array(ruleEditorInstance.customFieldKey, document.getElementById(ruleEditorInstance.customFieldReference).value);
        customArgs.push(ruleGuid);
    }
    
    var objRequest = new Ilog.RuleEditor.CallbackManagement.GetValueEditorRequest(timestamp, ruleEditorInstance.editorFieldId, customArgs, valueEditorKey);
    var encoded = Ext.util.JSON.encode(objRequest);
    return encoded;
};

Ilog.RuleEditor.CallbackManagement.GetCallbackArgumentForNodeDescriptionRequest = function(editorInstance, timestamp, node) {
    if (node == null || node == undefined) {
        Ilog.Debug.RuleEditorDebugger.PrintDebug('[GetCallbackArgumentForNodeDescriptionRequest] error: node is null/undefined');
        return;
    }

    if (node.id == null || node.id == undefined) {
        Ilog.Debug.RuleEditorDebugger.PrintDebug('[GetCallbackArgumentForNodeDescriptionRequest] error: node does not have id');
        return;
    }

    var stringBuilder = new Ilog.Utilities.StringBuilder('<Request>');
    stringBuilder.append('<EditorReference><![CDATA[');
    stringBuilder.append(editorInstance.editorFieldId);
    stringBuilder.append(']]></EditorReference>');
    stringBuilder.append('<Timestamp><![CDATA[');
    stringBuilder.append(timestamp);
    stringBuilder.append(']]></Timestamp>');
    stringBuilder.append('<RequestType><![CDATA[');
    stringBuilder.append(Ilog.Constants.RuleEditorConstants.CallbackStringRequestTypeNodeDescription);
    stringBuilder.append(']]></RequestType>');
    stringBuilder.append('<EditorHtmlRuleText><![CDATA[');
    stringBuilder.append(editorInstance.GetHtmlText());
    stringBuilder.append(']]></EditorHtmlRuleText>');
    stringBuilder.append('<EditorHtmlPlainText><![CDATA[');
    stringBuilder.append(editorInstance.GetPlainText());
    stringBuilder.append(']]></EditorHtmlPlainText>');

    if (editorInstance.customFieldKey && editorInstance.customFieldReference && document.getElementById(editorInstance.customFieldReference) != null) {
        stringBuilder.append('<CustomArgument><Name><![CDATA[');
        stringBuilder.append(editorInstance.customFieldKey);
        stringBuilder.append(']]></Name><Value><![CDATA[');
        stringBuilder.append(document.getElementById(editorInstance.customFieldReference).value);
        stringBuilder.append(']]></Value></CustomArgument>');
    }
    else {
        Ilog.Debug.RuleEditorDebugger.PrintDebug('[GetCallbackArgumentForNodeDescriptionRequest] : ' + editorInstance.customFieldReference + ' could not be found');
    }

    stringBuilder.append('</Request>');
    return stringBuilder.toString();
};
    
Ilog.RuleEditor.CallbackManagement.GetCallbackArgumentForSaveRequest = function(ruleEditorInstance, timestamp){
    var plainText = ruleEditorInstance.GetPlainText();
    var customArgs = new Array();
    if (document.getElementById(ruleEditorInstance.customFieldReference) != null) {
        var ruleGuid = new Array(ruleEditorInstance.customFieldKey, document.getElementById(ruleEditorInstance.customFieldReference).value);
        customArgs.push(ruleGuid);
    }
            
    var objRequest = new Ilog.RuleEditor.CallbackManagement.SaveRequest(timestamp, ruleEditorInstance.editorFieldId, customArgs, plainText);
    var encoded = Ext.util.JSON.encode(objRequest);
    return { argument:encoded, text:plainText };
};
