// --------------------------------------------------------------- -*- JS -*-
// IBM Confidential
// OCO Source Materials
// 5724-Y00 5724-Y02
//  Copyright IBM Corp. 2004, 2010. All Rights Reserved.
// The source code for this program is not published or otherwise
// divested of its trade secrets, irrespective of what has
// been deposited with the U.S. Copyright Office.
// --------------------------------------------------------------------------

Ext.namespace('Ilog', 'Ilog.RuleEditor');

Ilog.RuleEditor.PredictionsView = function(editorInstance, predictionsToShow, xCoordinate, yCoordinate, editorTextRange) {
    this.EditorInstance = editorInstance;
    this.EditorIdentifier = editorInstance.editorFieldId;
    editorInstance.PredictionsView = this;
    this.Identifier = this.EditorIdentifier + '_PredictionsView';

    // Paginated mode (large number of items)
    // displaying first page
    if (predictionsToShow && predictionsToShow.length >= Ilog.Constants.RuleEditorConstants.PaginationThreshold && Ilog.Constants.RuleEditorConstants.UsePagination === true) {
        this.Paginated = true;
        this.TotalItems = predictionsToShow.length;
        this.AllPredictionsUnfiltered = predictionsToShow;
        this.NbPages = Math.floor(predictionsToShow.length / Ilog.Constants.RuleEditorConstants.PageSize);
        if (this.NbPages * Ilog.Constants.RuleEditorConstants.PageSize < predictionsToShow.length) {
            this.NbPages++;
        }
        this.CurrentPageIndex = 0;
        predictionsToShow = new Array();
        for (var k = 0; k < Ilog.Constants.RuleEditorConstants.PageSize; k++) {
            predictionsToShow.push(this.AllPredictionsUnfiltered[k]);
        }
    }

    this.OriginalPredictions = predictionsToShow;
    this.Predictions = predictionsToShow; // current predictions = predictions received
    this.X = xCoordinate;
    this.Y = yCoordinate;
    this.EditorTextRange = editorTextRange;
    this.CurrentSelectedPredictionIndex = -1;
    this.PredictionsLayer = null; // to be created when init()
    this.FilterBox = null; // to be created when init()

    if (!Ilog.Constants.RuleEditorConstants.CurrentPredictionViewPresentationMode) {
        var presentationMode = Ilog.Constants.RuleEditorConstants.PredictionPresentationModeFlat; 
        this.CurrentPresentationMode = presentationMode;
        Ilog.Constants.RuleEditorConstants.CurrentPredictionViewPresentationMode = presentationMode;
    }
    else {
        this.CurrentPresentationMode = Ilog.Constants.RuleEditorConstants.CurrentPredictionViewPresentationMode;
    }

    if (Ilog.Constants.RuleEditorConstants.CurrentPredictionViewSearchMode == null || Ilog.Constants.RuleEditorConstants.CurrentPredictionViewSearchMode == undefined) {
        var searchMode = Ilog.Constants.RuleEditorConstants.SearchDisabled;
        this.CurrentSearchMode = searchMode;
        Ilog.Constants.RuleEditorConstants.CurrentPredictionViewSearchMode = searchMode;
    }
    else {
        this.CurrentSearchMode = Ilog.Constants.RuleEditorConstants.CurrentPredictionViewSearchMode;
    }

    this.CurrentSearchFilter = '';
};

Ext.extend(Ilog.RuleEditor.PredictionsView, Ext.util.Observable, {

    HasPredictions: function() {
        if (this.Predictions != null && this.Predictions.length > 0) {
            return true;
        }

        return false;
    },

    UpdateFilterBoxValue: function(filterValue) {
        if (this.FilterBox != null && this.FilterBox != undefined) {
            this.FilterBox.value = filterValue;
        }
    },

    GetCurrentSelectedPredictionValue: function() {
        if (this.CurrentSelectedPredictionIndex != null && this.CurrentSelectedPredictionIndex >= 0) {
            if (this.PredictionsIndexesMapping && this.CurrentPresentationMode == Ilog.Constants.RuleEditorConstants.PredictionPresentationModeTreeview) {
                var val = this.Predictions[this.PredictionsIndexesMapping[this.CurrentSelectedPredictionIndex]];
                if (val != null && val != undefined) {
                    return val;
                }
                else {
                    return '';
                }
            }

            if (this.Predictions != null && this.Predictions[this.CurrentSelectedPredictionIndex] != null) {
                var val = this.Predictions[this.CurrentSelectedPredictionIndex];
                return val;
            }
            else {
                return null;
            }
        }
        else {
            return null;
        }
    },

    GetCurrentSelectedPredictionValueEditor: function() {
        if (this.CurrentSelectedPredictionIndex != null && this.CurrentSelectedPredictionIndex >= 0) {
            if (this.PredictionsIndexesMapping && this.CurrentPresentationMode == Ilog.Constants.RuleEditorConstants.PredictionPresentationModeTreeview) {
                var val = this.Predictions[this.PredictionsIndexesMapping[this.CurrentSelectedPredictionIndex]].ValueEditor;
                if (val != null && val != undefined) {
                    return val;
                }
                else {
                    return '';
                }
            }

            if (this.Predictions != null && this.Predictions[this.CurrentSelectedPredictionIndex] != null) {
                var val = this.Predictions[this.CurrentSelectedPredictionIndex].ValueEditor;
                return val;
            }
            else {
                return null;
            }
        }
        else {
            return null;
        }
    },


    HasSelectedPrediction: function() {
        if (this.CurrentSelectedPredictionIndex != null && this.CurrentSelectedPredictionIndex >= 0) {
            return true;
        }
        else {
            return false;
        }
    },

    GetTotalNbItems: function() {
        if (this.Predictions) {
            return this.Predictions.length;
        }
        return 0;
    },

    SelectLastPredictionItem: function() {
        var length = this.GetTotalNbItems();
        this.CurrentSelectedPredictionIndex = length - 1;
        this.HighlightItem(length - 1);
    },

    HighlightItem: function(itemIndex, notUseMapping) {
        var itemIndexForSelection = itemIndex;
        var predictionsTableId = this.Identifier + "_PredictionsTable";
        var containerDiv = document.getElementById(predictionsTableId)

        // unselecting the item previously selected
        var itemIndexForUnselection = this.CurrentHighlightedItemIndex;
        var itemIndexForUnselection2 = this.CurrentSelectedPredictionIndex;

        var previouslySelectedPredictionIdentifier = this.Identifier + "_Container_Prediction_" + itemIndexForUnselection;
        var elt = document.getElementById(previouslySelectedPredictionIdentifier);
        if (elt) {
            elt.className = 'ilog-predictions-row-unselected';

            if (this.CurrentPresentationMode == Ilog.Constants.RuleEditorConstants.PredictionPresentationModeTreeview) {
                if (elt.childNodes) {
                    for (var p = 0; p < elt.childNodes.length; p++) {
                        var childElt = new Ext.Element(elt.childNodes[p]);
                        childElt.removeClass('ilog-predictions-row-selected');
                    }
                }
            }
        }
        if (itemIndexForUnselection2) {
            var previouslySelectedPredictionIdentifier2 = this.Identifier + "_Container_Prediction_" + itemIndexForUnselection2;

            var elt2 = document.getElementById(previouslySelectedPredictionIdentifier2);
            if (elt2) {
                elt2.className = 'ilog-predictions-row-unselected';

                if (this.CurrentPresentationMode == Ilog.Constants.RuleEditorConstants.PredictionPresentationModeTreeview) {
                    if (elt2.childNodes) {
                        for (var p = 0; p < elt2.childNodes.length; p++) {
                            var childElt = new Ext.Element(elt2.childNodes[p]);
                            childElt.removeClass('ilog-predictions-row-selected');
                        }
                    }
                }
            }
        }

        // highlighting the item at the selected index
        var predictionIdentifier = this.Identifier + "_Container_Prediction_" + itemIndexForSelection;
        var elm = new Ext.Element(predictionIdentifier);

        if (elm && elm.dom) {
            // changing style to highlight
            elm.dom.className = 'ilog-predictions-row-selected';

            if (this.CurrentPresentationMode == Ilog.Constants.RuleEditorConstants.PredictionPresentationModeTreeview) {
                if (elm.dom.childNodes) {
                    for (var p = 0; p < elm.dom.childNodes.length; p++) {
                        var childElt = new Ext.Element(elm.dom.childNodes[p]);
                        childElt.addClass('ilog-predictions-row-selected');
                    }
                }
            }

            // scrolling manually if need, so that the newly selected item is visible
            var predictionsTableId = this.Identifier + "_PredictionsTable";
            var containerDiv = document.getElementById(predictionsTableId)
            var height = containerDiv.offsetHeight;
            var scrollTop = containerDiv.scrollTop;
            var scrollBottom = containerDiv.scrollBottom;
            var selectedElementTop = elm.dom.offsetTop;
            if (this.CurrentPresentationMode == Ilog.Constants.RuleEditorConstants.PredictionPresentationModeTreeview) {
                selectedElementTop = selectedElementTop + elm.dom.parentNode.parentNode.offsetTop;
            }
            var selectedElementHeight = elm.getHeight();
            var elementBottom = selectedElementTop + selectedElementHeight;
            var currentViewBottom = scrollTop + height;
            if (elementBottom > currentViewBottom) {
                containerDiv.scrollTop = selectedElementTop;
            }
            else if (selectedElementTop < (scrollTop - 5)) {
                containerDiv.scrollTop = selectedElementTop;
            }
        }

        var tableId = this.Identifier + '_predictionsContentTable';
        var table = document.getElementById(tableId);
        if (table) {
            table.className = 'ilog-predictions-content-table';
        }

        if (this.CurrentHighlightedItemIndex == itemIndex) {
            return;
        }



        this.CurrentHighlightedItemIndex = itemIndex;
    },

    UpdateSelectedIndex: function(indexUpdate) {
        var length = this.GetTotalNbItems();

        if (this.CurrentSelectedPredictionIndex >= length) {
            this.CurrentSelectedPredictionIndex = 0;
        }
        if (indexUpdate == 1) {
            this.CurrentSelectedPredictionIndex += 1;
            if (this.CurrentSelectedPredictionIndex > length - 1) { this.CurrentSelectedPredictionIndex = 0; }
        }
        else if (indexUpdate == -1) {
            this.CurrentSelectedPredictionIndex -= 1;
            if (this.CurrentSelectedPredictionIndex < 0) { this.CurrentSelectedPredictionIndex = length - 1; }
        }

        this.HighlightItem(this.CurrentSelectedPredictionIndex);
    },

    DoPageUp: function() {
        // computing nb items per page
        var predictionIdentifier = this.Identifier + "_Container_Prediction_0"
        var predictionsTableId = this.Identifier + "_PredictionsTable";
        var extElmFirstPred = new Ext.Element(predictionIdentifier);
        var extElmPredictionTable = new Ext.Element(predictionsTableId);
        if (extElmFirstPred == null || extElmFirstPred == undefined || extElmPredictionTable == null || extElmPredictionTable == undefined) {
            return;
        }

        var nbItemsPerPage = Math.floor(extElmPredictionTable.getHeight() / extElmFirstPred.getHeight());

        var index = 0;
        if (this.CurrentSelectedPredictionIndex != null || this.CurrentSelectedPredictionIndex >= 0) {
            index = this.CurrentSelectedPredictionIndex;
        }
        index = index - nbItemsPerPage;

        if (index < 0) {
            index = 0;
        }

        this.CurrentSelectedPredictionIndex = index;
        this.HighlightItem(index);
    },

    DoPageDown: function() {
        // computing nb items per page
        var predictionIdentifier = this.Identifier + "_Container_Prediction_0"
        var predictionsTableId = this.Identifier + "_PredictionsTable";
        var extElmFirstPred = new Ext.Element(predictionIdentifier);
        var extElmPredictionTable = new Ext.Element(predictionsTableId);
        if (extElmFirstPred == null || extElmFirstPred == undefined || extElmPredictionTable == null || extElmPredictionTable == undefined) {
            return;
        }

        var nbItemsPerPage = Math.floor(extElmPredictionTable.getHeight() / extElmFirstPred.getHeight());

        var index = 0;
        if (this.CurrentSelectedPredictionIndex != null || this.CurrentSelectedPredictionIndex >= 0) {
            index = this.CurrentSelectedPredictionIndex;
        }
        index = index + nbItemsPerPage;

        if (index > (this.GetTotalNbItems() - 1)) {
            index = this.GetTotalNbItems() - 1;
        }

        this.CurrentSelectedPredictionIndex = index;
        this.HighlightItem(index);
    },

    Hide: function() {
        var predictionsViewLayer = document.getElementById(this.Identifier);
        if (predictionsViewLayer != null) {
            this.DisableTransparency();
            predictionsViewLayer.style.visibility = 'hidden';
            this.Visible = false;
            return true;
        }

        return false;
    },

    Show: function() {
        var predictionsViewLayer = document.getElementById(this.Identifier);

        if (predictionsViewLayer != null) {
            predictionsViewLayer.style.visibility = 'visible';
            this.Visible = true;
            return true;
        }

        return false;
    },

    HandleScroll: function(extEvt) {
        /*
        // only setting the text range at its current location:
        // it does nothing but after that, events fire correctly
        // whereas if we do nothing, editor events are not fired anymore 
        // (and so the prediction view cannot be closed using by using the keyboard shortcut, selecting items with the keyboard doesn't work anymore...)
        try {
        var editor = this.EditorInstance;
        var range = editor.GetTextRange();
        editor.SelectTextRange(range);
        } catch (ex) {
        Ilog.Debug.RuleEditorDebugger.PrintDebug('[HandleScroll] error: ' + ex.message);
        }*/
    },

    DisplayWaitingAnimation: function() {
        var waitingDiv;
        var predictionsTableId = this.Identifier + "_PredictionsTable";
        var predictionsTable = document.getElementById(predictionsTableId);
        if (predictionsTable) {
            predictionsTable.disabled = true;

            if (predictionsTable.offsetHeight > 35) {
                var waitingDivIdentifier = this.Identifier + "_PredictionsFilterImg";
                var waitingDiv = document.getElementById(waitingDivIdentifier);

                if (waitingDiv != null && waitingDiv != undefined) {
                    waitingDiv.style.visibility = 'visible';
                }
                else {
                    waitingDiv = document.createElement('div');
                    waitingDiv.id = waitingDivIdentifier;
                    var imgURl = Ilog.Constants.RuleEditorConstants.CompletionFilteringImgUrl;
                    waitingDiv.innerHTML = '<img src="' + imgURl + '" />';
                    waitingDiv.style.background = '';
                    waitingDiv.zindex = 999;
                    waitingDiv.style.zindex = 999;
                    waitingDiv.style.position = 'absolute';
                    document.body.appendChild(waitingDiv);
                    waitingDiv.style.visibility = 'visible';
                }

                var predictionsTableId = this.Identifier + "_PredictionsTable";
                var predictionsTable = document.getElementById(predictionsTableId);

                var extElm = new Ext.Element(predictionsTableId);
                var top = extElm.getTop();
                var left = extElm.getLeft();
                waitingDiv.style.top = top + predictionsTable.offsetHeight / 2 - 16 + 'px';
                waitingDiv.style.left = left + predictionsTable.offsetWidth / 2 - 16 + 'px';
                waitingDiv.style.visibility = 'visible';
            }
        }
    },

    HideWaitingAnimation: function() {
        var predictionsTableId = this.Identifier + "_PredictionsTable";
        var predictionsTable = document.getElementById(predictionsTableId);
        if (predictionsTable) {
            predictionsTable.disabled = false;
        }

        var waitingDivIdentifier = this.Identifier + "_PredictionsFilterImg";
        var waitingDiv = document.getElementById(waitingDivIdentifier);

        if (waitingDiv != null && waitingDiv != undefined) {
            document.body.removeChild(waitingDiv);
        }
    },

    RepaintPredictions: function(changeSize) {
        var currentEditorTextRange = this.EditorInstance.GetTextRange();
        this.CommandsLocked = true;
        this.DisplayWaitingAnimation();

        var self = this;
        if (this.RepaintPredictionThreadId) {
            window.clearTimeout(this.RepaintPredictionThreadId);
        }

        this.RepaintPredictionThreadId = window.setTimeout(function() {
            self.UpdatePredictionsLayer();
            self.HideWaitingAnimation();

            // Setting back focus to the text buffer, because the click caused it to lose focus             
            if (self.EditorInstance.contentDocument) {
                self.EditorInstance.contentWindow.focus();
                self.EditorInstance.SelectTextRange(self.ReplacementArea);
            }
            else {
                self.EditorInstance.editorDocument.body.focus();
                self.EditorInstance.SelectTextRange(self.ReplacementArea);
                self.EditorInstance.editorDocument.body.focus();
            }

            self.CommandsLocked = false;
            if (changeSize === true) {
                self.Resize();
            }
        }, 0);
    },

    HandleToggleTreeviewModeClick: function(evt) {
        if (this.CommandsLocked === true) {
            return;
        }

        var editorId = "'" + this.EditorIdentifier + "'";
        var toggleToggleTreeviewModeCommandId = "predictionWindowToggleTreeviewCommand_" + editorId;
        var elt = document.getElementById(toggleToggleTreeviewModeCommandId);
        if (!elt) {
            return;
        }

        if (this.CurrentPresentationMode == Ilog.Constants.RuleEditorConstants.PredictionPresentationModeTreeview) {
            elt.className = 'ilog-predictionView-TopCaptionIcon';
            this.CurrentPresentationMode = Ilog.Constants.RuleEditorConstants.PredictionPresentationModeFlat;
            Ilog.Constants.RuleEditorConstants.CurrentPredictionViewPresentationMode = this.CurrentPresentationMode;
            this.RepaintPredictions(true);
        }
        else {
            elt.className = 'ilog-predictionView-TopCaptionIconSelected';
            this.CurrentPresentationMode = Ilog.Constants.RuleEditorConstants.PredictionPresentationModeTreeview;
            Ilog.Constants.RuleEditorConstants.CurrentPredictionViewPresentationMode = this.CurrentPresentationMode;
            this.RepaintPredictions(true);
        }

        this.CurrentSelectedPredictionIndex = -1;
    },

    HideSearch: function() {
        var editorId = "'" + this.EditorIdentifier + "'";
        var toggleToggleTreeviewModeCommandId = "predictionWindowToggleSearchCommand_" + editorId;
        var elt = document.getElementById(toggleToggleTreeviewModeCommandId);
        if (!elt) {
            return;
        }
        elt.className = 'ilog-predictionView-TopCaptionIcon';
        this.CurrentSearchMode = Ilog.Constants.RuleEditorConstants.SearchDisabled;
        Ilog.Constants.RuleEditorConstants.CurrentPredictionViewSearchMode = this.CurrentSearchMode;
        this.HidePredictionFilterArea();

        this.Resize();
    },

    ShowSearch: function() {
        var editorId = "'" + this.EditorIdentifier + "'";
        var toggleToggleTreeviewModeCommandId = "predictionWindowToggleSearchCommand_" + editorId;
        var elt = document.getElementById(toggleToggleTreeviewModeCommandId);
        if (!elt) {
            return;
        }
        elt.className = 'ilog-predictionView-TopCaptionIconSelected';
        this.CurrentSearchMode = Ilog.Constants.RuleEditorConstants.SearchEnabled;
        Ilog.Constants.RuleEditorConstants.CurrentPredictionViewSearchMode = this.CurrentSearchMode;
        this.ShowPredictionFilterArea();

        this.Resize();

        try {
            if (this.FilterBox) {
                this.FilterBox.focus();
            }
        }
        catch (ex) {
            Ilog.Debug.RuleEditorDebugger.PrintDebug('[ShowSearch] error on focus filter: ' + ex.message);
        }
    },

    Resize: function() {
        try {
            var predictionsTableId = this.Identifier + "_PredictionsTable";
            var predDiv = document.getElementById(predictionsTableId);
            var browserWindowDimensions = Ilog.Utilities.GetBrowserWindowDimensions();
            var extElm = new Ext.Element(this.PredictionsLayer);
            var topRef = extElm.getTop();
            var maxVal = browserWindowDimensions.height + Ilog.Utilities.GetDocumentScrollTop() - topRef - 2;
            var currentPredHeight = (new Ext.Element(predDiv)).getHeight();

            // minimum size to be displayed, so that it is usable
            if (maxVal < Ilog.Constants.RuleEditorConstants.CompletionMinimumHeight) {
                maxVal = Ilog.Constants.RuleEditorConstants.CompletionMinimumHeight;
            }

            var higherDiv = this.PredictionsLayer;
            var topCaptionIdentifier = this.Identifier + "_TopCaption";
            var topElt = new Ext.Element(topCaptionIdentifier);

            var searchDivIdentifier = this.Identifier + "_FilterArea";
            var searchElt = new Ext.Element(searchDivIdentifier);
            var headerHeight = topElt.getHeight();
            var searchHeight = (searchElt && searchElt.isVisible() === true && searchElt.isDisplayed() === true) ? searchElt.getHeight() : 0;


            // update width if needed for search area
            try {
                if (searchElt && searchElt.isVisible() === true && searchElt.isDisplayed() === true) {
                    var predictionsFilterLabelId = this.Identifier + "_FilterLabel";
                    var predictionsFilterInputId = this.Identifier + "_FilterInput";
                    var searchLabelElt = new Ext.Element(predictionsFilterLabelId);
                    var searchInputElt = new Ext.Element(predictionsFilterInputId);
                    var searchLabelWidth = searchLabelElt.getWidth();
                    var searchInputWidth = searchInputElt.getWidth();
                    var currentGlobalWidth = (new Ext.Element(this.PredictionsLayer)).getWidth();
                    var minSearch = searchLabelWidth + searchInputWidth + Ilog.Constants.RuleEditorConstants.SearchAreaMargin;
                    if (minSearch > currentGlobalWidth) {
                        this.PredictionsLayer.style.width = minSearch + 'px';
                    }
                }
            }
            catch (exc) { }

            var bottomHeight = 0;
            if (this.Paginated === true) {
                var bottomIdentifier = this.Identifier + "_PredictionsBottomLayer";
                var bottomLayer = document.getElementById(bottomIdentifier)
                if (bottomLayer != null && bottomLayer != undefined && bottomLayer.style && bottomLayer.style.display != 'none') {
                    var bottomElt = new Ext.Element(bottomIdentifier);
                    bottomHeight = (bottomElt) ? bottomElt.getHeight() : 0;
                }
            }

            // no resize if the completion menu already fits in the remaining space
            var currentTotalHeight = headerHeight + searchHeight + currentPredHeight + bottomHeight;
            var proposedHeight = currentTotalHeight;

            if (this.Paginated === true) {
                if (this.PaginatedHeight) {
                    proposedHeight = this.PaginatedHeight;
                }
            }

            // no resize needed
            if (proposedHeight < maxVal) {
                if (this.Paginated === true) {
                    predDiv.style.height = (proposedHeight - (headerHeight + searchHeight + bottomHeight)) + 'px';
                    this.PaginatedHeight = proposedHeight;
                }
            }
            else {
                var heightPredictions = maxVal - (headerHeight + searchHeight + bottomHeight);
                predDiv.style.height = heightPredictions + 'px';
                predDiv.style.overflowY = "scroll";
                if (this.Paginated === true) {
                    this.PaginatedHeight = maxVal;
                }
            }
        }
        catch (ex) {
            Ilog.Debug.RuleEditorDebugger.PrintDebug('[Resize] error: ' + ex.message);
        }
    },

    HandleToggleSearchClick: function(evt) {
        if (this.CommandsLocked === true) {
            return;
        }

        if (this.CurrentSearchMode == Ilog.Constants.RuleEditorConstants.SearchEnabled) {
            this.FilterBox.value = '';
            this.CurrentSearchFilter = '';
            this.TriggerFiltering();
            this.HideSearch();
        }
        else {
            this.ShowSearch();
        }
    },

    HandleToggleSearchMouseOut: function(evt) {
        var editorId = "'" + this.EditorIdentifier + "'";
        var toggleToggleSearchCommandId = "predictionWindowToggleSearchCommand_" + editorId;
        var elt = document.getElementById(toggleToggleSearchCommandId);
        if (!elt) {
            return;
        }

        if (this.CurrentSearchMode == Ilog.Constants.RuleEditorConstants.SearchDisabled) {
            elt.className = 'ilog-predictionView-TopCaptionIcon';
        }
    },

    HandleToggleTreeviewModeMouseOut: function(evt) {
        var editorId = "'" + this.EditorIdentifier + "'";
        var toggleToggleTreeviewModeCommandId = "predictionWindowToggleTreeviewCommand_" + editorId;
        var elt = document.getElementById(toggleToggleTreeviewModeCommandId);
        if (!elt) {
            return;
        }

        if (this.CurrentPresentationMode == Ilog.Constants.RuleEditorConstants.PredictionPresentationModeFlat) {
            elt.className = 'ilog-predictionView-TopCaptionIcon';
        }
    },

    HandleLayerMouseDown: function(evt) {
        Ilog.Utilities.CancelEventBehaviour(evt);
        Ilog.Utilities.StopEventPropagation(evt);
        return false;
    },

    HandleClick: function(evt) {
        if (evt.type && (evt.type.toString() == "mouseout" || evt.type.toString() == "mousemove")) {
            return;
        }

        if (this.CommandsLocked === true) {
            return;
        }

        // doing nothing if div is disabled (during filtering)
        var predictionsTableId = this.Identifier + "_PredictionsTable";
        var predictionsTable = document.getElementById(predictionsTableId);
        if (predictionsTable && predictionsTable.disabled) {
            Ilog.Utilities.CancelEventBehaviour(evt);
            return false;
        }

        var predictionViewInstance = this;

        var ie_var = "srcElement";
        var moz_var = "target";

        var useFilterModeKey = "usingFilterMode";
        evt[moz_var] ? e_out = evt[moz_var][useFilterModeKey] : e_out = evt[ie_var][useFilterModeKey];
        var useFilterMode = e_out;

        var predictionIndexKey = "predictionIndex";
        evt[moz_var] ? e_out = evt[moz_var][predictionIndexKey] : e_out = evt[ie_var][predictionIndexKey];
        var predictionIndex = e_out;

        var predictionIndexValueKey = "predictionIndexValue";
        evt[moz_var] ? e_out = evt[moz_var][predictionIndexValueKey] : e_out = evt[ie_var][predictionIndexValueKey];
        var predictionIndexValue = e_out;

        var displayIndexKey = "displayIndex";
        evt[moz_var] ? e_out = evt[moz_var][displayIndexKey] : e_out = evt[ie_var][displayIndexKey];
        var displayIndex = e_out;

        // Retrieving selected prediction
        var predIndex = (this.PredictionsIndexesMapping && this.CurrentPresentationMode == Ilog.Constants.RuleEditorConstants.PredictionPresentationModeTreeview) ? displayIndex : predictionIndexValue;
        predictionObj = this.Predictions[predIndex];

        if (predictionObj == null || predictionObj == undefined) {
            return false;
        }

        // Predictions that involve a value editor:
        // firing special event
        try {
            if (predictionObj && predictionObj.ValueEditor) {
                this.Hide();
                this.fireEvent("ValidateValueEditor", predictionObj.ValueEditor);
                return false;
            }
        }
        catch (ex) {
            Ilog.Debug.RuleEditorDebugger.PrintDebug('[ HandleClick] error: on validate value editor: ' + ex.message);
        }

        var predictionValue = (useFilterMode === true && this.FilteredPredictionsValues) ? this.FilteredPredictionsValues[predIndex] : predictionObj.ValueToInsert;
        if (predictionValue == null || predictionValue == undefined) {
            return;
        }

        // Sending prediction value to the editor through event argument
        predictionValue = predictionValue.replace(/\r\n/g, '<br/>').replace(/\n/g, '<br/>').replace(/\t/g, '&nbsp;&nbsp;&nbsp;');
        predictionObj.ValueToInsert = predictionValue; // important
        this.Hide();
        this.fireEvent("ValidatePrediction", predictionObj, false);
        return false;
    },

    HandleMouseOver: function(evt) {
        // doing nothing if div is disabled (during filtering)
        var predictionsTableId = this.Identifier + "_PredictionsTable";
        var predictionsTable = document.getElementById(predictionsTableId);
        if (predictionsTable && predictionsTable.disabled) {
            Ilog.Utilities.CancelEventBehaviour(evt);
            return false;
        }

        var ie_var = "srcElement";
        var moz_var = "target";
        var predictionIndexKey = "predictionIndex";
        var displayIndexKey = "displayIndex";
        try {
            evt[moz_var] ? e_out = evt[moz_var][predictionIndexKey] : e_out = evt[ie_var][predictionIndexKey];
            var predictionIndex = e_out;
            evt[moz_var] ? e_out = evt[moz_var][displayIndexKey] : e_out = evt[ie_var][displayIndexKey];
            var displayIndex = e_out;
        }
        catch (ex) {
            Ilog.Debug.RuleEditorDebugger.PrintDebug('[HandleMouseOver] error on get prediction: ' + ex.message);
            return;
        }
        var predictionViewInstance = this;

        if (predictionIndex == null || predictionIndex == undefined) {
            return;
        }

        this.HighlightItem(predictionIndex, true);
        this.CurrentSelectedPredictionIndex = predictionIndex;
    },

    HandleMouseOut: function(evt) {
        var ie_var = "srcElement";
        var moz_var = "target";
        var predictionIndexKey = "predictionIndex";
        evt[moz_var] ? e_out = evt[moz_var][predictionIndexKey] : e_out = evt[ie_var][predictionIndexKey];
        var predictionIndex = e_out;

        var predictionViewInstance = this;

        // Unselecting prediction (un-highlight)
        for (var p = 0; p < predictionViewInstance.Predictions.length; p++) {
            var predictionIdentifier = predictionViewInstance.Identifier + "_Container_Prediction_" + p;
            if (p == predictionIndex) {
                if (document.getElementById(predictionIdentifier)) {
                    document.getElementById(predictionIdentifier).className = 'ilog-predictions-row-unselected';
                }
            }
        }
    },

    BuildPredictions: function(filterOnlyOnBeginning) {
        var PredictionsInnerHTML = '<table width="100%" cellpadding="0px" cellspacing="0px" class="ilog-predictions-content-table" id="' + this.Identifier + '_predictionsContentTable">';
        var maxTextLength = 0;

        var predictionsToWork = this.OriginalPredictions; // represents all predictions unfiltered if not paginated,  and 1st page of items if paginated
        if (this.Paginated === true) {
            predictionsToWork = this.AllPredictionsUnfiltered;
        }
        var allMatchingFilteredPredictions;
        
        if (this.CurrentPresentationMode == Ilog.Constants.RuleEditorConstants.PredictionPresentationModeTreeview) {
        	// if the display mode is 'tree view'
            // sort the predictions by type and then by concept to be able to split the tree correctly if it is paginated
            predictionsToWork.sort(function(a, b) {
                if (a.PredictionType != b.PredictionType) {
                    if (a.PredictionType === 1) {
                        return 1;
                    } else if (b.PredictionType === 1) {
                        return -1;
                    } else {
                        return a.PredictionType - b.PredictionType;
                    }
                } else {
                    if (a.Concept === null && b.Concept === null) {
                        return 0;
                    } else if (a.Concept === null) {
                        return 1;
                    } else if (b.Concept === null) {
                        return -1;
                    } else {
                        if (a.Concept === b.Concept) {
                            return 0;
                        } else if (a.Concept < b.Concept) {
                            return -1;
                        } else {
                            return 1;
                        }
                    }
                }
            });
        } else {
        	// otherwise sort the predictions alphabetically and ascending
        	predictionsToWork.sort(function(a, b) {
        		if (a.TextToDisplay == b.TextToDisplay) {
        			return 0;
        		} else {
        			return a.TextToDisplay < b.TextToDisplay ? -1 : 1; 
        		}
        	});
        }

        if (this.CurrentSearchFilter != '') {
            if (this.Paginated === true) {
                allMatchingFilteredPredictions = this.FilterPredictions(filterOnlyOnBeginning, predictionsToWork);
                this.NbPages = Math.floor(allMatchingFilteredPredictions.length / Ilog.Constants.RuleEditorConstants.PageSize);
                if (this.NbPages * Ilog.Constants.RuleEditorConstants.PageSize < allMatchingFilteredPredictions.length) {
                    this.NbPages++;

                }
                predictionsToWork = new Array();

                // computing page
                if (this.CurrentPageIndex != 0) {
                    var firstItemIndex = Ilog.Constants.RuleEditorConstants.PageSize * this.CurrentPageIndex;
                    var max = (allMatchingFilteredPredictions.length > (firstItemIndex + Ilog.Constants.RuleEditorConstants.PageSize)) ? firstItemIndex + Ilog.Constants.RuleEditorConstants.PageSize : allMatchingFilteredPredictions.length;
                }
                else {
                    var firstItemIndex = 0;
                    var max = (allMatchingFilteredPredictions.length > Ilog.Constants.RuleEditorConstants.PageSize) ? Ilog.Constants.RuleEditorConstants.PageSize : allMatchingFilteredPredictions.length;
                }
                for (var k = firstItemIndex; k < max; k++) {
                    try {
                        predictionsToWork.push(allMatchingFilteredPredictions[k]);
                    }
                    catch (ex) {
                        Ilog.Debug.RuleEditorDebugger.PrintDebug('[BuildPredictions] error on push prediction: ' + ex.message);
                    }
                }

                this.Predictions = predictionsToWork;
            }
            else {
                allMatchingFilteredPredictions = this.FilterPredictions(filterOnlyOnBeginning, predictionsToWork);
                this.Predictions = allMatchingFilteredPredictions;
            }
        } else {
            if (this.Paginated === true) {
                this.NbPages = Math.floor(predictionsToWork.length / Ilog.Constants.RuleEditorConstants.PageSize);
                if (this.NbPages * Ilog.Constants.RuleEditorConstants.PageSize < predictionsToWork.length) {
                    this.NbPages++;

                }

                var tmp = new Array();

                // computing page
                if (this.CurrentPageIndex != 0) {
                    var firstItemIndex = Ilog.Constants.RuleEditorConstants.PageSize * this.CurrentPageIndex;
                    var max = (predictionsToWork.length > (firstItemIndex + Ilog.Constants.RuleEditorConstants.PageSize)) ? firstItemIndex + Ilog.Constants.RuleEditorConstants.PageSize : predictionsToWork.length;
                }
                else {
                    var firstItemIndex = 0;
                    var max = (predictionsToWork.length > Ilog.Constants.RuleEditorConstants.PageSize) ? Ilog.Constants.RuleEditorConstants.PageSize : predictionsToWork.length;
                }

                for (var k = firstItemIndex; k < max; k++) {
                    try {
                        tmp.push(predictionsToWork[k]);
                    }
                    catch (ex) {
                        Ilog.Debug.RuleEditorDebugger.PrintDebug('[BuildPredictions] error on push prediction for pagination: ' + ex.message);
                    }
                }

                this.Predictions = tmp;
            }
            else {
                this.Predictions = this.predictionsToWork;
            }
        }

        var Predictions = this.Predictions;

        if (Predictions != null && Predictions.length > 0) {
            var predictionTree = new Object();
            var predictionTypeOthers = 999;
            var predCount = 0;

            // displaying as treeview
            if (this.CurrentPresentationMode == Ilog.Constants.RuleEditorConstants.PredictionPresentationModeTreeview) {
                var valuesNode = false, variablesNode = false, sentencesNode = false, conceptsNode = false, othersNode = false;
                for (var k = 0; k < Predictions.length; k++) {
                    var currentPrediction = Predictions[k];

                    if (currentPrediction != null && currentPrediction.PredictionType) {
                        currentPrediction.PredictionIndex = k;
                        var predictionType = currentPrediction.PredictionType;

                        if (predictionType == Ilog.Constants.RuleEditorConstants.PredictionConceptValue) {
                            valuesNode = true;
                        }
                        else if (predictionType == Ilog.Constants.RuleEditorConstants.PredictionConceptVariable) {
                            variablesNode = true;
                        }
                        else if (predictionType == Ilog.Constants.RuleEditorConstants.PredictionConceptSentence) {
                            sentencesNode = true;
                        }
                        else if (predictionType == Ilog.Constants.RuleEditorConstants.PredictionConceptTarget) {
                            conceptsNode = true;
                        }
                        else {
                            predictionType = predictionTypeOthers; // same (fake) prediction type for "others"
                            othersNode = true;
                        }

                        var concept = currentPrediction.Concept;
                        if (concept == null || concept == undefined) {
                            concept = '';
                        }

                        if (!predictionTree[predictionType]) {
                            predictionTree[predictionType] = new Object();
                        }

                        if (!predictionTree[predictionType][concept]) {
                            predictionTree[predictionType][concept] = new Array();
                        }

                        var currentLength = predictionTree[predictionType][concept].length;
                        predictionTree[predictionType][concept][currentLength] = currentPrediction;
                    }
                }

                var predictionsIndexesMapping = new Array();
                var predictionsIndexesCount = 0;
                var typeCount = 0;

                var typeOrdered = new Array();
                if (valuesNode === true) {
                    typeOrdered.push(Ilog.Constants.RuleEditorConstants.PredictionConceptValue);
                }
                if (variablesNode === true) {
                    typeOrdered.push(Ilog.Constants.RuleEditorConstants.PredictionConceptVariable);
                }
                if (sentencesNode === true) {
                    typeOrdered.push(Ilog.Constants.RuleEditorConstants.PredictionConceptSentence);
                }
                if (conceptsNode === true) {
                    typeOrdered.push(Ilog.Constants.RuleEditorConstants.PredictionConceptTarget);
                }
                if (othersNode === true) {
                    typeOrdered.push(predictionTypeOthers);
                }

                for (var q = 0; q < typeOrdered.length; q++) {
                    var type = typeOrdered[q];

                    // Displaying the Type        
                    var typeName = Ilog.Utilities.GetElementPredictionTypeName(type);
                    var text = Ext.util.Format.htmlEncode(typeName);
                    PredictionsInnerHTML += '<tr style="width:100%;height:20px;"><td style="text-align:left;vertical-align:middle;padding-left:5px;"><span style="vertical-align:middle;"><img src="' + Ilog.Constants.RuleEditorConstants.UrlTreeviewIconType + '" style="margin-right:5px;padding-top:2px;" /></span><span style="vertical-align:middle;">' + text + '</span></td></tr>';
                    var length = Ilog.Utilities.GetTextLength(text) + 16;
                    if (length > maxTextLength) {
                        maxTextLength = length;
                    }

                    var typeNode = predictionTree[type];
                    var conceptCount = 0;
                    for (var concept in typeNode) {
                        var itemsUnderConceptNodeId = this.Identifier + '_type' + typeCount + '_concept' + conceptCount;
                        var itemsUnderConceptNodeId = this.Identifier + '_type' + typeCount + '_concept' + conceptCount + '_nodes';

                        var text = Ext.util.Format.htmlEncode(concept) + '';
                        PredictionsInnerHTML += '<tr style="width:100%;height:20px"><td style="text-align:left;vertical-align:middle;">';

                        if (concept != null && concept != undefined && concept != '') {
                            var divItemsUnderConcept = '';
                            if (Ilog.Constants.RuleEditorConstants.TreenodeDefaultStatus == Ilog.Constants.RuleEditorConstants.TreenodeCollapsed) {
                                PredictionsInnerHTML += '<span style="padding-left:5px;vertical-align:middle;"><img src="' + Ilog.Constants.RuleEditorConstants.UrlTreeviewIconExpand + '" onclick=" Ilog.Utilities.ManagePredictionViewTreeNodeClick(this, \'' + itemsUnderConceptNodeId + '\'); "/></span>';
                                divItemsUnderConcept = '<div id=' + itemsUnderConceptNodeId + ' style="display:none;">';
                            }
                            else {
                                PredictionsInnerHTML += '<span style="padding-left:5px;vertical-align:middle;"><img src="' + Ilog.Constants.RuleEditorConstants.UrlTreeviewIconCollapse + '" onclick=" Ilog.Utilities.ManagePredictionViewTreeNodeClick(this, \'' + itemsUnderConceptNodeId + '\'); "/></span>';
                                divItemsUnderConcept = '<div id=' + itemsUnderConceptNodeId + ' style="display:inline;">';
                            }

                            PredictionsInnerHTML += '<span style="vertical-align:middle;"><img src="' + Ilog.Constants.RuleEditorConstants.UrlTreeviewIconConcept + '" style="margin-right:2px;"/></span>'
                            PredictionsInnerHTML += '<span style="vertical-align:middle;">' + text + '</span>';
                            PredictionsInnerHTML += divItemsUnderConcept;

                            // "Concept" row length
                            var length = 5 + 16 + 16 + Ilog.Utilities.GetTextLength(text);
                            if (length > maxTextLength) {
                                maxTextLength = length;
                            }
                        }
                        var itemsArray = typeNode[concept];

                        // checking if items need to be ordered using sort indexes
                        var orderedArray = new Array();
                        var secondSort = false;
                        var maxIndex = -1;
                        if (itemsArray.length > 0 && itemsArray[0].SortIndex != null && itemsArray[0].SortIndex != undefined && itemsArray[0].SortIndex >= 0) {
                            orderedArray = new Array(itemsArray.length);

                            for (var p = 0; p < itemsArray.length; p++) {
                                var predSortIndex = itemsArray[p].SortIndex;

                                if (predSortIndex > maxIndex) {
                                    maxIndex = predSortIndex;
                                }

                                if (predSortIndex >= itemsArray.length) {
                                    secondSort = true;
                                }

                                if (orderedArray[predSortIndex] != null && orderedArray[predSortIndex] != undefined) { // if there is already an item with same index, no sorting
                                    orderedArray = new Array(itemsArray.length);
                                    orderedArray = itemsArray;
                                    break;
                                }
                                orderedArray[predSortIndex] = itemsArray[p];
                            }
                        }
                        else {
                            orderedArray = itemsArray;
                        }

                        if (secondSort === true) {
                            var finalArray = new Array();
                            var itemsCount = 0;
                            for (var j = 0; j <= maxIndex; j++) {
                                var predItem = orderedArray[j];
                                if (predItem != null && predItem != undefined) {
                                    finalArray[itemsCount] = predItem;
                                    itemsCount++;
                                }
                            }

                            orderedArray = finalArray;
                        }

                        for (var p = 0; p < orderedArray.length; p++) {
                            var item = orderedArray[p];
                            var text = item.TextToDisplay;
                            var leftMargin = 5 + 16 - 3;


                            var imageUrl = Ilog.Utilities.GetPredictionIconUrlForIconId(item.IconId);
                            var predictionDivId = this.Identifier + "_Container_Prediction_" + predCount; //item.PredictionIndex;
                            var predictionImageId = this.Identifier + "_Image_Prediction_" + predCount; //item.PredictionIndex;
                            var predictionTextId = this.Identifier + "_Text_Prediction_" + predCount; //item.PredictionIndex;

                            var cssClass = (concept == '') ? Ilog.Constants.RuleEditorConstants.CssClassCompletionMenuTreeItemNoConcept : Ilog.Constants.RuleEditorConstants.CssClassCompletionMenuTreeItemConcept;
                            PredictionsInnerHTML += '<span id="' + predictionDivId + '"><div class="' + cssClass + '"><img src="' + Ilog.Constants.RuleEditorConstants.UrlTreeviewIconTreeLink + '" /><span style="vertical-align:middle;"><img id="' + predictionImageId + '" src="' + imageUrl + '" style="margin-right:5px;" /></span><span id="' + predictionTextId + '" style="vertical-align:middle;">' + text + '</span></div></span>';

                            predictionsIndexesMapping[predCount] = item.PredictionIndex;

                            var length = leftMargin + 16 + 5 + Ilog.Utilities.GetTextLength(text);
                            if (length > maxTextLength) {
                                maxTextLength = length;
                            }

                            predictionsIndexesCount++;
                            predCount++;
                        }

                        PredictionsInnerHTML += '</div></td></tr>';

                        conceptCount++;
                    }

                    typeCount++;
                }

                this.PredictionsIndexesMapping = predictionsIndexesMapping;
            }
            // display as a flat list :
            else {
                var maxTextLength = 0;

                for (var k = 0; k < Predictions.length; k++) {
                    var item = Predictions[k];

                    if (item != null) {
                        var predictionIdentifierRow = this.Identifier + "_Container_Prediction_" + k;
                        var predictionIdentifier = this.Identifier + "_Text_Prediction_" + k;
                        var predictionIdentifierCell = this.Identifier + "_SubContainer_Prediction_" + k;
                        var predictionIdentifierImg = this.Identifier + "_Image_Prediction_" + k;
                        var imageUrl = Ilog.Utilities.GetPredictionIconUrlForIconId(item.IconId);
                        var predictionToDisplay = Predictions[k].TextToDisplay;
                        var predictionValue = Predictions[k].ValueToInsert;

                        // visible length of text     
                        var length = Ilog.Utilities.GetTextLength(predictionToDisplay) + 10;
                        if (length > maxTextLength) {
                            maxTextLength = length;
                        }

                        PredictionsInnerHTML += "<tr id='" + predictionIdentifierRow + "' name='" + predictionIdentifierRow + "' style='width:100%;height:20px;align:left;horizontal-align:left;'><td width='100%' style='align:left;horizontal-align:left;' id='" + predictionIdentifierCell + "' name='" + predictionIdentifierCell + "'><span style='padding-left:5px;horizontal-align:center;width:18px;'><img id='" + predictionIdentifierImg + "' name='" + predictionIdentifierImg + "' src='" + imageUrl + "' style='vertical-align:middle;'/></span><span style='padding-left:5px;padding-right:5px;'><a id='" + predictionIdentifier + "' name='" + predictionIdentifier + "'>" + predictionToDisplay + "</a></span></td></tr>";
                    }
                }

                this.MaxTextLength = maxTextLength;
            }
        }
        else {
            PredictionsInnerHTML += "<tr width='100%'><td style='width:100%;height:20px;vertical-align:middle;padding-left:5px;'>" + Ilog.Constants.RuleEditorConstants.NoSuggestionsMessage + "</td></tr>";
        }

        PredictionsInnerHTML += "</table>";
        this.MaxTextLength = maxTextLength;
        return { html: PredictionsInnerHTML, match: allMatchingFilteredPredictions };
    },

    FilterPredictions: function(filterOnlyBeginning, predictions) {
        var filteredPredictions = new Array();
        try {
            var filteredPredictionsCount;
            var filterValue = this.CurrentSearchFilter.toUpperCase();

            if (filterOnlyBeginning == null || filterOnlyBeginning == undefined) {
                filterOnlyBeginning = false;
            }

            for (var p = 0; p < predictions.length; p++) {
                var currentPrediction = predictions[p];
                var currentText = Ext.util.Format.htmlDecode(currentPrediction.TextToDisplay.toUpperCase()).replace(/&LT;/g, '<').replace(/&GT;/g, '>').replace(/&AMP;/g, '&');

                var bCondOk = false;
                if (filterOnlyBeginning === true) {
                    if (currentText.indexOf(filterValue) == 0) {
                        bCondOk = true;
                    }
                }
                else {
                    if (currentText.indexOf(filterValue) >= 0) {
                        bCondOk = true;
                    }
                }

                if (bCondOk) {
                    filteredPredictions.push(currentPrediction);
                }
            }

            this.FilteredPredictions = filteredPredictions;
        }
        catch (ex) {
            Ilog.Debug.RuleEditorDebugger.PrintDebug('[FilterPredictions] error: ' + ex.message);
        }

        return filteredPredictions;
    },

    HandleKeyUp: function(evt) {
        var editorInstance = this.EditorInstance;
        var keyCode = evt.getCharCode();
        var shiftPressed = evt.shiftKey;

        if (keyCode == Ext.EventObject.SHIFT && Ilog.Constants.RuleEditorConstants.ShiftModifierForSwitchToBuffer === true) {
            Ilog.Utilities.CancelEventBehaviour(evt);
            Ilog.Utilities.StopEventPropagation(evt);
            return false;
        }

        // Keys to ignore
        if (evt != null && (keyCode == Ext.LEFT || keyCode == Ext.RIGHT)) {
            return;
        }

        // [PageUp / PageDown] keys
        if (evt != null && (keyCode == Ext.EventObject.PAGEUP || keyCode == Ext.EventObject.PAGEDOWN)) {
            Ilog.Utilities.CancelEventBehaviour(evt);
            Ilog.Utilities.StopEventPropagation(evt);
            return false;
        }

        // [Enter] or [Tab] key
        if (evt != null && (keyCode == 13 || keyCode == Ext.EventObject.ENTER || keyCode == Ilog.Constants.RuleEditorConstants.CharKeyCodeTabulation)) {
            if (editorInstance != null) {
                // doing nothing if div is disabled (during filtering)
                var predictionsTableId = this.Identifier + "_PredictionsTable";
                var predictionsTable = document.getElementById(predictionsTableId);
                if (predictionsTable && predictionsTable.disabled) {
                    Ilog.Utilities.CancelEventBehaviour(evt);
                    return false;
                }


                var selectedPrediction = this.GetCurrentSelectedPredictionValue();
                var valueEditor = this.GetCurrentSelectedPredictionValueEditor();

                // Predictions that involve a value editor:
                // firing special event
                if (valueEditor) {
                    this.Hide();
                    this.fireEvent("ValidateValueEditor", valueEditor);
                }

                if (selectedPrediction != null) {
                    selectedPrediction.ValueToInsert = selectedPrediction.ValueToInsert.replace(/\r\n/g, '<br/>').replace(/\n/g, '<br/>').replace(/\t/g, '&nbsp;&nbsp;&nbsp;');
                    this.Hide();
                    this.fireEvent("ValidatePrediction", selectedPrediction, true);
                }
            }

            Ilog.Utilities.CancelEventBehaviour(evt);
            return false;
        }
        // [Escape]
        else if (evt != null && keyCode == 27) {
            return;
        }
        else if (evt != null && keyCode == Ilog.Constants.RuleEditorConstants.CharKeyCodeForSwitchToBuffer && (Ilog.Constants.RuleEditorConstants.ShiftModifierForSwitchToBuffer === false || (Ilog.Constants.RuleEditorConstants.ShiftModifierForSwitchToBuffer === true && shiftPressed === true))) {
            Ilog.Utilities.CancelEventBehaviour(evt);

            if (editorInstance) {
                var editorIdentifier = editorInstance.editorFieldId;

                if (editorInstance.contentDocument) {
                    editorInstance.contentWindow.focus();
                }
                else {
                    editorInstance.editorDocument.body.focus();
                    var caretPosition = this.ReplacementArea;
                    editorInstance.SelectTextRange(caretPosition);
                }
            }
        }
        else {
            // up/down keys
            if (keyCode == Ext.EventObject.UP || keyCode == Ext.EventObject.DOWN) {
                Ilog.Utilities.CancelEventBehaviour(evt);
                return false;
            }

            this.TriggerFiltering();
        }
    },
    TriggerFiltering: function() {
        this.CurrentSearchFilter = this.FilterBox.value;

        // doing nothing if filter has not changed
        if (this.CurrentSearchFilter == this.PreviousSearchFilter || (this.CurrentSearchFilter == '' && (this.PreviousSearchFilter == null || this.PreviousSearchFilter == undefined))) {
            return;
        }

        this.CurrentPageIndex = 0; // resetting page index, predictions will be rebuilt and 1st page of results will be displayed
        this.CommandsLocked = true;
        this.DisplayWaitingAnimation();
        var self = this;
        var delay = Ilog.Constants.RuleEditorConstants.PredictionsFilteringDelay;

        if (Ilog.Constants.RuleEditorConstants.UseAdaptativeDelaysForFilter === true) {
            var filterLength = this.FilterBox.value.length;

            if (filterLength <= Ilog.Constants.RuleEditorConstants.AdaptativeDelaysRange1.max) {
                delay = Ilog.Constants.RuleEditorConstants.AdaptativeDelaysRange1.delay;
            }
            else if (filterLength >= Ilog.Constants.RuleEditorConstants.AdaptativeDelaysRange2.min && filterLength <= Ilog.Constants.RuleEditorConstants.AdaptativeDelaysRange2.max) {
                delay = Ilog.Constants.RuleEditorConstants.AdaptativeDelaysRange2.delay;
            }
            else {
                delay = Ilog.Constants.RuleEditorConstants.AdaptativeDelaysRange3.delay;
            }
        }

        if (Ilog.Constants.RuleEditorConstants.UseExtraDelayForLargeNbItems === true && this.OriginalPredictions.length >= Ilog.Constants.RuleEditorConstants.LargeNbItems) {
            delay += Ilog.Constants.RuleEditorConstants.LargeNbItemsExtraDelay;
        }

        // blank filter : updating immediately, items are already computed
        if (this.FilterBox.value.length == 0) {
            delay = 0;
        }

        if (this.delayedFilteringId) {
            window.clearTimeout(this.delayedFilteringId);
        }
        this.delayedFilteringId = window.setTimeout(function() {
            self.CurrentSearchFilter = self.FilterBox.value;

            // IE(6): storing caret position
            if (Ext.isIE) {
                var range = document.selection.createRange().duplicate();
                range.moveStart('character', -self.FilterBox.value.length);
                var caretPos = range.text.length;
            }

            self.UpdatePredictionsLayer();

            self.PreviousSearchFilter = self.CurrentSearchFilter;
            self.delayedFilteringId = null;
            self.HideWaitingAnimation();
            self.CommandsLocked = false;
            try {
                self.FilterBox.focus();

                // IE(6): setting back caret position
                if (Ext.isIE) {
                    var range = self.FilterBox.createTextRange();
                    range.collapse(true);
                    range.moveEnd('character', caretPos);
                    range.moveStart('character', caretPos);
                    range.select();
                }
            }
            catch (ex) {
                Ilog.Debug.RuleEditorDebugger.PrintDebug('[ TriggerFiltering] error on focus filter: ' + ex.message);
            }
        }, delay);
    },


    ManagePageUpPageDown: function(keyCode, shiftPressed) {
        // if the pagination mode is set, shift+PAGEUP/PAGEDOWN will change page
        if (this.Paginated === true && shiftPressed === true) {
            if (keyCode == Ext.EventObject.PAGEUP) {
                this.SwitchToPreviousPage();
            }
            else {
                this.SwitchToNextPage();
            }
            this.CurrentSelectedPredictionIndex = -1;
        }
        else {
            if (keyCode == Ext.EventObject.PAGEUP) {
                this.DoPageUp();
            }
            else {
                this.DoPageDown();
            }
        }
    },

    ManagePageUp: function(shiftPressed) {
        this.ManagePageUpPageDown(Ext.EventObject.PAGEUP, shiftPressed);
    },

    ManagePageDown: function(shiftPressed) {
        this.ManagePageUpPageDown(Ext.EventObject.PAGEDOWN, shiftPressed);
    },

    HandleKeyDown: function(evt) {
        var keyCode = evt.getCharCode();
        var shiftPressed = evt.shiftKey;

        if (evt != null && keyCode == Ilog.Constants.RuleEditorConstants.CharKeyCodeForSwitchToBuffer && (Ilog.Constants.RuleEditorConstants.ShiftModifierForSwitchToBuffer === false || (Ilog.Constants.RuleEditorConstants.ShiftModifierForSwitchToBuffer === true && shiftPressed === true))) {
            Ilog.Utilities.CancelEventBehaviour(evt);
            return;
        }

        // [Enter] or [Tab] key
        if (evt != null && (keyCode == 13 || keyCode == Ext.EventObject.ENTER || keyCode == Ilog.Constants.RuleEditorConstants.CharKeyCodeTabulation)) {
            // doing nothing if div is disabled (during filtering)
            var predictionsTableId = this.Identifier + "_PredictionsTable";
            var predictionsTable = document.getElementById(predictionsTableId);
            if (predictionsTable && predictionsTable.disabled) {
                return false;
            }

            Ilog.Utilities.CancelEventBehaviour(evt);
            return false;
        }

        // [Escape] or [TAB] key
        if (evt != null && (keyCode == 27 || (keyCode == 9 && Ilog.Constants.RuleEditorConstants.CharKeyCodeForSwitchToBuffer != 9))) {
            Ilog.Utilities.CancelEventBehaviour(evt);
            this.DoCancel();
            return false;
        }

        // [PageUp / PageDown] keys
        if (evt != null && (keyCode == Ext.EventObject.PAGEUP || keyCode == Ext.EventObject.PAGEDOWN)) {
            Ilog.Utilities.CancelEventBehaviour(evt);
            Ilog.Utilities.StopEventPropagation(evt);
            this.ManagePageUpPageDown(keyCode, shiftPressed);
            var box = this.FilterBox;
            window.setTimeout(function() { try { box.focus(); } catch (ex) { Ilog.Debug.RuleEditorDebugger.PrintDebug('[HandleKeyDown] error on set focus to filter box: ' + ex.message); } }, 10);
            return false;
        }

        // [END] key
        if (evt != null && (keyCode == evt.END)) {
            this.SelectLastPredictionItem();
            Ilog.Utilities.CancelEventBehaviour(evt);
            return false;
        }

        // Up/Down arrows
        if (evt != null && (keyCode == Ext.EventObject.UP || keyCode == Ext.EventObject.DOWN)) {
            // doing nothing if div is disabled (during filtering)
            var predictionsTableId = this.Identifier + "_PredictionsTable";
            var predictionsTable = document.getElementById(predictionsTableId);
            if (predictionsTable && predictionsTable.disabled) {
                return false;
            }

            // up
            if (keyCode == Ext.EventObject.UP) {
                Ilog.Utilities.CancelEventBehaviour(evt);
                this.UpdateSelectedIndex(-1);
                return false;
            }
            // down
            else if (keyCode == Ext.EventObject.DOWN) {
                Ilog.Utilities.CancelEventBehaviour(evt);
                this.UpdateSelectedIndex(1);
                return false;
            }
        }

        // Keys to ignore
        if (evt != null && (keyCode == Ext.LEFT || keyCode == Ext.RIGHT || keyCode == 37 || keyCode == 39 || (keyCode == Ilog.Constants.RuleEditorConstants.CharKeyCodeForSwitchToBuffer && (Ilog.Constants.RuleEditorConstants.ShiftModifierForSwitchToBuffer === false || (Ilog.Constants.RuleEditorConstants.ShiftModifierForSwitchToBuffer === true && shiftPressed === true))))) {
            return;
        }

        // nothing to do here, search is triggered from keyup
    },

    HandleFilterBoxMouseEvt: function() {
        this.FilterBox.focus();
        return false;
    },

    SwitchToFilter: function() {
        this.ShowSearch();
    },

    DoCancel: function() {
        this.Hide();

        if (this.EditorInstance) {
            var editorIdentifier = this.EditorInstance.editorFieldId;

            if (this.EditorInstance.contentDocument) {
                this.EditorInstance.contentWindow.focus();
            }
            else {
                this.EditorInstance.editorDocument.body.focus();
            }

            try {
                this.EditorInstance.SelectTextRange(this.EditorTextRange);
            }
            catch (ex) {
                Ilog.Debug.RuleEditorDebugger.PrintDebug('[DoCancel] error on select range: ' + ex.message);
            }

            this.EditorInstance.PredictionsView = null;
        }

        return false;
    },

    ShowPredictionFilterArea: function() {
        var predictionsFilterAreaId = this.Identifier + "_FilterArea";
        var predictionsFilterArea = document.getElementById(predictionsFilterAreaId);
        if (predictionsFilterArea) {
            predictionsFilterArea.style.display = 'block';
        }
    },

    HidePredictionFilterArea: function() {
        var predictionsFilterAreaId = this.Identifier + "_FilterArea";
        var predictionsFilterArea = document.getElementById(predictionsFilterAreaId);
        if (predictionsFilterArea) {
            predictionsFilterArea.style.display = 'none';
        }
    },

    UpdatePredictionsLayer: function(filterOnlyOnBeginning) {
        if (this.RequestTimestamp != this.EditorInstance.CompletionMenuRequested) {
            return;
        }
        var editorId = "'" + this.EditorIdentifier + "'";
        var topCaptionIdentifier = this.Identifier + "_TopCaption";
        var predictionsFilterLabelId = this.Identifier + "_FilterLabel";
        var predictionsFilterInputId = this.Identifier + "_FilterInput";
        var predictionsFilterAreaId = this.Identifier + "_FilterArea";
        var predictionsTableId = this.Identifier + "_PredictionsTable";
        var closeCommandId = "predictionWindowCloseCommand_" + editorId;
        var toggleToggleTreeviewModeCommandId = "predictionWindowToggleTreeviewCommand_" + editorId;
        var toggleToggleSearchCommandId = "predictionWindowToggleSearchCommand_" + editorId;

        this.PredictionsLayer.innerHTML = '';
        var treeviewIconClass = "ilog-predictionView-TopCaptionIcon";
        if (this.CurrentPresentationMode == Ilog.Constants.RuleEditorConstants.PredictionPresentationModeTreeview) {
            treeviewIconClass = 'ilog-predictionView-TopCaptionIconSelected';
        }
        var searchIconClass = "ilog-predictionView-TopCaptionIcon";
        if (this.CurrentSearchMode == Ilog.Constants.RuleEditorConstants.SearchEnabled) {
            searchIconClass = 'ilog-predictionView-TopCaptionIconSelected';
        }

        var PredictionsInnerHTML = '<div id="' + topCaptionIdentifier + '" class="ilog-predictionsView-TopCaption"><table width="100%"><tr height="20px"><td style="text-align:left;"><span style="text-align:left;"><img id="' + toggleToggleTreeviewModeCommandId + '" src="' + Ilog.Constants.RuleEditorConstants.UrlPictureToggleTreeview + '" class="' + treeviewIconClass + '" title="' + Ilog.Constants.RuleEditorConstants.ToggleTreeviewTooltipMessage + '" onmouseover=" this.className=\'ilog-predictionView-TopCaptionIconSelected\'; " /><img id="' + toggleToggleSearchCommandId + '" src="' + Ilog.Constants.RuleEditorConstants.UrlPictureSearch + '" class="' + searchIconClass + '" onmouseover=" this.className=\'ilog-predictionView-TopCaptionIconSelected\'; " title="' + Ilog.Constants.RuleEditorConstants.ToggleSearchTooltipMessage + '" /></span></td><td style="text-align:right;"><span style="text-align:right;" align="right"><span class="ilog-predictionsView-TopCaptionCloseCommand"><span id="' + closeCommandId + '" title="' + Ilog.Constants.RuleEditorConstants.TooltipCloseCompletionMenu + '" class="ilog-closeWindow-unselected" onmouseover="this.className=\'ilog-closeWindow-selected\';" onmouseout="this.className=\'ilog-closeWindow-unselected\';">' + Ilog.Constants.RuleEditorConstants.CloseCompletionMenuText + '</span></span></td></tr></table></div>';
        PredictionsInnerHTML += '<div class="ilog-predictions-content">';

        // Filter input :
        var searchDisplay = 'display:none';
        if (this.CurrentSearchMode == Ilog.Constants.RuleEditorConstants.SearchEnabled) {
            searchDisplay = 'display:block';
        }
        PredictionsInnerHTML += '<div id="' + predictionsFilterAreaId + '" class="ilog-predictionView-filterLayer" style="' + searchDisplay + '"><span id="' + predictionsFilterLabelId + '">' + Ilog.Constants.RuleEditorConstants.FilterPredictions + '</span><input id="' + predictionsFilterInputId + '" type="text" class="ilog-predictionView-filterBox" value="' + this.CurrentSearchFilter + '"/></div>';

        // Prediction table : 
        PredictionsInnerHTML += '<div id="' + predictionsTableId + '" style="overflow-y:auto;overflow-x:hidden;padding-top:0px;padding-bottom:0px;">';

        var PredictionsImagesUrl = new Array();
        var PredictionsToDisplay = new Array();
        var PredictionsValues = new Array();
        var absoluteIndex = 0;

        var maxTextLength = 0;
        // displaying predictions
        var predictionsBuilt = this.BuildPredictions(filterOnlyOnBeginning);
        PredictionsInnerHTML += predictionsBuilt.html;
        PredictionsInnerHTML += '</div></div>';

        var firstPageEltId = "completionFirstPage_" + editorId;
        var previousPageEltId = "completionPreviousPage_" + editorId;
        var nextPageEltId = "completionNextPage_" + editorId;
        var lastPageEltId = "completionLastPage_" + editorId;

        this.ListLinksToPages = null;

        // Building bottomlayer
        if (this.Paginated === true) {
            // filter mode
            if (this.CurrentSearchFilter != '') {
                var totalItems = predictionsBuilt.match.length;
                var nbPages = Math.floor(totalItems / Ilog.Constants.RuleEditorConstants.PageSize);
                if (nbPages * Ilog.Constants.RuleEditorConstants.PageSize < totalItems) {
                    nbPages++;
                }
                var currentPageIndex = this.CurrentPageIndex;
            }
            else {
                var totalItems = this.TotalItems;
                var nbPages = this.NbPages;
                var currentPageIndex = this.CurrentPageIndex;
            }

            if (totalItems > 0) {
                var bottomLayerId = this.Identifier + "_PredictionsBottomLayer";
                var bottomLayerContent = this.BuildBottomLayer(currentPageIndex, nbPages, totalItems, firstPageEltId, previousPageEltId, nextPageEltId, lastPageEltId);
                var bottomLayer = '<div id="' + bottomLayerId + '" class="ilog-predictionsView-BottomCaption">' + bottomLayerContent + '</div>';
                PredictionsInnerHTML += bottomLayer;
            }
        }

        if (this.Predictions != null && this.Predictions.length > 0) {
            // Setting window width
            if (this.MaxTextLength) {
                var totalWidth = this.MaxTextLength + 100;
                if (totalWidth > Ilog.Constants.RuleEditorConstants.CompletionDefaultWidth) {
                    this.PredictionsLayer.style.width = totalWidth + 'px';
                }
            }

            this.PredictionsLayer.innerHTML = PredictionsInnerHTML;
            this.PredictionsLayer.zindex = 998;
            this.PredictionsLayer.style.visibility = 'hidden';
            document.body.appendChild(this.PredictionsLayer);
            this.PredictionsLayer.style.visibility = 'visible';
        }
        else {
            this.PredictionsLayer.innerHTML = PredictionsInnerHTML;
            this.PredictionsLayer.zindex = 998;
            document.body.appendChild(this.PredictionsLayer);
            this.PredictionsLayer.style.visibility = 'visible';
        }


        var editionBox = document.getElementById(predictionsFilterInputId);
        if (editionBox) {
            this.FilterBox = editionBox;
        }

        this.CurrentSelectedPredictionIndex = -1;

        this.Resize();

        // Adding event handlers
        var filterValue = this.FilterBox.value;
        if (filterValue.length > 0) {
            this.RegisterEventHandlers(closeCommandId, nextPageEltId, previousPageEltId, firstPageEltId, lastPageEltId, true);
        }
        else {
            this.RegisterEventHandlers(closeCommandId, nextPageEltId, previousPageEltId, firstPageEltId, lastPageEltId, false);
        }
    },

    BuildBottomLayer: function(currentPageIndex, nbPages, totalItems, firstPageEltId, previousPageEltId, nextPageEltId, lastPageEltId) {
        var editorId = "'" + this.EditorIdentifier + "'";
        var linksToPreviousPages = '';
        var linksToNextPages = '';


        // link to previous and first page
        if (currentPageIndex > 0) {
            linksToPreviousPages += '<span id="' + firstPageEltId + '" class="' + Ilog.Constants.RuleEditorConstants.CssClassLinkToOtherPages + '" onmouseover="this.className = \'' + Ilog.Constants.RuleEditorConstants.CssClassLinkToOtherPagesSelected + '\';" onmouseout="this.className=\'' + Ilog.Constants.RuleEditorConstants.CssClassLinkToOtherPages + '\';" title="' + Ilog.Constants.RuleEditorConstants.TooltipGotoFirstPage.replace(/"/g, "&quot;") + '">&lt;&lt;</span><span id="' + previousPageEltId + '" class="' + Ilog.Constants.RuleEditorConstants.CssClassLinkToOtherPages + '" onmouseover="this.className = \'' + Ilog.Constants.RuleEditorConstants.CssClassLinkToOtherPagesSelected + '\';" onmouseout="this.className=\'' + Ilog.Constants.RuleEditorConstants.CssClassLinkToOtherPages + '\';" title="' + Ilog.Constants.RuleEditorConstants.TooltipGotoPreviousPage.replace(/"/g, "&quot;") + '">&lt;</span>';
        }

        // links to pages
        var minIndexDisplayed = ((currentPageIndex - 2) > 0) ? currentPageIndex - 2 : 0;
        var maxIndexDisplayed = ((currentPageIndex + 2) < nbPages) ? currentPageIndex + 2 : nbPages - 1;

        var linksToPages = "";
        if (nbPages > 5 && currentPageIndex > 2) {
            linksToPages += '<span class="' + Ilog.Constants.RuleEditorConstants.CssClassOtherPages + '">...</span>'
        }

        var listLinksToPages = new Array();
        for (var l = minIndexDisplayed; l < maxIndexDisplayed + 1; l++) {
            var linkToPageId = "completionLinkToPage" + l + "_" + editorId;

            var cssClass = Ilog.Constants.RuleEditorConstants.CssClassLinkToPage;
            var cssClassSelected = Ilog.Constants.RuleEditorConstants.CssClassLinkToPageSelected;
            if (l == currentPageIndex) {
                cssClass = Ilog.Constants.RuleEditorConstants.CssClassCurrentPage;
                cssClassSelected = Ilog.Constants.RuleEditorConstants.CssClassCurrentPage; ;
            }
            else {
                listLinksToPages.push({ id: linkToPageId, index: l });
            }
            linksToPages += '<span id="' + linkToPageId + '" class="' + cssClass + '" onmouseover="this.className = \'' + cssClassSelected + '\';" onmouseout="this.className=\'' + cssClass + '\';">' + (l + 1) + '</span>';
        }

        this.ListLinksToPages = listLinksToPages;

        if (nbPages > 5 && (currentPageIndex + 2) < nbPages - 1) {
            linksToPages += '<span class="' + Ilog.Constants.RuleEditorConstants.CssClassOtherPages + '">...</span>'
        }

        // link to next and last page
        if ((currentPageIndex + 1) < nbPages) {
            linksToNextPages += '<span id="' + nextPageEltId + '" class="' + Ilog.Constants.RuleEditorConstants.CssClassLinkToOtherPages + '" onmouseover="this.className = \'' + Ilog.Constants.RuleEditorConstants.CssClassLinkToOtherPagesSelected + '\';" onmouseout="this.className=\'' + Ilog.Constants.RuleEditorConstants.CssClassLinkToOtherPages + '\';" title="' + Ilog.Constants.RuleEditorConstants.TooltipGotoNextPage.replace(/"/g, "&quot;") + '">&gt;</span><span id="' + lastPageEltId + '" class="' + Ilog.Constants.RuleEditorConstants.CssClassLinkToOtherPages + '" onmouseover="this.className = \'' + Ilog.Constants.RuleEditorConstants.CssClassLinkToOtherPagesSelected + '\';" onmouseout="this.className=\'' + Ilog.Constants.RuleEditorConstants.CssClassLinkToOtherPages + '\';" title="' + Ilog.Constants.RuleEditorConstants.TooltipGotoLastPage.replace(/"/g, "&quot;") + '">&gt;&gt;</span>';
        }

        var bottomLayerContent = '<table width="100%"><tr height="20px"><td class="ilog-predictionsView-BottomCaptionLinks"><span clas="ilog-predictionsView-BottomCaptionLinks">' + linksToPreviousPages + linksToPages + linksToNextPages + '</span></td></tr></table>';
        return bottomLayerContent;
    },

    HandleMouseEvt: function(evt) {
        Ilog.Utilities.CancelEventBehaviour(evt);
        Ilog.Utilities.StopEventPropagation(evt);
        return false;
    },

    HandleCloseCommandClick: function(extEvt) {
        Ilog.Utilities.StopEventPropagation(extEvt);
        Ilog.Utilities.CancelEventBehaviour(extEvt.browserEvent);

        if (Ext.isIE) {
            var scrollPos = { x: this.EditorInstance.GetScrollXPosition(), y: this.EditorInstance.GetScrollYPosition() };
        }

        this.PredictionsLayer.parentNode.removeChild(this.PredictionsLayer);
        this.EditorInstance.Focus();

        this.DoCancel();

        if (Ext.isIE) {
            var self = this;
            window.setTimeout(function() { self.EditorInstance.SetScrollXYPosition(scrollPos.x, scrollPos.y); }, 40);
        }

        return false;
    },

    HandleFirstPageCommandClick: function(extEvt) {
        if (this.CommandsLocked === true) {
            return;
        }

        // Updating page index
        var newPageIndex = 0;

        // setting new page index
        this.CurrentPageIndex = newPageIndex;

        // Refreshing display
        this.RepaintPredictions();
    },

    SwitchToPreviousPage: function() {
        if (!(this.Paginated === true && this.NbPages > 1)) {
            return;
        }

        if (this.CommandsLocked === true) {
            return;
        }

        // Updating page index
        var newPageIndex = this.CurrentPageIndex - 1;
        if (newPageIndex < 0) {
            newPageIndex = 0;
        }

        if (newPageIndex == this.CurrentPageIndex) {
            return;
        }

        // setting new page index
        this.CurrentPageIndex = newPageIndex;

        // Refreshing display
        this.RepaintPredictions();
    },

    HandlePreviousPageCommandClick: function(extEvt) {
        this.SwitchToPreviousPage();
    },

    SwitchToNextPage: function() {
        if (!(this.Paginated === true && this.NbPages > 1)) {
            return;
        }

        // Updating page index
        var newPageIndex = this.CurrentPageIndex + 1;
        if (newPageIndex > (this.NbPages - 1)) {
            newPageIndex = this.NbPages - 1;
        }
        if (newPageIndex < 0) {
            newPageIndex = 0;
        }

        if (newPageIndex == this.CurrentPageIndex) {
            return;
        }

        // setting new page index
        this.CurrentPageIndex = newPageIndex;

        // Refreshing display
        this.RepaintPredictions();
    },

    HandleNextPageCommandClick: function(extEvt) {
        this.SwitchToNextPage();
    },

    HandleLastPageCommandClick: function(extEvt) {
        if (this.CommandsLocked === true) {
            return;
        }

        // Updating page index
        var newPageIndex = this.NbPages - 1;
        if (newPageIndex < 0) {
            newPageIndex = 0;
        }

        // setting new page index
        this.CurrentPageIndex = newPageIndex;

        // Refreshing display
        this.RepaintPredictions();
    },

    HandleJumpToPageCommandClick: function(extEvt) {
        var scope = this;
        var pageIndex = scope.index;
        var self = scope.context;

        // setting new page index
        self.CurrentPageIndex = pageIndex;

        // Refreshing display
        self.RepaintPredictions();
    },

    EnableTransparency: function() {
        var extElement = new Ext.Element(this.PredictionsLayer);
        extElement.setOpacity(0.15);
    },

    DisableTransparency: function() {
        var extElement = new Ext.Element(this.PredictionsLayer);
        extElement.setOpacity(1);
    },

    init: function(filter, filterOnlyOnBeginning) {
        if (this.RequestTimestamp != this.EditorInstance.CompletionMenuRequested) {
            return;
        }
    
        // if a filter is passed, 
        // need to enable search mode
        if (filter != null && filter != undefined && filter != "") {
            this.CurrentSearchFilter = filter;
        }

        var tmp = document.getElementById(this.Identifier);
        if (tmp != null) {
            this.PredictionsLayer = tmp;
        }
        else {
            this.PredictionsLayer = document.createElement('div');
        }

        if (this.EditorInstance.isEmbeddedInDT === true) {
            this.PredictionsLayer.className = "ilog-predictions-dtEmbedded";
        }
        else {
            this.PredictionsLayer.className = "ilog-predictions";
        }

        this.PredictionsLayer.style.position = 'absolute';
        this.PredictionsLayer.style.top = this.Y + 'px';
        this.PredictionsLayer.style.left = this.X + 'px';
        this.PredictionsLayer.style.width = Ilog.Constants.RuleEditorConstants.CompletionDefaultWidth + 'px'; // default width
        this.PredictionsLayer.id = this.Identifier;

        var self = this;
        window.setTimeout(function() { self.UpdatePredictionsLayer(filterOnlyOnBeginning); }, 100);
        
        this.Visible = true;
        this.addEvents({
            "ValidatePrediction": true,
            "ValidateValueEditor": true
        });
    },

    Update: function(textFilterBeginning, posX, posY, editorTextRange) {
        this.UpdatePosition(posX, posY);
        this.CurrentSearchFilter = textFilterBeginning;
        this.EditorTextRange = editorTextRange;
        this.UpdatePredictionsLayer(true);
    },

    UpdatePosition: function(scrollLeft, scrollTop) {
        try {
            this.PredictionsLayer.style.left = scrollLeft + 'px';
            this.PredictionsLayer.style.top = scrollTop + 'px';
        }
        catch (ex) { }
    },

    RegisterEventHandlers: function(closeCommandId, nextPageEltId, previousPageEltId, firstPageEltId, lastPageEltId, filterMode) {
        if (filterMode == null || filterMode == undefined) {
            filterMode = false;
        }

        if (!closeCommandId) {
            var editorId = "'" + this.EditorIdentifier + "'";
            closeCommandId = "predictionWindowCloseCommand_" + editorId;
        }

        try {
            Ext.EventManager.addListener(document.getElementById(closeCommandId), "click", this.HandleCloseCommandClick, this);
            Ext.EventManager.addListener(document.getElementById(closeCommandId), "mouseup", function(e) { Ilog.Utilities.StopEventPropagation(e); Ilog.Utilities.CancelEventBehaviour(e); return false; }, this);
            Ext.EventManager.addListener(this.PredictionsLayer, "mousedown", this.HandleMouseEvt, this);
            Ext.EventManager.addListener(this.PredictionsLayer, "mouseup", this.HandleMouseEvt, this);

            if (firstPageEltId) {
                var firstPageElt = document.getElementById(firstPageEltId);
                if (firstPageElt) {
                    Ext.EventManager.addListener(firstPageElt, "click", this.HandleFirstPageCommandClick, this);
                }
            }

            if (previousPageEltId) {
                var previousPageElt = document.getElementById(previousPageEltId);
                if (previousPageElt) {
                    Ext.EventManager.addListener(previousPageElt, "click", this.HandlePreviousPageCommandClick, this);
                }
            }

            if (nextPageEltId) {
                var nextPageElt = document.getElementById(nextPageEltId);
                if (nextPageElt) {
                    Ext.EventManager.addListener(nextPageElt, "click", this.HandleNextPageCommandClick, this);
                }
            }

            if (lastPageEltId) {
                var lastPageElt = document.getElementById(lastPageEltId);
                if (lastPageElt) {
                    Ext.EventManager.addListener(lastPageElt, "click", this.HandleLastPageCommandClick, this);
                }
            }

            // direct page links
            if (this.ListLinksToPages) {
                for (var k = 0; k < this.ListLinksToPages.length; k++) {
                    var linkPage = this.ListLinksToPages[k];
                    if (linkPage.id) {
                        var linkObj = document.getElementById(linkPage.id);
                        if (linkObj != null && linkObj != undefined && linkPage.index != null && linkPage.index != undefined) {
                            Ext.EventManager.addListener(linkObj, "click", this.HandleJumpToPageCommandClick, { context: this, index: linkPage.index });
                        }
                    }
                }
            }

            // unregister if needed
            try {
                Ext.EventManager.removeListener(this.FilterBox, "keyup", this.HandleKeyUp);
            } catch (ex) { }
            try {
                Ext.EventManager.removeListener(this.FilterBox, "keydown", this.HandleKeyDown);
            } catch (ex) { }
            try {
                Ext.EventManager.removeListener(this.FilterBox, "click", this.HandleFilterBoxMouseEvt);
            } catch (ex) { }
            try {
                Ext.EventManager.removeListener(this.FilterBox, "mousedown", this.HandleFilterBoxMouseEvt);
            } catch (ex) { }
            try {
                Ext.EventManager.removeListener(this.FilterBox, "mouseup", this.HandleFilterBoxMouseEvt);
            } catch (ex) { }
            // register
            Ext.EventManager.addListener(this.FilterBox, "keyup", this.HandleKeyUp, this);
            Ext.EventManager.addListener(this.FilterBox, "keydown", this.HandleKeyDown, this);
            Ext.EventManager.addListener(this.FilterBox, "click", this.HandleFilterBoxMouseEvt, this);
            Ext.EventManager.addListener(this.FilterBox, "mousedown", this.HandleFilterBoxMouseEvt, this);
            Ext.EventManager.addListener(this.FilterBox, "mouseup", this.HandleFilterBoxMouseEvt, this);

            var editorId = "'" + this.EditorIdentifier + "'";
            var toggleToggleTreeviewModeCommandId = "predictionWindowToggleTreeviewCommand_" + editorId;
            var treeviewModeCommandElement = document.getElementById(toggleToggleTreeviewModeCommandId);
            var toggleToggleSearchCommandId = "predictionWindowToggleSearchCommand_" + editorId;
            var toggleToggleSearchCommandElement = document.getElementById(toggleToggleSearchCommandId);
            if (treeviewModeCommandElement) {
                // unregister if needed
                try {
                    Ext.EventManager.removeListener(treeviewModeCommandElement, "click", this.HandleToggleTreeviewModeClick);
                } catch (ex) { }
                try {
                    Ext.EventManager.removeListener(treeviewModeCommandElement, "mouseout", this.HandleToggleTreeviewModeMouseOut);
                } catch (ex) { }

                // register
                Ext.EventManager.addListener(treeviewModeCommandElement, "click", this.HandleToggleTreeviewModeClick, this);
                Ext.EventManager.addListener(treeviewModeCommandElement, "mouseout", this.HandleToggleTreeviewModeMouseOut, this);
            }

            if (toggleToggleSearchCommandElement) {
                // unregister if needed
                try {
                    Ext.EventManager.removeListener(toggleToggleSearchCommandElement, "click", this.HandleToggleSearchClick);
                } catch (ex) { }
                try {
                    Ext.EventManager.removeListener(toggleToggleSearchCommandElement, "mouseout", this.HandleToggleSearchMouseOut);
                } catch (ex) { }
                // register
                Ext.EventManager.addListener(toggleToggleSearchCommandElement, "click", this.HandleToggleSearchClick, this);
                Ext.EventManager.addListener(toggleToggleSearchCommandElement, "mouseout", this.HandleToggleSearchMouseOut, this);
            }

            if (!this.Predictions) {
                return;
            }
            for (var k = 0; k < this.Predictions.length; k++) {
                // Flat view
                var displayIndex = k;
                // Tree view
                if (this.CurrentPresentationMode == Ilog.Constants.RuleEditorConstants.PredictionPresentationModeTreeview) {
                    displayIndex = this.PredictionsIndexesMapping[displayIndex];
                }

                var predictionIdentifierTest = this.Identifier + "_Text_Prediction_" + k;
                var predictionIdentifierTestImg = this.Identifier + "_Image_Prediction_" + k;
                var predictionIdentifierTestCell = this.Identifier + "_Container_Prediction_" + k;
                var predictionIdentifierTestRealCell = this.Identifier + "_SubContainer_Prediction_" + k;

                var imgNode = document.getElementById(predictionIdentifierTestImg);
                if (imgNode != null) {
                    imgNode.predictionIndex = k;
                    imgNode.predictionIndexValue = k;
                    imgNode.usingFilterMode = filterMode;
                    imgNode.displayIndex = displayIndex;
                }
                var linkNode = document.getElementById(predictionIdentifierTest);
                if (linkNode != null) {

                    linkNode.predictionIndex = k;
                    linkNode.predictionIndexValue = k;
                    linkNode.usingFilterMode = filterMode;
                    linkNode.displayIndex = displayIndex;
                }
                var cellNode = document.getElementById(predictionIdentifierTestRealCell);
                if (cellNode != null) {
                    cellNode.predictionIndex = k;
                    cellNode.predictionIndexValue = k;
                    cellNode.usingFilterMode = filterMode;
                    cellNode.displayIndex = displayIndex;
                }

                var rowNode = document.getElementById(predictionIdentifierTestCell);

                if (rowNode != null) {
                    rowNode.predictionIndex = k;
                    rowNode.predictionIndexValue = k;
                    rowNode.usingFilterMode = filterMode;
                    rowNode.displayIndex = displayIndex;

                    Ext.EventManager.addListener(rowNode, "click", this.HandleClick, this);
                    Ext.EventManager.addListener(rowNode, "mouseover", this.HandleMouseOver, this);
                    Ext.EventManager.addListener(rowNode, "mouseout", this.HandleMouseOut, this);
                }
            } // for

            // to workaround IE bug, need to catch the scroll event
            if (Ext.isIE) {
                var idDiv = this.Identifier + "_PredictionsTable";
                var el = document.getElementById(idDiv);
                Ext.EventManager.addListener(el, "scroll", this.HandleScroll, this);
            }
        }
        catch (ex) {
            Ilog.Debug.RuleEditorDebugger.PrintDebug('[RegisterEventHandlers] error on register event handlers: ' + ex.message)
        }
    }
});
