// --------------------------------------------------------------- -*- JS -*-
// IBM Confidential
// OCO Source Materials
// 5724-Y00 5724-Y02
//  Copyright IBM Corp. 2004, 2010. All Rights Reserved.
// The source code for this program is not published or otherwise
// divested of its trade secrets, irrespective of what has
// been deposited with the U.S. Copyright Office.
// --------------------------------------------------------------------------

Ext.namespace("Ilog");

/**
 * Box Wrapper for the "generic" intellirule editor
 * 
 *
 * example of ruleeditor config
 * defaultRuleEditorConfig : {
 *			"StyleSheetUrl":"/web-dt-sandbox/skins/common/css/RuleEditor.css",
 *			"CustomFieldReference":"",
			"SeparatorListIdeographsRanges":null,
			"IsMultiline":true,
			"PbListFieldId":"intelliRuleEditorPbListId",
			"UseSentenceCompletion":true,
			"EditorFieldId":"intelliRuleEditorId",
			"DataCacheEnabled":false,
			"IsEmbeddedInDT":true,
			"IsReadOnly":false,
			"SpecificUrlForSaveRequests":"/web-dt-sandbox/servlet/IlrBRLServlet",
			"IsAspNetCallbackHandlerAvailable":false,
			"Culture":{ "FullDatePatternWithoutTime":"M\/d\/yyyy {0}",
						"DatePattern":"M\/d\/yyyy",
						"FullDatePattern":"M\/d\/yyyy {0}",
						"FullTimePattern":"h:mm:ss a",
						"UniversalDatePattern":"M\/d\/yyyy {0}",
						"MonthNames":["January","February","March","April","May","June","July","August","September","October","November","December",""],
						"UniversalTimePattern":"h:mm:ss a Z",
						"DayNames":["","Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"],
						"UniversalDatePatternWithoutTime":"M\/d\/yyyy {0}",
						"Culture":"en"},
			"SeparatorWhitespaces":null,
			"InitialText":"",
			"InitialTooltips":null,
			"AutoCompletionEnabledAtStartup":true,
			"SeparatorListSingleIdeographs":null,
			"CustomFieldKey":"",
			"ServiceUrl":"/web-dt-sandbox/servlet/IlrBRLServlet",
			"FrozenRegions":null},

 * 
 * 
 */
Ilog.RuleEditorComponent = Ext.extend(Ext.BoxComponent, {
	
	// the intellirule editor
	ruleEditor : null, 
	// initial config
	ruleEditorConfig : null,
	// editor ID
	ruleEditorID : null,
	// keep track of the actual text displayed to filter change events
	text : "",
	
	initEditor : function(){
		if( this.ruleEditor && this.ruleEditor.canPerformInit === true ){
			this.ruleEditor.init();
			this.ruleEditor.ToolbarContainerID = 'ilog-ruleEditor-toolbarPlaceholderLayerId';
		}
	},
    
	setText: function(html, dtEditionContext){
		this.ruleEditor.DtEditionContext = dtEditionContext;
		if (html){
			this.ruleEditor.SetHtmlText(html.text, false);
			if (html.frozenRegions){
				this.ruleEditor.FrozenRegions =  html.frozenRegions;
			}
			else{
				this.ruleEditor.FrozenRegions = null;
			}
		}
		else{
			this.ruleEditor.SetHtmlText("", false);
		}
		this.text = this.ruleEditor.GetPlainText();
	},
	
	getText: function(){
		return this.ruleEditor.GetPlainText();
	},
	
	getEditorFieldId : function(){
		return this.ruleEditor.editorFieldId;
	},
	
    drawEditor : function(){
		// produces the innerHTML for rendering the editor
		var s = '<div id="'+this.getEditorFieldId()+'-editorContainerId" class="ilog-ruleEditor-editorContainer" style="width:700px;">';
		
		s+='<div align="center">';
		s+='<iframe id="'+this.getEditorFieldId()+'" name="intelliRuleEditor" frameborder="0px" bgcolor="Color [Empty]" designMode="On" src="about:blank" style="border:0px none;" width="100%"></iframe>';
		s+='</div>';
        
		s+='</div>';
		this.el.dom.innerHTML = s;
    },
    
    initComponent:function() {
    	this.ruleEditor = new Ilog.RuleEditor({
    		"StyleSheetUrl":this.ruleEditorConfig.StyleSheetUrl,
			"CustomFieldReference":"",
			"SeparatorListIdeographsRanges":null,
			"IsMultiline":true,
			"PbListFieldId":"intelliRuleEditorPbListId",
			"UseSentenceCompletion":true,
			"EditorFieldId": this.ruleEditorID,
			"DataCacheEnabled":false,
			"IsEmbeddedInDT":true,
			"IsReadOnly":false,
			"SpecificUrlForSaveRequests": this.ruleEditorConfig.SpecificUrlForSaveRequests,
			"IsAspNetCallbackHandlerAvailable":false,
			"Culture":this.ruleEditorConfig.Culture,
			"SeparatorWhitespaces": this.ruleEditorConfig.SeparatorWhitespaces,
			"InitialText":"",
			"InitialTooltips":null,
			"AutoCompletionEnabledAtStartup":true,
			"SeparatorListSingleIdeographs":null,
			"CustomFieldKey":"",
			"ServiceUrl": this.ruleEditorConfig.ServiceUrl,
			"FrozenRegions":null
    	});
    	
        ilog.dt.web.DTRuleEditor.superclass.initComponent.apply(this, arguments);
        this.addEvents('textChanged');
    },
    
    onRender : function() {
        // invoke super method
        ilog.dt.web.DTRuleEditor.superclass.onRender.apply(this, arguments);
        this.drawEditor();
        this.initEditor();
        // bridge between event schemes
    	this.ruleEditor.addListener("textchanged", function(newText) {
    		if (newText.text != this.text){
    			this.text =  newText.text;
    			this.fireEvent("textChanged");
    		}
    	},this);
    },

    onResize:function(iWidth, iHeight) {
        var div = Ext.get(this.getEditorFieldId()+'-editorContainerId');
        div.setWidth(iWidth);
        div.setHeight(iHeight);
    }

});

