// --------------------------------------------------------------- -*- JS -*-
// IBM Confidential
// OCO Source Materials
// 5724-Y00 5724-Y02 5724-Y21
//  Copyright IBM Corp. 2004, 2010. All Rights Reserved.
// The source code for this program is not published or otherwise
// divested of its trade secrets, irrespective of what has
// been deposited with the U.S. Copyright Office.
// --------------------------------------------------------------------------

Ext.namespace('Ilog', 'Ilog.Utilities');

String.prototype.endsWith = function(str){
    return (this.match(str+"$")==str);
};

String.prototype.startsWith = function(str){
    return (this.match("^"+str)==str);
};

String.prototype.replaceAt = function(index, character) {
    return this.substr(0, index) + character + this.substr(index+character.length);
};

Ilog.Utilities.CountOccurences = function(mainString, searchString){
    return mainString.split(searchString).length - 1;
};

Ilog.Utilities.Store = function(){
    this.StoreObject = new Object();
    this.Count = 0;
};

Ilog.Utilities.GetLongRuleAdditionalDelay = function(characterCount) {
    var delay = 0;

    if (characterCount > 500) {
        delay = characterCount/2;
    }

    return delay;
};

Ilog.Utilities.GetElementActualStyle = function(element, IEStyleProp, CSSStyleProp) {
    try {
        if (element.currentStyle) {
            return element.currentStyle[IEStyleProp];
        }
        else if (window.getComputedStyle) {
            var computedStyle = window.getComputedStyle(element, "");
            return computedStyle.getPropertyValue(CSSStyleProp);
        }
    }
    catch (ex) { }

    return null;
};

Ilog.Utilities.CleanEditorResources = function(){
    if( Ilog.Utilities.RuleEditorStore ){
        var store = Ilog.Utilities.RuleEditorStore.GetStoreArray();
        for(var key in store){
            var editor = store[key];
            if( editor ){
                if( editor.ClearResources ){               
                    editor.ClearResources();
                }
                else{
                    eval(editor+'.ClearResources(); ');
                }
            }
        }
    }
};

Ilog.Utilities.MouseUpHandler = null;
Ilog.Utilities.MouseUpCheckThread = null;

Ilog.Utilities.RuleEditorMouseUpHandler = function(evt) {
    window.setTimeout(function() {
        // Special case for FF :
        // checking that the click is not happening on the browser window scrollbars
        // -> if it is the case, we don't consider that the "focus" is lost
        if (Ext.isGecko) {
            try {
                var scrollbarWidth = (Ilog.Constants.RuleEditorConstants.ScrollbarWidth != null && Ilog.Constants.RuleEditorConstants.ScrollbarWidth != undefined) ? Ilog.Constants.RuleEditorConstants.ScrollbarWidth : Ilog.Utilities.GetScrollbarWidth();
                var xMax = window.innerWidth - scrollbarWidth;
                var yMax = window.innerHeight - scrollbarWidth;
                var xClick = evt.clientX;
                var yClick = evt.clientY;
                
                if (xClick == 0 && yClick == 0 || ((xClick == undefined || xClick == null) && (yClick == undefined || yClick == null)) ) {
                    xClick = evt.getPageX() - Ilog.Utilities.GetDocumentScrollLeft();
                    yClick = evt.getPageY() - Ilog.Utilities.GetDocumentScrollTop();
                    
                    // browser and/or ext bug : click on scrollbar sometimes make evt.getPageX and evt.getPageY give 0/0
                    try{
                        if( xClick <= 0 && yClick <= 0 ){
                            return;
                        }
                    }
                    catch(ex){}
                }

                if (xClick > xMax || yClick > yMax) {
                    return;
                }
            }
            catch (ex) {
                Ilog.Debug.RuleEditorDebugger.PrintDebug('[RuleEditorMouseUpHandler] error: ' + ex.message);
            }
        }
        
        // checking if click is not happening on calendar
        if( Ilog.Utilities.RuleEditorStore ){
            var store = Ilog.Utilities.RuleEditorStore.GetStoreArray();
            for(var key in store){
                var editor = store[key];
                if( editor ){
                    if( typeof(editor) === 'string' ){
                        editor = eval(editor);
                    }

                    try{
                        if( editor && editor.DatePickerLayer ){
                            // checking if the click is happening on this layer
                            var extLayer = new Ext.Element(editor.DatePickerLayer.childNodes[0]);

                            if( extLayer ){
                                var xClick = evt.getPageX();
                                var yClick = evt.getPageY();
                                
                                box = extLayer.getBox(true);
                                var boxWidth = extLayer.dom.offsetWidth;
                                var boxHeight = extLayer.dom.offsetHeight;
                                var bottom = extLayer.getBottom(false);
                                var right = extLayer.getRight(false);
                                
                                if( editor.DatePickerLayer.childNodes.length > 1 ){
                                    var timeLayer = new Ext.Element(editor.DatePickerLayer.childNodes[1]);
                                    bottom = timeLayer.getBottom(false);
                                }
                                
                                if( xClick >= box.x && xClick <= right && yClick >= box.y && yClick <= bottom ){
                                    return;
                                }
                            }
                        }
                    }
                    catch(ex){}
                }
            }
        }

        Ilog.Utilities.CleanEditorResources();

        if (Ilog.Utilities.MouseUpHandler) {
            try {
                return Ilog.Utilities.MouseUpHandler();
            }
            catch (ex) {
                Ilog.Debug.RuleEditorDebugger.PrintDebug('[RuleEditorMouseUpHandler] error#2: ' + ex.message);
            }
        }
    }, Ilog.Constants.RuleEditorConstants.CleanEditorResourcesDelay);

};

Ilog.Utilities.GetScrollbarWidth = function() {
    var tempDiv = null, innerDiv = null;

    // creating temporary invisible div
    var tempDiv = document.createElement('div');
    tempDiv.style.position = 'absolute';
    tempDiv.style.top = '-1000px';
    tempDiv.style.left = '-1000px';
    tempDiv.style.width = '100px';
    tempDiv.style.height = '50px';
    tempDiv.style.overflow = 'hidden';
    // creating temporary inner div which is bigger
    // and adding it to the previous div
    var tempInnerDiv = document.createElement('div');
    tempInnerDiv.style.width = '100%';
    tempInnerDiv.style.height = '200px';
    tempDiv.appendChild(tempInnerDiv);
     
    document.body.appendChild(tempDiv);
 
    var widthWithoutScrollbar = tempInnerDiv.offsetWidth; // getting the width of the inner div, while scrolling is not enabled
    tempDiv.style.overflow = 'auto'; // activating display of scrollbar (inner div is bigger)
    var scrollbarWidth = widthWithoutScrollbar - tempInnerDiv.offsetWidth; // getting the width of the inner div, should be smaller because of the scrollbar
    document.body.removeChild(tempDiv); // removing the divs from the doc
    return (scrollbarWidth);
};

Ilog.Utilities.CheckMouseUpScript = function() {
    try {
        try {
            Ext.EventManager.removeListener(document, 'mouseup', Ilog.Utilities.RuleEditorMouseUpHandler);
        }
        catch (ex) { }

        Ext.EventManager.addListener(document, "mouseup", Ilog.Utilities.RuleEditorMouseUpHandler, this);
    }
    catch (ex) {
        Ilog.Debug.RuleEditorDebugger.PrintDebug('[CheckMouseUpScript] error: ' + ex.message);
    }
};

Ilog.Utilities.StartMouseUpThread = function(){
    if( Ilog.Utilities.MouseUpCheckThread == null ){
        Ilog.Utilities.CheckMouseUpScript();
        Ilog.Utilities.MouseUpCheckThread = window.setTimeout(function(){
            Ilog.Utilities.CheckMouseUpScript();
        }, Ilog.Constants.RuleEditorConstants.MouseUpGlobalScriptCheckInterval);
    }
};

Ilog.Utilities.Store.prototype.GetStoreArray = function(){
    return this.StoreObject;
};

Ilog.Utilities.Store.prototype.AddItem = function(key, value){
    this.StoreObject[key] = value;
    this.Count++;
};

Ilog.Utilities.Store.prototype.GetItem = function(key){
    for(var storedKey in this.StoreObject){
        if( storedKey == key ){
            return this.StoreObject[storedKey];
        }
    }
    return null;
};

Ilog.Utilities.RuleEditorStore = new Ilog.Utilities.Store(); // global (page) object store

Ilog.Utilities.StopEventPropagation = function(e){
    if( e.browserEvent ){
        e.stopPropagation();
    }
    else{
        if( e.stopPropagation ){
            e.stopPropagation();
        }
        
        e.cancelBubble = true;
    }
};

Ilog.Utilities.CancelEventBehaviour = function(e){
    if( e.browserEvent ){
        e.stopEvent();
        if( e.browserEvent.preventDefault ){
            e.browserEvent.preventDefault();
        }
        e.browserEvent.returnValue = false;
    }
    else{
        if( e.preventDefault ){
            e.preventDefault();
        }
        
        e.returnValue = false;
    }
};

Ilog.Utilities.IsFirefox2 = function(){
    if( Ext.isGecko && navigator && navigator.userAgent ){
        var ua = navigator.userAgent;
        if( ua ){
            var tokens = ua.toString().split("/");
            if( tokens && tokens.length > 0 ){
                var lastToken = tokens[tokens.length-1];
                if( lastToken && lastToken.length > 0 ){
                    var minorVersion = lastToken.substring(0,1);
                    if( minorVersion == "2" ){
                        return true;
                    }
                }
            }
        }
    }
    
    return false;
}

Ilog.Utilities.IsLinux = function(){
    if( navigator && navigator.userAgent ){
        var ua = navigator.userAgent;
        if( ua.toString().toUpperCase().indexOf("LINUX", 0) >= 0 ){
            return true;
        }
    }
    return false;
};

Ilog.Utilities.RenderRuleEditorStatusBar = function(containerId){
    var panel = new Ext.Panel({width:100});
    panel.render(containerId);
};

Ilog.Utilities.FindButtonInElt = function(rootElt) {
    var tag = (rootElt.tagName != null && rootElt.tagName != undefined) ? rootElt.tagName : rootElt.nodeName;
    if (!tag) {
        return null;
    }
    
    if (tag.toUpperCase() == "BUTTON") {
        return rootElt;
    }

    for (var p = 0; p < rootElt.childNodes.length; p++) {
        var buttonElt = Ilog.Utilities.FindButtonInElt(rootElt.childNodes[p]);
        if (buttonElt != null) {
            return buttonElt;
        }
    }

    return null;
};

Ilog.Utilities.UpdateToolbarButton = function(button, imageUrl, enabled) {
    if (!button) { return; }

    try {
        var elt = button.getEl().dom;
        var btn = Ilog.Utilities.FindButtonInElt(elt);

        if (imageUrl) {
            btn.style.background = "url(" + imageUrl + ")";
        }
        if (enabled != null && enabled != undefined) {
            button.setDisabled(!enabled);
        }
    }
    catch (ex) { Ilog.Debug.RuleEditorDebugger.PrintDebug('[UpdateToolbarButton] error: ' + ex.message); }
};

Ilog.Utilities.RenderRuleEditorToolbar = function(ruleEditorObject, completionMenuEnabled, sentenceCompletionEnabled) {
    if (!ruleEditorObject) {
        return;
    }

    if (!ruleEditorObject.ToolbarContainerID) {
        return;
    }

    var containerId = ruleEditorObject.ToolbarContainerID;
    if (!document.getElementById(containerId)) {
        return;
    }

    var toolBar = new Ext.Toolbar();
    toolBar.render(containerId);

    toolBar.addSeparator();

    var toolBarIconUndoDisabledConfig = new Object();
    toolBarIconUndoDisabledConfig.minWidth = 25;
    toolBarIconUndoDisabledConfig.disabledClass = '';
    toolBarIconUndoDisabledConfig.disabled = true;
    toolBarIconUndoDisabledConfig.icon = Ilog.Constants.RuleEditorConstants.UrlToolbarUndoDisabledIcon;
    toolBarIconUndoDisabledConfig.iconCls = 'ilog-ruleEditor-toolbarButton';
    toolBarIconUndoDisabledConfig.tooltip = Ilog.Constants.RuleEditorConstants.ToolbarUndoTooltip;
    toolBarIconUndoDisabledConfig.tooltipType = "title";
    toolBarIconUndoDisabledConfig.handler = function() { ruleEditorObject.Focus(); if (ruleEditorObject.IsUndoPossible() === true) { ruleEditorObject.HandleUndoCommand(); } };
    var buttonUndo = new Ext.Toolbar.Button(toolBarIconUndoDisabledConfig);
    toolBar.addButton(buttonUndo);

    if (ruleEditorObject.canPerformInit === true) {
        ruleEditorObject.on("UndoPossible", function(e) {
            Ilog.Utilities.UpdateToolbarButton(buttonUndo, Ilog.Constants.RuleEditorConstants.UrlToolbarUndoIcon, true);
        });

        ruleEditorObject.on("UndoImpossible", function(e) {
            Ilog.Utilities.UpdateToolbarButton(buttonUndo, Ilog.Constants.RuleEditorConstants.UrlToolbarUndoDisabledIcon, false);
        });
    }


    var toolBarIconRedoDisabledConfig = new Object();
    toolBarIconRedoDisabledConfig.minWidth = 25;
    toolBarIconRedoDisabledConfig.disabled = true;
    toolBarIconRedoDisabledConfig.disabledClass = '';
    toolBarIconRedoDisabledConfig.icon = Ilog.Constants.RuleEditorConstants.UrlToolbarRedoDisabledIcon;
    toolBarIconRedoDisabledConfig.iconCls = 'ilog-ruleEditor-toolbarButton';
    toolBarIconRedoDisabledConfig.tooltip = Ilog.Constants.RuleEditorConstants.ToolbarRedoTooltip;
    toolBarIconRedoDisabledConfig.tooltipType = "title";
    toolBarIconRedoDisabledConfig.handler = function() { ruleEditorObject.Focus(); if (ruleEditorObject.IsRedoPossible() === true) { ruleEditorObject.HandleRedoCommand(); } };
    var buttonRedo = new Ext.Toolbar.Button(toolBarIconRedoDisabledConfig);
    toolBar.addButton(buttonRedo);

    if (ruleEditorObject.canPerformInit === true) {
        ruleEditorObject.on("RedoPossible", function(e) {
            Ilog.Utilities.UpdateToolbarButton(buttonRedo, Ilog.Constants.RuleEditorConstants.UrlToolbarRedoIcon, true);
        });

        ruleEditorObject.on("RedoImpossible", function(e) {
            Ilog.Utilities.UpdateToolbarButton(buttonRedo, Ilog.Constants.RuleEditorConstants.UrlToolbarRedoDisabledIcon, false);
        });
    }

    toolBar.addSeparator();

    var toolBarIconCutConfig = new Object();
    toolBarIconCutConfig.minWidth = 25;
    toolBarIconCutConfig.disabled = true;
    toolBarIconCutConfig.disabledClass = '';
    toolBarIconCutConfig.icon = Ilog.Constants.RuleEditorConstants.UrlToolbarCutDisabledIcon;
    toolBarIconCutConfig.iconCls = 'ilog-ruleEditor-toolbarButton';
    toolBarIconCutConfig.tooltip = Ilog.Constants.RuleEditorConstants.ToolbarCutTooltip;
    toolBarIconCutConfig.tooltipType = "title";
    toolBarIconCutConfig.handler = function() { ruleEditorObject.Focus(); ruleEditorObject.HandleCutCommand(); };
    var buttonCut = new Ext.Toolbar.Button(toolBarIconCutConfig);
    buttonCut.IsEnabled = false;
    toolBar.addButton(buttonCut);

    if (ruleEditorObject.canPerformInit === true) {
        ruleEditorObject.on("CutPossible", function(e) {
            Ilog.Utilities.UpdateToolbarButton(buttonCut, Ilog.Constants.RuleEditorConstants.UrlToolbarCutIcon, true);
        });

        ruleEditorObject.on("CutImpossible", function(e) {
            Ilog.Utilities.UpdateToolbarButton(buttonCut, Ilog.Constants.RuleEditorConstants.UrlToolbarCutDisabledIcon, false);
        });
    }

    var toolBarIconCopyConfig = new Object();
    toolBarIconCopyConfig.minWidth = 25;
    toolBarIconCopyConfig.disabled = true;
    toolBarIconCopyConfig.disabledClass = '';
    toolBarIconCopyConfig.icon = Ilog.Constants.RuleEditorConstants.UrlToolbarCopyDisabledIcon;
    toolBarIconCopyConfig.iconCls = 'ilog-ruleEditor-toolbarButton';
    toolBarIconCopyConfig.tooltip = Ilog.Constants.RuleEditorConstants.ToolbarCopyTooltip;
    toolBarIconCopyConfig.tooltipType = "title";
    toolBarIconCopyConfig.handler = function() { ruleEditorObject.Focus(); ruleEditorObject.HandleCopyCommand(); };
    var buttonCopy = new Ext.Toolbar.Button(toolBarIconCopyConfig);
    buttonCopy.IsEnabled = false;
    toolBar.addButton(buttonCopy);

    if (ruleEditorObject.canPerformInit === true) {
        ruleEditorObject.on("CopyPossible", function(e) {
            Ilog.Utilities.UpdateToolbarButton(buttonCopy, Ilog.Constants.RuleEditorConstants.UrlToolbarCopyIcon, true);
        });

        ruleEditorObject.on("CopyImpossible", function(e) {
            Ilog.Utilities.UpdateToolbarButton(buttonCopy, Ilog.Constants.RuleEditorConstants.UrlToolbarCopyDisabledIcon, false);
        });
    }

    var toolBarIconPasteConfig = new Object();
    toolBarIconPasteConfig.minWidth = 25;
    toolBarIconPasteConfig.icon = Ilog.Constants.RuleEditorConstants.UrlToolbarPasteIcon;
    toolBarIconPasteConfig.iconCls = 'ilog-ruleEditor-toolbarButton';
    toolBarIconPasteConfig.tooltip = Ilog.Constants.RuleEditorConstants.ToolbarPasteTooltip;
    toolBarIconPasteConfig.tooltipType = "title";
    toolBarIconPasteConfig.handler = function() { ruleEditorObject.Focus(); if (Ext.isGecko) { ruleEditorObject.HandlePasteGecko(null); } else { ruleEditorObject.HandlePasteCommand(); } };
    toolBar.addButton(toolBarIconPasteConfig);
    toolBar.addSeparator();

    var toolBarIconClearAllConfig = new Object();
    toolBarIconClearAllConfig.minWidth = 25;
    toolBarIconClearAllConfig.icon = Ilog.Constants.RuleEditorConstants.UrlToolbarClearAllIcon;
    toolBarIconClearAllConfig.iconCls = 'ilog-ruleEditor-toolbarButton';
    toolBarIconClearAllConfig.tooltip = Ilog.Constants.RuleEditorConstants.ToolbarClearAllTooltip;
    toolBarIconClearAllConfig.tooltipType = "title";
    toolBarIconClearAllConfig.handler = function() { ruleEditorObject.Focus(); ruleEditorObject.HandleClearAllCommand(); };
    toolBar.addButton(toolBarIconClearAllConfig);

    var toolBarSelectAllConfig = new Object();
    toolBarSelectAllConfig.minWidth = 25;
    toolBarSelectAllConfig.icon = Ilog.Constants.RuleEditorConstants.UrlToolbarSelectAllIcon;
    toolBarSelectAllConfig.iconCls = 'ilog-ruleEditor-toolbarButton';
    toolBarSelectAllConfig.tooltip = Ilog.Constants.RuleEditorConstants.ToolbarSelectAllTooltip;
    toolBarSelectAllConfig.tooltipType = "title";
    toolBarSelectAllConfig.handler = function() { ruleEditorObject.Focus(); ruleEditorObject.HandleSelectAllCommand(); };
    toolBar.addButton(toolBarSelectAllConfig);

    toolBar.addSeparator();

    var toolBarCompletionMenuConfig = new Object();
    toolBarCompletionMenuConfig.minWidth = 25;
    toolBarCompletionMenuConfig.icon = Ilog.Constants.RuleEditorConstants.UrlToolbarCompletionMenuIcon;
    toolBarCompletionMenuConfig.iconCls = 'ilog-ruleEditor-toolbarButton';
    toolBarCompletionMenuConfig.enableToggle = true;
    toolBarCompletionMenuConfig.tooltipType = "title";
    toolBarCompletionMenuConfig.toggleHandler = function(button, state) { if (state === true) { ruleEditorObject.AutoCompletion = true; } else { ruleEditorObject.AutoCompletion = false; } ruleEditorObject.Focus(); };
    toolBarCompletionMenuConfig.tooltip = Ilog.Constants.RuleEditorConstants.ToolbarCompletionMenuTooltip;

    var completionMenuButton = new Ext.Toolbar.Button(toolBarCompletionMenuConfig);
    toolBar.addButton(completionMenuButton);
    completionMenuButton.toggle(completionMenuEnabled);

    var toolBarSentenceCompletionConfig = new Object();
    toolBarSentenceCompletionConfig.minWidth = 25;
    toolBarSentenceCompletionConfig.icon = Ilog.Constants.RuleEditorConstants.UrlToolbarSentenceCompletionIcon;
    toolBarSentenceCompletionConfig.iconCls = 'ilog-ruleEditor-toolbarButton';
    toolBarSentenceCompletionConfig.enableToggle = true;
    toolBarSentenceCompletionConfig.tooltipType = "title";
    toolBarSentenceCompletionConfig.toggleHandler = function(button, state) { if (state == true) { ruleEditorObject.SentenceCompletion = true; } else { ruleEditorObject.SentenceCompletion = false; } ruleEditorObject.Focus(); };
    toolBarSentenceCompletionConfig.tooltip = Ilog.Constants.RuleEditorConstants.ToolbarSentenceCompletionTooltip;

    var sentenceCompletionButton = new Ext.Toolbar.Button(toolBarSentenceCompletionConfig);
    toolBar.addButton(sentenceCompletionButton);
    sentenceCompletionButton.toggle(sentenceCompletionEnabled);

    toolBar.addSeparator();

    ruleEditorObject.HasToolBar = true;
};

Ilog.Utilities.ConvertDateStringFormatToExt = function(dateStringFormat) {
        // Day:
        dateStringFormat = dateStringFormat.replace(/dddd/g, "l"); // full day name (Monday)
        dateStringFormat = dateStringFormat.replace(/EEEE/g, "l"); // full day name (Monday)
        dateStringFormat = dateStringFormat.replace(/EEE/g, "D"); // abbreviation of day name (Mon)

        if (dateStringFormat.indexOf("dd", 0) >= 0) {
            dateStringFormat = dateStringFormat.replace(/dd/g, "d"); // day of the month, 2 digits with leading zeros (01)
        }
        else {
            dateStringFormat = dateStringFormat.replace(/d/g, "j"); // day of the month without leading zeros (1)
        }

        // Month:
        dateStringFormat = dateStringFormat.replace(/MMMM/g, "F"); // full month name (January)

        if (dateStringFormat.indexOf("MMM", 0) >= 0) {
            dateStringFormat = dateStringFormat.replace(/MMM/g, "M"); // abbreviation of month name (Jan)
        }
        else {
            dateStringFormat = dateStringFormat.replace(/MM/g, "m"); // Numeric representation of a month, with leading zeros (01)
            dateStringFormat = dateStringFormat.replace(/M/g, "n"); // Numeric representation of a month, without leading zeros (1)
        }

        // Year:
        dateStringFormat = dateStringFormat.replace(/yyyy/g, "Y"); // A full numeric representation of a year, 4 digits  (2008)
        dateStringFormat = dateStringFormat.replace(/yy/g, "y"); // A two digit representation of a year (08)

        // Hours:
        // 24hrs format
        if (dateStringFormat.indexOf("HH", 0) >= 0) {
            dateStringFormat = dateStringFormat.replace(/HH/g, "H");
        }
        else {
            dateStringFormat = dateStringFormat.replace(/H/g, "G");
        }
        // 12 hrs format
        if (dateStringFormat.indexOf("hh", 0) >= 0) {
            dateStringFormat = dateStringFormat.replace(/hh/g, "h");
        }
        else {
            dateStringFormat = dateStringFormat.replace(/h/g, "g");
        }

        // Minutes:
        dateStringFormat = dateStringFormat.replace(/mm/g, "i");

        // Seconds:
        dateStringFormat = dateStringFormat.replace(/ss/g, "s");

        // AM/PM
        dateStringFormat = dateStringFormat.replace(/tt/g, "A");
        dateStringFormat = dateStringFormat.replace(/aa/g, "A");
        dateStringFormat = dateStringFormat.replace(/a/g, "A");

        /// time zones
        if (Ilog.Constants.RuleEditorConstants.Environment == 1){
            dateStringFormat = dateStringFormat.replace(/Z/g, "O"); // Z means the offset here, whithout separator
            dateStringFormat = dateStringFormat.replace(/zzzz/g, "T"); /// no equivalent of timezone full name, so using the abbreviation
            dateStringFormat = dateStringFormat.replace(/zzz/g, "T");
            dateStringFormat = dateStringFormat.replace(/zz/g, "T");
            dateStringFormat = dateStringFormat.replace(/z/g, "T");
        }
        else{
            dateStringFormat = dateStringFormat.replace(/Z/g, "P"); // Z has no meaning here, so using the offset instead
            dateStringFormat = dateStringFormat.replace(/zzzz/g, "P"); // zzzz has no meaning here, so using the offset instead
            // no distinction possible in ExtJs between offset formatting (+2/+02/+02:00 -> only +02:00)
            dateStringFormat = dateStringFormat.replace(/zzz/g, "P");
            dateStringFormat = dateStringFormat.replace(/zz/g, "P"); 
            dateStringFormat = dateStringFormat.replace(/z/g, "P");
        }
    
    return dateStringFormat;
};

Ilog.Utilities.InjectScriptIntoPage = function(scriptResourceIdentifier, scriptResource, scriptType) {
    try {
        var registeredScript = Ext.DomQuery.select("script[id=" + scriptResourceIdentifier + "]");
        if (registeredScript && registeredScript.length > 0) {
            return true;
        }

        var oHead = document.getElementsByTagName("head").item(0);
        var oScript = document.createElement("script");
        oScript.language = "javascript";
        oScript.type = "text/javascript";

        if (scriptType == Ilog.Constants.RuleEditorConstants.FullScriptMode) {
            oScript.text = scriptResource;
        }
        else {
            // received <script src="..."..></script>
            if (scriptResource.length > 7 && scriptResource.substr(0, 7).toUpperCase() == "<SCRIPT") {
                var regExp = '<script([^>]*)src=\"([^"]*)\"([^>]*)>';
                var rgx = new RegExp(regExp, "gi");
                var res = rgx.exec(scriptResource);

                if (res && res.length >= 3) {
                    oScript.src = res[2];
                }
                else {
                    return false;
                }
            }
            // received the URL only
            else {
                oScript.src = scriptResource;
            }

        }

        oScript.id = scriptResourceIdentifier;
        oHead.appendChild(oScript);
        return true;
    }
    catch (ex) {
        Ilog.Debug.RuleEditorDebugger.PrintDebug('[InjectScriptIntoPage] error: ' + ex.message);
        return false;
    }
};

Ilog.Utilities.IsNodeFrozen = function(node){
    if( node && node.className && node.className.toUpperCase() == Ilog.Constants.RuleEditorConstants.FrozenTextCssClass.toUpperCase() ){
        return true;
    } 
    
    return false;
};

Ilog.Utilities.GetPredictionIconUrlForIconId = function(iconId){
    var iconFileName = '';
    
    if( iconId == Ilog.Constants.RuleEditorConstants.IcVocabulary ){
        iconFileName = 'Vocabulary';
    }
    else if( iconId == Ilog.Constants.RuleEditorConstants.IcVocabularyDomain ){
        iconFileName = 'VocabularyDomain';
    }
    else if( iconId == Ilog.Constants.RuleEditorConstants.IcConcept ){
        iconFileName = 'Concept';
    }
    else if( iconId == Ilog.Constants.RuleEditorConstants.IcConceptInstance ){
        iconFileName = 'ConceptInstance';
    }
    else if( iconId == Ilog.Constants.RuleEditorConstants.IcSentence ){
        iconFileName = 'Sentence';
    }
    else if( iconId == Ilog.Constants.RuleEditorConstants.IcSentenceAction ){
        iconFileName = 'SentenceAction';
    }
    else if( iconId == Ilog.Constants.RuleEditorConstants.IcSentenceGetter ){
        iconFileName = 'SentenceGetter';
    }
    else if( iconId == Ilog.Constants.RuleEditorConstants.IcSentenceSetter ){
        iconFileName = 'SentenceSetter';
    }
    else if( iconId == Ilog.Constants.RuleEditorConstants.IcSentenceOperator ){
        iconFileName = 'SentenceOperator';
    }
    else if( iconId == Ilog.Constants.RuleEditorConstants.IcSentencePredicate ){
        iconFileName = 'SentencePredicate';
    }
    else if( iconId == Ilog.Constants.RuleEditorConstants.IcVariable ){
        iconFileName = 'Variable';
    }
    else if( iconId == Ilog.Constants.RuleEditorConstants.IcAutomaticVariable ){
        iconFileName = 'AutomaticVariable';
    }
    else if( iconId == Ilog.Constants.RuleEditorConstants.IcImplicitVariable ){
        iconFileName = 'ImplicitVariable';
    }
    else if( iconId == Ilog.Constants.RuleEditorConstants.IcText ){
        iconFileName = 'Text';
    }
    else if( iconId == Ilog.Constants.RuleEditorConstants.IcValue ){
        iconFileName = 'Value';
    }
    else if( iconId == Ilog.Constants.RuleEditorConstants.IcEmpty ){
        iconFileName = 'Empty';
    }
    else if( iconId == Ilog.Constants.RuleEditorConstants.IcNamespace ){
        iconFileName = 'Namespace';
    }
    else if( iconId == Ilog.Constants.RuleEditorConstants.IcFolder ){
        iconFileName = 'Folder';
    }
    
    var resourceToEval = 'Ilog.Constants.RuleEditorConstants.UrlPredictionIcon'+iconFileName;
    var resource = window.eval(resourceToEval);
    return resource;
};

Ilog.Utilities.UpdateStatusBar = function(status, containerId, cssClass, saveIndicatorContainerId){
    var container = document.getElementById(containerId);
    if( ! container ){
        return;
    }
    
    if( saveIndicatorContainerId ){
        var saveIndicatorContainer = document.getElementById(saveIndicatorContainerId);
    }
    
    if( status == 0 ){
        container.innerHTML = "<img src=\""+Ilog.Constants.RuleEditorConstants.UrlStatusBarImageOK+"\" alt=\""+Ilog.Constants.RuleEditorConstants.MessageStatusBarOK+"\" class=\""+cssClass+"\" />"+Ilog.Constants.RuleEditorConstants.MessageStatusBarOK;
    }
    else if( status == 1 ){
        container.innerHTML = "<img src=\""+Ilog.Constants.RuleEditorConstants.UrlStatusBarImageLoader+"\" alt=\""+Ilog.Constants.RuleEditorConstants.MessageStatusBarLoader+"\" class=\""+cssClass+"\" />"+Ilog.Constants.RuleEditorConstants.MessageStatusBarLoader;
    }
    else if( status == 2 ){
        container.innerHTML = "<img src=\""+Ilog.Constants.RuleEditorConstants.UrlStatusBarImageTimeout+"\" alt=\""+Ilog.Constants.RuleEditorConstants.MessageStatusBarTimeout+"\" class=\""+cssClass+"\" />"+Ilog.Constants.RuleEditorConstants.MessageStatusBarTimeout;
    }
    else if( status == 3 ){
        container.innerHTML = "<img src=\""+Ilog.Constants.RuleEditorConstants.UrlStatusBarImageKO+"\" alt=\""+Ilog.Constants.RuleEditorConstants.MessageStatusBarKO+"\" class=\""+cssClass+"\" />"+Ilog.Constants.RuleEditorConstants.MessageStatusBarKO;
    }
    // save sent
    else if( status == 4 ){
        if( saveIndicatorContainer ){
            saveIndicatorContainer.innerHTML = "<img src=\""+Ilog.Constants.RuleEditorConstants.UrlSaveImage+"\" alt=\"\" class=\""+cssClass+"\" />";
        }
    }
    // save OK
    else if( status == 5 ){
        if( saveIndicatorContainer ){
            window.setTimeout(function(){ saveIndicatorContainer.innerHTML = ''; }, 500);
        }
    }

};

Ilog.Utilities.ManagePredictionViewTreeNodeClick = function(treenodeImage, treenodeChildrenAreaId){
    var treenodeStatus = Ilog.Constants.RuleEditorConstants.TreenodeDefaultStatus;
    if( treenodeImage.treenodeStatus != null && treenodeImage.treenodeStatus != undefined ){
        treenodeStatus = treenodeImage.treenodeStatus;
    }
    
    if( treenodeStatus == Ilog.Constants.RuleEditorConstants.TreenodeCollapsed ){
        document.getElementById(treenodeChildrenAreaId).style.display = 'inline'; 
        treenodeImage.src = Ilog.Constants.RuleEditorConstants.UrlTreeviewIconCollapse;
        treenodeImage.treenodeStatus = Ilog.Constants.RuleEditorConstants.TreenodeExpanded;
    }else{
        document.getElementById(treenodeChildrenAreaId).style.display = 'none'; 
        treenodeImage.src = Ilog.Constants.RuleEditorConstants.UrlTreeviewIconExpand;
        treenodeImage.treenodeStatus = Ilog.Constants.RuleEditorConstants.TreenodeCollapsed;
    }
};

Ilog.Utilities.GetDocumentScrollTop = function() {
     var scrollTop = (window.pageYOffset!=undefined)?window.pageYOffset:document.documentElement.scrollTop;
     return scrollTop;
};

Ilog.Utilities.GetDocumentScrollLeft = function() {
     var scrollLeft = (window.pageXOffset!=undefined)?window.pageXOffset:document.documentElement.scrollLeft;
     return scrollLeft;
};

Ilog.Utilities.GetBrowserWindowDimensions = function(){
    var browserWindowWidth, browserWindowHeight;
    
    if (window.innerWidth) {
        browserWindowWidth = window.innerWidth;
    }
    else if (document.documentElement && document.documentElement.clientWidth) {
        browserWindowWidth = document.documentElement.clientWidth;
    }
    else if (document.body) {
        browserWindowWidth = document.body.clientWidth;
    }
    
    if (window.innerHeight) {
        browserWindowHeight = window.innerHeight;
    }
    else if (document.documentElement && document.documentElement.clientHeight) {
        browserWindowHeight = document.documentElement.clientHeight;
    }
    else if (document.body) {
        browserWindowHeight = document.body.clientHeight;
    }
    
    return { width:browserWindowWidth, height:browserWindowHeight};
};

Ilog.Utilities.GetTimeStamp = function() {
    var currentDatetime = new Date();
    var timestamp = currentDatetime.getTime();
    return timestamp;
};

Ilog.Utilities.addLoadEvent = function(func){
    var oldonload = window.onload;
    if (typeof window.onload != 'function') {
        window.onload = func;
    }
    else {
        window.onload = function() {
            oldonload();
            func();
        }
    }
};

Ilog.Utilities.StringBuilder = function(value){
	this.strings = new Array("");
	this.append(value);
};

Ilog.Utilities.StringBuilder.prototype.append = function (value){
	if (value){
		this.strings.push(value);
	}
};

Ilog.Utilities.StringBuilder.prototype.clear = function (){
	this.strings.length = 1;
};

Ilog.Utilities.StringBuilder.prototype.toString = function (){
	return this.strings.join("");
};

Ilog.Utilities.GetNodeStartOffset = function(bodyNode, nodeToSearch){
    var offset = 0;
    for(var p=0;p<bodyNode.childNodes.length;p++){
        var currentNode = bodyNode.childNodes[p];
        if( currentNode == nodeToSearch ){
            break;
        }
        else if( currentNode.contains && currentNode.contains(nodeToSearch)  ){
            break;
        }
        else if( currentNode.compareDocumentPosition && ( nodeToSearch.compareDocumentPosition(currentNode) & 16) ){
            break;
        }
        else{
            offset += Ilog.Utilities.GetNodeLength(currentNode);
        }
    }
    return offset;
};

Ilog.Utilities.GetNodeEndOffset = function(bodyNode, nodeToSearch){
    return Ilog.Utilities.GetNodeStartOffset(bodyNode, nodeToSearch)+Ilog.Utilities.GetNodeLength(nodeToSearch);
};

Ilog.Utilities.FindAdjacentNodes = function(bodyNode, originalNode, recognitionPattern){
    if( ! originalNode ){
        return null;
    }
    
    var groupId = originalNode.getAttribute(recognitionPattern);
    var beginningOffset = Ilog.Utilities.GetNodeStartOffset(bodyNode, originalNode);
    var endOffset = beginningOffset+Ilog.Utilities.GetNodeLength(originalNode);
    
    var node = originalNode;
    if( node.previousSibling ){
        node = node.previousSibling;
        
        while( node && node.getAttribute && node.getAttribute(recognitionPattern) && node.getAttribute(recognitionPattern) == groupId ){
            var old = beginningOffset.toString()+'';
            beginningOffset = Ilog.Utilities.GetNodeStartOffset(bodyNode, node);
            node = node.previousSibling;
        }
    }
    
    node = originalNode;
    if( node.nextSibling ){
        node = node.nextSibling;
        while( node && node.getAttribute && node.getAttribute(recognitionPattern) && node.getAttribute(recognitionPattern) == groupId ){
            endOffset = Ilog.Utilities.GetNodeStartOffset(bodyNode, node)+Ilog.Utilities.GetNodeLength(node);
            node = node.nextSibling;
        }
    }
    
    return {start:beginningOffset, end:endOffset};
}

Ilog.Utilities.CheckNodeInsideNode = function(nodeToFind, nodeToCheck){
    if( ! nodeToFind || ! nodeToCheck ){
        return;
    }

    if( nodeToCheck.childNodes && nodeToCheck.childNodes.length > 0 ){
        var caretSum = 0;
        for(var k=0;k<nodeToCheck.childNodes.length;k++){
            var childNode = nodeToCheck.childNodes[k];
            if( childNode == nodeToFind ){
                return caretSum;
            }
            else{
                var val = Ilog.Utilities.CheckNodeInsideNode(nodeToFind, childNode);
                if( val >= 0 ){
                    return caretSum+val;
                }
                else if (childNode.nodeName == 'br' || childNode.nodeName == 'BR') {
                    caretSum += 1;
                }
                else if( childNode.textContent ){
                    caretSum += childNode.textContent.length;
                }
            }
        }
    }
    else{
        if( nodeToFind == nodeToCheck ){
            return 0;
        }
    }    

    return -1;
}
    
Ilog.Utilities.GetNodeAndRelativeOffsetForAbsoluteOffset = function(editedDocument, positionToMoveTo){
    var bodyNode = editedDocument.body;
    var currentLength = 0;
    var bOk = false;
    var positionNodeFound = false;

    for(var p=0;p<bodyNode.childNodes.length;p++){
    var currentNode = bodyNode.childNodes[p];
    var nodeLength = Ilog.Utilities.GetNodeLength(currentNode);

        if( positionToMoveTo >= currentLength && positionToMoveTo <= (currentLength+nodeLength) && ! bOk ) {
            var relativePositionInNode = positionToMoveTo - currentLength;
            
            if( currentNode.textContent && currentNode.textContent.length < relativePositionInNode ){
                relativePositionInNode = currentNode.textContent.length;
            }
                           
            if( relativePositionInNode <= nodeLength && ! positionNodeFound )
            {
                var positionNode = currentNode;
                if( currentNode.nodeType == 1 ){
                    if( currentNode.childNodes.length > 0 ) {
                        for(var m=0;m<currentNode.childNodes.length && ! positionNodeFound;m++) {
                            if( currentNode.childNodes[m].nodeType == 3 ) {
                                positionNode = currentNode.childNodes[m];
                                positionNodeFound = true;
                            }
                        }
                    }
                }
                
                var shouldAddRange = false;
                try {
                    // Fix for positioning caret AFTER a BR node
                    // works but the caret is not blinking anymore
                     if( positionNode.nodeName.toUpperCase() == 'BR' || (positionNode.tagName && positionNode.tagName.toUpperCase() == 'BR') ){
                        Ilog.Debug.RuleEditorDebugger.PrintDebug('[GetNodeAndRelativeOffsetForAbsoluteOffset] positionNode is a BR');
                        if( positionNode.nextSibling ){
                            if( positionNode.nextSibling.nodeName && positionNode.nextSibling.nodeName.toUpperCase() == 'BR' ){
                                // inserting a blank node just for positioning
                                    var elt = editedDocument.createElement('span');
                                    editedDocument.body.insertBefore(elt, positionNode.nextSibling);
                                    positionNode = elt;
                                    var relativePositionInNode = 0;
                                    return { node:positionNode, offset:relativePositionInNode };
                            }
                            else{
                                positionNode = positionNode.nextSibling;
                                var relativePositionInNode = 0;
                                return { node:positionNode, offset:relativePositionInNode };
                            }
                        }
                        
                    }
                    
                    return { node:positionNode, offset:relativePositionInNode };
                }
                catch(rangeException) {
                    Ilog.Debug.RuleEditorDebugger.PrintDebug("[GetNodeAndRelativeOffsetForAbsoluteOffset] error while setting range (on node " + positionNode.nodeName + " - " + positionNode.nodeValue + "): " + rangeException.message);
                }
            }
            else{
                return { node:currentNode, offset:relativePositionInNode };
            }
                    
            try {
                if( shouldAddRange ){
                     return { node:currentNode, offset:relativePositionInNode };
                }
             }
             catch(rangeException) {
                 Ilog.Debug.RuleEditorDebugger.PrintDebug("[GetNodeAndRelativeOffsetForAbsoluteOffset] error while setting range in selection ('" + positionNode.nodeName + "'): " + rangeException.message);
             }
            
            if( ! shouldAddRange ){
                Ilog.Debug.RuleEditorDebugger.PrintDebug("[GetNodeAndRelativeOffsetForAbsoluteOffset] range could not be added (position " + positionToMoveTo + ")");
            }
            
            bOk = true;
            }
            currentLength += nodeLength;
        }
        
    return null;
}

Ilog.Utilities.ShowClipboardErrorWindow = function(ruleEditorInstance){
    var clipboardErrorLayer = ruleEditorInstance.ClipboardErrorLayer;
    
    if( clipboardErrorLayer == null ){       
        var protocolHostAndPort = window.location.protocol+'//'+window.location.host;
        var stringBuilder = new Ilog.Utilities.StringBuilder('');
        stringBuilder.append('<div style="padding:5px">');
        
        if( Ilog.Constants.RuleEditorConstants.MozillaClipboardErrorWindowMessage1 ){
            stringBuilder.append(Ilog.Constants.RuleEditorConstants.MozillaClipboardErrorWindowMessage1);
            stringBuilder.append('<br/>');
        }
                
        stringBuilder.append('<ul>');
        
        if( Ilog.Constants.RuleEditorConstants.MozillaClipboardErrorWindowMessage2 ){
            stringBuilder.append('<li>');
            stringBuilder.append(Ilog.Constants.RuleEditorConstants.MozillaClipboardErrorWindowMessage2);
            stringBuilder.append('</li>');
        }
        if( Ilog.Constants.RuleEditorConstants.MozillaClipboardErrorWindowMessage3 ){
            stringBuilder.append('<li>');
            stringBuilder.append(Ilog.Constants.RuleEditorConstants.MozillaClipboardErrorWindowMessage3);
            stringBuilder.append('</li>');
        }
        if( Ilog.Constants.RuleEditorConstants.MozillaClipboardErrorWindowMessage4 ){
            stringBuilder.append("<li>");
            stringBuilder.append(Ilog.Constants.RuleEditorConstants.MozillaClipboardErrorWindowMessage4);
            stringBuilder.append("</li>");
        }
        if( Ilog.Constants.RuleEditorConstants.MozillaClipboardErrorWindowMessage5 ){
            stringBuilder.append('<li>');
            stringBuilder.append(Ilog.Constants.RuleEditorConstants.MozillaClipboardErrorWindowMessage5);
            
            if( Ilog.Constants.RuleEditorConstants.MozillaClipboardErrorWindowMessage6a ){
                stringBuilder.append('<br/><table align="center" style="border:1px black solid;padding:3px;"><tr><td>');
                stringBuilder.append(Ilog.Constants.RuleEditorConstants.MozillaClipboardErrorWindowMessage6a.replace('{0}', protocolHostAndPort));
                stringBuilder.append('</td></tr><tr><td>');
                stringBuilder.append(Ilog.Constants.RuleEditorConstants.MozillaClipboardErrorWindowMessage6b.replace('{0}', protocolHostAndPort));
                stringBuilder.append('</td></tr><tr><td>');
                stringBuilder.append(Ilog.Constants.RuleEditorConstants.MozillaClipboardErrorWindowMessage6c.replace('{0}', protocolHostAndPort));
                stringBuilder.append('</td></tr><tr><td>');
                stringBuilder.append(Ilog.Constants.RuleEditorConstants.MozillaClipboardErrorWindowMessage6d.replace('{0}', protocolHostAndPort));
                stringBuilder.append('</td></tr></table>');
            }
            stringBuilder.append('</li>');
        }
        if( Ilog.Constants.RuleEditorConstants.MozillaClipboardErrorWindowMessage7 ){
            stringBuilder.append('<li>');
            stringBuilder.append(Ilog.Constants.RuleEditorConstants.MozillaClipboardErrorWindowMessage7);
            stringBuilder.append('</li>');
        }
        stringBuilder.append('</ul>');
        stringBuilder.append('</div>');
        
             var xy = ruleEditorInstance.GetXYAbsolutePositionOfCaret();
             if( xy == null || xy == undefined ){
                xy = {x:0,y:0};
             }
             if( ! xy.x ){
                xy.x = 0;
             }
             if( ! xy.y ){
                xy.y = 0;
             }
             
             var xposition = xy.x;
             var yposition = xy.y;

                // Checking absolute caret position vs iframe bounds  ("blocking" position to the border of the iframe)
                 var extElm = new Ext.Element(ruleEditorInstance.editorField);
                 var iframePosition = {x:extElm.getLeft(), y:extElm.getTop(), width:extElm.getWidth(), height:extElm.getHeight() };
                 if( xposition < iframePosition.x ){
                    xposition = iframePosition;
                 }
                 if( xposition > (iframePosition.x+iframePosition.width) ){
                    xposition = iframePosition.x+iframePosition.width;
                 }
                 if( yposition < iframePosition.y ){
                    yposition = iframePosition.y;
                 }
                 if( yposition > (iframePosition.y+iframePosition.height) ){
                    yposition = iframePosition.y+iframePosition.height;
                 }
      
        clipboardErrorLayer = new Ext.Window({
                    modal: false,
                    closeAction:'hide',
                    title:Ilog.Constants.RuleEditorConstants.MozillaClipboardErrorWindowTitle,
                    cls:'ilog-ruleEditor-ClipboardErrorLayer',
                    plain:true,
                    layout:'fit',
                    pageX: iframePosition.x,
                    pageY:yposition,
                    width:iframePosition.width,
                    resizable: false,
                    html: stringBuilder.toString().replace('{0}', '<b>'+Ilog.Constants.RuleEditorConstants.FirefoxSetting0+'</b>').replace('{1}', '<b>'+Ilog.Constants.RuleEditorConstants.FirefoxSetting1+'</b>')
                });
        
        clipboardErrorLayer.show();
        
        ruleEditorInstance.ClipboardErrorLayer = clipboardErrorLayer;
        clipboardErrorLayer.on("close", function(e){ ruleEditorInstance.ClipboardErrorLayer = null; });
    }
    else{
        clipboardErrorLayer.show();
    }
};

Ilog.Utilities.GetTextDimensions = function(text) {
    var rulerId = 'IlogRuleEditorRuler';
    var ruler = document.getElementById(rulerId);
    if (ruler == null) {
        ruler = document.createElement('span');
        ruler.id = rulerId;
        ruler.style.visibility = 'hidden';
        document.body.appendChild(ruler);
    }

    ruler.innerHTML = text;
    return { width: ruler.offsetWidth, height: ruler.offsetHeight };
};

Ilog.Utilities.GetTextLength = function(text) {
    return Ilog.Utilities.GetTextDimensions(text).width;
};

Ilog.Utilities.GetTextHeight = function(text) {
    return Ilog.Utilities.GetTextDimensions(text).height;
};

Ilog.Utilities.ExtractTextFromNode = function(node){
    var text = '';

    if( node != null ){
        if( node.tagName && node.tagName.toUpperCase() == 'BR' ){
            text = '\n';
        }
        else if( node.nodeType != null && node.nodeType == 3 ){
            var nodeTextContent = node.nodeValue;
            nodeTextContent = nodeTextContent.replace(/\n/g, '');
            text = nodeTextContent;
        }
        else{
            if( node.childNodes != null && node.childNodes.length > 0 ){
                for(var p=0;p<node.childNodes.length;p++){
                    text = text+Ilog.Utilities.ExtractTextFromNode(node.childNodes[p]);
                }
            }
        }
    }

    return text;
};

Ilog.Utilities.CleanNode = function(potentialNodeToClean){
    var result = new Array();
    
    if( potentialNodeToClean != null ){
        // Full text node : no need to clean
        if( potentialNodeToClean.nodeType == 3 ){
            result[0] = 0;
            return result;
        }
        
        if( potentialNodeToClean.tagName && potentialNodeToClean.tagName != "" && potentialNodeToClean.tagName.toUpperCase() != "SPAN" && potentialNodeToClean.tagName.toUpperCase() != "BR" ){
            var replacedNode = null;
            var text = Ilog.Utilities.ExtractTextFromNode(potentialNodeToClean);
            if( text != null && text != '' ){
                replacedNode = document.createElement('span');
                replacedNode.innerHTML = text;
            }
            
            result[0] = 1;
            result[1] = replacedNode;
            return result;
        }
                
        if( potentialNodeToClean.childNodes &&  potentialNodeToClean.childNodes.length > 0 ){
            for(var k=0;k<potentialNodeToClean.childNodes.length;k++){
                var subNode = potentialNodeToClean.childNodes[k];
                
                var cleanedSubNodeStatus = Ilog.Utilities.CleanNode(subNode);
                
                if( cleanedSubNodeStatus != null && cleanedSubNodeStatus[0] == 1 ){
                    var newNode = cleanedSubNodeStatus[1];
                    
                    if( newNode != null ){
                        potentialNodeToClean.replaceChild(cleanedSubNodeStatus[1], potentialNodeToClean.childNodes[k]);
                    }
                    else{
                        potentialNodeToClean.removeChild(potentialNodeToClean.childNodes[k]);
                    }
                    
                    result[0] = 1;
                    result[1] = potentialNodeToClean;
                    return result;
                }
            }
        }
    }    
    
    result[0] = 0;
    return result;
};

Ilog.Utilities.GetUndoCommandHandler = function(ruleEditorInstance) {
    if( ruleEditorInstance != null ) {
        return ruleEditorInstance.HandleUndoCommand();
    }
};

Ilog.Utilities.GetRedoCommandHandler = function(ruleEditorInstance) {
    if( ruleEditorInstance != null ) {
        return ruleEditorInstance.HandleRedoCommand();
    }
};

Ilog.Utilities.GetCutCommandHandler = function(ruleEditorInstance) {
    if( ruleEditorInstance != null ) {
        return ruleEditorInstance.HandleCutCommand();
    }
};

Ilog.Utilities.GetCopyCommandHandler = function(ruleEditorInstance) {
    if( ruleEditorInstance != null ) {
        return ruleEditorInstance.HandleCopyCommand();
    }
};

Ilog.Utilities.GetPasteCommandHandler = function(ruleEditorInstance) {
    if( ruleEditorInstance != null ) {
        return ruleEditorInstance.HandlePasteCommand();
    }
};

Ilog.Utilities.GetClearAllCommandHandler = function(ruleEditorInstance) {
    if( ruleEditorInstance != null ) {
        return ruleEditorInstance.HandleClearAllCommand();
    }
};

Ilog.Utilities.GetSelectAllCommandHandler = function(ruleEditorInstance) {
    if( ruleEditorInstance != null ) {
        return ruleEditorInstance.HandleSelectAllCommand();
    }
};

Ilog.Utilities.HidePredictionsView = function (evt){
     var ruleEditorInstance = this;
     
     if( ruleEditorInstance.PredictionsView ){  
         ruleEditorInstance.PredictionsView.Hide();    
                 
         if( ruleEditorInstance.contentDocument ){
            ruleEditorInstance.contentWindow.focus();
         }
         else{
            ruleEditorInstance.editorDocument.body.focus();
            
            var caretPosition =  ruleEditorInstance.PredictionsView.ReplacementArea;
            
            if( ruleEditorInstance.PredictionsView.CurrentCaretPosition ){
                caretPosition = ruleEditorInstance.PredictionsView.CurrentCaretPosition;
            }
            
            ruleEditorInstance.SelectTextRange(caretPosition);
            ruleEditorInstance.editorDocument.body.focus();           
         }
         
         ruleEditorInstance.SendRefreshRequest();
     }
};

Ilog.Utilities.GetNodeLength = function (node){
    if( node == null || ! node.nodeType ) { return 0; }

    // TEXT nodes
    if( node.nodeType == 3 ) {
        if( node.textContent ){
            return node.textContent.length;
        } 
        else{ 
            return node.nodeValue.length;
        }
    }

    // ELEMENT nodes
    if( node.nodeType == 1 ) {
        var nodeLength = 0;
        
         if( node.nodeName.toUpperCase() == 'BR' || (node.tagName && node.tagName.toUpperCase() == 'BR') ) {
            nodeLength = 1;
         }
         else{
            if( node.textContent ){
                nodeLength = node.textContent.length;
            }
            else{
                if( node.nodeValue ) { nodeLength += node.nodeValue; } 
                if( node.childNodes && node.childNodes.length > 0 ) { 
                    for(var p=0;p<node.childNodes.length;p++) {
                        nodeLength += Ilog.Utilities.GetNodeLength(node.childNodes[p]);
                    }        
                } 
            }
         }
    
         return nodeLength;
    }

    return 0;
};

Ilog.Utilities.GetElementPredictionTypeName = function(type){ 
    if( type == Ilog.Constants.RuleEditorConstants.PredictionConceptValue ){
        return Ilog.Constants.RuleEditorConstants.LabelPredictionConceptValue;
    }
    else if( type == Ilog.Constants.RuleEditorConstants.PredictionConceptVariable ){
        return Ilog.Constants.RuleEditorConstants.LabelPredictionConceptVariable;
    }
    else if( type == Ilog.Constants.RuleEditorConstants.PredictionConceptSentence ){
        return Ilog.Constants.RuleEditorConstants.LabelPredictionConceptSentence;
    }
    else if( type == Ilog.Constants.RuleEditorConstants.PredictionConceptTarget ){
        return Ilog.Constants.RuleEditorConstants.LabelPredictionConceptTarget;
    }

    return Ilog.Constants.RuleEditorConstants.LabelPredictionConceptOther;
}; 

Ilog.Utilities.TestTimeoutSession = function (){
    if( Ilog.RTSUtilities && Ilog.RTSUtilities.TestTimeoutSession ){
        Ilog.RTSUtilities.TestTimeoutSession();
    } 
};

// extending node object, adding "contains" method
if (window.Node && Node.prototype && !Node.prototype.contains){
    Node.prototype.contains = function (arg) {
	    return !!(this.compareDocumentPosition(arg) & 16)
    }
}