// --------------------------------------------------------------- -- JS --
// IBM Confidential     
// OCO Source Materials  
// 5724-Y00 
//  Copyrights by IBM Corp 1987, 2010. - All rights Reserved
// The source code for this program is not published or otherwise
// divested of its trade secrets, irrespective of what has
// been deposited with the U.S. Copyright Office.
// --------------------------------------------------------------------------

/**
 * Decision table component
 * 
 * listen to key events to simulate keyboard controls
 */
function IlrDTWTable(port, id, onKeyUpAction){
  this.base = IlxWComponent;
  this.base(port, id);
  this.stateToXml = IlrDTWTable_stateToXml;
  this.resetState = IlrDTWTable_resetState;
  this.onKeyUp = IlrDTWTable_onKeyUp;
  this.onKeyPress = IlrDTWTable_onKeyPress;
  this.setFocusOnEditor= IlrDTWTable_setFocusOnEditor;
  this.onKeyUpAction = onKeyUpAction;
  this.keyCode=-1;
  this.charCode=0;
}

/**
 * listen keypressed events and keep the char code in memory
 * the value will be used during the keyup event
 */
function IlrDTWTable_onKeyPress(event){
	if ( typeof event == 'undefined' ) event = window.event;
	// in IE on keyCode, in FF charCode in keypressed events
	this.charCode = event.charCode || event.keyCode;
}

/**
 * collect all on key up events on the table 
 * mouvement must be captured through on key up as 
 * onkeypress is not triggered by IE for control keys like arrows 
 */
function IlrDTWTable_onKeyUp(event){
	if ( typeof event == 'undefined' ) event = window.event;
	wkey = event.keyCode;
	if ( document.layers ) wkey = event.which;
	var target = event.target || event.srcElement;
	if ((target.tagName != 'INPUT')&&(target.tagName != 'SELECT')&&(target.tagName != 'TEXTAREA')){
		var onKeyUpAction = this.onKeyUpAction;
  		if (IlxWUtil.instance.isDefined(onKeyUpAction)) {
  			// arrows
  			if ((wkey == 37)||(wkey == 38)||(wkey == 39)||(wkey == 40)){
  	  			this.invalidate(false);
  	  			this.keyCode = wkey;
  	  			onKeyUpAction.perform(this);
  	  		}
  	  		else if ((wkey == 13) || (wkey == 17)){
  	  			// do nothing this might be an event from validating an opened input
  	  			// prevent ctrl key also 
  	  			// TODO: for combination keys like ctrl+C the second keypresssed is passed through in FF
  	  			// and gives special character in IE
  	  		}
  	  		else if (wkey == 46){ // escape
  	  			// set focus on currently selected editor
  	  			this.setFocusOnEditor(-1);
  	  		}
  	  		//filter out control characters like Ctrl+V, Ctrl+Z
  	  		else if (this.charCode > 31){
  	  			// set focus on currently selected editor
  	  			this.setFocusOnEditor(this.charCode);
  	  		}
		}
	}
}
 
 
/**
 * set the focus on the first editor found inside the table
 */
function IlrDTWTable_setFocusOnEditor(charCode){
	table = IlrDT_returnObjById(this.path);
	editor = IlrDT_returnFirstObjByTags(table, new Array('input','select'), new Array('table'));
	if (editor != null){
		editor.focus();
		if (charCode != -1){
			editor.value = editor.value+String.fromCharCode(charCode);
		}
		editor.blur();
		editor.focus();
	} 	
}

function IlrDTWTable_stateToXml(){
	return "<keyCode value=\"" + this.keyCode + "\"/>";
}
 
function IlrDTWTable_resetState(){
	this.keyCode = -1;
}