// --------------------------------------------------------------- -- JS --
// IBM Confidential     
// OCO Source Materials  
// 5724-Y00 
//  Copyrights by IBM Corp 1987, 2010. - All rights Reserved
// The source code for this program is not published or otherwise
// divested of its trade secrets, irrespective of what has
// been deposited with the U.S. Copyright Office.
// --------------------------------------------------------------------------

/*
 *  (C) Copyright IBM Corp. 1987, 2009
 *  All rights reserved.
 */

/**
 * <code>IlvMapInteractor</code> is an interactor that lets you select a point
 * on the <code>IlvView</code> to perform an action on the server.
 * The action is specified by a name that can be set by
 * <code>setActionName</code>. Once the interactor has been
 * installed on the view, when the user clicks on the image, a new
 * request is sent to the server with additional parameters.
 * To implement the action on the server side, you will have to implement
 * a subclass of <code>ServerActionListener</code> on the server side.
 * @param actionName The name of the action.
 */
function IlvMapInteractor(actionName) {
  this.superConstructor();
  this.setCursor("crosshair");
  this.setActionName(actionName);
}

IlvMapInteractor.prototype = new IlvInteractor;
IlvMapInteractor.prototype.setClassName("IlvMapInteractor");

/**
 * Returns the name of the action to perform on the server side.
 */
IlvMapInteractor.prototype.getActionName = function() {
  return this.action;
}

/**
 * Changes the name of the action to perform on the server side.
 */
IlvMapInteractor.prototype.setActionName = function(actionName) {
  return this.action = actionName;
}

/*
 * Handles mouse down events.
 */
IlvMapInteractor.prototype.onmousedown = function(e) {
  var view = this.getView();
  var x = e.mouseX;
  var y = e.mouseY;
  var bbox = view.getViewBBox();

  var oldx = bbox.left + x*(bbox.width/view.getWidth());
  var oldy = bbox.top + y*(bbox.height/view.getHeight());
//  e.window.alert(oldx+" : "+oldy);

  x = bbox.left + x * ((view.maxx - view.minx) / (view.imagePanel.width * bbox.width));
  y = bbox.top +  y * ((view.maxy - view.miny) / (view.imagePanel.height * bbox.height));

//  e.window.alert(x+" : "+y);
//  x = oldx;
//  y = oldy; 

  var params = new Array();
  params[0] = x;
  params[1] = y
  view.performAction(this.action, params);
  return false;
}
