// --------------------------------------------------------------- -*- JS -*-
// IBM Confidential
// OCO Source Materials
// 5724-Y00 5724-Y02 5724-Y21
//  Copyright IBM Corp. 2004, 2010. All Rights Reserved.
// The source code for this program is not published or otherwise
// divested of its trade secrets, irrespective of what has
// been deposited with the U.S. Copyright Office.
// --------------------------------------------------------------------------

Ext.namespace('Ilog');

/***************************/
/* Rule Editor constructor */
/***************************/
Ilog.RuleEditor = function(jsonInit){
    var isEmbeddedInDT = false;
    var isReadOnly = false;
    var isMultiline = true;
    
    var culture;
    
    var selfVariableId = null;
    var dtFieldId = null;
    var dtEditionContext = null;
    var frozenRegions = null;
    var dataCacheEnabled = null;
    var dtSessionId = null;
    var dtDescriptor = null;
    var additionalCssStyleSheetUrl = null;
    
    try{
        var jsonObj = jsonInit;
        if( typeof(jsonObj) == "string" ){
            jsonObj = Ext.util.JSON.decode(jsonObj.replace(/&splquot;/g, "'"));
        }
                
        if( jsonObj && jsonObj.EditorFieldId != null && jsonObj.EditorFieldId != undefined ){
            editorFieldId = jsonObj.EditorFieldId;
            
            pbListContainerId = null;
            if( jsonObj.PbListFieldId ){
                pbListContainerId = jsonObj.PbListFieldId;
                this.pbListContainerId = pbListContainerId;
            }
            initialText = jsonObj.InitialText;
            cssStyleSheetUrl = jsonObj.StyleSheetUrl;
            if (jsonObj.AdditionnalStyleSheetUrl) {
                additionalCssStyleSheetUrl = jsonObj.AdditionnalStyleSheetUrl;
            }
            customFieldKey = jsonObj.CustomFieldKey;
            customFieldReference = jsonObj.CustomFieldReference;
            serviceUrl = jsonObj.ServiceUrl;
            isAspNetCallbackHandlerAvailable = jsonObj.IsAspNetCallbackHandlerAvailable;
            autoCompletionEnabledAtStartup = jsonObj.AutoCompletionEnabledAtStartup;
            useSentenceCompletion = jsonObj.UseSentenceCompletion;
            specificUrlForSaveRequests = jsonObj.SpecificUrlForSaveRequests;
            isEmbeddedInDT = jsonObj.IsEmbeddedInDT;
            if( jsonObj.IsReadOnly != undefined && jsonObj.IsReadOnly != null ){
                isReadOnly = jsonObj.IsReadOnly;
            }
            if( jsonObj.IsMultiline != undefined && jsonObj.IsMultiline != null ){
                isMultiline = jsonObj.IsMultiline;
            }
           
            if( jsonObj.SelfVariableId ){
                selfVariableId = jsonObj.SelfVariableId;
            }
            
            if( jsonObj.Culture ){
                culture = jsonObj.Culture;
            }
            
            if( jsonObj.DtFieldId ){
                dtFieldId = jsonObj.DtFieldId;
            }
            
            if( jsonObj.DtSessionId){
                dtSessionId = jsonObj.DtSessionId;
            }
            
            if( jsonObj.DtEditionContext != null && jsonObj.DtEditionContext != undefined ){
                dtEditionContext = jsonObj.DtEditionContext;
            }
            
            if( jsonObj.FrozenRegions ){
                frozenRegions = jsonObj.FrozenRegions;
            }
            
            if( jsonObj.DtDescriptor ){
                dtDescriptor = jsonObj.DtDescriptor;
            }
            
            if( jsonObj.DataCacheEnabled != null && jsonObj.DataCacheEnabled != undefined ){
                dataCacheEnabled = jsonObj.DataCacheEnabled;
            }

            if (jsonObj.SeparatorWhitespaces) {
                this.SeparatorWhitespaces = jsonObj.SeparatorWhitespaces;
                var keycodes = new Array();
                for (var k = 0; k < this.SeparatorWhitespaces.length; k++) {
                    keycodes[k] = this.SeparatorWhitespaces.charCodeAt(k);
                }
                this.SeparatorWhitespacesKeycodes = keycodes;
            }

            if (jsonObj.SeparatorListSingleIdeographs) {
                this.SeparatorListSingleIdeographs = jsonObj.SeparatorListSingleIdeographs;
                this.SeparatorListSingleIdeographsString = "";

                var keycodes = new Array();
                for (var k = 0; k < this.SeparatorListSingleIdeographs.length; k++) {
                    keycodes[k] = this.SeparatorListSingleIdeographs[k].toString().charCodeAt(0);
                    this.SeparatorListSingleIdeographsString += this.SeparatorListSingleIdeographs[k].toString().charAt(0) + "";
                }
                this.SeparatorListSingleIdeographsKeycodes = keycodes;
            }

            if (jsonObj.SeparatorListIdeographsRanges) {
                this.SeparatorListIdeographsRanges = jsonObj.SeparatorListIdeographsRanges;

                var keycodeRanges = new Array();
                for (var k = 0; k < this.SeparatorListIdeographsRanges.length; k++) {
                    var range = this.SeparatorListIdeographsRanges[k];

                    if (range.length >= 2) {
                        var lowerBound = range[0];
                        var higherBound = range[1];
                        keycodeRanges[k] = new Array();
                        keycodeRanges[k][0] = lowerBound.charCodeAt(0);
                        keycodeRanges[k][1] = higherBound.charCodeAt(0);
                    }
                }
                this.SeparatorListIdeographsRangesKeycodes = keycodeRanges;
            }
        }
        else{
            Ilog.Debug.RuleEditorDebugger.PrintDebug('[Ilog.RuleEditor] cannot init');
            this.canPerformInit = false;
            return;
        }
    }
    catch(ex){
        Ilog.Debug.RuleEditorDebugger.PrintDebug('[Ilog.RuleEditor] invalid json object received: ' + ex.message);
        this.canPerformInit = false;
        return;
    }

    if( jsonObj.InitialTooltips ){
        this.Tooltips = jsonObj.InitialTooltips;
    }
    
    this.IsReadOnly = isReadOnly;
    this.IsMultiline = isMultiline;
    
    if( culture ){
        this.Culture = culture;
    }
    
    if( selfVariableId ){
        this.SelfVariableId = selfVariableId;
    }
    
    if( dtFieldId ){
        this.DtFieldId = dtFieldId;
    }
    if( dtSessionId ){
        this.DtSessionId = dtSessionId;
    }
    if( dtEditionContext ){
        this.DtEditionContext = dtEditionContext;
    }
    if( frozenRegions ){
        this.FrozenRegions = frozenRegions;
    }
    if( dtDescriptor ){
        this.DtDescriptor = dtDescriptor;
    }
    if( dataCacheEnabled ){
        this.DataCacheEnabled = dataCacheEnabled;
    }
    
    this.canPerformInit = true;
    this.isEmbeddedInDT = isEmbeddedInDT;
    this.editorFieldId = editorFieldId;
    
    this.initialText = initialText;
    this.cssStylesheet = cssStyleSheetUrl;
    this.additionalCssStyleSheetUrl = additionalCssStyleSheetUrl;

    // Completion modes :
        // Sentence completion
        this.SentenceCompletion = useSentenceCompletion;
        // Auto-completion
        this.AutoCompletion = autoCompletionEnabledAtStartup;

    if( customFieldKey && customFieldReference ) {
        this.customFieldKey = customFieldKey;
        this.customFieldReference = customFieldReference;
    }

    this.ServiceUrl = serviceUrl;
    
     // object that will help to manage the editor status
    this.OngoingRefreshRequestObject = null; // XMLHttpRequest object in case of direct ajax, always null in case of AspNetCallbackHandler
    this.OngoingRefreshRequestTimeStamp = null;
    this.OngoingRefreshRequestContext = null;
    this.OngoingRefreshRequestDelayedActionIdentifier = null;
    this.OngoingPredictionsRequestObject = null; // XMLHttpRequest object in case of direct ajax, always null in case of AspNetCallbackHandler
    this.OngoingPredictionsRequestTimeStamp = null;
    this.OngoingPredictionsRequestDelayedActionIdentifier = null;
    this.HtmlDocumentToDisplayOnRefresh = null;

    this.CurrentTextRange = {start:0, end:0};
    this.IsCurrentTextRangeUpToDate = true;
    
    // potentially using another URL for save requests (Java)
        if( specificUrlForSaveRequests != null && specificUrlForSaveRequests != undefined ){
            this.SpecificSaveRequestsServiceUrl = specificUrlForSaveRequests;
        }
        this.IsAspNetCallbackHandlerAvailable = isAspNetCallbackHandlerAvailable;
};
/***************************/
/***************************/

Ext.extend(Ilog.RuleEditor, Ext.util.Observable, {
    InitFields: function() {
        var editorFieldId = this.editorFieldId;
        var pbListContainerId = this.pbListContainerId;

        if (!document.getElementById(editorFieldId)) {
            return false;
        }

        if (pbListContainerId != null && pbListContainerId != undefined) {
            try {
                this.pbListContainerField = document.getElementById(pbListContainerId);
            } catch (ex) {
                Ilog.Debug.RuleEditorDebugger.PrintDebug('[InitFields] unable to get pb list container field');
            }
        }

        this.editorField = document.getElementById(editorFieldId);

        if (this.editorField.contentWindow) {
            this.editorDocument = this.editorField.contentWindow.document;
            this.contentWindow = this.editorField.contentWindow;
            this.contentDocument = this.editorField.contentDocument;
        }
        else if (this.editorField.document) {
            this.editorDocument = this.editorField.document;
        }
        else {
            return false;
        }
    },

    SetPbListHtmlContent: function(pbListHtmlContentToSet) {
        if (this.pbListContainerField == null) {
            return false;
        }
        if (pbListHtmlContentToSet) {
            this.pbListContainerField.innerHTML = pbListHtmlContentToSet;
        }
    },

    GetPlainText: function() {
        if (!this.editorDocument) {
            return null;
        }

        var ruleText = "";
        try {
            // FireFox
            if (this.contentDocument && !Ext.isIE) {
                // Replacing special ascii chars            
                var tempRuleText = Ilog.Utilities.ExtractTextFromNode(this.contentDocument.body);
                for (var p = 0; p < tempRuleText.length; p++) {
                    if (tempRuleText.charCodeAt(p) == 160) {
                        ruleText = ruleText + ' ';
                    }
                    else {
                        ruleText = ruleText + tempRuleText.charAt(p);
                    }
                }

                return ruleText;
            }
            else {
                ruleText = this.editorDocument.body.innerText;

                // Fix for IE
                if (Ext.isIE) {
                    ruleText = ruleText.replace(/\r\n/g, "\n");

                    var htmlText = this.GetHtmlText();
                    var matches = htmlText.split('<span type="lineBreak"></span>').length - 1;
                    var matchesUp = htmlText.split('<SPAN type="lineBreak"></SPAN>').length - 1;

                    var count = matches + matchesUp;
                    if (count > 0) {
                        for (var k = 0; k < count; k++) {
                            ruleText += "\n";
                        }
                    }

                    var regExp = "</SPAN>((<BR>)+)(<SPAN>)*(</SPAN>)*";
                    var rgx = new RegExp(regExp, "gi");
                    var res = rgx.exec(htmlText);
                    var lineBreaksString = RegExp.$1;
                    var nb = Ilog.Utilities.CountOccurences(lineBreaksString, "<BR>");
                    if (nb) {
                        for (var k = 0; k < nb; k++) {
                            ruleText += "\n";
                        }
                    }
                }
                return ruleText;
            }
        }
        catch (ex) { }

        return ruleText;
    },

    GetHtmlText: function() {
        if (!this.editorDocument) {
            return null;
        }
        if (this.contentDocument && !Ext.isIE) {
            return this.contentDocument.body.innerHTML;
        }
        else {
            return this.editorDocument.body.innerHTML;
        }
    },

    SelectPlaceholderNode: function(node) {
        // closing pop-ups (completion menu, date picker) of potential other editors on page  (DT mode)
        Ilog.Utilities.CleanEditorResources();

        this.SelectedPlaceholderIndex = node.placeholderIndex;
        this.SelectedPlaceholder = node;

        var placeholderConcept = null;
        // placeholder concept
        if (node.getAttribute('placeHolderConcept')) {
            placeholderConcept = node.getAttribute('placeHolderConcept');
        }

        var documentToCheck = this.editorDocument;
        var selection, range, doc, win;

        var textRangeToKeep = null;
        var bodyNode = this.GetDocumentBodyNode();
        var textrangeWholeValue = Ilog.Utilities.FindAdjacentNodes(bodyNode, node, Ilog.Constants.RuleEditorConstants.PatternPlaceHolderGroupRecognition);

        if (Ext.isIE) {
            var scrollLeftToKeep = this.GetScrollXPosition();
            var scrollTopToKeep = this.GetScrollYPosition();

            try {
                if (node) {
                    node.focus();
                    var textRange = this.editorDocument.selection.createRange();
                    textRange.moveToElementText(node);
                    textRange.select();
                }
            }
            catch (exc) {
                this.Focus();
                this.SelectTextRange(textrangeWholeValue);
            }
        }
        else {
            this.Focus();
            this.SelectTextRange(textrangeWholeValue);
        }

        textRangeToKeep = textrangeWholeValue; // getting back the text range this way, so as to get absolute start and end index of the range

        this.SelectedPlaceholder = node;

        // send request to get the predictions for this placeholder :
        this.CancelGetPredictionsRequest();
        this.CancelRefreshRequest();
        var ts = Ilog.Utilities.GetTimeStamp();
        this.LatestGetPredictionsTimeStamp = ts;
        this.SendGetPredictionsRequest(ts, textRangeToKeep, 0, true, placeholderConcept);
    },

    HandleTimeFieldSpecialKey: function(timefield, evt) {
        if (evt.getKey() == Ext.EventObject.ENTER) {
            if (this.DatePickerLayer) {
                this.DatePickerValidate(this.DatePickerLayer.Picker.getValue());
            }
        }
    },
    HandleDateTimePickerLayerKeyEvt: function(evt) {
        var keyCode = evt.getCharCode();
        if (keyCode == Ext.EventObject.ESC || keyCode == 27) {
            if (this.DatePickerLayer) {
                this.DatePickerLayer.parentNode.removeChild(this.DatePickerLayer);
                this.DatePickerLayer = null;
            }
        }
    },
    HandleDateTimePickerLayerMouseEvt: function(evt) {
    },

    SelectPlaceholder: function(evt) {
        Ilog.Utilities.StopEventPropagation(evt);

        if (evt != null) {
            var node = evt.target || evt.srcElement;

            if (node) {
                this.SelectPlaceholderNode(node);
            }
        }
    },

    HandleDateValueEditorValidate: function(extDatePicker, extDate) {
        this.DatePickerValidate(extDate);
    },

    DatePickerValidate: function(extDate) {
        if (this.DatePickerLayer) {
            var extFormat = this.DatePickerLayer.ExtFormat;
            if (this.DatePickerLayer.OutputDateFormat) {
                extFormat = this.DatePickerLayer.OutputDateFormat;
            }

            var dateStringLocaleFormatted = extDate.format(extFormat);

            if (this.DatePickerLayer.TimeField) {
                var timeValue = this.DatePickerLayer.TimeField.getValue();
                dateStringLocaleFormatted = dateStringLocaleFormatted.replace(/\{0\}/g, timeValue); //" " + timeValue;
            }

            var range = null;
            if (this.DatePickerLayer.AssociatedRange) {
                range = this.DatePickerLayer.AssociatedRange;
            }

            // delaying action so that the potential keyup of [ENTER] is not detected as a new line
            var self = this;
            window.setTimeout(function() {
                self.DatePickerLayer.parentNode.removeChild(self.DatePickerLayer);
                self.DatePickerLayer = null;
                self.Focus();
                self.SelectTextRange(range);
                self.InsertTextAtCaret(dateStringLocaleFormatted);
                self.CancelGetPredictionsRequest();
                self.SendRefreshRequest();
            }, Ilog.Constants.RuleEditorConstants.DelayCalendarValidation);
        }
    },

    SelectValueNode: function(node) {
        if (this.PredictionsView) {
            this.PredictionsView.Hide();
        }

        var documentToCheck = this.editorDocument;
        var selection, range, doc, win;

        //var textRangeToKeep = null;
        var valueText = null;
        var bodyNode = this.GetDocumentBodyNode();
        var textrangeWholeValue = Ilog.Utilities.FindAdjacentNodes(bodyNode, node, Ilog.Constants.RuleEditorConstants.PatternValueEditorRecognition);
        this.Focus();
        var valueEditorKey = node.getAttribute(Ilog.Constants.RuleEditorConstants.PatternValueEditorRecognition);
        this.ManageValueEditor(valueEditorKey, textrangeWholeValue, node);
    },

    ManageValueEditor: function(valueEditorKey, textrange, customValueEditorNode) {
        try {
            var valueText = this.SelectTextRange(textrange).text;
        }
        catch (ex) {
            Ilog.Debug.RuleEditorDebugger.PrintDebug('[ManageValueEditor] unable to get value text: ' + ex.message);
        }

        // built-in editors (date)
        if (valueEditorKey.toString().toUpperCase() == Ilog.Constants.RuleEditorConstants.FullDateValueEditor.toString().toUpperCase() || valueEditorKey.toString().toUpperCase() == Ilog.Constants.RuleEditorConstants.ShortDateValueEditor.toString().toUpperCase() || valueEditorKey.toString().toUpperCase() == Ilog.Constants.RuleEditorConstants.UniversalDateValueEditor.toString().toUpperCase()) {
            var pickerLayer = document.createElement('div');

            if (textrange) {
                this.Focus();
                this.SelectTextRange(textrange);
            }
            var xy = this.GetXYAbsolutePositionOfCaret();
            pickerLayer.style.background = '';
            pickerLayer.zindex = 99;
            pickerLayer.style.zindex = 99;
            pickerLayer.style.position = 'absolute';
            pickerLayer.style.top = xy.y + 'px';
            pickerLayer.style.left = xy.x + 'px';

            // FF Fix :
            // need to set a size, and it has to be smaller than the calendar size so that it is rendered correctly
            if (Ext.isGecko) {
                pickerLayer.style.width = '0px';
                pickerLayer.style.height = '0px';
            }

            document.body.appendChild(pickerLayer);
            this.DatePickerLayer = pickerLayer;
            this.DatePickerLayer.AssociatedRange = textrange;
            var timeLayer = null;
            var timeformat = null;
            var outputDateFormat = null;

            // datetime format
            var dateTimeFormat = "";
            if (valueEditorKey.toString().toUpperCase() == Ilog.Constants.RuleEditorConstants.FullDateValueEditor.toString().toUpperCase()) {
                dateTimeFormat = this.Culture.FullDatePattern;
                timeformat = this.Culture.FullTimePattern;
                outputDateFormat = this.Culture.FullDatePatternWithoutTime;
            }
            else if (valueEditorKey.toString().toUpperCase() == Ilog.Constants.RuleEditorConstants.ShortDateValueEditor.toString().toUpperCase()) {
                dateTimeFormat = this.Culture.DatePattern;
                outputDateFormat = dateTimeFormat;
            }
            else if (valueEditorKey.toString().toUpperCase() == Ilog.Constants.RuleEditorConstants.UniversalDateValueEditor.toString().toUpperCase()) {
                dateTimeFormat = this.Culture.UniversalDatePattern;
                timeformat = this.Culture.UniversalTimePattern;
                outputDateFormat = this.Culture.UniversalDatePatternWithoutTime;
            }
            var extFormat = Ilog.Utilities.ConvertDateStringFormatToExt(dateTimeFormat);
            this.DatePickerLayer.ExtFormat = extFormat;

            if (outputDateFormat) {
                this.DatePickerLayer.OutputDateFormat = Ilog.Utilities.ConvertDateStringFormatToExt(outputDateFormat);
            }

            // current value
            if (valueText && valueText.length > 0) {
                var valueToParse = valueText;
                if (valueToParse && valueToParse.length > 1 && valueToParse.substr(0, 1) == '"' && valueToParse.substr(valueToParse.length - 1, 1) == '"') {
                    valueToParse = valueToParse.substr(1, valueToParse.length - 2) + "";
                }
                try {
                    var currentDateTimeValue = Date.parseDate(valueToParse, extFormat);
                }
                catch (ex) {
                    Ilog.Debug.RuleEditorDebugger.PrintDebug('[ManageValueEditor] date parse error (' + valueToParse + ' ** ' + extFormat + ') : ' + ex.message);
                    currentDateTimeValue = null;
                }
            }

            var config = { format: extFormat };

            // L10N/I18N
            if (this.Culture.MonthNames && this.Culture.DayNames) {
                config.dayNames = this.Culture.DayNames;
                config.monthNames = this.Culture.MonthNames;
            }
            if (Ilog.Constants.RuleEditorConstants.CalendarConfirmLabel && Ilog.Constants.RuleEditorConstants.CalendarCancelLabel && Ilog.Constants.RuleEditorConstants.CalendarTodayLabel && Ilog.Constants.RuleEditorConstants.CalendarPreviousLabel && Ilog.Constants.RuleEditorConstants.CalendarNextLabel) {
                config.cancelText = Ilog.Constants.RuleEditorConstants.CalendarCancelLabel;
                config.okText = Ilog.Constants.RuleEditorConstants.CalendarConfirmLabel;
                config.todayText = Ilog.Constants.RuleEditorConstants.CalendarTodayLabel;
                config.prevText = Ilog.Constants.RuleEditorConstants.CalendarPreviousLabel;
                config.nextText = Ilog.Constants.RuleEditorConstants.CalendarNextLabel;
            }

            var datePicker = new Ext.DatePicker(config);
            if (currentDateTimeValue) {
                datePicker.setValue(currentDateTimeValue);
            }
            datePicker.on('select', this.HandleDateValueEditorValidate, this);

            pickerLayer.Picker = datePicker;
            datePicker.render(pickerLayer);
            datePicker.show();

            Ext.EventManager.addListener(pickerLayer, "mousedown", this.HandleDateTimePickerLayerMouseEvt, this);
            Ext.EventManager.addListener(pickerLayer, "mouseup", this.HandleDateTimePickerLayerMouseEvt, this);
            Ext.EventManager.addListener(pickerLayer, "click", this.HandleDateTimePickerLayerMouseEvt, this);
            Ext.EventManager.addListener(pickerLayer, "keydown", this.HandleDateTimePickerLayerKeyEvt, this);

            var calendarWidth = datePicker.getEl().getWidth();

            if (timeformat != null && timeformat != undefined) {
                try {
                    var extTimeFormat = Ilog.Utilities.ConvertDateStringFormatToExt(timeformat);
                    var config = { format: extTimeFormat, increment: 15, width: (calendarWidth - 16 - 3 - 3 - 5), listClass: 'ilog-timeDDL' };

                    if (currentDateTimeValue) {
                        var timeValue = currentDateTimeValue.format(extTimeFormat);

                        if (timeValue) {
                            config.value = timeValue;
                        }
                    }

                    timeLayer = new Ext.form.TimeField(config);

                    // managing "focus out" with mouse down event
                    // (here we are setting mousedown on time drop down list panel -> no focus out)(stopping event propagation, so that highler level handler does not catch and consider it as a focus out)
                    timeLayer.on("expand", function(e) {
                        window.setTimeout(function() {
                            var comboLists = Ext.DomQuery.select("div[class*=ilog-timeDDL]", document.body);
                            for (var p = 0; p < comboLists.length; p++) {
                                Ext.EventManager.addListener(comboLists[p], "mousedown", function(e) { Ilog.Utilities.StopEventPropagation(e); }, this);
                                Ext.EventManager.addListener(comboLists[p], "mouseup", function(e) { Ilog.Utilities.StopEventPropagation(e); }, this);
                            }
                        }, 10); // 10 ms to be sure that the drop down list is created and added to the document
                    });
                }
                catch (ex) {
                    Ilog.Debug.RuleEditorDebugger.PrintDebug('[ManageValueEditor] time field management error: ' + ex.message);
                }
            }

            if (timeLayer) {
                timeLayer.on("select", function(e) { Ilog.Utilities.StopEventPropagation(e); }, this);

                var timeContainer = document.createElement('div');
                timeContainer.style.background = '#FFFFFF';
                timeContainer.style.height = '25px';
                timeContainer.style.width = (calendarWidth - 5) + 'px';
                timeContainer.style.paddingTop = '3px';
                timeContainer.style.paddingLeft = '3px';
                timeContainer.style.marginTop = '0px';
                timeContainer.style.borderLeft = '1px black solid';
                timeContainer.style.borderRight = '1px black solid';
                timeContainer.style.borderBottom = '1px black solid';
                timeContainer.innerHTML = '<div style="float:left;padding-top:4px;"><img src="' + Ilog.Constants.RuleEditorConstants.UrlCalendarTimeIcon + '" /></div>';

                var container2 = document.createElement('div');
                container2.style.paddingLeft = '3px';
                container2.cssFloat = 'left';
                container2.styleFloat = 'left';
                container2.style.cssFloat = 'left';
                container2.style.styleFloat = 'left';
                timeContainer.appendChild(container2);

                pickerLayer.appendChild(timeContainer);
                timeLayer.render(container2);
                timeLayer.on("specialkey", this.HandleTimeFieldSpecialKey, this);
                this.DatePickerLayer.TimeField = timeLayer;
            }

            return;
        }

        // custom editors
        var timestamp = Ilog.Utilities.GetTimeStamp();
        var editorIdentifier = this.editorFieldId;
        var directAjaxCall = true;
        var xmlToPost = Ilog.RuleEditor.CallbackManagement.GetCallbackArgumentForValueEditorRequest(this, timestamp, valueEditorKey);
        var serviceUrl = this.ServiceUrl;

        if (directAjaxCall) {
            if (serviceUrl != null && serviceUrl != '') {
                var objForRequest = Ilog.RuleEditor.CallbackManagement.GetEditorRequestWithDirectAjaxCall(serviceUrl, xmlToPost, Ilog.RuleEditor.CallbackManagement.OnGetValueEditorSuccessfulResponseReceived, this, Ilog.RuleEditor.CallbackManagement.OnGetValueEditorDirectAjaxErrorResponseReceived, timestamp, valueText, customValueEditorNode, valueEditorKey, textrange);
                Ilog.RuleEditor.CallbackManagement.SendEditorRequestWithDirectAjaxCall(objForRequest.Request, objForRequest.Xml, objForRequest.ServiceUrl, Ilog.Constants.RuleEditorConstants.CallbackStringRequestTypeGetValueEditor);
            }
        }
    },

    SelectValue: function(evt) {
        Ilog.Utilities.StopEventPropagation(evt);

        if (evt != null) {
            var node = evt.target || evt.srcElement;

            if (node) {
                this.SelectValueNode(node);
            }
        }
    },

    HandlePlaceholderMouseOver: function(evt) {
        var placeholderNode = evt.getTarget();
        if (placeholderNode != null && placeholderNode != undefined) {
            // remark: IE seems to refuse to apply this cursor style inside the iframe
            placeholderNode.style.cursor = 'pointer';

            if (this.contentDocument && !Ext.isIE) {
                this.contentDocument.getElementsByTagName('body')[0].style.cursor = 'pointer';
            }
            else if (this.editorDocument) {
                this.editorDocument.getElementsByTagName('body')[0].style.cursor = 'pointer';
            }

        }
    },

    HandlePlaceholderMouseOut: function(evt) {
        var placeholderNode = evt.getTarget();
        if (placeholderNode != null && placeholderNode != undefined) {
            placeholderNode.style.cursor = 'default';

            if (this.contentDocument && !Ext.isIE) {
                this.contentDocument.getElementsByTagName('body')[0].style.cursor = 'default';
            }
            else if (this.editorDocument) {
                this.editorDocument.getElementsByTagName('body')[0].style.cursor = 'default';
            }
        }
    },

    EnterKeyOnPlaceHolder: function(evt) {
        if (!evt) {
            return;
        }

        var keyCode = evt.getCharCode();
        var altPressed = evt.altKey;

        // ALT + arrow left/right : should switch placeholder
        if (altPressed === true && (keyCode == Ext.EventObject.LEFT || keyCode == Ext.EventObject.RIGHT)) {
            // Cancelling event
            Ilog.Utilities.CancelEventBehaviour(evt);
        }

        return false;
    },

    UpdatePlaceHolders: function() {
        var bodyNode = this.GetDocumentBodyNode();
        var placeHolders = Ext.DomQuery.select("span[class*=" + Ilog.Constants.RuleEditorConstants.PatternPlaceHolderRecognition + "]", bodyNode);
        var valuesWithEditors = Ext.DomQuery.select("span[" + Ilog.Constants.RuleEditorConstants.PatternValueEditorRecognition + "]", bodyNode);

        if (valuesWithEditors && valuesWithEditors.length > 0) {
            for (var k = 0; k < valuesWithEditors.length; k++) {
                Ext.EventManager.addListener(valuesWithEditors[k], "dblclick", this.SelectValue, this);
            }
        }

        if (placeHolders && placeHolders.length > 0) {
            this.PlaceHolders = placeHolders;
            this.SelectedPlaceHolderIndex = -1;

            for (var k = 0; k < placeHolders.length; k++) {
                var currentPlaceholder = placeHolders[k];
                currentPlaceholder.placeholderIndex = k;
                Ext.EventManager.addListener(currentPlaceholder, "click", this.SelectPlaceholder, this);
                Ext.EventManager.addListener(currentPlaceholder, "mouseover", this.HandlePlaceholderMouseOver, this);
                Ext.EventManager.addListener(currentPlaceholder, "mouseout", this.HandlePlaceholderMouseOut, this);
            }
        }
        else {
            this.PlaceHolders = null;
            this.SelectedPlaceHolderIndex = -1;
        }
    },

    UpdateDisplay: function() {
        if (this.HtmlDocumentToDisplayOnRefresh == null || this.HtmlDocumentToDisplayOnRefresh == undefined) {
            return;
        }

        try {
            this.InputDisabled = true;
            var shouldRebindHandlers = false;
            var scrollLeftToKeep = this.GetScrollXPosition();
            var scrollTopToKeep = this.GetScrollYPosition();
            var textRangeToKeep = this.GetTextRange();

            if (this.contentDocument && !Ext.isIE) {
                this.Tooltips = this.TooltipsToSetOnRefresh;
                this.SetHtmlText(this.HtmlDocumentToDisplayOnRefresh, false);
                this.HtmlDocumentToDisplayOnRefresh = null;
                this.TooltipsToSetOnRefresh = null;
            }
            else {
                var temp = Ext.util.Format.htmlEncode(this.HtmlDocumentToDisplayOnRefresh.replace(/\n/, "CR"));
                this.Tooltips = this.TooltipsToSetOnRefresh;
                var docSet = this.HtmlDocumentToDisplayOnRefresh + '';

                if (scrollLeftToKeep > 0 || scrollTopToKeep > 0) {
                    if (Ext.isIE) {
                        // Getting iframe body background color
                        var iframeBg = Ilog.Utilities.GetElementActualStyle(this.editorField, 'background', 'background');
                        var iframeBgColor = Ilog.Utilities.GetElementActualStyle(this.editorField, 'backgroundColor', 'backgroundColor');
                        var docBodyBg = Ilog.Utilities.GetElementActualStyle(this.editorDocument.body, 'background', 'background');
                        var docBodyBgColor = Ilog.Utilities.GetElementActualStyle(this.editorDocument.body, 'backgroundColor', 'backgroundColor');

                        var hideLayerColor = '#FFFFFF';
                        if (docBodyBgColor) {
                            hideLayerColor = docBodyBgColor;
                        }
                        else if (docBodyBg) {
                            hideLayerColor = docBodyBg;
                        }
                        else if (iframeBgColor) {
                            hideLayerColor = iframeBgColor;
                        }
                        else if (iframeBg) {
                            hideLayerColor = iframeBg;
                        }

                        var hideLayer = document.createElement('div');
                        hideLayer.style.background = hideLayerColor;
                        var iframeExtElt = new Ext.Element(this.editorField);
                        var scrollFixHideLayerMargin = Ilog.Constants.RuleEditorConstants.IEScrollFixHideLayerMargin;
                        var iframeX = iframeExtElt.getBox().x + scrollFixHideLayerMargin;
                        var iframeY = iframeExtElt.getBox().y + scrollFixHideLayerMargin;
                        var iframeWidth = iframeExtElt.getBox().width;
                        var iframeHeight = iframeExtElt.getBox().height;
                        hideLayer.style.position = 'absolute';
                        hideLayer.style.top = iframeY + 'px';
                        hideLayer.style.left = iframeX + 'px';
                        hideLayer.style.width = (iframeWidth - scrollFixHideLayerMargin) + 'px';
                        hideLayer.style.height = (iframeHeight - scrollFixHideLayerMargin) + 'px';
                        hideLayer.zindex = 200;
                        hideLayer.style.zindex = 200;
                        document.body.appendChild(hideLayer);

                        var restorePredictions = false;
                        if (this.PredictionsView && this.PredictionsView.Visible) {
                            restorePredictions = true;
                            this.PredictionsView.Hide();
                        }
                    }
                }

                var documentSet = false;
                if (Ext.isIE) {
                    // fixing position for empty doc
                    if (docSet.toUpperCase() == "<SPAN ID='EMPTY'>&NBSP;</SPAN>") {
                        textRangeToKeep.start = 0;
                        textRangeToKeep.end = 0;
                    }
                    else if (textRangeToKeep.start == textRangeToKeep.end) {
                        var length = this.GetPlainText().length;
                        if (textRangeToKeep.start > length) {
                            var diff = textRangeToKeep.start - length;
                            for (var p = 0; p < diff; p++) {
                                docSet += '<br/><span></span>';
                            }
                            this.SetHtmlText(docSet, false);
                            docSet = null;
                            documentSet = true;
                        }
                    }
                }

                if (!documentSet) {
                    this.SetHtmlText(docSet, false);
                }
                this.HtmlDocumentToDisplayOnRefresh = null;
                this.TooltipsToSetOnRefresh = null;
            }

            if (scrollLeftToKeep > 0 || scrollTopToKeep > 0) {
                if (Ext.isIE) {
                    this.SelectTextRange(textRangeToKeep);
                    this.SetScrollXYPosition(scrollLeftToKeep, scrollTopToKeep);
                    var self = this;
                    window.setTimeout(function() {
                        document.body.removeChild(hideLayer);
                        if (self.PredictionsView && restorePredictions === true) {
                            self.PredictionsView.Show();
                        }
                    }, Ilog.Constants.RuleEditorConstants.IEScrollFixHideLayerDelay);
                }
                else {
                    this.SetScrollXYPosition(scrollLeftToKeep, scrollTopToKeep);
                    this.SelectTextRange(textRangeToKeep);
                }
            }
            else {
                this.SelectTextRange(textRangeToKeep);
            }

            if (shouldRebindHandlers) {
                this.UnregisterEventHandlers();
                this.RegisterEventHandlers(); // need to rebind the event handlers, otherwise no events are catched anymore because the whole document (which had the handlers bound to) has been changed
            }
        }
        catch (ex) {
            Ilog.Debug.RuleEditorDebugger.PrintDebug('[UpdateDisplay] error: ' + ex.message);
        }

        this.InputDisabled = false;
    },


    SetHtmlText: function(textToSet, addHtmlStructure, delayForPlaceholderComputation) {
        if (!textToSet || !textToSet.toUpperCase().startsWith('<SPAN>')) {
            textToSet = '<span></span>' + textToSet;
        }
        textToSet = textToSet.replace(/&#13;/g, '&nbsp;');

        if (!this.editorDocument) {
            return false;
        }

        this.CleanTooltips(); // so that tooltips are reset, to prevent a tooltip to keep on being displayed

        if (addHtmlStructure == undefined || addHtmlStructure == null || addHtmlStructure == true) {
            var documentBodyClass = "ilog-ruleEditor-editionArea";

            // defining document style
            var padd = "";
            if (!(this.isEmbeddedInDT === true)) {
                if (window.getSelection) {
                    padd = "padding:10px";
                }
                else {
                    padd = "padding-top:5px;padding-left:5px";
                }
            }
            else {
                documentBodyClass = "ilog-ruleEditor-editionArea-dtEmbedded";
            }
            var additionalCss = "";
            if (this.additionalCssStyleSheetUrl) {
                additionalCss = '<link type = "text/css" href="' + this.additionalCssStyleSheetUrl + '" rel="stylesheet" />';
            }
            var docContent = textToSet;
            this.editorDocument.writeln('<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"><html><head><link type = "text/css" href="' + this.cssStylesheet + '" rel="stylesheet" />' + additionalCss + '<style type="text/css"> p { margin: 0; }</style></head><body class="' + documentBodyClass + '" style="' + padd + '" alt="' + this.editorFieldId + '" intelliruleeditorreference="' + this.editorFieldId + '">' + docContent + '</body></html>');
            this.editorDocument.close();
        }
        else if (addHtmlStructure == false) {
            this.editorDocument.body.innerHTML = textToSet;
        }

        // Setting up tooltips:
        var bodyNode = this.GetDocumentBodyNode();

        var self = this;
        window.setTimeout(function() {
            var spanNodes = new Array();
            if (self.contentDocument && !Ext.isIE) {
                spanNodes = self.contentDocument.getElementsByTagName('span');
            }
            else {
                spanNodes = self.editorDocument.getElementsByTagName('span');
            }

            for (var p = 0; p < spanNodes.length; p++) {
                var node = spanNodes[p];
                var tooltipData = null;

                // looking for "title" attribute on the node,
                // on his parent or 1st child node
                var hasTitle = false;
                if (node.getAttribute('title') != null && node.getAttribute('title') != undefined && node.getAttribute('title') != '') {
                    hasTitle = true;
                }
                else {
                    if (node.childNodes && node.childNodes.length > 0) {
                        var firstChild = node.childNodes[0];
                        if (firstChild.getAttribute && firstChild.getAttribute('title') != null && firstChild.getAttribute('title') != undefined && firstChild.getAttribute('title') != '') {
                            hasTitle = true;
                        }
                    }
                    if (node.parentNode && node.parentNode.getAttribute && node.parentNode.getAttribute('title') != null && node.parentNode.getAttribute('title') != undefined && node.parentNode.getAttribute('title') != '') {
                        hasTitle = true;
                    }
                }

                // Priority on titles :
                // if node or parent or 1st child has a title, it indicates an error
                // and so 
                if (hasTitle) {
                    // special case for IE, because elements with a "title" attribute inside an iframe will not show tooltip (seems to be a bug in IE - works perfectly in FF)
                    // so we display custom tooltips for 'title' as well
                    var parentHasTitle = false;
                    if (node.parentNode && node.parentNode.getAttribute('title') != null && node.parentNode.getAttribute('title') != undefined && node.parentNode.getAttribute('title') != '') {
                        parentHasTitle = true;
                    }

                    var val = node.getAttribute('title');
                    node.removeAttribute('title');
                    node.setAttribute('tt', val);
                    tooltipData = Ext.util.Format.htmlEncode(val);
                }
                else if (node.getAttribute(Ilog.Constants.RuleEditorConstants.TooltipAttributeName) != null && node.getAttribute(Ilog.Constants.RuleEditorConstants.TooltipAttributeName) != undefined) {
                    tooltipData = node.getAttribute(Ilog.Constants.RuleEditorConstants.TooltipAttributeName);
                }

                if (tooltipData != null) {
                    // Getting real tooltip value
                    if (tooltipData.toString().startsWith('tt') && tooltipData.toString().length > 2) {
                        tooltipData = tooltipData.toString().substring(2);
                        if (self.Tooltips) {
                            try {
                                tooltipData = self.Tooltips[parseInt(tooltipData)];
                            }
                            catch (ex) {
                                Ilog.Debug.RuleEditorDebugger.PrintDebug('[SetHtmlText] tooltip data error: ' + ex.message);
                            }
                        }
                    }

                    var scope = { editor: self, node: node, data: tooltipData };
                    Ext.EventManager.addListener(node, "mouseover", self.HandleNodeMouseOver, scope);
                    Ext.EventManager.addListener(node, "mouseout", self.HandleNodeMouseOut, scope);
                }
            }
        }, 50);

        if (this.IsReadOnly === true) {
            return;
        }

        if (delayForPlaceholderComputation === true) {
            var self = this;
            window.setTimeout(function() { self.UpdatePlaceHolders(); }, 15);
        }
        else {
            this.UpdatePlaceHolders();
        }
    },

    HandleNodeMouseOver: function(evt) {
        try {
            if (Ilog.Constants.RuleEditorConstants.TooltipsActivated === false) {
                return;
            }

            var scope = this;
            var editor = scope.editor;
            var data = scope.data;
            var node = scope.node;

            if (!data || !node) {
                return;
            }

            var edArea = editor.GetEditionAreaAbsolutePosition();
            var extNode = new Ext.Element(node);
            var xy = extNode.getBox();

            var parent = node.parentNode;
            while (parent) {
                if (parent.scrollLeft) {
                    xy.x -= parent.scrollLeft;
                }
                if (parent.scrollTop) {
                    xy.y -= parent.scrollTop;
                }
                parent = parent.parentNode;
            }

            parent = editor.editorField;
            while (parent) {
                if (parent.scrollLeft) {
                    xy.x -= parent.scrollLeft;
                }
                if (parent.scrollTop) {
                    xy.y -= parent.scrollTop;
                }
                parent = parent.parentNode;
            }

            var x = edArea.x + xy.x + editor.GetScrollXPosition();
            var xMin = edArea.x;
            if (x < xMin) { x = xMin; }
            var y = edArea.y + xy.y + editor.GetScrollYPosition() + Ilog.Constants.RuleEditorConstants.TooltipYmargin;

            data = data.replace(/<br><\/br>/g, '<br/>');
            editor.DisplayTooltipForNode(x, y, node, data);
        }
        catch (ex) { }
    },

    HandleNodeMouseOut: function(evt) {
        try {
            var scope = this;
            var editor = scope.editor;
            var node = scope.node;
            editor.HideTooltipForNode(node);
        }
        catch (ex) { }
    },

    CleanTooltips: function() {
        var tooltips = Ext.DomQuery.select("div[class=" + Ilog.Constants.RuleEditorConstants.TooltipCssClass + "]");
        for (var k = 0; k < tooltips.length; k++) {
            try {
                tooltips[k].parentNode.removeChild(tooltips[k]);
            }
            catch (ex) { }
        }
    },

    DisplayTooltipForNode: function(x, y, node, data) {
        if (data == null || data == undefined || data == '') {
            return;
        }

        data = data.replace(/\\"/g, '"');

        var tooltip = document.createElement('div');
        tooltip.style.visibility = 'hidden';
        tooltip.id = node.id + '_tooltip';
        tooltip.className = Ilog.Constants.RuleEditorConstants.TooltipCssClass;
        tooltip.innerHTML = data;
        tooltip.style.position = 'absolute';
        tooltip.style.left = x + 'px';
        tooltip.style.top = y + 'px';
        document.body.appendChild(tooltip);

        window.setTimeout(function() {
            if (tooltip != null && tooltip != undefined) {
                tooltip.style.visibility = 'visible';
            }
        }, Ilog.Constants.RuleEditorConstants.TooltipDelayBeforeDisplay);
    },

    HideTooltipForNode: function(node) {
        var tooltipId = node.id + '_tooltip';
        var tooltip = document.getElementById(tooltipId);

        if (tooltip) {
            tooltip.parentNode.removeChild(tooltip);
        }
    },

    EnsureCaretVisible: function(xposToSet, yPosToSet, prevScrollX, prevScrollY, pastedText) {
        var scrollX, scrollY;
        try {
            if (prevScrollX && pastedText) { scrollX = prevScrollX; }
            if (prevScrollY && pastedText) { scrollY = prevScrollY; }
        }
        catch (ex) { }

        if (!scrollX) { scrollX = this.GetScrollXPosition(); }
        if (!scrollY) { scrollY = this.GetScrollYPosition(); }
        
        var car = this.GetCaretRelativePositionInsideEditionArea();
        var caretRelativeXPosition = car.x;
        var caretRelativeYPosition = car.y + Ilog.Constants.RuleEditorConstants.CharacterSquareSize; // bottom of the caret should be visible


        var iframeExtElt = new Ext.Element(this.editorField);
        var iframeBox = iframeExtElt.getBox();
        var iframeWidth = iframeBox.width;
        var iframeHeight = iframeBox.height;

        var margin = Ilog.Constants.RuleEditorConstants.IECaretVisibleMargin;
        if (caretRelativeXPosition > iframeWidth + margin || caretRelativeYPosition > iframeHeight + margin) {
            var addW = 0;
            var addH = 0;
            if (pastedText) { var dims = Ilog.Utilities.GetTextDimensions(pastedText); addW = dims.width; addH = dims.height; }
            var xpos = (caretRelativeXPosition > iframeWidth + margin) ? (scrollX + addW + Ilog.Constants.RuleEditorConstants.ScrollMargin) : scrollX;
            var ypos = (caretRelativeYPosition > iframeHeight + margin) ? (scrollY + addH + Ilog.Constants.RuleEditorConstants.ScrollMargin) : scrollY;
            if (xposToSet != null && xposToSet != undefined) { if (xposToSet == -1) { xpos = scrollX; } else { xpos = xposToSet; } }
            if (yPosToSet != null && yPosToSet != undefined) { if (yPosToSet == -1) { ypos = scrollY; } else { ypos = yPosToSet; } }
            var self = this;
            window.setTimeout(function() {
                self.SetScrollXYPosition(xpos, ypos);
            }, 0);
            return;
        }
        else if (caretRelativeXPosition < 0 || caretRelativeYPosition < 0) {
            if (caretRelativeXPosition < 0) {
                var caretPos = scrollX + caretRelativeXPosition - Ilog.Constants.RuleEditorConstants.ScrollMargin;
                var xpos = (caretPos >= 0) ? caretPos : 0;
            }
            else {
                var xpos = scrollX;
            }

            if (caretRelativeYPosition < 0) {
                var caretPos = scrollY + caretRelativeYPosition - Ilog.Constants.RuleEditorConstants.CharacterSquareSize - Ilog.Constants.RuleEditorConstants.ScrollMargin;
                var ypos = (caretPos >= 0) ? caretPos : 0;
            }
            else {
                var ypos = scrollY;
            }

            if (xposToSet != null && xposToSet != undefined) { if (xposToSet == -1) { xpos = scrollX; } else { xpos = xposToSet; } }
            if (yPosToSet != null && yPosToSet != undefined) { if (yPosToSet == -1) { ypos = scrollY; } else { ypos = yPosToSet; } }
            this.SetScrollXYPosition(xpos, ypos);
            return;
        }
    },

    InsertLineBreakAtCaret: function(endOfDoc) {
        if (document.selection) {
            this.InsertTextAtCaret('</span><span><br/>', true);
            var self = this;
            window.setTimeout(function() {
                self.EnsureCaretVisible(0); // manually scrolling iframe if caret is not visible anymore
            }, 50);
            return true;
        }
        else if ((document.getSelection || window.getSelection) && this.contentWindow) {
            return this.InsertTextAtCaret('</span><br/><span>');
        }

        return false;
    },

    ClearAllText: function() {
        if (this.editorDocument && this.editorDocument.body) {
            this.editorDocument.body.innerHTML = '';
            this.Focus();
        }

        this.CancelRefreshRequest();
        this.SendRefreshRequest();
    },

    InsertTextAtCaret: function(htmlTextToInsert, directPaste) {
        // IE
        if (document.selection && this.editorDocument) {
            var rangeCreated = this.editorDocument.selection.createRange();
            // need to keep scroll position in IE, otherwise it is reset
            // getting scroll position:
            var scrollPos = { x: this.GetScrollXPosition(), y: this.GetScrollYPosition() };

            // pasting text
            if (directPaste === true) {
                rangeCreated.pasteHTML(htmlTextToInsert);
            }
            else {
                rangeCreated.pasteHTML("</span>" + htmlTextToInsert + "<span>");
            }

            // setting back scroll position
            this.SetScrollXYPosition(scrollPos.x, scrollPos.y);

            return true;
        }
        // FF
        else if ((document.getSelection || window.getSelection) && this.contentWindow) {
            this.contentWindow.focus();

            try {
                this.contentWindow.document.execCommand("inserthtml", false, htmlTextToInsert);
            }
            catch (ex) {
                Ilog.Debug.RuleEditorDebugger.PrintDebug("[InsertTextAtCaret] inserthtml error: " + ex.message);
            }

            return true;
        }
    },

    MoveCaretToEndOfNode: function(nodeToMoveTo) {
        // FF
        if (window.getSelection) {
            if (!nodeToMoveTo) {
                return false;
            }

            if (!this.contentDocument) {
                return false;
            }

            var range = this.contentDocument.createRange();
            var lastOffset = nodeToMoveTo.textContent.length;
            range.setStart(nodeToMoveTo, lastOffset);
            range.setEnd(nodeToMoveTo, lastOffset);

            if (range != null) {
                this.contentWindow.getSelection().removeAllRanges();
                this.contentWindow.getSelection().addRange(range);
                return true;
            }
        }
        // not implemented for other browsers than FF
        else {
            return false;
        }

    },

    MoveCaretToBeginningOfNode: function(nodeToMoveTo) {
        // FF
        if (window.getSelection) {
            if (!nodeToMoveTo) {
                return false;
            }

            if (!this.contentDocument) {
                return false;
            }

            var range = this.contentDocument.createRange();
            range.setStart(nodeToMoveTo, 0);
            range.setEnd(nodeToMoveTo, 0);

            if (range != null) {
                this.contentWindow.getSelection().removeAllRanges();
                this.contentWindow.getSelection().addRange(range);
                return true;
            }
        }
        // not implemented for other browsers than FF
        else {
            return false;
        }
    },

    Focus: function() {
        if (this.contentDocument && !Ext.isIE) {
            try {
                this.contentWindow.focus();
            }
            catch (ex) {
                Ilog.Debug.RuleEditorDebugger.PrintDebug("[Focus] error on set focus to contentWindow: " + ex.message);
            }
            return true;
        }
        else if (this.editorDocument && this.editorField) {
            try {
                this.editorField.focus();
            }
            catch (ex) {
                Ilog.Debug.RuleEditorDebugger.PrintDebug("[Focus] error on set focus to editorField: " + ex.message);
            }

            try {
                this.editorDocument.body.focus();
            }
            catch (ex) {
                Ilog.Debug.RuleEditorDebugger.PrintDebug("[Focus] error on set focus to editorDocument.body: " + ex.message);
            }

            var range = this.editorDocument.selection.createRange();
            range.select();
            return true;
        }

        return false;
    },

    SelectTextRange: function(textRangeToSelect) {
        if (!textRangeToSelect) {
            return;
        }

        // IE
        if (this.editorDocument && this.editorDocument.selection) {
            // important. without focus the caret cannot be placed.
            try {
                this.editorField.focus();
            }
            catch (ex) {
                Ilog.Debug.RuleEditorDebugger.PrintDebug('[SelectTextRange] error on set focus to editorField: ' + ex.message);
            }


            try {
                this.editorField.document.focus();
            }
            catch (ex) {
                Ilog.Debug.RuleEditorDebugger.PrintDebug('[SelectTextRange] error on set focus to editorField.document: ' + ex.message);
            }

            // text range start should be lower than its end
            if (textRangeToSelect.start > textRangeToSelect.end) {
                var start = textRangeToSelect.start;
                textRangeToSelect.start = textRangeToSelect.end;
                textRangeToSelect.end = start;
            }

            // Step 1 : resetting caret position (setting to 0)
            // because IE only works with relative positionning
            var resetRange = this.editorDocument.selection.createRange();
            resetRange.moveStart("character", -this.GetCurrentTextLength()); // not elegant solution but nothing equivalent found
            resetRange.collapse(true);
            resetRange.select();

            // Step 2 : now that caret is at first position in the text buffer,
            // setting the text range using the absolute start offset and range length                        
            var textRange = this.editorDocument.selection.createRange();
            textRange.moveStart("character", textRangeToSelect.start);
            textRange.moveEnd("character", textRangeToSelect.end - textRangeToSelect.start);
            textRange.select();

            return { text: textRange.text };
        }
        // FF
        else {
            if (this.contentDocument && !Ext.isIE) {
                var bodyNode = this.contentDocument.body;
                var currentLength = 0;

                var range = null;
                var bOk = false;
                var positionNodeFound = false;
                var positionStart = textRangeToSelect.start;
                var positionEnd = textRangeToSelect.end;
                range = this.contentDocument.createRange();

                var startNodeAndOffset = Ilog.Utilities.GetNodeAndRelativeOffsetForAbsoluteOffset(this.contentDocument, positionStart);
                var endNodeAndOffset = null;
                if (positionEnd == positionStart) {
                    endNodeAndOffset = startNodeAndOffset;
                }
                else {
                    endNodeAndOffset = Ilog.Utilities.GetNodeAndRelativeOffsetForAbsoluteOffset(this.contentDocument, positionEnd);
                }

                if (range != null && startNodeAndOffset != null && endNodeAndOffset != null) {
                    try {
                        range.setStart(startNodeAndOffset.node, startNodeAndOffset.offset);
                    }
                    catch (ex) {
                        // even if error is met, it doesn't affect the process thanks to the trick which is following
                        if (startNodeAndOffset.node.firstChild) {
                            try {
                                range.setStart(startNodeAndOffset.node.firstChild, 1);
                            }
                            catch (ex2) {
                                Ilog.Debug.RuleEditorDebugger.PrintDebug('[SelectTextRange] error on setStart: ' + ex2.message);
                            }
                        }
                    }

                    try {
                        // the following case should only happen because of a bug in Firefox
                        if (endNodeAndOffset.node.data != null && endNodeAndOffset.node.data != undefined && endNodeAndOffset.node.data.length < endNodeAndOffset.offset) {
                            // need to rewrite the content of the node (fix for Firefox)
                            if (endNodeAndOffset.node.parentNode && endNodeAndOffset.node.parentNode.innerHTML) {
                                var innerHTML = endNodeAndOffset.node.parentNode.innerHTML;

                                try {
                                    endNodeAndOffset.node.parentNode.innerHTML = innerHTML; // simply "rewriting" the node
                                    endNodeAndOffset = Ilog.Utilities.GetNodeAndRelativeOffsetForAbsoluteOffset(this.contentDocument, positionEnd);
                                    startNodeAndOffset = Ilog.Utilities.GetNodeAndRelativeOffsetForAbsoluteOffset(this.contentDocument, positionStart); // simple security, in case start node and end nodes are the same
                                    // ATTENTION : a small trick is used here :
                                    // it's important to do setStart AFTER setEnd,
                                    // otherwise, an exception is thrown
                                    range.setEnd(endNodeAndOffset.node, endNodeAndOffset.offset);
                                    range.setStart(startNodeAndOffset.node, startNodeAndOffset.offset);

                                    this.contentWindow.getSelection().removeAllRanges();
                                    this.contentWindow.getSelection().addRange(range);

                                    return { text: range.toString(), endNode: endNodeAndOffset.node };
                                }
                                catch (ex) {
                                    Ilog.Debug.RuleEditorDebugger.PrintDebug('[SelectTextRange] unexpected error: ' + ex.message);
                                    return "";
                                }
                            }
                        }
                        range.setEnd(endNodeAndOffset.node, endNodeAndOffset.offset);
                    }
                    catch (ex) {
                        if (endNodeAndOffset.node.firstChild) {
                            try {
                                range.setEnd(endNodeAndOffset.node.firstChild, 1);
                            }
                            catch (ex2) {
                                Ilog.Debug.RuleEditorDebugger.PrintDebug('[SelectTextRange] #2 error on setStart: ' + ex2.message);
                            }
                        }
                    }

                    try {
                        this.contentWindow.getSelection().removeAllRanges();
                    }
                    catch (ex) {
                        Ilog.Debug.RuleEditorDebugger.PrintDebug('[SelectTextRange] error on removeAllRanges: ' + ex.message);
                    }

                    try {
                        this.contentWindow.getSelection().addRange(range);
                    } catch (ex) {
                        Ilog.Debug.RuleEditorDebugger.PrintDebug('[SelectTextRange] error on add range: ' + ex.message);
                    }

                    return { text: range.toString(), endNode: endNodeAndOffset.node };

                    if (startNode != null && startOffset != null && endNode != null & endOffset != null) {
                        return;
                    }
                }
                return range.toString();
            }
        }
    },

    SelectTextRangeAndChangeValue: function(textRangeToSelect, newValue) {
        if (!textRangeToSelect) {
            return;
        }

        // IE
        if (this.editorDocument && this.editorDocument.selection) {
            // important. without focus the caret cannot be placed.
            try {
                this.editorField.focus();
            }
            catch (ex) {
                Ilog.Debug.RuleEditorDebugger.PrintDebug('[SelectTextRangeAndChangeValue] error on set focus to editorField: ' + ex.message);
            }

            try {
                this.editorField.document.focus();
            }
            catch (ex) {
                Ilog.Debug.RuleEditorDebugger.PrintDebug('[SelectTextRangeAndChangeValue] error on set focus to editorField.document: ' + ex.message);
            }

            // text range start should be lower than its end
            if (textRangeToSelect.start > textRangeToSelect.end) {
                var start = textRangeToSelect.start;
                textRangeToSelect.start = textRangeToSelect.end;
                textRangeToSelect.end = start;
            }

            // Step 1 : resetting caret position (setting to 0)
            // because IE only works with relative positionning
            var resetRange = this.editorDocument.selection.createRange();
            resetRange.moveStart("character", -this.GetCurrentTextLength()); // not elegant solution but nothing equivalent found
            resetRange.collapse(true);
            resetRange.select();

            // Step 2 : now that caret is at first position in the text buffer,
            // setting the text range using the absolute start offset and range length                        
            var textRange = this.editorDocument.selection.createRange();
            textRange.moveStart("character", textRangeToSelect.start);
            textRange.moveEnd("character", textRangeToSelect.end - textRangeToSelect.start);
            textRange.select();
            textRange.text = newValue;

            return { text: textRange.text };
        }
        // FF
        else {
            if (this.contentDocument && !Ext.isIE) {
                var bodyNode = this.contentDocument.body;
                var currentLength = 0;

                var range = null;
                var bOk = false;
                var positionNodeFound = false;
                var positionStart = textRangeToSelect.start;
                var positionEnd = textRangeToSelect.end;
                range = this.contentDocument.createRange();

                var startNodeAndOffset = Ilog.Utilities.GetNodeAndRelativeOffsetForAbsoluteOffset(this.contentDocument, positionStart);
                var endNodeAndOffset = null;

                if (positionEnd == positionStart) {
                    endNodeAndOffset = startNodeAndOffset;
                }
                else {
                    endNodeAndOffset = Ilog.Utilities.GetNodeAndRelativeOffsetForAbsoluteOffset(this.contentDocument, positionEnd);
                }

                if (range != null && startNodeAndOffset != null && endNodeAndOffset != null) {
                    try {
                        range.setStart(startNodeAndOffset.node, startNodeAndOffset.offset);
                    }
                    catch (ex) {
                        // even if error is met, it doesn't affect the process thanks to the trick which is following
                        if (startNodeAndOffset.node.firstChild) {
                            try {
                                range.setStart(startNodeAndOffset.node.firstChild, 1);
                            }
                            catch (ex2) {
                                Ilog.Debug.RuleEditorDebugger.PrintDebug('[SelectTextRangeAndChangeValue] error on setStart : ' + ex2.message);
                            }
                        }
                    }

                    try {
                        // the following case should only happen because of a bug in Firefox
                        if (endNodeAndOffset.node.data != null && endNodeAndOffset.node.data != undefined && endNodeAndOffset.node.data.length < endNodeAndOffset.offset) {
                            // need to rewrite the content of the node (fix for Firefox)
                            if (endNodeAndOffset.node.parentNode && endNodeAndOffset.node.parentNode.innerHTML) {
                                var innerHTML = endNodeAndOffset.node.parentNode.innerHTML;

                                try {
                                    endNodeAndOffset.node.parentNode.innerHTML = innerHTML; // simply "rewriting" the node
                                    endNodeAndOffset = Ilog.Utilities.GetNodeAndRelativeOffsetForAbsoluteOffset(this.contentDocument, positionEnd);
                                    startNodeAndOffset = Ilog.Utilities.GetNodeAndRelativeOffsetForAbsoluteOffset(this.contentDocument, positionStart); // simple security, in case start node and end nodes are the same
                                    // ATTENTION : a small trick is used here :
                                    // it's important to do setStart AFTER setEnd,
                                    // otherwise, an exception is thrown
                                    range.setEnd(endNodeAndOffset.node, endNodeAndOffset.offset);
                                    range.setStart(startNodeAndOffset.node, startNodeAndOffset.offset);

                                    this.contentWindow.getSelection().removeAllRanges();
                                    this.contentWindow.getSelection().addRange(range);

                                    return { text: range.toString(), endNode: endNodeAndOffset.node };
                                }
                                catch (ex) {
                                    Ilog.Debug.RuleEditorDebugger.PrintDebug('[SelectTextRangeAndChangeValue] unexpected error: ' + ex.message);
                                    return "";
                                }
                            }
                        }
                        range.setEnd(endNodeAndOffset.node, endNodeAndOffset.offset);
                    }
                    catch (ex) {
                        if (endNodeAndOffset.node.firstChild) {
                            try {
                                range.setEnd(endNodeAndOffset.node.firstChild, 1);
                            }
                            catch (ex2) {
                                Ilog.Debug.RuleEditorDebugger.PrintDebug('[SelectTextRangeAndChangeValue] error on setStart: ' + ex2.message);
                            }
                        }
                    }

                    // deleting the content of the current range
                    range.deleteContents();

                    // creating and appending new node with the new value
                    var newValueNode = document.createElement('span');
                    newValueNode.innerHTML = Ext.util.Format.htmlEncode(newValue);
                    range.insertNode(newValueNode);

                    // selecting range corresponding to the new node
                    range.selectNodeContents(newValueNode);

                    return { text: range.toString() };
                }
                return range.toString();
            }
        }
    },

    GoToErrorOffset: function(offset, length) {
        if (Ext.isIE) {
            var ifr = document.getElementById(this.editorFieldId);
            ifr.focus(); //call for IE only
            ifr.contentWindow.focus(); //call for both IE and FF

            // Step 1 : resetting caret position (setting to 0)
            // because IE only works with relative positionning
            var resetRange = this.editorDocument.selection.createRange();
            resetRange.moveStart("character", -this.GetCurrentTextLength()); // not elegant solution but nothing equivalent found
            resetRange.collapse(true);
            resetRange.select();

            // Step 2 : now that caret is at first position in the text buffer,
            // creating a collapsed text range at the desired offset
            var range = this.editorDocument.selection.createRange();
            if (length == 0) {
                range.collapse(true);
            }
            range.moveStart('character', offset);
            if (length > 0) {
                range.moveEnd('character', length);
            }
            range.select();
        }
        else {
            this.Focus();

            var max = this.GetPlainText().length;
            try {
                var startPos = offset;
                if (startPos > max) {
                    startPos = max;
                }
                var endPos = offset + length;
                if (endPos > max) {
                    endPos = max;
                }
                this.SelectTextRange({ start: startPos, end: endPos });
            }
            catch (ex) {
                Ilog.Debug.RuleEditorDebugger.PrintDebug('[GoToErrorOffset] error on SelectTextRange: ' + ex.message);
            }

            var self = this;
            window.setTimeout(function() {
                self.EnsureCaretVisible();
            }, 20);
        }

        return;
    },

    GetCurrentTextLength: function() {
        var text = this.GetPlainText();
        return text.length;
    },

    /*****************************************/
    /* Move the caret to a specific position */
    /*****************************************/
    MoveCaretToPosition: function(positionToMoveTo) {
        if (!positionToMoveTo) {
            Ilog.Debug.RuleEditorDebugger.PrintDebug("[MoveCaretToPosition] position set to 0 because it was null");
            positionToMoveTo = 0;
        }

        // IE
        if (this.editorDocument && this.editorDocument.selection) {
            try {
                this.editorField.focus();
            }
            catch (ex) {
                Ilog.Debug.RuleEditorDebugger.PrintDebug('[MoveCaretToPosition] error to set focus on editorField: ' + ex.message);
            }
            try {
                this.editorField.document.focus();
            }
            catch (ex) {
                Ilog.Debug.RuleEditorDebugger.PrintDebug('[MoveCaretToPosition] error to set focus on editorField.document: ' + ex.message);
            }

            // Step 1 : resetting caret position (setting to 0)
            // because IE only works with relative positionning
            var resetRange = this.editorDocument.selection.createRange();
            resetRange.moveStart("character", -this.GetCurrentTextLength()); // not elegant solution but nothing equivalent found
            resetRange.collapse(true);
            resetRange.select();

            // Step 2 : now that caret is at first position in the text buffer,
            // creating a collapsed text range at the desired offset
            var range = this.editorDocument.selection.createRange();
            range.collapse(true);
            range.moveStart('character', positionToMoveTo);
            range.select();
        }
        // FF :
        else {
            if (this.contentDocument && !Ext.isIE) {
                var bodyNode = this.contentDocument.body;
                var currentLength = 0;

                var range = null;
                var bOk = false;
                var positionNodeFound = false;

                for (var p = 0; p < bodyNode.childNodes.length; p++) {
                    var currentNode = bodyNode.childNodes[p];
                    var nodeLength = Ilog.Utilities.GetNodeLength(currentNode);

                    if (positionToMoveTo >= currentLength && positionToMoveTo <= (currentLength + nodeLength) && !bOk) {
                        var relativePositionInNode = positionToMoveTo - currentLength;
                        range = this.contentDocument.createRange();

                        if (relativePositionInNode <= nodeLength && !positionNodeFound) {
                            var positionNode = currentNode;
                            if (currentNode.nodeType == 1) {
                                if (currentNode.childNodes.length > 0) {
                                    for (var m = 0; m < currentNode.childNodes.length; m++) {
                                        if (currentNode.childNodes[m].nodeType == 3) {
                                            positionNode = currentNode.childNodes[m];
                                            positionNodeFound = true;
                                        }
                                    }
                                }
                            }

                            var shouldAddRange = false;
                            try {
                                // Fix for positioning caret AFTER a BR node
                                // it works but the caret is not blinking anymore
                                // TO DO : check for a workaround so that the carets blinks
                                if (positionNode.nodeName.toUpperCase() == 'BR') {

                                    Ilog.Debug.RuleEditorDebugger.PrintDebug('[MoveCaretToPosition] position to BR node');

                                    if (bodyNode.childNodes[p + 1]) {
                                        var positionNode = bodyNode.childNodes[p + 1];
                                        var relativePositionInNode = 0;
                                        range.setStart(positionNode, relativePositionInNode);
                                        range.setEnd(positionNode, relativePositionInNode);
                                        this.contentWindow.getSelection().removeAllRanges();
                                        this.contentWindow.getSelection().addRange(range);
                                        return;
                                    }
                                }

                                range.setStart(positionNode, relativePositionInNode);
                                range.setEnd(positionNode, relativePositionInNode);
                                shouldAddRange = true;
                            }
                            catch (rangeException) {
                                Ilog.Debug.RuleEditorDebugger.PrintDebug("[MoveCaretToPosition] error while setting range (on node " + positionNode.nodeName + " - " + positionNode.nodeValue + "): " + rangeException.message);
                            }
                        }
                        else {
                            range.setStart(currentNode, 0);
                            range.setEnd(currentNode, 0);
                            shouldAddRange = true;
                            Ilog.Debug.RuleEditorDebugger.PrintDebug("[MoveCaretToPosition] position error : relativePositionInNode = " + relativePositionInNode + " while currentLength = " + nodeLength);
                        }

                        try {
                            if (shouldAddRange) {
                                this.contentWindow.getSelection().removeAllRanges();
                                this.contentWindow.getSelection().addRange(range);
                            }
                        }
                        catch (rangeException) {
                            Ilog.Debug.RuleEditorDebugger.PrintDebug("[MoveCaretToPosition] error while setting range in selection ('" + positionNode.nodeName + "'): " + rangeException.message);
                        }

                        if (!shouldAddRange) {
                            Ilog.Debug.RuleEditorDebugger.PrintDebug("[MoveCaretToPosition] range could not be added (position " + positionToMoveTo + ")");
                        }

                        bOk = true;
                    }
                    currentLength += nodeLength;
                }
            }
        }
    },

    GetCaretRelativePositionInsideEditionArea: function() {
        if (!this.editorField || !this.editorDocument) {
            return null;
        }

        if (this.editorDocument.selection) {
            try {
                var rng = this.editorDocument.selection.createRange().duplicate();
            }
            catch (ex) {
                var rng = this.editorDocument.selection.createRange();
            }
            var position = { x: rng.offsetLeft, y: rng.offsetTop };
            return position;
        }
        else if (this.contentWindow && this.contentWindow.getSelection()) {
            var el = this.contentWindow;
            var off = { 'x': 0, 'y': 0, 'h': 0 };
            var r = null;

            try {
                r = el.getSelection().getRangeAt(0);
            }
            catch (ex) {
                Ilog.Debug.RuleEditorDebugger.PrintDebug('[GetCaretRelativePositionInsideEditionArea] could not get range at 0: ' + ex.message);
                return null;
            }

            var rangeToRestore = { startContainer: r.startContainer, startOffset: r.startOffset, endContainer: r.endContainer, endOffset: r.endOffset };

            var e = r.endOffset
               , s = this.editorDocument.createElement('span')
               , n = s;

            s.style.borderLeft = '1px solid red';
            s.id = "Ilog.RuleEditorPredictionsPositionNode_" + this.editorFieldId;

            try {
                r.insertNode(s);
                off.h = n.offsetHeight;
                while (n.offsetParent) {
                    off.x += n.offsetLeft;
                    off.y += n.offsetTop;
                    n = n.offsetParent
                }
                s.parentNode.removeChild(s);
            }
            // when several nodes are involved by text range, insertion of positioning node may fail,
            // workaround :
            catch (ex) {
                try {
                    var nodeStart = r.startContainer;
                    var offsetStart = r.startOffset;
                    var nodeLength = nodeStart.textContent.length;
                    if (nodeLength == offsetStart) { // it is the case that fails
                        r.startContainer.parentNode.insertBefore(s, r.startContainer.nextSibling);
                        off.h = n.offsetHeight;
                        while (n.offsetParent) {
                            off.x += n.offsetLeft;
                            off.y += n.offsetTop;
                            n = n.offsetParent
                        }
                        s.parentNode.removeChild(s);
                    }
                }
                catch (exc) {
                    Ilog.Debug.RuleEditorDebugger.PrintDebug('[GetCaretRelativePositionInsideEditionArea] workaround error: ' + exc.message);
                }
            }

            if (e - r.endOffset) {
                try {
                    r.setEnd(r.endContainer.nextSibling, e - r.endOffset);
                    el.getSelection().addRange(r);
                }
                catch (ex) { // error happens here in Chrome when displaying the completion menu
                    Ilog.Debug.RuleEditorDebugger.PrintDebug('[GetCaretRelativePositionInsideEditionArea] error while setting end of range and setting range: ' + ex.message);
                }
            }
            off.x -= this.GetScrollXPosition();
            off.y -= this.GetScrollYPosition();

            var rngOK = false;
            try {
                if (rangeToRestore.startContainer.textContent != undefined && rangeToRestore.startContainer.textContent != null && rangeToRestore.startContainer.textContent.length < rangeToRestore.startOffset) {
                    r.setStart(rangeToRestore.startContainer, rangeToRestore.startContainer.textContent.length);
                }
                else {
                    r.setStart(rangeToRestore.startContainer, rangeToRestore.startOffset);
                }

                rngOK = true;
            }
            catch (ex) {
                Ilog.Debug.RuleEditorDebugger.PrintDebug('[GetCaretRelativePositionInsideEditionArea] range set start failed: ' + ex.message);
            }

            if (rngOK === true) {
                try {
                    if (rangeToRestore.endContainer.textContent != undefined && rangeToRestore.endContainer.textContent != null && rangeToRestore.endContainer.textContent.length < rangeToRestore.endOffset) {
                        r.setEnd(rangeToRestore.endContainer, rangeToRestore.endContainer.textContent.length);
                    }
                    else {
                        r.setEnd(rangeToRestore.endContainer, rangeToRestore.endOffset);
                    }
                }
                catch (ex) {
                    Ilog.Debug.RuleEditorDebugger.PrintDebug('[GetCaretRelativePositionInsideEditionArea] range set end failed: ' + ex.message);
                }
            }

            return off;
        }

        return null;
    },

    // FF specific
    GetAbsoluteOffsetOfNodeBeginning: function(node) {
        var bodyNode = this.contentDocument.body;

        if (!bodyNode) {
            return 0;
        }

        var index = Ilog.Utilities.CheckNodeInsideNode(node, bodyNode);

        if (index < 0) {
            return 0;
        }

        return index;
    },

    GetTextForTextRange: function(textRange) {
        var txt = this.GetPlainText();
        return txt.substring(textRange.start, textRange.end);
    },

    DeleteCurrentTextRange: function() {
        if (document.selection && this.editorDocument) {
            var range = this.editorDocument.selection.createRange();
            if (range && range.text && range.text.length > 0) {
                range.text = '';
            }
        }
        else if ((document.getSelection || window.getSelection) && this.contentWindow) {
            this.Focus();
            try {
                var range = this.contentWindow.getSelection().getRangeAt(0);
                if (range && range.deleteContents) {
                    range.deleteContents();
                }
            }
            catch (ex) {
                Ilog.Debug.RuleEditorDebugger.PrintDebug('[DeleteCurrentTextRange] error on delete range: ' + ex.message);
            }
        }
    },

    FindRangePosition: function(textLength, startOrEnd, range, originalRange) {
        if (textLength == 0) {
            return 0;
        }

        var lowerBound = 0;
        var upperBound = textLength;
        var found = false;
        var ite = 0;
        var pos = null;
        while (!found) {
            var delta = Math.floor((upperBound - lowerBound) / 2);
            if (delta == 0) { delta = 1; }
            var moveDelta = lowerBound + delta;

            var nbUnitsMove = null;
            if (startOrEnd == 0) {
                nbUnitsMove = range.moveStart("character", -moveDelta);
            }
            else {
                nbUnitsMove = range.moveEnd("character", -moveDelta);
            }

            var canMove = (nbUnitsMove == -moveDelta);
            if (canMove) {
                lowerBound = lowerBound + delta;
                if (delta == 1) { found = true; pos = moveDelta + 1; range = originalRange.duplicate(); var canMoveCandidate = 0; if (startOrEnd == 0) { canMoveCandidate = (range.moveStart("character", -pos) == -pos); } else { canMoveCandidate = (range.moveEnd("character", -pos) == -pos); } if (!canMoveCandidate) { pos = pos - 1; } break; }
            }
            else {
                upperBound = moveDelta;
                if (delta == 1) { found = true; pos = moveDelta - 1; range = originalRange.duplicate(); var canMoveCandidate = 0; if (startOrEnd == 0) { canMoveCandidate = (range.moveStart("character", -pos) == -pos); } else { canMoveCandidate = (range.moveEnd("character", -pos) == -pos); } if (!canMoveCandidate) { pos = pos - 1; } break; }
            }

            range = originalRange.duplicate();
            ite++;
        }

        return pos;
    },

    GetTextRange: function() {
        // IE
        if (document.selection && this.editorDocument) {
            var range = null;
            try {
                var originalRange = this.editorDocument.selection.createRange();
                var range = originalRange.duplicate();
            }
            catch (ex) {
                range = this.editorDocument.selection.createRange();
            }

            try {
                var startPos = 0;
                var endPos = 0;

                var textLength = this.GetCurrentTextLength();

                try {
                    startPos = this.FindRangePosition(textLength, 0, range, originalRange);
                    range = originalRange.duplicate();
                    endPos = this.FindRangePosition(textLength, 1, range, originalRange);
                    range = originalRange.duplicate();
                }
                catch (ex) {
                    startPos = 0;
                    endPos = 0;
                    range = originalRange.duplicate();

                    while (range.moveStart("character", -1) != 0) {
                        startPos++;
                    }
                    while (range.moveEnd("character", -1) != 0) {
                        endPos++;
                    }
                }

                return { start: startPos, end: endPos };
            }
            catch (ex) {
                Ilog.Debug.RuleEditorDebugger.PrintDebug('[GetTextRange] text range computing failed: ' + ex.message);
                return { start: 0, end: 0 };
            }
        }
        // FF : iterating through nodes and summing length of each node until the right node (for which only the offset is added, not the whole length)
        else if ((document.getSelection || window.getSelection) && this.contentWindow) {
            var range = null;
            this.Focus();
            try {
                range = this.contentWindow.getSelection().getRangeAt(0);
            }
            catch (ex) {
                this.SelectTextRange({ start: 0, end: 0 });

                try {
                    range = this.contentWindow.getSelection().getRangeAt(0);
                }
                catch (ex) {
                    Ilog.Debug.RuleEditorDebugger.PrintDebug('[GetTextRange] error on get range: ' + ex.message);
                }

                return { start: 0, end: 0 };
            }

            var startNode = range.startContainer;
            var endNode = range.endContainer;
            var endOffset = range.endOffset;
            var startOffset = range.startOffset;

            if (startNode == this.contentDocument.body && endNode == this.contentDocument.body) {
                Ilog.Debug.RuleEditorDebugger.PrintDebug('[GetTextRange] selection is set to body');
                if (this.contentDocument.body.childNodes.length == (startOffset + 1)) {
                    var elt = this.contentDocument.createElement('span');
                    this.contentDocument.body.appendChild(elt);
                    startNode = elt;
                    startOffset = 0;
                    endNode = elt;
                    endOffset = 0;
                }
                else {

                    try {
                        var elt = this.contentDocument.createElement('span');
                        range.insertNode(elt);
                        startNode = elt;
                        startOffset = 0;
                        endNode = elt;
                        endOffset = 0;
                    }
                    catch (ex) {
                        Ilog.Debug.RuleEditorDebugger.PrintDebug('[GetTextRange] error on create and insert node: ' + ex.message);
                    }
                }
            }


            var startNodeAbsoluteOffset = this.GetAbsoluteOffsetOfNodeBeginning(startNode);
            var endNodeAbsoluteOffset = null;
            if (endNode == startNode) {
                endNodeAbsoluteOffset = startNodeAbsoluteOffset;
            }
            else {
                endNodeAbsoluteOffset = this.GetAbsoluteOffsetOfNodeBeginning(endNode);
            }

            var startPos = startNodeAbsoluteOffset + startOffset;
            var endPos = endNodeAbsoluteOffset + endOffset;
            return { start: startPos, end: endPos, startNode: startNode, endNode: endNode, startOffset: startOffset, endOffset: endOffset };
        }

        return { start: 0, end: 0 };
    },

    GetXYAbsolutePositionOfCaret: function() {
        var positionXandY = this.GetCaretRelativePositionInsideEditionArea();

        if (!positionXandY) {
            return null;
        }

        var extElm = new Ext.Element(this.editorField);
        var iframePosition = { x: extElm.getLeft(), y: extElm.getTop() };

        var caretAbsoluteX = iframePosition.x + positionXandY.x;
        var caretAbsoluteY = iframePosition.y + positionXandY.y + Ilog.Constants.RuleEditorConstants.CaretBottomMargin;

        // FF
        if ((document.getSelection || window.getSelection) && positionXandY.y < 0) {
            var el = this.contentWindow;
            var r = el.getSelection().getRangeAt(0);
            var elm = new Ext.Element(r.startContainer);

            try {
                caretAbsoluteY = iframePosition.y + elm.getTop() + elm.getHeight();
            }
            catch (ex) {
                Ilog.Debug.RuleEditorDebugger.PrintDebug('[GetXYAbsolutePositionOfCaret] error while computing caret Y: ' + ex.message);
            }

            if (Ilog.Constants.RuleEditorConstants.FfScrollFix === true) {
                caretAbsoluteY += this.contentDocument.body.scrollLeft; // to overcome a weird FF bug, adding scrollLeft to the top value so that the result is correct
            }
        }

        return { x: caretAbsoluteX, y: caretAbsoluteY };
    },

    GetScrollXPosition: function() {
        if (this.ScrollbarLock === true) {
            return -1;
        }

        var iframe = document.getElementById(this.editorFieldId);

        if (this.contentWindow && this.contentWindow.pageXOffset) {
            return this.contentWindow.pageXOffset;
        }
        else if (this.contentWindow) {
            return this.contentWindow.document.body.scrollLeft;
        }
        else {
            Ilog.Debug.RuleEditorDebugger.PrintDebug('[GetScrollXPosition] Unable to get scrollX position');
        }
    },

    SetScrollXYPosition: function(xoffset, yoffset) {
        var self = this;

        try {
            self.LastScrollPos = { x: xoffset, y: yoffset };
            window.setTimeout(function() {
                self.contentWindow.scrollTo(xoffset, yoffset);
            }, 0); // using a setTimeout is mandatory to make it work inside IE
        }
        catch (ex) {
        }
    },

    GetScrollYPosition: function() {
        if (this.ScrollbarLock === true) {
            return -1;
        }

        var iframe = document.getElementById(this.editorFieldId);

        if (this.contentDocument && !Ext.isIE) {
            return this.contentDocument.body.scrollTop;
        }
        else if (this.editorDocument) {
            return this.editorDocument.body.scrollTop;
        }
        else {
            return 0;
        }
    },

    GetEditionAreaAbsolutePosition: function() {
        var srcElem = this.editorField;
        var oTop = 0.0;
        var oLeft = 0.0;
        var elem = srcElem;
        while (elem.offsetParent) {
            oTop += elem.offsetParent.offsetTop;
            oLeft += elem.offsetParent.offsetLeft;
            elem = elem.offsetParent;
        };

        var pos = { x: (oLeft + srcElem.offsetLeft), y: (oTop + srcElem.offsetTop) };
        return pos;
    },

    CheckDropOrPasteTargetAgainstFrozenZones: function() {
        if (!this.FrozenZones) {
            return true;
        }

        var textRange = this.GetTextRange();

        for (var p = 0; p < this.FrozenZones.length; p++) {
            var frozenZone = this.FrozenZones[p];

            if (textRange.start == textRange.end) {
                var singleCaretPos = textRange.start;
                if (singleCaretPos > frozenZone.start && singleCaretPos < frozenZone.end) {
                    return false;
                }
                else if (singleCaretPos == frozenZone.start) {
                    return false;
                }
                else if (singleCaretPos == frozenZone.end) {
                    return true;
                }
            }
            else {
                // current text selection has a part inside a frozen zone :
                // canceling everything
                if ((textRange.start >= frozenZone.start && textRange.start <= frozenZone.end) || (textRange.end >= frozenZone.start && (textRange.end + 1) <= frozenZone.end)) {
                    return false;
                }
            }
        }

        return true;
    },

    CheckKeyInputAgainstFrozenZones: function(keyCode, ctrlPressed, altPressed, shiftPressed, evt) {
        if (!this.FrozenZones) {
            return true;
        }

        // Arrow left/right/up/down = always allowed
        if (!(keyCode == Ext.EventObject.UP || keyCode == Ext.EventObject.LEFT || keyCode == Ext.EventObject.RIGHT || keyCode == Ext.EventObject.DOWN)) {
            // Commands always allowed :
            if (ctrlPressed === true) {
                // CTRL+S - Save
                if (keyCode == Ilog.Constants.RuleEditorConstants.CharKeyCodeForSaveShortcut) {
                    return true;
                }

                // "SELECT ALL" command
                if (keyCode == 65) { // 65 = A/a
                    return true;
                }

                // "PASTE" command
                if (evt && evt.browserEvent) {
                    var e = evt.browserEvent;
                    if ((keyCode == e.DOM_VK_V) || (shiftPressed === true && keyCode == e.DOM_VK_INSERT)) {
                    }
                }

                if (keyCode == 86) { // 86 = V/v
                }

                // "COPY" command
                if (keyCode == 67) { // 67 = C/c
                    return true;
                }

                // "cut" command
                if (keyCode == 88) { // 88 = X/x
                }

                // "undo" command
                if (keyCode == 90) { // 90 = Z/z
                    return true;
                }

                // "redo" command
                if (keyCode == 89) { // 89 = Y/y
                    return true;
                }
            }

            var textRange = this.GetTextRange();

            for (var p = 0; p < this.FrozenZones.length; p++) {
                var frozenZone = this.FrozenZones[p];

                // single caret
                if (textRange.start == textRange.end) {
                    var singleCaretPos = textRange.start;
                    if (singleCaretPos > frozenZone.start && singleCaretPos < frozenZone.end) {
                        return false;
                    }
                    else if (singleCaretPos == frozenZone.start) {
                        return false;
                    }
                    else if (singleCaretPos == frozenZone.end) {
                        if (keyCode == Ext.EventObject.BACKSPACE) {
                            return false;
                        }
                    }
                }
                else {
                    // current text selection has a part inside a frozen zone :
                    // canceling everything
                    if ((textRange.start >= frozenZone.start && textRange.start <= frozenZone.end) || (textRange.end >= frozenZone.start && (textRange.end + 1) <= frozenZone.end)) {
                        return false;
                    }
                }
            }
        }

        return true;
    },

    HandleKeyUp: function(evt) {
        return this.AnalyzeKeyEvent(evt, Ilog.Constants.RuleEditorConstants.KeyUpEvent);
    },

    AnalyzeKeyEvent: function(keyEvt) {
        if (!keyEvt) {
            return;
        }

        var eventType = null;
        if (keyEvt.type == "keydown") {
            eventType = Ilog.Constants.RuleEditorConstants.KeyDownEvent;
        }
        if (keyEvt.type == "keypress") {
            eventType = Ilog.Constants.RuleEditorConstants.KeyPressEvent;
        }
        if (keyEvt.type == "keyup") {
            eventType = Ilog.Constants.RuleEditorConstants.KeyUpEvent;
        }

        if (eventType == null) {
            return;
        }

        // preventing event from bubbling
        Ilog.Utilities.StopEventPropagation(keyEvt);
        // checking session
        Ilog.Utilities.TestTimeoutSession();

        if (!this) {
            return;
        }

        if (this.InputDisabled === true) {
            Ilog.Utilities.CancelEventBehaviour(keyEvt);
            return false;
        }

        // getting key code and modifiers
        var keyCode = keyEvt.getCharCode();
        var altPressed = keyEvt.altKey;
        var ctrlPressed = keyEvt.ctrlKey;
        var shiftPressed = keyEvt.shiftKey;
        var charCode = (keyEvt.browserEvent && keyEvt.browserEvent.charCode) ? keyEvt.browserEvent.charCode : keyCode;

        if (!keyCode && !charCode) {
            Ilog.Utilities.CancelEventBehaviour(keyEvt);
            return false;
        }

        // general actions upon keydown/press
        if (eventType == Ilog.Constants.RuleEditorConstants.KeyDownEvent) {
            if (this.DatePickerLayer) {
                this.DatePickerLayer.parentNode.removeChild(this.DatePickerLayer);
                this.DatePickerLayer = null;
            }

            // Special behaviour for firefox on the delete key :
            // storing the text range, because on key up the text range is not accurate anymore
            // do not delete, it is used to overcome a bug in firefox
            if (Ext.isGecko && keyCode == Ext.EventObject.DELETE) {
                this.TextRangeOnDeleteForFirefox = this.GetTextRange();
                return;
            }
        }

        // general actions upon keyup
        if (eventType == Ilog.Constants.RuleEditorConstants.KeyUpEvent) {
            //"cleanup"
            this.HideValueEditor();
            if (this.PredictionsView && this.PredictionsView.Visible) {
                this.PredictionsView.DisableTransparency();
            }

            // workarounds
            // FF linux workaround for arrow up/down
            if (keyCode == Ext.EventObject.DOWN) {
                try {
                    if (this.TimerLinuxFFArrowDown) {
                        try {
                            window.clearTimeout(this.TimerLinuxFFArrowDown);
                        } catch (exc) { }

                        window.clearInterval(this.TimerLinuxFFArrowDown);
                        this.TimerLinuxFFArrowDown = null;
                    }
                }
                catch (ex) { }
            }
            if (keyCode == Ext.EventObject.UP) {
                try {
                    if (this.TimerLinuxFFArrowUp) {
                        try {
                            window.clearTimeout(this.TimerLinuxFFArrowUp);
                        } catch (exc) { }

                        window.clearInterval(this.TimerLinuxFFArrowUp);
                        this.TimerLinuxFFArrowUp = null;
                    }
                }
                catch (ex) { }
            }

            // special behaviour for Firefox with the Delete key
            // forcing the text range to be set, using the text range save
            // do not delete, it is used to overcome a bug in firefox
            if (Ext.isGecko && keyCode == Ext.EventObject.DELETE) {
                if (this.TextRangeOnDeleteForFirefox != null) {
                    var startRange = this.TextRangeOnDeleteForFirefox.start;
                    var self = this;
                    window.setTimeout(function() { self.SelectTextRange({ start: startRange, end: startRange }); }, 10);
                }
            }
        }

        if (eventType == Ilog.Constants.RuleEditorConstants.KeyDownEvent && this.toolbarButtonsUpdateTaskId) {
            window.clearTimeout(this.toolbarButtonsUpdateTaskId);
        }

        /********************************/
        // Frozen regions
        /********************************/
        // same behaviour on keyup/keydown/keypress
        if (this.FrozenZones && this.FrozenZones.length > 0) {
            // blocking contextual menu shortcut in FF2 when there are some frozen region
            if (keyCode == 93 && Ilog.Utilities.IsFirefox2() === true) {
                if (keyEvt.preventDefault) {
                    keyEvt.preventDefault();
                }
                Ilog.Utilities.CancelEventBehaviour(keyEvt);
                Ilog.Utilities.StopEventPropagation(keyEvt);
                return false;
            }
            // Checking entered key against frozen zones
            var inputAllowed = this.CheckKeyInputAgainstFrozenZones(keyCode, ctrlPressed, altPressed, shiftPressed, keyEvt);
            if (inputAllowed === false) {
                Ilog.Utilities.CancelEventBehaviour(keyEvt);
                return false;
            }
        }
        /********************************/

        /********************************/
        // DT Integration
        /********************************/
        if (this.isEmbeddedInDT === true) {
            if ((keyCode == 13 || keyCode == Ext.EventObject.ENTER) && !this.IsMultiline && (!this.PredictionsView || !this.PredictionsView.Visible)) {
                Ilog.Utilities.CancelEventBehaviour(keyEvt);
                Ilog.Utilities.StopEventPropagation(keyEvt);

                if (eventType == Ilog.Constants.RuleEditorConstants.KeyDownEvent) {
                    this.fireEvent("EditionValidate");
                }

                return false;
            }

            if (!this.IsMultiline && (!this.PredictionsView || !this.PredictionsView.Visible)) {

                if (keyCode == Ilog.Constants.RuleEditorConstants.CharKeyCodeForDtValidation) {
                    Ilog.Utilities.CancelEventBehaviour(keyEvt);
                    if (eventType == Ilog.Constants.RuleEditorConstants.KeyUpEvent) {
                        this.fireEvent("EditionValidate");
                    }
                    return false;
                }
                if (keyCode == Ilog.Constants.RuleEditorConstants.CharKeyCodeForDtCancellation) {
                    Ilog.Utilities.CancelEventBehaviour(keyEvt);

                    if (eventType == Ilog.Constants.RuleEditorConstants.KeyUpEvent) {
                        try {
                            this.fireEvent("EditionCancel");
                        }
                        catch (ex) { }
                    }
                    return false;
                }
            }
        }
        /********************************/


        /********************************/
        // Placeholders
        /********************************/
        if (this.PlaceHolders != null && this.PlaceHolders != undefined) {
            // navigation between placeholders with ALT+arrow left/right
            if (this.SelectedPlaceholder != null && this.SelectedPlaceholderIndex > -1 && altPressed === true && (keyCode == Ext.EventObject.LEFT || keyCode == Ext.EventObject.RIGHT)) {
                Ilog.Utilities.CancelEventBehaviour(keyEvt);

                // performing the actual swith on keyup
                if (eventType == Ilog.Constants.RuleEditorConstants.KeyUpEvent) {
                    var nextPlaceholderIndex = this.SelectedPlaceholderIndex;

                    if (this.PlaceHolders.length < 2) {
                        // no other placeholder to select
                        return;
                    }
                    else {
                        if (this.PredictionsView) {
                            this.PredictionsView.Hide();
                        }

                        // going to "next" placeholder
                        if (keyCode == Ext.EventObject.RIGHT) {
                            if (this.SelectedPlaceholderIndex == (this.PlaceHolders.length - 1)) {
                                nextPlaceholderIndex = 0;
                            }
                            else {
                                nextPlaceholderIndex = this.SelectedPlaceholderIndex + 1;
                            }
                        }
                        // going to "previous" placeholder
                        else {
                            if (this.SelectedPlaceholderIndex == 0) {
                                nextPlaceholderIndex = this.PlaceHolders.length - 1;
                            }
                            else {
                                nextPlaceholderIndex = this.SelectedPlaceholderIndex - 1;
                            }
                        }
                    }

                    if (nextPlaceholderIndex >= 0 && nextPlaceholderIndex < this.PlaceHolders.length && this.SelectedPlaceholderIndex != nextPlaceholderIndex) {
                        var placeholderToSelect = this.PlaceHolders[nextPlaceholderIndex];
                        this.CancelGetPredictionsRequest();
                        this.SelectPlaceholderNode(placeholderToSelect);
                    }

                    return false;
                }

                // on keydown/keypress, canceling behaviour
                return false;
            }

            // ALT+down on a placeholder : selecting the placeholder
            if (Ilog.Constants.RuleEditorConstants.UseKeyboardShortCutForPlaceholderSelect === true && altPressed === true && keyCode == Ext.EventObject.DOWN) {
                Ilog.Utilities.CancelEventBehaviour(keyEvt);

                if (eventType == Ilog.Constants.RuleEditorConstants.KeyUpEvent) {
                    var self = this;
                    window.setTimeout(function() {
                        try {
                            var textRange = self.GetTextRange();
                            if (textRange.start == textRange.end) {
                                var doc = (self.editorDocument != undefined && self.editorDocument != null) ? self.editorDocument : self.contentDocument;
                                var node = Ilog.Utilities.GetNodeAndRelativeOffsetForAbsoluteOffset(doc, textRange.start).node;

                                if (node.nodeType == 3) {
                                    node = node.parentNode;
                                }

                                var cssClass = node.className;
                                if (cssClass.indexOf(Ilog.Constants.RuleEditorConstants.PatternPlaceHolderRecognition) >= 0) {
                                    self.Focus();
                                    self.SelectPlaceholderNode(node);
                                    return false;
                                }
                            }
                        }
                        catch (ex) { Ilog.Debug.RuleEditorDebugger.PrintDebug('[HandleKeyDown] Placeholder could not be selected : ' + ex.message); }
                    }, 100);
                    return false;
                }
                return false;
            }
        }
        /********************************/

        /********************************/
        // Interactions with a displayed completion menu
        /********************************/
        if (this.PredictionsView && this.PredictionsView.Visible === true) {
            //CTRL on keydown : enabling transparency
            if (keyCode == 17 && eventType == Ilog.Constants.RuleEditorConstants.KeyDownEvent) {
                this.PredictionsView.EnableTransparency();
            }

            // Trapping [Escape] key
            if (keyCode == 27) {
                if (eventType == Ilog.Constants.RuleEditorConstants.KeyUpEvent) {
                    this.PredictionsView.Hide();
                    this.SendRefreshRequest();
                }
                return false;
            }

            // Switching to prediction filter
            if (keyCode == Ilog.Constants.RuleEditorConstants.CharKeyCodeForSwitchToPredFilter && (Ilog.Constants.RuleEditorConstants.ShiftModifierForSwitchToPredFilter === false || (Ilog.Constants.RuleEditorConstants.ShiftModifierForSwitchToPredFilter === true && shiftPressed === true))) {
                Ilog.Utilities.CancelEventBehaviour(keyEvt);

                var range = null;

                if (Ext.isIE) {
                    range = this.GetTextRange();
                }

                if (eventType == Ilog.Constants.RuleEditorConstants.KeyUpEvent) {
                    Ilog.Utilities.StopEventPropagation(keyEvt);

                    try {
                        this.PredictionsView.SwitchToFilter();
                        if (Ext.isIE && range) {
                            this.PredictionsView.ReplacementArea = range;
                            this.PredictionsView.ReplacementArea.NoFix = true;
                        }
                    }
                    catch (ex) {
                        Ilog.Debug.RuleEditorDebugger.PrintDebug('[HandleKeyUp] error on switch to filter: ' + ex.message);
                    }

                    if (Ext.isIE && range) {
                        this.SelectTextRange(range);
                        this.PredictionsView.FilterBox.focus();
                    }

                    return false;
                }

                // cancelling behaviour on keydown/keypress
                return false;
            }

            // [ENTER] key
            if (keyCode == 13 || keyCode == Ext.EventObject.ENTER) {
                // validating prediction on keyup if a prediction is selected
                if (this.PredictionsView.GetCurrentSelectedPredictionValue()) {
                    Ilog.Utilities.CancelEventBehaviour(keyEvt);

                    if (eventType == Ilog.Constants.RuleEditorConstants.KeyDownEvent || eventType == Ilog.Constants.RuleEditorConstants.KeyPressEvent) {
                        this.MarkDocumentAsDirty();
                        return false;
                    }

                    var predictionObj = this.PredictionsView.GetCurrentSelectedPredictionValue();
                    var selectedPrediction = predictionObj.ValueToInsert;
                    var valueEditor = this.PredictionsView.GetCurrentSelectedPredictionValueEditor();

                    this.PredictionsView.Hide();

                    // Predictions that involve a value editor:
                    // firing special event
                    if (valueEditor) {
                        this.ValidateValueEditorPrediction(valueEditor);
                        return false;
                    }

                    if (selectedPrediction != null) {
                        selectedPrediction = selectedPrediction.replace(/\r\n/g, '<br/>').replace(/\n/g, '<br/>').replace(/\t/g, '&nbsp;&nbsp;&nbsp;');
                        predictionObj.ValueToInsert = selectedPrediction;
                        this.ValidatePrediction(predictionObj, false);
                        Ilog.Utilities.CancelEventBehaviour(keyEvt);
                        return false;
                    }
                }
            } //[ENTER]

            // Arrow UP/DOWN
            if (keyCode == Ext.EventObject.UP || keyCode == Ext.EventObject.DOWN) {
                if (eventType == Ilog.Constants.RuleEditorConstants.KeyDownEvent) {
                    Ilog.Utilities.CancelEventBehaviour(keyEvt);

                    var self = this;
                    // up
                    if (keyCode == Ext.EventObject.UP) {
                        this.PredictionsView.UpdateSelectedIndex(-1);
                        if (Ext.isGecko && Ilog.Utilities.IsLinux() === true) {
                            this.TimerLinuxFFArrowUp = window.setTimeout(function() { self.PredictionsView.UpdateSelectedIndex(-1); self.TimerLinuxFFArrowUp = null; self.TimerLinuxFFArrowUp = window.setInterval(function() { self.PredictionsView.UpdateSelectedIndex(-1); }, Ilog.Constants.RuleEditorConstants.LinuxFFArrowDelay); }, Ilog.Constants.RuleEditorConstants.LinuxFFArrowFirstDelay);
                        }
                        return false;

                    }
                    // down
                    else if (keyCode == Ext.EventObject.DOWN) {
                        this.PredictionsView.UpdateSelectedIndex(1);
                        if (Ext.isGecko && Ilog.Utilities.IsLinux() === true) {
                            this.TimerLinuxFFArrowDown = window.setTimeout(function() { self.PredictionsView.UpdateSelectedIndex(1); self.TimerLinuxFFArrowDown = null; self.TimerLinuxFFArrowDown = window.setInterval(function() { self.PredictionsView.UpdateSelectedIndex(1); }, Ilog.Constants.RuleEditorConstants.LinuxFFArrowDelay); }, Ilog.Constants.RuleEditorConstants.LinuxFFArrowFirstDelay);
                        }
                        return false;
                    }
                }

                return false;
            } // UP/DOWN

            // PageUP/PageDOWN
            if (keyCode == keyEvt.PAGEUP || keyCode == keyEvt.PAGEDOWN) {
                if (eventType == Ilog.Constants.RuleEditorConstants.KeyDownEvent) {
                    var range = null;
                    if (Ext.isIE) {
                        range = this.GetTextRange();
                    }

                    // canceling regular behaviour
                    Ilog.Utilities.CancelEventBehaviour(keyEvt);

                    if (keyCode == keyEvt.PAGEUP) {
                        this.PredictionsView.ManagePageUp(shiftPressed);
                    }
                    else {
                        this.PredictionsView.ManagePageDown(shiftPressed);
                    }

                    if (Ext.isIE && range) {
                        this.PredictionsView.ReplacementArea = range;
                        this.PredictionsView.ReplacementArea.NoFix = true;
                    }

                    return false;
                }

                return false;
            }

            // [End] key
            if (keyCode == keyEvt.END) {
                // if predictions view is displayed, going to last item
                if (eventType == Ilog.Constants.RuleEditorConstants.KeyDownEvent) {
                    this.PredictionsView.SelectLastPredictionItem();
                    Ilog.Utilities.CancelEventBehaviour(keyEvt);
                    return false;
                }

                return false;
            }

            // [Tab]
            if (keyCode == Ilog.Constants.RuleEditorConstants.CharKeyCodeTabulation) {
                // validating prediction on keyup if a prediction is selected
                if (this.PredictionsView.GetCurrentSelectedPredictionValue()) {
                    Ilog.Utilities.CancelEventBehaviour(keyEvt);

                    if (eventType == Ilog.Constants.RuleEditorConstants.KeyDownEvent || eventType == Ilog.Constants.RuleEditorConstants.KeyPressEvent) {
                        return false;
                    }

                    if (eventType == Ilog.Constants.RuleEditorConstants.KeyUpEvent) {
                        var predictionObj = this.PredictionsView.GetCurrentSelectedPredictionValue();
                        if (predictionObj) {
                            var selectedPrediction = predictionObj.ValueToInsert;
                            var valueEditor = this.PredictionsView.GetCurrentSelectedPredictionValueEditor();

                            this.PredictionsView.Hide();

                            // Specific behaviour for predictions that involve a value editor
                            if (valueEditor) {
                                this.ValidateValueEditorPrediction(valueEditor);
                                return false;
                            }

                            if (selectedPrediction != null) {
                                selectedPrediction = selectedPrediction.replace(/\r\n/g, '<br/>').replace(/\n/g, '<br/>').replace(/\t/g, '&nbsp;&nbsp;&nbsp;');
                                predictionObj.ValueToInsert = selectedPrediction;
                                this.Focus();
                                this.ValidatePrediction(predictionObj, false);
                                return false;
                            }
                        }
                    }
                }
            }

            // for any other key pressed, filtering predictions on the fly
            if (eventType == Ilog.Constants.RuleEditorConstants.KeyUpEvent) {
                var bStop = false;
                if (keyCode == Ext.EventObject.DELETE || keyCode == Ext.EventObject.PAGEUP || keyCode == Ext.EventObject.PAGEDOWN || keyCode == Ext.EventObject.END || keyCode == Ext.EventObject.UP || keyCode == Ext.EventObject.RIGHT || keyCode == Ext.EventObject.DOWN || keyCode == Ext.EventObject.LEFT || keyCode == Ext.EventObject.ESC || keyCode == Ext.EventObject.HOME || keyCode == Ext.EventObject.ENTER || keyCode == 13 || keyCode == Ext.EventObject.SPACE || keyCode == Ext.EventObject.TAB) {
                    bStop = true;
                }

                // Managing predictions view
                var currentRange = this.GetTextRange();
                var OriginalCaretPosition = this.PredictionsView.OriginalCaretPosition;
                var NewCaretPosition = currentRange.start;
                var diff = NewCaretPosition - OriginalCaretPosition;

                if (diff < 0) {
                    this.PredictionsView.Hide();
                    this.SendRefreshRequest();
                    var ts = Ilog.Utilities.GetTimeStamp();
                    this.LatestGetPredictionsTimeStamp = ts;
                    this.SendGetPredictionsRequest(ts);
                    return;
                }
                else {

                    // updating replacement area
                    try {
                        this.predictionsView.ReplacementArea = this.ComputeReplacementRange(this.GetPlainText(), OriginalCaretPosition);
                        this.predictionsView.ReplacementArea.end = NewCaretPosition;
                    }
                    catch (ex) {
                        Ilog.Debug.RuleEditorDebugger.PrintDebug('[HandleKeyUp] error on compute replacement range: ' + ex.message);
                    }

                    // getting the call point of the completion menu
                    if (!bStop && this.PredictionsView.ReplacementArea != null && this.PredictionsView.ReplacementArea != undefined) {
                        var tr = { start: this.predictionsView.ReplacementArea.start, end: currentRange.start };

                        if (diff < 0) {
                            this.PredictionsView.Hide();
                        }

                        this.PredictionsView.ReplacementArea = tr;

                        // getting text between call point and current position
                        var text = '';
                        try {
                            text = this.GetTextForTextRange({ start: this.predictionsView.ReplacementArea.start, end: NewCaretPosition });
                        }
                        catch (ex) {
                            // Fix for Firefox
                            // DO NOT REMOVE
                            if (Ext.isGecko) {
                                var startNodeAndOffset = Ilog.Utilities.GetNodeAndRelativeOffsetForAbsoluteOffset(this.contentDocument, tr.start);
                                var endNodeAndOffset = Ilog.Utilities.GetNodeAndRelativeOffsetForAbsoluteOffset(this.contentDocument, tr.end);

                                // The fix consists of simply re-writing the content of the node(s)
                                if (startNodeAndOffset.node.parentNode) {
                                    startNodeAndOffset.node.parentNode.innerHTML = startNodeAndOffset.node.parentNode.innerHTML;
                                }

                                if (endNodeAndOffset.node != startNodeAndOffset.node) {
                                    if (endNodeAndOffset.node.parentNode) {
                                        endNodeAndOffset.node.parentNode.innerHTML = endNodeAndOffset.node.parentNode.innerHTML;
                                    }
                                }

                                try {
                                    text = this.GetTextForTextRange(tr);
                                }
                                catch (e) {
                                    Ilog.Debug.RuleEditorDebugger.PrintDebug('[HandleKeyUp] error on get text for text range: ' + e.message);
                                }
                            }
                            else {
                                Ilog.Debug.RuleEditorDebugger.PrintDebug('[HandleKeyUp] error #2 on get text for text range: ' + ex.message);
                            }
                        }

                        // updating filter box in prediction view
                        this.PredictionsView.UpdateFilterBoxValue(text);
                        var textRange = this.GetTextRange();
                        this.PredictionsView.CurrentCaretPosition = textRange;

                        // updating location of completion menu
                        var xy = this.GetXYAbsolutePositionOfCaret();
                        if (xy == null) {
                            return;
                        }

                        var xposition = xy.x;
                        var yposition = xy.y;

                        // Checking absolute caret position vs iframe bounds  ("blocking" position to the border of the iframe)
                        var extElm = new Ext.Element(this.editorField);
                        var iframePosition = { x: extElm.getLeft(), y: extElm.getTop(), width: extElm.getWidth(), height: extElm.getHeight() };
                        if (xposition < iframePosition.x) {
                            xposition = iframePosition;
                        }
                        if (xposition > (iframePosition.x + iframePosition.width)) {
                            xposition = iframePosition.x + iframePosition.width;
                        }
                        if (yposition < iframePosition.y) {
                            yposition = iframePosition.y;
                        }
                        if (yposition > (iframePosition.y + iframePosition.height)) {
                            yposition = iframePosition.y + iframePosition.height;
                        }

                        this.PredictionsView.Update(text, xposition, yposition, textRange);
                        return;
                    }
                    else {
                        this.PredictionsView.Hide();
                        this.SendRefreshRequest();
                    }
                }
            }

            //    return false;
            //}

            //return false;
        }
        /********************************/

        /********************************/
        // Commands : Cut/Copy/Paste/SelectAll/Undo/Redo
        /********************************/
        // "undo" command
        if (keyCode == 90 && ctrlPressed) { // 90 = Z/z
            Ilog.Utilities.CancelEventBehaviour(keyEvt);
            if (eventType == Ilog.Constants.RuleEditorConstants.KeyUpEvent) {
                this.HandleUndoCommand();
                this.MarkDocumentAsDirty();
            }
            return false;
        }

        // "redo" command
        if (keyCode == 89 && ctrlPressed) { // 89 = Y/y
            Ilog.Utilities.CancelEventBehaviour(keyEvt);
            if (eventType == Ilog.Constants.RuleEditorConstants.KeyUpEvent) {
                this.HandleRedoCommand();
                this.MarkDocumentAsDirty();
            }
            return false;
        }

        // "COPY" command
        if (keyCode == 67 && ctrlPressed) { // 67 = C/c
            return; //keyup
        }
        // "CUT" command
        if (keyCode == 88 && ctrlPressed) { // 88 = X/x
            if (eventType == Ilog.Constants.RuleEditorConstants.KeyUpEvent && !Ext.isIE) {
                this.SendRefreshRequest();
            }
            return;
        }

        // "PASTE" command
        var e = keyEvt.browserEvent;
        if ((e.ctrlKey && e.keyCode == e.DOM_VK_V) || (e.shiftKey && e.keyCode == e.DOM_VK_INSERT)) {
            Ilog.Utilities.CancelEventBehaviour(keyEvt);

            if (eventType == Ilog.Constants.RuleEditorConstants.KeyUpEvent) {
                this.HandlePasteCommand();
                this.SendRefreshRequest();
                return;
            }

            this.MarkDocumentAsDirty();
            return false;
        }

        // "SELECT ALL" command
        if (keyCode == 65 && ctrlPressed) { // 65 = A/a
            return;
        }

        /********************************/

        /********************************/
        // Special keys
        /********************************/
        // do nothing special on [CTRL]/[ALT]/[SHIFT] key
        if (keyCode == 17 || keyCode == 16 || keyCode == 18) {
            return;
        }

        // TAB management
        if (keyCode == Ilog.Constants.RuleEditorConstants.CharKeyCodeTabulation && !altPressed && !ctrlPressed && !shiftPressed) {
            Ilog.Utilities.CancelEventBehaviour(keyEvt);

            if (eventType == Ilog.Constants.RuleEditorConstants.KeyUpEvent) {
                // adding a tabulation-like
                this.InsertTextAtCaret('&nbsp;&nbsp;&nbsp;');
                this.SendRefreshRequest();
            }
            else {
                this.MarkDocumentAsDirty();
            }

            return false;
        }

        // keys that do not modify the text nor the caret
        // [End] key
        if (keyCode == Ext.EventObject.ESC || keyCode == 27) {
            this.CancelGetPredictionsRequest();
            this.CancelRefreshRequest();
            this.SendRefreshRequest();
            return;
        }

        // keys that do not modify the text but move the caret
        if (keyCode == Ext.EventObject.HOME || keyCode == keyEvt.END || keyCode == keyEvt.PAGEUP || keyCode == keyEvt.PAGEDOWN || keyCode == Ext.EventObject.UP || keyCode == Ext.EventObject.RIGHT || keyCode == Ext.EventObject.DOWN || keyCode == Ext.EventObject.LEFT) {
            this.CancelGetPredictionsRequest();
            this.CancelRefreshRequest();
            this.SendRefreshRequest();
            return;
        }

        // [Space] management
        if (keyCode == 32 && !ctrlPressed) {
            if (eventType == Ilog.Constants.RuleEditorConstants.KeyUpEvent) {
                if (Ext.isIE) {
                    this.EnsureCaretVisible(null, -1); // manually scrolling iframe if caret is not visible anymore
                }
            }
            else {
                this.MarkDocumentAsDirty();
            }
        }

        // [ENTER]
        if (keyCode == 13 || keyCode == Ext.EventObject.ENTER) {
            Ilog.Utilities.CancelEventBehaviour(keyEvt);

            if (eventType == Ilog.Constants.RuleEditorConstants.KeyUpEvent) {
                var text = this.GetPlainText();
                var pos = this.GetTextRange();
                var requirePredictions = false;
                try {
                    if (this.AutoCompletion === true && (pos.end >= text.length || text.charAt(pos.end) == ' ' || text.charAt(pos.end) == '\r' || text.charAt(pos.end) == '\n')) {
                        requirePredictions = true;
                    }
                }
                catch (ex) { }

                this.InsertLineBreakAtCaret();
                this.CancelGetPredictionsRequest();
                this.CancelRefreshRequest();
                this.SendRefreshRequest();
                if (requirePredictions === true) {
                    var ts = Ilog.Utilities.GetTimeStamp();
                    this.LatestGetPredictionsTimeStamp = ts;
                    this.SendGetPredictionsRequest(ts, null, null, null, null, text);
                }
            }

            return false;
        }

        // for Firefox, canceling default action when pressing CTRL+space on the keydown
        // because it causes the iframe to lose focus, and the keyup event is not executed
        if (eventType == Ilog.Constants.RuleEditorConstants.KeyDownEvent && Ext.isGecko && keyCode == Ilog.Constants.RuleEditorConstants.CharKeyCodeForPredictions && ctrlPressed) {
            Ilog.Utilities.CancelEventBehaviour(keyEvt);
            return false;
        }


        if (eventType == Ilog.Constants.RuleEditorConstants.KeyDownEvent) {
            this.CancelGetPredictionsRequest();

            if (!(keyCode == Ilog.Constants.RuleEditorConstants.CharKeyCodeForPredictions && this.AutoCompletion)) {
                this.CancelRefreshRequest();
            }

            this.MarkDocumentAsDirty();
        }

        var regularPredictionCondition = (keyCode == Ilog.Constants.RuleEditorConstants.CharKeyCodeForPredictions && (this.AutoCompletion === true || ctrlPressed === true));

        // Checked additional delimiters on keypress (the real character entered is only known for sure on keypress)
        var additionalPredictionCondition = false;
        if (eventType == Ilog.Constants.RuleEditorConstants.KeyPressEvent) {
            try {
                var charPressed = String.fromCharCode(keyCode) + "";
                var lowerCaseKeycode = (charPressed.toLocaleLowerCase() + "").charCodeAt(0);
                additionalPredictionCondition = this.IsAdditionalCompletionDelimiter(ctrlPressed, keyCode, lowerCaseKeycode);
            }
            catch (ex) { }
        }

        // Predictions requests
        if (regularPredictionCondition || additionalPredictionCondition) {
            // keydown
            if (eventType == Ilog.Constants.RuleEditorConstants.KeyDownEvent) {
                return;
            }
            // keypress
            else if (eventType == Ilog.Constants.RuleEditorConstants.KeyPressEvent) {
                this.CancelGetPredictionsRequest();

                // Sending prediction request immediately if CTRL was pressed (manual request)
                // but in case of CTRL+space, keypress should not be fired, so this command will be trapped in keyup as well
                if (ctrlPressed === true) {
                    var ts = Ilog.Utilities.GetTimeStamp();
                    this.LatestGetPredictionsTimeStamp = ts;
                    this.SendGetPredictionsRequest(ts, null, 0);
                }
                else {
                    this.CancelRefreshRequest();

                    var text = this.GetPlainText();
                    var ts = Ilog.Utilities.GetTimeStamp();
                    this.LatestGetPredictionsTimeStamp = ts;
                    var delay = 10 + Ilog.Utilities.GetLongRuleAdditionalDelay(text.length);
                    var self = this;
                    window.setTimeout(function() {
                        self.SendGetPredictionsRequest(ts, null, null, null, null, text);
                    }, delay);
                }

                // this.SendRefreshRequest();

                return;
            }
            // keyup
            else {
                if (!ctrlPressed) {
                    this.CancelRefreshRequest();

                    var self = this;
                    window.setTimeout(function() {
                        self.SendRefreshRequest();
                    }, 5);
                }
                else {
                    var ts = Ilog.Utilities.GetTimeStamp();
                    this.LatestGetPredictionsTimeStamp = ts;
                    this.SendGetPredictionsRequest(ts, null, 0);
                }

                return;
            }
        }

        // requesting refresh/predictions
        if (eventType == Ilog.Constants.RuleEditorConstants.KeyPressEvent || eventType == Ilog.Constants.RuleEditorConstants.KeyUpEvent) {
            this.NotifyTextBufferModified();

            // if (keyCode == Ilog.Constants.RuleEditorConstants.CharKeyCodeForPredictions && (this.AutoCompletion === true || ctrlPressed === true)) {
            //     this.SendRefreshRequest();
            // }

            /*
            // Autocompletion turned on + alternate key (ENTER) for predictions = requiring predictions in addition to refresh
            if (this.AutoCompletion === true && Ilog.Constants.RuleEditorConstants.UseAlternateCharKeyCodeForPredictions === true && keyCode == Ilog.Constants.RuleEditorConstants.AlternateCharKeyCodeForPredictions) {
            this.CancelGetPredictionsRequest();
            this.CancelRefreshRequest();
            this.SendRefreshRequest();
            var ts = Ilog.Utilities.GetTimeStamp();
            this.LatestGetPredictionsTimeStamp = ts;
            this.SendGetPredictionsRequest(ts);
            return;
            }
            */

            if (eventType == Ilog.Constants.RuleEditorConstants.KeyPressEvent) {
                this.CancelGetPredictionsRequest();
            }

            // Clear predictions request
            var self = this;
            window.setTimeout(function() {
                if (eventType == Ilog.Constants.RuleEditorConstants.KeyPressEvent) {
                    self.CancelGetPredictionsRequest();
                }
                self.SendRefreshRequest();
            }, 10);
            //            this.CancelGetPredictionsRequest();
            //            this.SendRefreshRequest();
        }


        /********************************/
    },

    AddUndoRedoEntry: function(ruleText) {
        if (this.UndoRedoStack) {
            var state = this.GetEditorState();
            var prevState = this.UndoRedoStack.GetPreviousState();

            try {
                if (ruleText) {
                    var existingText = this.UndoRedoStack.GetCurrentState();
                    if (existingText && existingText.PlainText == ruleText) {
                        return;
                    }
                }
            }
            catch (ex) { }

            if (state && prevState) {
                if (state.PlainText == prevState.PlainText) {
                    return;
                }
            }

            this.UndoRedoStack.AddState(state);
            this.ChangeUndoStatus(true);

            var followingState = this.UndoRedoStack.GetFollowingState();
            if (!followingState) {
                this.ChangeRedoStatus(false);
            }
            else {
                this.ChangeRedoStatus(true);
            }
        }
    },

    GetEditorState: function() {
        var text = this.GetHtmlText();
        var plainText = this.GetPlainText();
        var textSelection = this.GetTextRange();
        var beginningCaretPosition = textSelection.start;
        var endCaretPosition = textSelection.end;
        var editorState = new Ilog.RuleEditor.EditorState(plainText, text, beginningCaretPosition, endCaretPosition);
        return editorState;
    },

    MarkDocumentAsDirty: function() {
    },

    MarkDocumentAsUpToDate: function() {
    },

    UpdateToolbarButtons: function() {
        if (this.HasToolBar !== true) {
            if (this.ThreadToolbarButtonsUpdate) {
                window.clearInterval(this.ThreadToolbarButtonsUpdate);
            }
            return;
        }

        if (this.toolbarButtonsUpdateTaskId) {
            window.clearTimeout(this.toolbarButtonsUpdateTaskId);
        }

        var self = this;
        this.toolbarButtonsUpdateTaskId = window.setTimeout(function() {
            if (Ext.isGecko) {
                try {
                    var bOk = true;
                    var sel = self.contentWindow.getSelection();
                    if (sel == null || sel == undefined || sel == '') {
                        bOk = false;
                    }
                }
                catch (ex) {
                    bOk = false;
                }

                if (bOk === false) {
                    self.fireEvent("CutImpossible");
                    self.fireEvent("CopyImpossible");
                    return;
                }

                self.fireEvent("CutPossible");
                self.fireEvent("CopyPossible");
                return;
            }

            var range = self.editorDocument.selection.createRange().duplicate();
            if (range != null && range != undefined && range.text != null && range.text != undefined && range.text != '') {
                self.fireEvent("CutPossible");
                self.fireEvent("CopyPossible");
            }
            else {
                self.fireEvent("CutImpossible");
                self.fireEvent("CopyImpossible");
            }
        }, Ilog.Constants.RuleEditorConstants.ToolbarUpdateCheckDelay);
    },

    HandleKeyDown: function(keyEvt) {
        var bwEvent = keyEvt.browserEvent;
        return this.AnalyzeKeyEvent(bwEvent, Ilog.Constants.RuleEditorConstants.KeyDownEvent);
    },


    CancelRefreshRequest: function() {
        var timeoutId = this.OngoingRefreshRequestDelayedActionIdentifier;
        if (timeoutId != null) {
            window.clearTimeout(timeoutId);

            if (this.OngoingRefreshRequestObject && this.OngoingRefreshRequestObjectTimestamp)
                this.fireEvent("RefreshCancelled");

            try {
                if (this.OngoingRefreshRequestObject.Request && this.OngoingRefreshRequestObject.Request.abort) {
                    this.OngoingRefreshRequestObject.Request.abort();
                }
            }
            catch (ex) {
                Ilog.Debug.RuleEditorDebugger.PrintDebug('[CancelRefreshRequest] error on abort request: ' + ex.message);
            }

            this.OngoingRefreshRequestObject = null;
            this.OngoingRefreshRequestDelayedActionIdentifier = null;
        }

        this.OngoingRefreshRequestTimeStamp = null;
        this.OngoingRefreshRequestContext = null;
    },

    CancelGetPredictionsRequest: function() {
        if (Ilog.RuleEditor.CallbackManagement.GetLastPredictionsRequestSentForEditor(this.editorFieldId) != null) {
            window.clearTimeout(Ilog.RuleEditor.CallbackManagement.GetLastPredictionsRequestSentForEditor(this.editorFieldId));
        }

        if (this.OngoingPredictionsRequestDelayedActionIdentifier != null) {
            window.clearTimeout(this.OngoingPredictionsRequestDelayedActionIdentifier);
        }

        var timeoutId = this.OngoingPredictionsRequestDelayedActionIdentifier;
        if (timeoutId != null) {
            this.fireEvent("CompletionCancelled");
            window.clearTimeout(timeoutId);

            try {
                if (this.OngoingPredictionsRequestObject && this.OngoingPredictionsRequestObject.Request && this.OngoingPredictionsRequestObject.Request.abort) {
                    this.OngoingPredictionsRequestObject.Request.abort();
                }
            }
            catch (ex) {
                Ilog.Debug.RuleEditorDebugger.PrintDebug('[CancelGetPredictionsRequest] error on abort request: ' + ex.message);
            }

            this.OngoingPredictionsRequestObject = null;
            this.OngoingPredictionsRequestDelayedActionIdentifier = null;
        }

        if (this.OngoingPredictionsRequestObject != null && this.OngoingPredictionsRequestObject.Request && this.OngoingPredictionsRequestObject.Request.abort) {
            this.OngoingPredictionsRequestObject.Request.abort();
        }

        Ilog.RuleEditor.CallbackManagement.SetLastPredictionsRequestTimeStampForEditor(this.editorFieldId, '');
    },

    SendGetPredictionsRequest: function(reqTimestamp, caretPositionForPlaceholder, reqDelay, isPlaceholder, placeholderConcept, ruleTextValue) {
        if (isPlaceholder == null || isPlaceholder == undefined) {
            isPlaceholder = false;
        }

        var timestamp = Ilog.Utilities.GetTimeStamp();
        this.CompletionMenuRequested = timestamp;
        var editorIdentifier = this.editorFieldId;
        var directAjaxCall = true;
        if (this.IsAspNetCallbackHandlerAvailable === true && this.IsAjaxRetryAllowed() === false) {
            directAjaxCall = false;
        }

        var replacementRange = null;
        var ruleText = this.GetPlainText();

        if (this.OngoingPredictionsRequestDelayedActionIdentifier != null) {
            window.clearTimeout(this.OngoingPredictionsRequestDelayedActionIdentifier);
        }

        var ruleEditorInstance = this;
        var self = this;

        var delay = reqDelay;
        if (delay == null || delay == undefined) {
            delay = Ilog.Constants.RuleEditorConstants.CallbackPredictionsDelay + Ilog.Utilities.GetLongRuleAdditionalDelay(ruleText.length);
        }

        this.OngoingPredictionsRequestDelayedActionIdentifier = window.setTimeout(
            function IlogRuleEditorCallbackManagementSendPredictionsRequest() {
                //alert(self.LatestGetPredictionsTimeStamp + ' -- ' + timestamp);
                //                if (self.LatestGetPredictionsTimeStamp != timestamp) {
                //                    return;
                //                }
                if (self.LatestGetPredictionsTimeStamp != reqTimestamp) {
                    alert(self.LatestGetPredictionsTimeStamp + ' -- ' + reqTimestamp);
                    return;
                }

                // Looking for separator
                // [replacement area]
                if (caretPositionForPlaceholder && caretPositionForPlaceholder == null || caretPositionForPlaceholder == undefined) {
                    var globalPos = self.GetTextRange();
                    var keepTextRangeSelected = false;

                    // if a text range is selected, using it as is for requesting and replacing predictions
                    if (globalPos.start != globalPos.end) {
                        keepTextRangeSelected = { start: globalPos.start, end: globalPos.end };
                    }

                    // otherwise, if we have a single caret, 2 possibilities
                    // 1. it is placed just after a separator
                    // 2. it is placed inside "a word" (before and after the caret, there is at least 1 non-separator character)
                    var singleCaretPosition = globalPos.start;

                    if (singleCaretPosition > ruleText.length) { // happens in IE
                        var pos = singleCaretPosition;
                        if (ruleText.length >= 1) { pos = ruleText.length - 1; }
                        replacementRange = { start: pos, end: pos };
                    }
                    else {
                        replacementRange = self.ComputeReplacementRange(ruleText, singleCaretPosition);
                    }
                }
                else {
                    replacementRange = caretPositionForPlaceholder;
                }

                if (replacementRange == null) {
                    replacementRange = globalPos;
                }

                var xmlToPost = Ilog.RuleEditor.CallbackManagement.GetCallbackArgumentForPredictionsRequest(self, timestamp, replacementRange.start, self.DtEditionContext, self.DtFieldId, self.DataCacheEnabled, self.DtSessionId, self.DtDescriptor, placeholderConcept);
                var serviceUrl = self.ServiceUrl;

                Ilog.RuleEditor.CallbackManagement.SetLastPredictionsRequestTimeStampForEditor(editorIdentifier, timestamp);

                self.OngoingPredictionsRequestTimeStamp = timestamp;

                if (directAjaxCall) {
                    if (serviceUrl != null && serviceUrl != '') {
                        var objForRequest = Ilog.RuleEditor.CallbackManagement.GetEditorRequestWithDirectAjaxCall(serviceUrl, xmlToPost, Ilog.RuleEditor.CallbackManagement.OnGetPredictionsSuccessfulResponseReceived, self, Ilog.RuleEditor.CallbackManagement.OnGetPredictionsDirectAjaxErrorResponseReceived, timestamp, replacementRange, isPlaceholder, keepTextRangeSelected);

                        if (objForRequest != null) {
                            Ilog.RuleEditor.CallbackManagement.SetLastDirectAjaxGetPredictionsRequest(editorIdentifier, objForRequest);
                        }

                        self.OngoingPredictionsRequestObject = objForRequest;
                    }
                }


                if (directAjaxCall === false) {
                    if (ruleEditorInstance.isEmbeddedInDT === true) {
                        ruleEditorInstance.fireEvent("RequestCallback", ruleEditorInstance, xmlToPost);
                        ruleEditorInstance.fireEvent("CompletionMenuRequestSent");
                    }
                    else {
                        if (ruleEditorInstance.SendEditorGetPredictionsRequestWithAspNetCallback) {
                            ruleEditorInstance.fireEvent("CompletionMenuRequestSent");
                            ruleEditorInstance.SendEditorGetPredictionsRequestWithAspNetCallback(xmlToPost);
                        }
                    }
                }
                else {
                    if (objForRequest != null) {
                        ruleEditorInstance.fireEvent("CompletionMenuRequestSent");
                        Ilog.RuleEditor.CallbackManagement.SendEditorRequestWithDirectAjaxCall(objForRequest.Request, objForRequest.Xml, objForRequest.ServiceUrl, Ilog.Constants.RuleEditorConstants.CallbackStringRequestTypeGetPredictions);

                        window.setTimeout(function() {
                            if (objForRequest && objForRequest.Request && objForRequest.Request.ResponseReceived != true && !objForRequest.Request._aborted) {
                                ruleEditorInstance.fireEvent("CompletionTimeout");
                                objForRequest.Request.onreadystatechange = null;
                                objForRequest.Request.abort();
                                objForRequest = null;

                                ruleEditorInstance.OngoingPredictionsRequestDelayedActionIdentifier = null;
                            }
                        }, Ilog.Constants.RuleEditorConstants.XmlHttpRequestTimeout);
                    }
                }
            }
            , delay);

        Ilog.RuleEditor.CallbackManagement.SetLastPredictionsRequestSentForEditor(editorIdentifier, this.OngoingPredictionsRequestDelayedActionIdentifier); //editorIdentifier
    },

    ComputeReplacementRange: function(ruleText, singleCaretPosition) {
        var replacementRange = null;

        // 1. it is placed just after a separator
        // we consider that if caret is at 0 it is placed next to a separator
        var placedNextToSeparator = false;
        if (singleCaretPosition == 0) {
            placedNextToSeparator = true;
            replacementRange = { start: singleCaretPosition, end: singleCaretPosition };
        }
        else {
            var wordSeparator = Ilog.Constants.RuleEditorConstants.CharForWordSeparator + "";
            var wordSeparatorCode = Ilog.Constants.RuleEditorConstants.KeycodeForWordSeparator;
            var lineSeparator = Ilog.Constants.RuleEditorConstants.CharForLineSeparator + "";
            var lineSeparatorCode = Ilog.Constants.RuleEditorConstants.KeycodeForLineSeparator;
            var semicolonSeparator = Ilog.Constants.RuleEditorConstants.CharForSemicolonSeparator + "";
            var semicolonSeparatorCode = Ilog.Constants.RuleEditorConstants.KeycodeForSemicolonSeparator;

            try {
                var previousChar = ruleText.charAt(singleCaretPosition - 1);
                var previousCharCode = ruleText.charCodeAt(singleCaretPosition - 1);
            }
            catch (ex) {
                Ilog.Debug.RuleEditorDebugger.PrintDebug('[ComputeReplacementRange] error on get previous char: ' + ex.message);
            }

            if (previousChar == wordSeparator || previousCharCode == wordSeparatorCode || previousChar == lineSeparator || previousCharCode == lineSeparatorCode) {
                placedNextToSeparator = true;
                replacementRange = { start: singleCaretPosition, end: singleCaretPosition };
            }
            else {
                // the caret is not placed after a separator, 
                // so it is placed inside a word (including at the end of this word)
                // so we now look at the bounds of this word
                var lowerBound = -1;
                var upperBound = -1;

                // Searching bounds of the word
                // lower
                for (var p = singleCaretPosition - 1; p >= 0; p--) {
                    try {
                        var charToCompare = ruleText.charAt(p);
                        var charCodeToCompare = ruleText.charCodeAt(p);

                        if (charToCompare == wordSeparator || charCodeToCompare == wordSeparatorCode || charToCompare == lineSeparator || charCodeToCompare == lineSeparatorCode) {
                            lowerBound = p + 1;
                            break;
                        }
                    }
                    catch (ex) {
                        Ilog.Debug.RuleEditorDebugger.PrintDebug('[ComputeReplacementRange] error on get word lower bound: ' + ex.message);
                    }
                }
                // upper
                for (var p = singleCaretPosition; p < ruleText.length; p++) {
                    try {
                        var charToCompare = ruleText.charAt(p);
                        var charCodeToCompare = ruleText.charCodeAt(p);
                        if (charToCompare == wordSeparator || charCodeToCompare == wordSeparatorCode || charToCompare == lineSeparator || charCodeToCompare == lineSeparatorCode || charToCompare == semicolonSeparator || charCodeToCompare == semicolonSeparatorCode) {
                            upperBound = p;
                            break;
                        }
                    }
                    catch (ex) {
                        Ilog.Debug.RuleEditorDebugger.PrintDebug('[ComputeReplacementRange] error on get word upper bound: ' + ex.message);
                    }
                }

                if (lowerBound == -1) {
                    lowerBound = 0;
                }
                if (upperBound == -1) {
                    upperBound = ruleText.length - 1;
                }

                if (lowerBound > upperBound) {
                    var tmp = lowerBound;
                    lowerBound = upperBound;
                    upperBound = tmp;
                }

                if (singleCaretPosition > upperBound) {
                    upperBound = singleCaretPosition;
                }

                replacementRange = { start: lowerBound, end: upperBound };
            }
        }

        return replacementRange;
    },


    PrepareAspNetCallback: function(jsonRequest) {
        var obj = Ext.util.JSON.decode(jsonRequest);

        if (obj.RequestType == Ilog.Constants.RuleEditorConstants.CallbackStringRequestTypeRefreshEditor) {
            this.OngoingRefreshRequestTimeStamp = obj.TimeStamp;

            var ruleText = this.GetPlainText();
            this.LastRuleTextRequestSent = ruleText;

            if (Ext.isIE) {
                ruleText = ruleText.replace(/&nbsp;&nbsp;&nbsp;/g, "\t");
                ruleText = ruleText.replace(/   /g, "\t");
            }

            if (this.OngoingRefreshRequestDelayedActionIdentifier != null) {
                window.clearTimeout(this.OngoingRefreshRequestDelayedActionIdentifier);
            }
        }
        else if (obj.RequestType == Ilog.Constants.RuleEditorConstants.CallbackStringRequestTypeGetPredictions) {
            Ilog.RuleEditor.CallbackManagement.SetLastPredictionsRequestTimeStampForEditor(this.editorFieldId, obj.TimeStamp); // editorIdentifier
            this.OngoingPredictionsRequestTimeStamp = obj.TimeStamp;

            if (this.OngoingPredictionsRequestDelayedActionIdentifier != null) {
                window.clearTimeout(this.OngoingPredictionsRequestDelayedActionIdentifier);
            }
        }
    },

    SendValueEditorRequest: function(valueEditorKey) {
        var timestamp = Ilog.Utilities.GetTimeStamp();
        var editorIdentifier = this.editorFieldId;
        var directAjaxCall = true;

        var customArgs = new Array();
        if (document.getElementById(this.customFieldReference) != null) {
            var ruleGuid = new Array(this.customFieldKey, document.getElementById(this.customFieldReference).value);
            customArgs.push(ruleGuid);
        }
    },

    NotifyTextBufferModified: function() {
        this.LastRuleTextRequestSent = null;
    },

    ResumeDone: function(state) {
        if (state) {
            this.RestoreState(state);
            this.SendRefreshRequest();

            // sending completion menu request if it the request that triggered the DT cache miss
            if (state.ResendGetPredictions === true) {
                var currentText = this.GetPlainText();
                if (state.PlainText == currentText) {
                    var ts = Ilog.Utilities.GetTimeStamp();
                    this.LatestGetPredictionsTimeStamp = ts;
                    this.SendGetPredictionsRequest(ts);
                }
            }
        }
    },

    ReportDirectAjaxError: function() {
        if (!this.AjaxRetryCount) {
            this.AjaxRetryCount = 1;
        }
        else {
            this.AjaxRetryCount++;
        }
    },

    IsAjaxRetryAllowed: function() {
        if (!(this.IsAspNetCallbackHandlerAvailable === true)) {
            return false;
        }

        if (!this.AjaxRetryCount) {
            this.AjaxRetryCount = 0;
        }

        return (this.AjaxRetryCount < Ilog.Constants.RuleEditorConstants.AjaxRetryCountBeforeCallbackHandler) ? true : false;
    },

    SendRefreshRequest: function(contextInfo, doNotAddUndoRedoEntry) {
        if (this.stopRefresh === true) {
            return;
        }

        var timestamp = Ilog.Utilities.GetTimeStamp();
        var editorIdentifier = this.editorFieldId;
        var directAjaxCall = true;

        if (this.IsAspNetCallbackHandlerAvailable === true && this.IsAjaxRetryAllowed() === false) {
            directAjaxCall = false;
        }

        var customArgs = new Array();
        if (document.getElementById(this.customFieldReference) != null) {
            var ruleGuid = new Array(this.customFieldKey, document.getElementById(this.customFieldReference).value);
            customArgs.push(ruleGuid);
        }

        var ruleText = this.GetPlainText();

        // Checking whether we should actually send the request :
        // except for firefox 2 (to prevent focus issues), 
        // sending the request if it has not already been sent and replied with a successful response
        var checkReponseReceived = !(Ilog.Utilities.IsFirefox2() === true);
        if (this.LastRuleTextRequestSent == ruleText) {
            if (!checkReponseReceived || (this.RuleTextUpdated && this.RuleTextUpdated[ruleText] === true)) {
                return;
            }
        }

        // at this point, 
        // the request will actually be sent
        this.LastRuleTextRequestSent = ruleText;
        this.RuleTextSent = new Object();
        this.RuleTextSent[timestamp] = ruleText;

        this.fireEvent("TextChanged", { text: ruleText });

        if (Ext.isIE) {
            ruleText = ruleText.replace(/&nbsp;&nbsp;&nbsp;/g, "\t");
            ruleText = ruleText.replace(/   /g, "\t");
        }

        var obj = new Ilog.RuleEditor.CallbackManagement.RefreshRequest(timestamp, editorIdentifier, customArgs, ruleText);

        // setting EditorClientInstanceReference only when not embedded in DT
        // warning : when embedded in DT, item stored is an oject, which cannot be used by Ext.util.JSON.encode
        if (Ilog.Utilities.RuleEditorStore && !(this.isEmbeddedInDT === true)) {
            var ref = Ilog.Utilities.RuleEditorStore.GetItem(editorIdentifier);
            if (ref && typeof (ref) == 'string') {
                obj.EditorClientInstanceReference = ref;
            }
        }

        if (this.DtEditionContext != null && this.DtEditionContext != undefined && this.DtEditionContext > 0) {
            obj.DtEditionContext = this.DtEditionContext;
            if (this.DtFieldId) {
                obj.DtFieldId = this.DtFieldId;
            }
            if (this.DtSessionId) {
                obj.DtSessionId = this.DtSessionId;
            }
        }

        if (this.FrozenRegions) {
            obj.FrozenRegions = this.FrozenRegions;
        }

        if (this.DtDescriptor) {
            obj.DtDescriptor = this.DtDescriptor;
        }

        if (this.DataCacheEnabled) {
            obj.DataCacheEnabled = this.DataCacheEnabled;
        }

        var encoded = "{}";
        try {
            encoded = Ext.util.JSON.encode(obj);
        }
        catch (ex) {
            Ilog.Debug.RuleEditorDebugger.PrintDebug('[SendRefreshRequest] JSON encoding failed : ' + ex.message);
            return;
        }

        var serviceUrl = this.ServiceUrl;
        this.OngoingRefreshRequestTimeStamp = timestamp;

        if (contextInfo != null && contextInfo != undefined && contextInfo.reason == Ilog.Constants.RuleEditorConstants.ContextValidatePrediction) {
            this.OngoingRefreshRequestContext = contextInfo;
        }
        else {
            this.OngoingRefreshRequestContext = null;
        }

        if (directAjaxCall) {
            if (serviceUrl != null && serviceUrl != '') {
                var objForRequest = Ilog.RuleEditor.CallbackManagement.GetEditorRequestWithDirectAjaxCall(serviceUrl, encoded, Ilog.RuleEditor.CallbackManagement.OnEditorRefreshSuccessfulResponseReceived, this, Ilog.RuleEditor.CallbackManagement.OnEditorRefreshDirectAjaxErrorResponseReceived, timestamp);
                if (objForRequest != null) {
                    this.OngoingRefreshRequestObject = objForRequest;
                }
            }
        }

        if (this.OngoingRefreshRequestDelayedActionIdentifier != null) {
            window.clearTimeout(this.OngoingRefreshRequestDelayedActionIdentifier);
        }

        var ruleEditorInstance = this;

        var requestDelay = Ilog.Constants.RuleEditorConstants.CallbackRefreshContentTextDelay;

        if (Ext.isIE) {
            requestDelay += Ilog.Utilities.GetLongRuleAdditionalDelay(ruleText.length);
        }
        this.OngoingRefreshRequestDelayedActionIdentifier = window.setTimeout(
            function IlogRuleEditorCallbackManagementSendRefreshRequest() {
                if (doNotAddUndoRedoEntry == null || doNotAddUndoRedoEntry == undefined || !(doNotAddUndoRedoEntry === true)) {
                    window.setTimeout(function() {
                        ruleEditorInstance.AddUndoRedoEntry(ruleText);
                    }, 20);
                }
                ruleEditorInstance.SendEditorSaveRequest();

                if (!directAjaxCall) {
                    if (ruleEditorInstance.isEmbeddedInDT === true) {
                        var xmlCallbackArgument = encoded;
                        ruleEditorInstance.fireEvent("RequestCallback", ruleEditorInstance, xmlCallbackArgument);
                        ruleEditorInstance.fireEvent("RefreshRequestSent");
                    }
                    else {
                        if (ruleEditorInstance.SendEditorRefreshRequestWithAspNetCallback) {
                            var xmlCallbackArgument = encoded;
                            ruleEditorInstance.fireEvent("RefreshRequestSent");
                            ruleEditorInstance.SendEditorRefreshRequestWithAspNetCallback(xmlCallbackArgument);
                        }
                    }
                }
                else {
                    if (objForRequest != null) {
                        ruleEditorInstance.fireEvent("RefreshRequestSent");

                        Ilog.RuleEditor.CallbackManagement.SendEditorRequestWithDirectAjaxCall(objForRequest.Request, objForRequest.Xml, objForRequest.ServiceUrl, Ilog.Constants.RuleEditorConstants.CallbackStringRequestTypeRefreshEditor);
                        window.setTimeout(function() {
                            if (objForRequest && objForRequest.Request && objForRequest.Request.ResponseReceived != true && !objForRequest.Request._aborted) {
                                ruleEditorInstance.fireEvent("RefreshTimeout");
                                objForRequest.Request.onreadystatechange = null;
                                objForRequest.Request.abort();
                                objForRequest = null;
                                ruleEditorInstance.OngoingRefreshRequestDelayedActionIdentifier = null;
                            }

                        }, Ilog.Constants.RuleEditorConstants.XmlHttpRequestTimeout);
                    }
                }
            }
            , requestDelay);
    },

    CallbackResponse: function(jsonResponse) {
        var obj = Ext.util.JSON.decode(jsonResponse);
        if (obj.RequestType + '' == Ilog.Constants.RuleEditorConstants.CallbackStringRequestTypeRefreshEditor + '') {
            Ilog.RuleEditor.CallbackManagement.OnEditorRefreshSuccessfulResponseReceived(jsonResponse, this, this, null);
        }
        else if (obj.RequestType + '' == Ilog.Constants.RuleEditorConstants.CallbackStringRequestTypeGetPredictions + '') {
            Ilog.RuleEditor.CallbackManagement.OnGetPredictionsSuccessfulResponseReceived(jsonResponse, this, this, null);
        }
    },

    SendEditorSaveRequest: function() {
        // do not save when embedded into DT
        if (this.isEmbeddedInDT === true) {
            return;
        }

        var timestamp = Ilog.Utilities.GetTimeStamp();
        var editorIdentifier = this.editorFieldId;
        var directAjaxCall = true;

        if (this.IsAspNetCallbackHandlerAvailable) {
            directAjaxCall = false;
        }

        var obj = Ilog.RuleEditor.CallbackManagement.GetCallbackArgumentForSaveRequest(this, timestamp);
        var xmlToPost = obj.argument;
        var serviceUrl;
        if (this.SpecificSaveRequestsServiceUrl != null && this.SpecificSaveRequestsServiceUrl != undefined && this.SpecificSaveRequestsServiceUrl != '') {
            serviceUrl = this.SpecificSaveRequestsServiceUrl;
        }
        else {
            serviceUrl = this.ServiceUrl;
        }

        if (obj.text == this.LastRequestedSavedText) {
            return;
        }

        this.LastSaveRequestTimestamp = timestamp;
        this.LastRequestedSavedText = obj.text;

        // if using .NET, using the callback handler function to perform save request
        if (this.SendEditorSaveRequestWithAspNetCallback) {
            this.SendEditorSaveRequestWithAspNetCallback(xmlToPost);
        }
        else {
            // if Java, sending a regular ajax request
            var customArgs = new Array();
            if (document.getElementById(this.customFieldReference) != null) {
                var ruleGuid = new Array(this.customFieldKey, document.getElementById(this.customFieldReference).value);
                customArgs.push(ruleGuid);
            }

            var ruleText = this.GetPlainText();
            var obj = new Ilog.RuleEditor.CallbackManagement.SaveRequest(timestamp, editorIdentifier, customArgs, ruleText);
            if (Ilog.Utilities.RuleEditorStore && Ilog.Utilities.RuleEditorStore.GetItem[editorIdentifier] != null && Ilog.Utilities.RuleEditorStore.GetItem[editorIdentifier] != undefined) {
                obj.EditorClientInstanceReference = Ilog.Utilities.RuleEditorStore.GetItem[editorIdentifier];
            }
            var encoded = Ext.util.JSON.encode(obj);
            var objForRequest = Ilog.RuleEditor.CallbackManagement.GetEditorRequestWithDirectAjaxCall(serviceUrl, encoded, Ilog.RuleEditor.CallbackManagement.OnSaveSuccessfulResponseReceived, this, Ilog.RuleEditor.CallbackManagement.OnSaveErrorResponseReceived, timestamp);
            Ilog.RuleEditor.CallbackManagement.SendEditorRequestWithDirectAjaxCall(objForRequest.Request, objForRequest.Xml, objForRequest.ServiceUrl, Ilog.Constants.RuleEditorConstants.CallbackStringRequestTypeSaveRule);
        }

        this.fireEvent("SaveRequestSent");

        return timestamp;
    },

    // function which is be called when the editor is embedded in DT,
    // when it needs to be unloaded
    Dispose: function() {
        this.ClearResources();
    },

    HandlePasteGecko: function(evt) {

        if (evt) {
            var stdEvent = evt.browserEvent;
            Ilog.Utilities.CancelEventBehaviour(evt);
            Ilog.Utilities.StopEventPropagation(evt);
        }

        try {
            netscape.security.PrivilegeManager.enablePrivilege('UniversalXPConnect');
        }
        catch (ex) {
            Ilog.Utilities.ShowClipboardErrorWindow(this);
        }

        try {
            var clip = Components.classes["@mozilla.org/widget/clipboard;1"].getService(Components.interfaces.nsIClipboard);
            var trans = Components.classes["@mozilla.org/widget/transferable;1"].createInstance(Components.interfaces.nsITransferable);
            trans.addDataFlavor("text/unicode");
            clip.getData(trans, clip.kGlobalClipboard);

            var str = new Object();
            var strLength = new Object();
            trans.getTransferData("text/unicode", str, strLength);

            if (str) str = str.value.QueryInterface(Components.interfaces.nsISupportsString);
            if (str) pastetext = str.data.substring(0, strLength.value / 2);

            var textToPaste = Ext.util.Format.htmlEncode(pastetext);

            // fixing line breaks for non-IE browsers
            if (!Ext.isIE) {
                var newText = "";
                for (var p = 0; p < textToPaste.length; p++) {
                    var charcode = textToPaste.charCodeAt(p);
                    var charTest = textToPaste.charAt(p);

                    if (charcode == 10 || charcode == 13 || charcode == Ext.EventObject.ENTER) {
                        newText += '<br/>';
                    }
                    else {
                        newText += charTest;
                    }
                }

                textToPaste = newText;
                pastetext = textToPaste;
            }

            try {
                if (Ilog.Constants.RuleEditorConstants.MaxSizeOnPaste && pastetext.length > Ilog.Constants.RuleEditorConstants.MaxSizeOnPaste) {
                    pastetext = pastetext.substring(0, Ilog.Constants.RuleEditorConstants.MaxSizeOnPaste);
                }
            }
            catch (ex) { }

            this.InsertTextAtCaret(pastetext);
            this.CancelGetPredictionsRequest();
            this.SendRefreshRequest();
            return;
        }
        catch (ex) {
            Ilog.Debug.RuleEditorDebugger.PrintDebug('[HandlePasteGecko] error: ' + ex.message);
        }

        // if the manual plain text copy from clipboard could not be completed, 
        // letting normal behaviour happen (built-in copy)
        return;
    },

    // used to managed browser/system commands
    HandleCut: function(evt) {
        // checking if the user is not trying to cut out a frozen region
        var frozenRegionSelected = this.CheckTextRangeContainsFrozenRegions();
        if (frozenRegionSelected === true) {
            Ilog.Utilities.CancelEventBehaviour(evt);
            return false;
        }

        var self = this;
        window.setTimeout(function() {
            self.CancelGetPredictionsRequest();
            self.SendRefreshRequest();
        }, 50);
    },

    // used to managed browser/system commands
    HandleCopy: function(evt) {
    },

    HandlePaste: function(evt) {
        var stdEvent = evt.browserEvent;
        Ilog.Utilities.StopEventPropagation(evt);

        // checking if the user is not trying to paste inside a frozen region
        if (this.FrozenZones && this.FrozenZones.length > 0) {
            var doable = this.CheckDropOrPasteTargetAgainstFrozenZones();
            if (doable === false) {
                Ilog.Utilities.CancelEventBehaviour(evt);
                return false;
            }
        }

        try {
            // Get clipboard content :
            var textData = window.clipboardData.getData("Text");

            // Getting instance of rule editor
            var ruleEditorInstance = this;
            if (!ruleEditorInstance) {
                return;
            }

            if (textData == null || textData == undefined || textData == '') {
                // return; // we are supposed to stop, but actually if clipboard programmatic access has been disabled, paste will occur but textdata stays blank
            }

            Ilog.Utilities.StopEventPropagation(evt);
            Ilog.Utilities.CancelEventBehaviour(evt);

            // textdata is blank :
            // either the clipboard is empty, or its access has been denied.
            if (textData != null && textData != undefined && textData != '') {
                // erasing text range if not empty
                ruleEditorInstance.DeleteCurrentTextRange();

                // Pasting text
                ruleEditorInstance.Focus();

                // limitation on data size to be pasted
                var textToPaste = textData;

                if (Ilog.Constants.RuleEditorConstants.MaxSizeOnPaste && textData.length > Ilog.Constants.RuleEditorConstants.MaxSizeOnPaste) {
                    textToPaste = textData.substring(0, Ilog.Constants.RuleEditorConstants.MaxSizeOnPaste);
                }

                textToPaste = Ext.util.Format.htmlEncode(textToPaste);
                textToPaste = textToPaste.replace(/\n/g, '<br/>');
                textToPaste = textToPaste.replace(/ /g, '&nbsp;');
                ruleEditorInstance.InsertTextAtCaret("<span>" + textToPaste + "</span>");
            }

            // Setting back focus
            // Give a little time before setting focus(), otherwise it doesn't work properly
            window.setTimeout(function() {
                // Setting focus to the edition area
                ruleEditorInstance.Focus();
                ruleEditorInstance.CancelGetPredictionsRequest();
                ruleEditorInstance.SendRefreshRequest();
            }, 10);

            return false;
        }
        catch (ex) {
            Ilog.Debug.RuleEditorDebugger.PrintDebug('[HandlePaste] error: ' + ex.message);

            window.setTimeout(function() {
                // Setting focus to the edition area
                ruleEditorInstance.Focus();
                ruleEditorInstance.CancelGetPredictionsRequest();
                ruleEditorInstance.SendRefreshRequest();
            }
            , 1000);
        }
    },

    CheckTextRangeContainsFrozenRegions: function() {
        if (this.FrozenZones && this.FrozenZones.length > 0) {
            var textRange = this.GetTextRange();

            for (var p = 0; p < this.FrozenZones.length; p++) {
                var frozenZone = this.FrozenZones[p];
                // current text selection has a part inside a frozen zone :
                // canceling everything
                if ((textRange.start >= frozenZone.start && textRange.start <= frozenZone.end) || (textRange.end >= frozenZone.start && (textRange.end + 1) <= frozenZone.end)) {
                    return true;
                }
            }
        }

        return false;
    },

    HideAllTooltips: function() {
        if (this.editorDocument == null || this.editorDocument == undefined) {
            return;
        }

        var spanNodes = this.editorDocument.getElementsByTagName("span");
        for (var p = 0; p < spanNodes.length; p++) {
            var spanNode = spanNodes[p];
            if (spanNode.tooltip) {
                spanNode.tooltip.Hide();
                spanNode.tooltip = null;
            }
        }
    },

    // For FF
    // Cleaning HTML document after node is inserted (drag'n drop or paste case)
    CleanDocument: function() {
    },

    // OnDrop Event Handler
    // IE-specific
    HandleDrop: function(evt) {
        if (!Ext.isIE) {
            return;
        }

        var transfertMode = "copy";
        try {
            transfertMode = evt.browserEvent.dataTransfer.dropEffect.toString();
        }
        catch (ex) {
            Ilog.Debug.RuleEditorDebugger.PrintDebug('[HandleDrop] error on get drop effect: ' + ex.message);
        }

        var dragAndDropInternal = true;
        try {
            var test = evt.browserEvent.dataTransfer.effectAllowed;
        }
        catch (ex) {
            dragAndDropInternal = false; // when exception is met, it means that the source is external to the browser
        }

        // checking if the user is not trying to paste inside a frozen region
        // or to drag a part of a frozen region somewhere else
        if (this.FrozenZones && this.FrozenZones.length > 0) {
            var doable = this.CheckDropOrPasteTargetAgainstFrozenZones();
            if (doable === false) {
                Ilog.Utilities.CancelEventBehaviour(evt);
                return false;
            }
        }

        var self = this;

        // if drag and drop is internal (in the editor itself or in the web page), just letting the normal behaviour happen
        if (dragAndDropInternal == true) {
            if (transfertMode == "none") {
                // Give a little time before setting focus(), otherwise it doesn't work properly
                window.setTimeout(function() {
                    // Setting focus to the edition area
                    self.Focus();
                    self.CancelGetPredictionsRequest();
                    self.SendRefreshRequest();
                }, 15);

                return;
            }
        }

        // if we are in "copy" mode or dropping something from an external source,
        // just canceling the regular behaviour
        Ilog.Utilities.CancelEventBehaviour(evt);

        // and then filtering the data which is dropped
        var textData = null;
        var urlData = null;

        if (evt.browserEvent) {
            evt = evt.browserEvent;
        }

        if (evt != null && evt.dataTransfer) {
            textData = evt.dataTransfer.getData("Text");
            urlData = evt.dataTransfer.getData("URL");
        }

        Ilog.Utilities.StopEventPropagation(evt);

        if (textData != null) {
            // limitation on data size to be pasted
            var textToPaste = textData;
            if (Ilog.Constants.RuleEditorConstants.MaxSizeOnPaste && textData.length > Ilog.Constants.RuleEditorConstants.MaxSizeOnPaste) {
                textToPaste = textData.substring(0, Ilog.Constants.RuleEditorConstants.MaxSizeOnPaste);
            }

            self.Focus();
            self.InsertTextAtCaret("<span>" + textToPaste + "</span>");

            // Give a little time before setting focus(), otherwise it doesn't work properly
            window.setTimeout(function() {
                // Setting focus to the edition area
                self.Focus();
                self.CancelGetPredictionsRequest();
                self.SendRefreshRequest();
            }, 15);
        }

        return false;
    },


    /*******************************************************/
    /* function that handles a 'click' event in the editor */
    /*******************************************************/
    HandleClick: function(evt) {
        Ilog.Utilities.TestTimeoutSession();

        var ruleEditorInstance = this;

        // closing pop-ups (completion menu, date picker) of potential other editors on page  (DT mode)
        Ilog.Utilities.CleanEditorResources();

        if (evt.getTarget) {
            var targetNode = evt.getTarget();
            if (targetNode && Ilog.Utilities.IsNodeFrozen(targetNode)) {
                Ilog.Utilities.CancelEventBehaviour(evt);
                Ilog.Utilities.StopEventPropagation(evt);
                return false;
            }
        }

        if (ruleEditorInstance == null) {
            return;
        }

        this.HideValueEditor();

        var editorIdentifier = ruleEditorInstance.editorFieldId;

        if (ruleEditorInstance.PredictionsView) {
            ruleEditorInstance.PredictionsView.Hide();
        }

        if (this.SelectedPlaceholder != null) {
            this.SelectedPlaceholder = null;
            this.SelectedPlaceHolderIndex = -1;
        }

        if (this.DatePickerLayer) {
            this.DatePickerLayer.parentNode.removeChild(this.DatePickerLayer);
            this.DatePickerLayer = null;
        }

        if (this.toolbarButtonsUpdateTaskId) {
            window.clearTimeout(this.toolbarButtonsUpdateTaskId);
        }

        // sending refresh is useful because the text may have changed :
        // ex : dynamic filtering of completion menu and then click elsewhere -> need a refresh
        // anyway the refresh will actually be sent only if text has changed since last refresh
        ruleEditorInstance.SendRefreshRequest();

        return;
    },

    HandleUndoCommand: function() {
        Ilog.Utilities.TestTimeoutSession();

        if (this.IsUndoPossible() === false) {
            return;
        }

        if (this.UndoRedoStack != null) {
            var previousState = this.UndoRedoStack.GetPreviousState();

            if (previousState != null) {
                this.CancelRefreshRequest();
                this.CancelGetPredictionsRequest();
                this.Tooltips = null;
                var restoreStatus = this.RestoreState(previousState);
                if (restoreStatus == true) {
                    this.UndoRedoStack.ValidatePreviousState();
                    this.SendRefreshRequest(null, true);

                    // Checking next undo
                    if (this.UndoRedoStack.GetPreviousState()) {
                        this.ChangeUndoStatus(true);
                    }
                    else {
                        this.ChangeUndoStatus(false);
                    }

                    // Checking redo
                    var followingState = this.UndoRedoStack.GetFollowingState();
                    if (followingState != null) {
                        this.ChangeRedoStatus(true);
                    }
                    else {
                        this.ChangeRedoStatus(false);
                    }
                }
            }
            else {
                this.ChangeUndoStatus(false);
                this.Focus();
            }
        }
        else {
            this.Focus();
        }
    },

    ChangeUndoStatus: function(isPossible) {
        this.isUndoPossible = isPossible;

        if (isPossible === true) {
            this.fireEvent("UndoPossible");
        }
        else {
            this.fireEvent("UndoImpossible");
        }
    },

    ChangeRedoStatus: function(isPossible) {
        this.isRedoPossible = isPossible;

        if (isPossible === true) {
            this.fireEvent("RedoPossible");
        }
        else {
            this.fireEvent("RedoImpossible");
        }
    },

    IsUndoPossible: function() {
        return (this.isUndoPossible === true);
    },

    IsRedoPossible: function() {
        return (this.isRedoPossible === true);
    },

    HandleRedoCommand: function() {
        Ilog.Utilities.TestTimeoutSession();

        if (this.IsRedoPossible() === false) {
            return;
        }

        if (this.editorDocument) {
            if (this.UndoRedoStack != null) {
                var followingState = this.UndoRedoStack.GetFollowingState();

                if (followingState != null) {
                    this.CancelRefreshRequest();
                    this.CancelGetPredictionsRequest();
                    this.Tooltips = null;
                    var restoreStatus = this.RestoreState(followingState);

                    if (restoreStatus == true) {
                        this.UndoRedoStack.ValidateFollowingState();
                        this.SendRefreshRequest(null, true);

                        // Checking next undo
                        if (this.UndoRedoStack.GetPreviousState()) {
                            this.ChangeUndoStatus(true);
                        }
                        else {
                            this.ChangeUndoStatus(false);
                        }

                        // Checking redo
                        var followingState = this.UndoRedoStack.GetFollowingState();
                        if (followingState != null) {
                            this.ChangeRedoStatus(true);
                        }
                        else {
                            this.ChangeRedoStatus(false);
                        }
                    }
                }
                else {
                    this.ChangeRedoStatus(false);
                }
            }
        }
    },

    HandleCutCommand: function() {
        Ilog.Utilities.TestTimeoutSession();

        // checking if the user is not trying to cut out a frozen region
        var frozenRegionSelected = this.CheckTextRangeContainsFrozenRegions();
        if (frozenRegionSelected === true) {
            return false;
        }

        var mustRefresh = false;

        if (this.editorDocument && this.editorDocument.selection) {
            this.editorDocument.execCommand("Cut");

            // Adding entry in undo/redo stack after a delay
            if (this.UndoRedoTimeout != null) {
                window.clearTimeout(this.UndoRedoTimeout);
            }
            var ruleEditorInstance = this;
            this.UndoRedoTimeout = window.setTimeout(function() { ruleEditorInstance.AddUndoRedoEntry(); ruleEditorInstance.SendEditorSaveRequest(); }, Ilog.Constants.RuleEditorConstants.UndoRedoSaveDelay);

            mustRefresh = true;
            this.Focus();
        }
        else if (this.contentDocument && !Ext.isIE) {
            try {
                this.contentDocument.execCommand("Cut", false, null);

                // Adding entry in undo/redo stack after a delay
                if (this.UndoRedoTimeout != null) {
                    window.clearTimeout(this.UndoRedoTimeout);
                }
                var ruleEditorInstance = this;
                this.UndoRedoTimeout = window.setTimeout(function() { ruleEditorInstance.AddUndoRedoEntry(); ruleEditorInstance.SendEditorSaveRequest(); }, Ilog.Constants.RuleEditorConstants.UndoRedoSaveDelay);
                mustRefresh = true;
                this.Focus();
            }
            catch (ex) {
                Ilog.Debug.RuleEditorDebugger.PrintDebug('[HandleCutCommand] unauthorized clipboard access: ' + ex.message);

                // Unauthorized access to the clipboard
                if (ex.name.toUpperCase() == 'NS_ERROR_DOM_XPCONNECT_ACCESS_DENIED') {
                    Ilog.Utilities.ShowClipboardErrorWindow(this);
                }
            }
        }

        // Sending refresh request if needed
        if (mustRefresh) {
            this.CancelGetPredictionsRequest();
            this.CancelRefreshRequest();
            this.SendRefreshRequest();
        }
    },

    HandleCopyCommand: function() {
        Ilog.Utilities.TestTimeoutSession();

        // Place current selection (if any) in the copy buffer AS PLAIN TEXT
        // IE
        if (this.editorDocument && this.editorDocument.selection) {
            this.editorDocument.execCommand("Copy");
        }
        else if (this.contentDocument && !Ext.isIE) {
            try {
                this.contentDocument.execCommand("Copy", false, null);
                this.Focus();
            }
            catch (ex) {
                Ilog.Debug.RuleEditorDebugger.PrintDebug('[HandleCopyCommand] unauthorized clipboard access: ' + ex.message);
                // Unauthorized access to the clipboard
                if (ex.name.toUpperCase() == 'NS_ERROR_DOM_XPCONNECT_ACCESS_DENIED') {
                    Ilog.Utilities.ShowClipboardErrorWindow(this);
                }
            }
        }
    },

    HandleSelectAllCommand: function() {
        Ilog.Utilities.TestTimeoutSession();

        // IE
        if (this.editorDocument && this.editorDocument.selection) {
            this.editorDocument.execCommand("SelectAll");
        }
        else if (this.contentDocument && !Ext.isIE) {
            this.contentDocument.execCommand("SelectAll", false, null);
            this.Focus();
        }
    },

    HandlePasteCommand: function() {
        Ilog.Utilities.TestTimeoutSession();

        // IE
        if (this.editorDocument && this.editorDocument.selection) {
            this.editorDocument.execCommand("Paste");

            // Adding entry in undo/redo stack after a delay
            if (this.UndoRedoTimeout != null) {
                window.clearTimeout(this.UndoRedoTimeout);
            }
            var ruleEditorInstance = this;
            this.UndoRedoTimeout = window.setTimeout(function() { ruleEditorInstance.AddUndoRedoEntry(); }, Ilog.Constants.RuleEditorConstants.UndoRedoSaveDelay);
        }
        else if (this.contentDocument && !Ext.isIE) {
            try {
                if (Ext.isGecko) {
                    this.HandlePasteGecko();
                }
                else {
                    this.contentDocument.execCommand("Paste", false, null);
                }
            }
            catch (ex) {
                Ilog.Debug.RuleEditorDebugger.PrintDebug('[HandlePasteCommand] unauthorized clipboard access: ' + ex.message);
                // Unauthorized access to the clipboard
                if (ex.name.toUpperCase() == 'NS_ERROR_DOM_XPCONNECT_ACCESS_DENIED') {
                    Ilog.Utilities.ShowClipboardErrorWindow(this);
                }
            }
        }

        // no need to send refresh (at least for IE) because the "onpaste" event will fire and the associated handler will send the request.
    },

    ShowWaitingMessage: function() {
        var extElm = new Ext.Element(this.editorField);
        var iframePosition = { x: extElm.getLeft(), y: extElm.getTop(), width: extElm.getWidth(), height: extElm.getHeight() };

        var waitingLayer = new Ext.Window({
            modal: false,
            cls: 'ilog-ruleEditor-ClipboardErrorLayer',
            plain: true,
            layout: 'fit',
            pageX: iframePosition.x - 2,
            pageY: iframePosition.y - 2,
            width: iframePosition.width + 2,
            height: iframePosition.height + 2,
            resizable: false,
            draggable: false,
            closable: false,
            html: '<div style="position:absolute;text-align:center;top:40%;width:100%;background-image:background-image:url(\'' + Ilog.Constants.RuleEditorConstants.UrlWaitingLayerPicture + '\');background-position:center;background-repeat:no-repeat;">' + Ilog.Constants.RuleEditorConstants.WaitMessage + '<br/><img src="' + Ilog.Constants.RuleEditorConstants.UrlWaitingLayerPicture + '" style="vertical-align:middle;" /></div>'
        });

        waitingLayer.show();
        this.WaitingLayer = waitingLayer;
    },

    HideWaitingMessage: function() {
        if (this.WaitingLayer) {
            this.WaitingLayer.close();
        }

        this.Focus();
    },

    HandleClearAllCommand: function() {
        Ilog.Utilities.TestTimeoutSession();

        this.ClearAllText();
    },

    HandleSaveCommand: function() {
        return this.SendEditorSaveRequest();
    },

    RestoreState: function(editorStateToRestore) {
        if (editorStateToRestore == undefined || editorStateToRestore == null) {
            return;
        }

        var text = editorStateToRestore.Text;
        var caretBeginning = editorStateToRestore.TextSelectionStart;
        var caretEnd = editorStateToRestore.TextSelectionEnd;
        var caret = { start: caretBeginning, end: caretEnd };

        if (this.PredictionsView && this.PredictionsView.Visible) {
            this.PredictionsView.Hide();
        }

        this.SetHtmlText(text, false);

        if (caretBeginning == caretEnd) {
            this.Focus();
            this.SelectTextRange(caret);
        }

        return true;
    },

    ValidateValueEditorPrediction: function(valueEditor) {
        var caretPosition = this.PredictionsView.ReplacementArea;
        this.ManageValueEditor(valueEditor, caretPosition);
    },

    ValidatePrediction: function(prediction, shouldMoveCaret) {
        var predictionValue = prediction.ValueToInsert + '';

        // DT : removing insertion of line breaks and replacing tabs by single space
        if (this.isEmbeddedInDT === true && !(this.IsMultiline === true)) {
            predictionValue = predictionValue.replace(/<br\/>/g, '').replace(/<BR\/>/g, '');
            predictionValue = predictionValue.replace(/&nbsp;&nbsp;&nbsp;/g, '&nbsp;');
            predictionValue = predictionValue.replace(/   /g, ' ');
        }

        var editorInstance = this;
        var ruleEditorInstance = this;
        var scrollPos = { x: this.GetScrollXPosition(), y: this.GetScrollYPosition() };
        var caretPosition = this.PredictionsView.ReplacementArea;
        var endPos = ruleEditorInstance.PredictionsView.ReplacementArea.start + predictionValue.length;
        if (Ext.isGecko) {
            endPos = ruleEditorInstance.PredictionsView.ReplacementArea.start + Ext.util.Format.htmlDecode(predictionValue.toString()).length;
        }

        var contextInfo = { reason: Ilog.Constants.RuleEditorConstants.ContextValidatePrediction, value: caretPosition, caretPositionAfterPrediction: endPos };
        editorInstance.contentWindow.focus();

        if (shouldMoveCaret) {
        }

        // FF
        if (Ext.isGecko) {
            // Placeholder
            if (caretPosition.startNode != null && caretPosition.startNode.className && caretPosition.startNode.className.toUpperCase().indexOf(Ilog.Constants.RuleEditorConstants.PatternPlaceHolderRecognition.toUpperCase()) >= 0) {
                editorInstance.Focus();
                caretPosition.startNode.innerHTML = predictionValue;
                caretPosition.startNode.className = "";
                editorInstance.MoveCaretToEndOfNode(caretPosition.startNode);
            }
            // regular replacement (no placeholder)
            else {
                var textRangeInfo = null;
                if (this.PredictionsView.ReplacementArea != null && this.PredictionsView.ReplacementArea != undefined) {
                    var currentRange = this.GetTextRange();
                    var tr = { start: ruleEditorInstance.PredictionsView.ReplacementArea.start, end: ruleEditorInstance.PredictionsView.ReplacementArea.end };
                    textRangeInfo = this.SelectTextRange(tr);
                }

                editorInstance.InsertTextAtCaret(predictionValue, true);
            }
        }
        // IE
        else if (document.selection) {
            if (this.PredictionsView.ReplacementArea != null && this.PredictionsView.ReplacementArea != undefined) {
                var tr = this.PredictionsView.ReplacementArea;
                if (tr.start == tr.end) {
                    var range = this.GetTextRange();
                    if (range.start == tr.start && range.end != tr.end && !(this.PredictionsView.ReplacementArea.NoFix === true)) {
                        tr.end = range.end;
                    }
                }
                this.SelectTextRange(tr);
            }
            editorInstance.InsertTextAtCaret(predictionValue, true);

            //setting back scroll position when needed
            if (scrollPos.x > 0 || scrollPos.y > 0) {
                this.SetScrollXYPosition(scrollPos.x, scrollPos.y);
            }
        }

        // if the prediction includes a selection to be done after paste
        if (prediction.SelectionOffset && prediction.SelectionLength) {

            try {
                var startPos = ruleEditorInstance.PredictionsView.ReplacementArea.start; // caretPosition.start
                var self = this;
                window.setTimeout(function() { self.SelectTextRange({ start: startPos + prediction.SelectionOffset, end: startPos + prediction.SelectionOffset + prediction.SelectionLength }); }, 15);
            }
            catch (ex) {
                Ilog.Debug.RuleEditorDebugger.PrintDebug('[ValidatePrediction] error on select range: ' + ex.message);
            }
        }
        // Firefox : moving caret according to the prediction's length  (on the contrary of IE, in FF the caret stays at the insertion point)
        else if (window.getSelection) {
            var plainPredictionValue = predictionValue.replace(/<br\/>/g, '\n').replace(/&nbsp;/g, ' ');
            var endPositionToMoveTo = ruleEditorInstance.PredictionsView.ReplacementArea.start + Ext.util.Format.htmlDecode(plainPredictionValue).length;
            editorInstance.Focus();
            editorInstance.MoveCaretToPosition(endPositionToMoveTo);
        }

        this.EnsureCaretVisible(null, null, scrollPos.x, scrollPos.y, prediction.TextToDisplay);
        if (Ext.isIE) {
            this.Focus();
        }

        this.NotifyTextBufferModified();

        editorInstance.CancelGetPredictionsRequest();
        editorInstance.CancelRefreshRequest();
        editorInstance.SendRefreshRequest(contextInfo);
    },

    HandleDropGecko: function(evt) {
        Ilog.Utilities.TestTimeoutSession();

        var contentWindow = evt.currentTarget;
        if (contentWindow == null || contentWindow == undefined || !contentWindow.mydata) {
            return;
        }

        var editor = contentWindow.mydata;

        if (editor == null || editor == undefined) {
            return;
        }

        // Checking permissions
        try {
            netscape.security.PrivilegeManager.enablePrivilege('UniversalXPConnect');
        }
        catch (ex) {
            // The typical error message is the following :
            // "A script from "http://host:port" was denied UniversalXPConnect privileges."
            Ilog.Utilities.StopEventPropagation(evt);
            Ilog.Utilities.CancelEventBehaviour(evt);

            Ext.Msg.show({
                cls: 'ilog-ruleEditor-DragDropErrorWindow',
                msg: Ilog.Constants.RuleEditorConstants.MozillaDragAndDropError,
                buttons: Ext.Msg.OK,
                animEl: editor.editorFieldId,
                icon: Ext.MessageBox.ERROR
            });

            return false;
        }

        var dragService = Components.classes["@mozilla.org/widget/dragservice;1"].getService(Components.interfaces.nsIDragService);
        var dragSession = dragService.getCurrentSession();

        // If the drag'n drop is happening inside the document
        if (dragSession.sourceDocument && dragSession.sourceDocument == editor.contentDocument) {
            if (editor.FrozenZones && editor.FrozenZones.length > 0) {
                if (evt.stopPropagation) {
                    evt.stopPropagation();
                    Ilog.Utilities.CancelEventBehaviour(evt);
                }

                // stopping propagation of event
                Ilog.Utilities.StopEventPropagation(evt);
                // stoppping event from built-in behaviour
                Ilog.Utilities.CancelEventBehaviour(evt);
                return false;
            }

            // letting normal behaviour happen,
            // just sending refresh 
            window.setTimeout(function() {
                editor.CancelGetPredictionsRequest();
                editor.SendRefreshRequest();
            }, 50);
            return;
        }
        // external drop
        else {
            if (editor.FrozenZones && editor.FrozenZones.length > 0) {
                if (evt.stopPropagation) {
                    evt.stopPropagation();
                    Ilog.Utilities.CancelEventBehaviour(evt);
                }

                // stopping propagation of event
                Ilog.Utilities.StopEventPropagation(evt);
                // stoppping event from built-in behaviour
                Ilog.Utilities.CancelEventBehaviour(evt);
                return false;
            }

            if (evt.stopPropagation) {
                evt.stopPropagation();
                Ilog.Utilities.CancelEventBehaviour(evt);
            }

            // stopping propagation of event
            Ilog.Utilities.StopEventPropagation(evt);
            // stoppping event from built-in behaviour
            Ilog.Utilities.CancelEventBehaviour(evt);
        }

        var transferObject = Components.classes["@mozilla.org/widget/transferable;1"].createInstance(Components.interfaces.nsITransferable);
        transferObject.addDataFlavor("text/unicode");
        var numItems = dragSession.numDropItems; // Get the number of items currently being dropped in this drag/drop operation.  

        var dataToPaste = "";
        for (var i = 0; i < numItems; i++) {

            try {
                // getting the data for the given drag item from the drag session into the transfer object    
                dragSession.getData(transferObject, i);

                var dataObj = new Object();
                var dropSizeObj = new Object();

                transferObject.getTransferData("text/unicode", dataObj, dropSizeObj);
                var stringToTransfer = dataObj.value.QueryInterface(Components.interfaces.nsISupportsString);
                stringToTransfer = stringToTransfer.toString() + '';

                var xDropPos = evt.clientX + editor.GetScrollXPosition();
                var yDropPos = evt.clientY + editor.GetScrollYPosition();

                var targetNodeFound = false;
                var targetNodeSearched;
                var xToUse;
                var yToUse;

                var pasteAfterNode = false;
                var nodeToPasteAfter;

                var lowestYlimit = -1;
                var highestYlimit = -1;
                for (var h = 0; h < editor.contentDocument.body.childNodes.length && targetNodeFound === false; h++) {
                    var extNode = new Ext.Element(editor.contentDocument.body.childNodes[h]);
                    var nodeX = extNode.dom.offsetLeft;
                    var nodeLength = extNode.getWidth();
                    var nodeY = extNode.dom.offsetTop;
                    var nodeHeight = extNode.getHeight();

                    var nodeOK = (extNode.dom.textContent != '');
                    if (nodeOK) {
                        if (lowestYlimit == -1 || (lowestYlimit >= 0 && nodeY < lowestYlimit)) {
                            lowestYlimit = nodeY;
                        }

                        if (highestYlimit == -1 || (highestYlimit >= 0 && (nodeY + nodeHeight) > highestYlimit)) {
                            highestYlimit = nodeY + nodeHeight;
                        }
                    }

                    if (xDropPos >= nodeX && xDropPos <= (nodeX + nodeLength) && yDropPos >= nodeY && yDropPos <= (nodeY + nodeHeight)) {
                        targetNodeSearched = extNode.dom;
                        targetNodeFound = true;
                        xToUse = xDropPos;
                        yToUse = yDropPos;
                    }
                }

                if (targetNodeFound === false) {
                    if (lowestYlimit > -1 && yDropPos < lowestYlimit) {
                        var maxX = -1;
                        var nodeLowFound = false;
                        var currentPotentialNode;
                        var nodeLow;
                        // looking for a node containing the X of the drop position, otherwise we take the last node on the right for this Y
                        for (var h = 0; h < editor.contentDocument.body.childNodes.length && nodeLowFound === false; h++) {
                            var extNode = new Ext.Element(editor.contentDocument.body.childNodes[h]);
                            var nodeX = extNode.dom.offsetLeft;
                            var nodeLength = extNode.getWidth();
                            var nodeY = extNode.dom.offsetTop;
                            var nodeHeight = extNode.getHeight();

                            if (nodeY == lowestYlimit) {
                                if (xDropPos >= nodeX && xDropPos <= (nodeX + nodeLength)) {
                                    nodeLowFound = true;
                                    nodeLow = extNode.dom;
                                }
                                else if (maxX == -1 || (nodeX + nodeLength) > maxX) {
                                    maxX = nodeX + nodeLength;
                                    currentPotentialNode = extNode.dom;
                                }
                            }
                        }

                        if (nodeLowFound) {
                            pasteAfterNode = false;
                            targetNodeFound = true;
                            targetNodeSearched = nodeLow;
                            xToUse = xDropPos;
                            yToUse = lowestYlimit;
                        }
                        else if (currentPotentialNode != null && currentPotentialNode != undefined) {
                            pasteAfterNode = true;
                            nodeToPasteAfter = currentPotentialNode;
                        }
                    }
                    else if (highestYlimit > -1 && yDropPos > highestYlimit) {
                        var maxX = -1;
                        var nodeHighFound = false;
                        var currentPotentialNode;
                        var nodeHigh;
                        // looking for a node containing the X of the drop position, otherwise we take the last node on the right for this Y
                        for (var h = 0; h < editor.contentDocument.body.childNodes.length && nodeHighFound === false; h++) {
                            var extNode = new Ext.Element(editor.contentDocument.body.childNodes[h]);
                            var nodeX = extNode.dom.offsetLeft;
                            var nodeLength = extNode.getWidth();
                            var nodeY = extNode.dom.offsetTop;
                            var nodeHeight = extNode.getHeight();

                            if ((nodeY + nodeHeight) == highestYlimit) {
                                if (xDropPos >= nodeX && xDropPos <= (nodeX + nodeLength)) {
                                    nodeHighFound = true;
                                    nodeHigh = extNode.dom;
                                }
                                else if (maxX == -1 || (nodeX + nodeLength) > maxX) {
                                    maxX = nodeX + nodeLength;
                                    currentPotentialNode = extNode.dom;
                                }
                            }
                        }

                        if (nodeHighFound) {
                            pasteAfterNode = false;
                            targetNodeFound = true;
                            targetNodeSearched = nodeHigh;
                            xToUse = xDropPos;
                            yToUse = highestYlimit;
                        }
                        else if (currentPotentialNode != null && currentPotentialNode != undefined) {
                            pasteAfterNode = true;
                            nodeToPasteAfter = currentPotentialNode;
                        }
                    }
                    else {
                        var maxX = -1;
                        var currentPotentialNode;
                        var nodeMiddle;
                        // looking for a node containing the X of the drop position, otherwise we take the last node on the right for this Y
                        for (var h = 0; h < editor.contentDocument.body.childNodes.length; h++) {
                            var extNode = new Ext.Element(editor.contentDocument.body.childNodes[h]);
                            var nodeX = extNode.dom.offsetLeft;
                            var nodeLength = extNode.getWidth();
                            var nodeY = extNode.dom.offsetTop;
                            var nodeHeight = extNode.getHeight();

                            if (yDropPos >= nodeY && yDropPos <= (nodeY + nodeHeight)) {
                                if (maxX == -1 || (nodeX + nodeLength) > maxX) {
                                    maxX = nodeX + nodeLength;
                                    currentPotentialNode = extNode.dom;
                                }
                            }
                        }

                        if (currentPotentialNode != null && currentPotentialNode != undefined) {
                            pasteAfterNode = true;
                            nodeToPasteAfter = currentPotentialNode;
                        }
                    }
                }

                var targetNodeCharacterOffset = -1;

                if (targetNodeFound === true) {
                    targetNode = targetNodeSearched;
                }
                else if (pasteAfterNode === true && nodeToPasteAfter != null && nodeToPasteAfter != undefined) {
                }
                else {
                    var targetNode = evt.target;
                }

                if (targetNode && targetNode.nodeName && targetNode.nodeName.toUpperCase() != 'HTML') {
                    targetNodeCharacterOffset = editor.GetAbsoluteOffsetOfNodeBeginning(targetNode);
                }

                if (targetNodeCharacterOffset < 0 && pasteAfterNode === false) {
                    return;
                }

                editor.ShowWaitingMessage();

                window.setTimeout(function() {
                    editor.Focus();

                    if (pasteAfterNode === false) {
                        editor.SelectTextRange({ start: targetNodeCharacterOffset, end: targetNodeCharacterOffset });
                        var pos = editor.GetCaretRelativePositionInsideEditionArea();
                        var dropPointSpacePositionX = evt.clientX;

                        // converting difference into a number of chars
                        var nodeText = '';

                        if (evt.target.textContent) {
                            nodeText = evt.target.textContent;
                        }

                        var previousPos = pos.x;
                        var isSet = false;
                        var posBeforePaste;
                        var posOk;


                        for (var k = 1; k < nodeText.length && isSet == false; k++) {
                            var start = targetNodeCharacterOffset;
                            var end = targetNodeCharacterOffset + k;

                            posBeforePaste = targetNodeCharacterOffset + k;
                            editor.SelectTextRange({ start: posBeforePaste, end: posBeforePaste });

                            var currentPos = editor.GetCaretRelativePositionInsideEditionArea().x;

                            if (dropPointSpacePositionX >= previousPos && dropPointSpacePositionX <= currentPos) {
                                posOk = posBeforePaste;
                                isSet = true;
                                break;
                            }
                            else {
                                previousPos = currentPos;
                            }
                        }

                        if (!isSet) {
                            try {
                                posBeforePaste = targetNodeCharacterOffset + nodeText.length;
                                editor.SelectTextRange({ start: posBeforePaste, end: posBeforePaste });
                                var currentPos = editor.GetCaretRelativePositionInsideEditionArea().x;

                                if (dropPointSpacePositionX >= previousPos && dropPointSpacePositionX <= currentPos) {
                                    posOk = posBeforePaste;
                                    editor.SelectTextRange({ start: posOk, end: posOk });
                                }
                                else {
                                    editor.MoveCaretToBeginningOfNode(evt.target);
                                }
                            }
                            catch (ex) {
                                editor.MoveCaretToBeginningOfNode(evt.target);
                            }
                        }
                        else {
                            editor.SelectTextRange({ start: posOk, end: posOk });
                        }
                    }
                    else {
                        editor.SelectTextRange({ start: (editor.GetAbsoluteOffsetOfNodeBeginning(nodeToPasteAfter) + nodeToPasteAfter.textContent.length), end: (editor.GetAbsoluteOffsetOfNodeBeginning(nodeToPasteAfter) + nodeToPasteAfter.textContent.length) });
                    }

                    // limitation on data size to be pasted
                    var textToPaste = stringToTransfer;
                    if (Ilog.Constants.RuleEditorConstants.MaxSizeOnPaste && stringToTransfer.length > Ilog.Constants.RuleEditorConstants.MaxSizeOnPaste) {
                        textToPaste = stringToTransfer.substring(0, Ilog.Constants.RuleEditorConstants.MaxSizeOnPaste);
                    }

                    // checking if the user is not trying to paste inside a frozen region
                    if (editor.FrozenZones && editor.FrozenZones.length > 0) {
                        var doable = editor.CheckDropOrPasteTargetAgainstFrozenZones();
                        if (doable === false) {
                            Ilog.Utilities.CancelEventBehaviour(evt);
                            return false;
                        }
                    }

                    editor.InsertTextAtCaret(textToPaste);
                    window.setTimeout(function() { editor.HideWaitingMessage(); }, 1500); // setting 1.5s delay so that the window doesn't disappear just after it appeared
                }, 500);

            }
            catch (ex) {
                Ilog.Debug.RuleEditorDebugger.PrintDebug('[HandleDropGecko] error: ' + ex.message);
                window.setTimeout(function() { editor.HideWaitingMessage(); }, 1500);
            }
        } // for

        window.setTimeout(function() { editor.CancelGetPredictionsRequest(); editor.SendRefreshRequest(); }, 1500);
        return false;
    },

    HandleNodeInserted: function(extEvt) {
        Ilog.Utilities.StopEventPropagation(extEvt);
        this.CleanAfterPaste();
    },

    CleanAfterPaste: function() {
        var editor = this;
        if (this.CleanTask != null) {
            window.clearTimeout(this.CleanTask);
            this.CleanTask = null;
        }

        this.CleanTask = window.setTimeout(function() {
            editor.ShowWaitingMessage();

            var docModified = false;
            for (var k = 0; k < editor.contentDocument.body.childNodes.length; k++) {
                var tagName = editor.contentDocument.body.childNodes[k].tagName;

                if (tagName != null && tagName != undefined) {
                    if (tagName.toUpperCase() != 'SPAN' && tagName.toUpperCase() != 'BR') {
                        var textContent = editor.contentDocument.body.childNodes[k].textContent;
                        var spanReplacementNode = editor.editorDocument.createElement('span');
                        spanReplacementNode.id = 'IlogRuleEditorTempNode_' + editor.editorFieldId + '_' + Ilog.Utilities.GetTimeStamp();
                        spanReplacementNode.innerHTML = textContent;
                        editor.contentDocument.body.replaceChild(spanReplacementNode, editor.contentDocument.body.childNodes[k]);
                        docModified = true;
                    }
                }
            }
            if (docModified) {
                editor.UnregisterEventHandlers();
                editor.RegisterEventHandlers();
            }

            window.setTimeout(function() { editor.HideWaitingMessage(); }, 1500); // setting 1.5s delay so that the window doesn't disappear just after it appeared
            editor.CleanTask = null;

            editor.CancelGetPredictionsRequest();
            editor.SendRefreshRequest();
        },
        100);
    },

    RegisterEventHandlers: function() {
        var instance = this;

        // Register event handlers
        // Gecko
        if (this.contentWindow && window.addEventListener) {
            Ext.EventManager.addListener(this.contentWindow, 'keyup', this.AnalyzeKeyEvent, this);
            Ext.EventManager.addListener(this.contentWindow, 'keydown', this.AnalyzeKeyEvent, this);
            Ext.EventManager.addListener(this.contentWindow, 'keypress', this.AnalyzeKeyEvent, this);
            Ext.EventManager.addListener(this.contentWindow, 'click', this.HandleClick, this);
            Ext.EventManager.addListener(window, 'scroll', this.HandleGlobalWindowScroll, this);

            // FF3 : paste event supported
            Ext.EventManager.addListener(this.contentWindow, 'paste', this.HandlePasteGecko, this);

            // Special support of drag and drop
            this.contentWindow.captureEvents(Event.DRAGDROP);
            this.contentWindow.addEventListener('dragdrop', this.HandleDropGecko, true); // not using Ext.EventManager because it doesn't work in that case
            this.contentWindow.myflag = "editor";
            this.contentWindow.mydata = this;

            // Permanent thread that checks the need for a refresh, to overcome the issue where cut/copy/paste is not
            // detected (ex: when done using the right click contextual menu), etc...
            // only used for Firefox 2, where some events cannot be detected
            if (Ilog.Utilities.IsFirefox2() === true) {
                window.setInterval(function() { instance.SendRefreshRequest(); }, Ilog.Constants.RuleEditorConstants.GeckoIntervalForAutomaticRefreshCheck);
            }
        }
        // IE
        else if (this.editorDocument) {
            Ext.EventManager.addListener(this.editorDocument, 'keyup', this.AnalyzeKeyEvent, this);
            Ext.EventManager.addListener(this.editorDocument, 'keydown', this.AnalyzeKeyEvent, this);
            Ext.EventManager.addListener(this.editorDocument, 'keypress', this.AnalyzeKeyEvent, this);
            Ext.EventManager.addListener(this.editorDocument, 'click', this.HandleClick, this);
            Ext.EventManager.addListener(this.contentWindow, 'scroll', this.HandleEditorWindowScroll, this);
            Ext.EventManager.addListener(window, 'scroll', this.HandleGlobalWindowScroll, this);

            if (this.editorDocument.body) {
                Ext.EventManager.addListener(this.editorDocument.body, 'paste', this.HandlePaste, this);
                Ext.EventManager.addListener(this.editorDocument.body, 'cut', this.HandleCut, this);
                Ext.EventManager.addListener(this.editorDocument.body, 'copy', this.HandleCopy, this);
                Ext.EventManager.addListener(this.editorDocument.body, 'drop', this.HandleDrop, this);
            }
        }
    },

    UnregisterEventHandlers: function() {
        var instance = this;

        // UnRegister event handlers
        // Gecko
        if (this.contentWindow && window.addEventListener) {
            Ext.EventManager.removeListener(this.contentWindow, 'keyup', this.AnalyzeKeyEvent);
            Ext.EventManager.removeListener(this.contentWindow, 'keydown', this.AnalyzeKeyEvent);
            Ext.EventManager.removeListener(this.contentWindow, 'keypress', this.AnalyzeKeyEvent);
            Ext.EventManager.removeListener(this.contentWindow, 'click', this.HandleClick);
            Ext.EventManager.removeListener(this.contentWindow, 'DOMNodeInserted', this.HandleNodeInserted);
            Ext.EventManager.removeListener(this.contentWindow, 'scroll', this.HandleEditorWindowScroll);
        }
        // IE
        else if (this.editorDocument) {
            Ext.EventManager.removeListener(this.editorDocument, 'keyup', this.AnalyzeKeyEvent);
            Ext.EventManager.removeListener(this.editorDocument, 'keydown', this.AnalyzeKeyEvent);
            Ext.EventManager.removeListener(this.editorDocument, 'keypress', this.AnalyzeKeyEvent);
            Ext.EventManager.removeListener(this.editorDocument, 'click', this.HandleClick);
            Ext.EventManager.removeListener(this.contentWindow, 'scroll', this.HandleEditorWindowScroll);

            if (this.editorDocument.body) {
                Ext.EventManager.removeListener(this.editorDocument.body, 'paste', this.HandlePaste);
                Ext.EventManager.removeListener(this.editorDocument.body, 'drop', this.HandleDrop);
            }
        }
    },

    HandleBrowserWindowResize: function(evt) {
        var ruleEditorInstance = this;
        // Updating location of prediction view
        // Notify completion view for position update
        if (this.PredictionsView && this.PredictionsView.Visible) {
            var xy = ruleEditorInstance.GetXYAbsolutePositionOfCaret();
            if (xy == null) {
                return;
            }

            var xposition = xy.x;
            var yposition = xy.y;

            // Checking absolute caret position vs iframe bounds  ("blocking" position to the border of the iframe)
            var extElm = new Ext.Element(ruleEditorInstance.editorField);
            var iframePosition = { x: extElm.getLeft(), y: extElm.getTop(), width: extElm.getWidth(), height: extElm.getHeight() };
            if (xposition < iframePosition.x) {
                xposition = iframePosition;
            }
            if (xposition > (iframePosition.x + iframePosition.width)) {
                xposition = iframePosition.x + iframePosition.width;
            }
            if (yposition < iframePosition.y) {
                yposition = iframePosition.y;
            }
            if (yposition > (iframePosition.y + iframePosition.height)) {
                yposition = iframePosition.y + iframePosition.height;
            }
            this.PredictionsView.UpdatePosition(xposition, yposition);
        }
    },

    HandleGlobalWindowScroll: function(evt) {
        Ilog.Utilities.CancelEventBehaviour(evt);
        Ilog.Utilities.StopEventPropagation(evt);
        return false;
    },

    HandleEditorWindowScroll: function(evt) {
        var ruleEditorInstance = this;
        // Notify completion view for position update
        if (this.PredictionsView && this.PredictionsView.Visible) {
            var xy = ruleEditorInstance.GetXYAbsolutePositionOfCaret();
            if (xy == null) {
                return;
            }

            var xposition = xy.x;
            var yposition = xy.y;

            // Checking absolute caret position vs iframe bounds  ("blocking" position to the border of the iframe)
            var extElm = new Ext.Element(ruleEditorInstance.editorField);
            var iframePosition = { x: extElm.getLeft(), y: extElm.getTop(), width: extElm.getWidth(), height: extElm.getHeight() };
            if (xposition < iframePosition.x) {
                xposition = iframePosition;
            }
            if (xposition > (iframePosition.x + iframePosition.width)) {
                xposition = iframePosition.x + iframePosition.width;
            }
            if (yposition < iframePosition.y) {
                yposition = iframePosition.y;
            }
            if (yposition > (iframePosition.y + iframePosition.height)) {
                yposition = iframePosition.y + iframePosition.height;
            }
            this.PredictionsView.UpdatePosition(xposition, yposition);
        }
    },

    init: function(editorRef) {
        // initialization of fields references
        // (DOM nodes references to be stored by the editor for immediate access)
        this.InitFields();

        if (Ilog.Utilities.RuleEditorStore) {
            if (!editorRef) {
                Ilog.Utilities.RuleEditorStore.AddItem(this.editorFieldId, this);
            }
            else {
                Ilog.Utilities.RuleEditorStore.AddItem(this.editorFieldId, editorRef);
            }
        }

        if (!(this.canPerformInit === true)) {
            return;
        }

        if (Ilog.Constants.RuleEditorConstants.UseMouseUpGlobalEventToManageFocusOut === true) {
            try {
                window.captureEvents(Event.MOUSEUP);
            }
            catch (ex) { }

            Ext.onReady(Ilog.Utilities.StartMouseUpThread, null);
        }

        try {
            this.addEvents({
                "RefreshRequestSent": true,
                "RefreshDoneSuccessfully": true,
                "RefreshTimeout": true,
                "RefreshInError": true,
                "RefreshCancelled": true,
                "CompletionMenuRequestSent": true,
                "CompletionDoneSuccessfully": true,
                "CompletionTimeout": true,
                "CompletionInError": true,
                "CompletionCancelled": true,
                "SaveRequestSent": true,
                "SaveCompletedSuccessfully": true,
                "SaveInError": true,
                "SaveTimeout": true,
                "TextChanged": true,
                "UndoPossible": true,
                "UndoImpossible": true,
                "RedoPossible": true,
                "RedoImpossible": true,
                "CutPossible": true,
                "CutImpossible": true,
                "CopyPossible": true,
                "CopyImpossible": true,
                "EditionValidate": true, // when pressing [enter] while in DT mode
                "EditionCancel": true, // when pressing [escape] while in DT mode
                "CacheMiss": true
            });
        }
        catch (ex) { Ilog.Debug.RuleEditorDebugger.PrintDebug('[init] error on add events: ' + ex.message); }

        // Activating design mode
        if (this.IsReadOnly === true) {
            this.editorDocument.designMode = 'Off';
            this.SetHtmlText(this.initialText, true, true);
            return true;
        }
        else {
            this.editorDocument.designMode = 'On';
        }

        // Setting initial text
        this.editorDocument.designMode = 'On';
        this.SetHtmlText(this.initialText, true, true);

        this.RuleTextUpdated = new Object();
        var plainText = this.GetPlainText();
        this.RuleTextUpdated[plainText] = true;
        this.LastRuleTextRequestSent = plainText;

        var self = this;
        window.setTimeout(function() {
            self.LastRuleTextRequestSent = self.GetPlainText();
        }, 200);

        // Creating undo/redo stack
        var editorInitialState = new Ilog.RuleEditor.EditorState(plainText, this.initialText, 0, 0);
        this.UndoRedoStack = new Ilog.RuleEditor.UndoRedoStack(Ilog.Constants.RuleEditorConstants.UndoRedoStackSize);
        this.UndoRedoStack.AddState(editorInitialState);
        this.RegisterEventHandlers();

        if (!(Ilog.Constants.RuleEditorConstants.EnableToolbarButtonsUpdate === false)) {
            // starting thread to check for cut&copy availability
            this.ThreadToolbarButtonsUpdate = window.setInterval(function() { self.UpdateToolbarButtons() }, Ilog.Constants.RuleEditorConstants.ToolbarUpdatePollInterval);
        }

        // Subscribing to browser window resize event
        Ext.EventManager.addListener(window, "resize", this.HandleBrowserWindowResize, this);

        // Setting focus to the edition area    
        if (this.isEmbeddedInDT === false || !this.isEmbeddedInDT) {
            // FF
            if (window.getSelection || document.getSelection) {
                var editor = this;
                window.setTimeout(function() {
                    self.Focus();
                    self.SelectTextRange({ start: 0, end: 0 });
                }, 200);
            }
            //IE
            else {
                window.setTimeout(function() {
                    var ifr = document.getElementById(self.editorFieldId);
                    ifr.focus();
                    ifr.contentWindow.focus();
                }, 200);
            }
        }

        this.LastRuleTextRequestSent = this.GetPlainText();
        this.InitFrozenZones();
    },

    InitFrozenZones: function() {
        var frozenZones = new Array();

        var self = this;
        window.setTimeout(function() {
            var bodyNode = self.GetDocumentBodyNode();
            // selecting frozen nodes
            var frozenNodes = Ext.DomQuery.select("span[class*=" + Ilog.Constants.RuleEditorConstants.PatternFrozenZoneRecognition + "]", bodyNode);

            var frozenRanges = new Array();

            for (var k = 0; k < frozenNodes.length; k++) {
                var frozenNode = frozenNodes[k];

                Ext.EventManager.addListener(frozenNode, "dragstart", function(e) { Ilog.Utilities.StopEventPropagation(e); Ilog.Utilities.CancelEventBehaviour(e); return false; }, self);
                var range = { start: Ilog.Utilities.GetNodeStartOffset(bodyNode, frozenNode), end: Ilog.Utilities.GetNodeEndOffset(bodyNode, frozenNode) };
                frozenZones.push(range);

                if (Ilog.Utilities.IsFirefox2() === true) {
                    frozenNode.style.MozUserFocus = "ignore";
                    frozenNode.style.MozUserInput = "none";
                    frozenNode.style.MozUserModify = "read-only";
                    frozenNode.style.MozUserSelect = "none";
                    frozenNode.style.cursor = "default";
                }

                Ext.EventManager.addListener(frozenNode, "dragdrop", function(e) { Ilog.Utilities.StopEventPropagation(e); Ilog.Utilities.CancelEventBehaviour(e); return false; }, this);
                Ext.EventManager.addListener(frozenNode, "ondragdrop", function(e) { Ilog.Utilities.StopEventPropagation(e); Ilog.Utilities.CancelEventBehaviour(e); return false; }, this);

                Ext.EventManager.addListener(frozenNode, "cut", function(e) { Ilog.Utilities.StopEventPropagation(e); Ilog.Utilities.CancelEventBehaviour(e); return false; }, this);
                Ext.EventManager.addListener(frozenNode, "oncut", function(e) { Ilog.Utilities.StopEventPropagation(e); Ilog.Utilities.CancelEventBehaviour(e); return false; }, this);

                Ext.EventManager.addListener(frozenNode, "paste", function(e) { Ilog.Utilities.StopEventPropagation(e); Ilog.Utilities.CancelEventBehaviour(e); return false; }, this);
                Ext.EventManager.addListener(frozenNode, "onpaste", function(e) { Ilog.Utilities.StopEventPropagation(e); Ilog.Utilities.CancelEventBehaviour(e); return false; }, this);

                Ext.EventManager.addListener(frozenNode, "dragover", function(e) { Ilog.Utilities.StopEventPropagation(e); Ilog.Utilities.CancelEventBehaviour(e); return false; }, this);
                Ext.EventManager.addListener(frozenNode, "dragenter", function(e) { Ilog.Utilities.StopEventPropagation(e); Ilog.Utilities.CancelEventBehaviour(e); return false; }, this);

                Ext.EventManager.addListener(frozenNode, "ondragover", function(e) { Ilog.Utilities.StopEventPropagation(e); Ilog.Utilities.CancelEventBehaviour(e); return false; }, this);
                Ext.EventManager.addListener(frozenNode, "ondragenter", function(e) { Ilog.Utilities.StopEventPropagation(e); Ilog.Utilities.CancelEventBehaviour(e); return false; }, this);


                if (Ilog.Utilities.IsFirefox2() === true) {
                    var scope = { editor: self, node: frozenNode, text: frozenNode.textContent };
                    Ext.EventManager.addListener(frozenNode, "DOMCharacterDataModified", self.HandleFrozenNodeDataModified, scope);
                    frozenRanges.push({ textRange: { start: range.start, end: range.end }, text: frozenNode.textContent, id: frozenNode.id });
                }
            }
            self.FrozenZones = frozenZones;
            self.FrozenRanges = frozenRanges; // FF2-specific
        }, 30);
    },

    // only called with FF2, where frozen node can modified in some cases
    HandleFrozenNodeDataModified: function(e) {
        var scope = this;
        scope.editor.stopRefresh = true;

        try {
            var bodyInnerHTML = scope.editor.contentDocument.body.innerHTML;
            window.setTimeout(function() { try { scope.editor.SetHtmlText(bodyInnerHTML, false); scope.editor.InitFrozenZones(); } catch (ex) { Ilog.Debug.RuleEditorDebugger.PrintDebug('[HandleFrozenNodeDataModified] error on InitFrozenZones: ' + ex.message); } }, 30);
        }
        catch (ex) {
            Ilog.Debug.RuleEditorDebugger.PrintDebug('[HandleFrozenNodeDataModified] error: ' + ex.message);
        }

        scope.editor.stopRefresh = false;
    },

    GetDocumentBodyNode: function() {
        if (this.contentDocument && !Ext.isIE) {
            return this.contentDocument.body;
        }
        else {
            return this.editorDocument.body;
        }
    },

    IsAdditionalCompletionDelimiter: function(isCtrlPressed, charCode, lowerCaseCharCode) {
        // Whitespaces
        if (this.SeparatorWhitespaces && (this.SeparatorWhitespaces.indexOf(String.fromCharCode(charCode)) >= 0 || this.SeparatorWhitespaces.indexOf(String.fromCharCode(lowerCaseCharCode)) >= 0)) {
            return true;
        }

        if (this.SeparatorWhitespacesKeycodes) {
            for (var k = 0; k < this.SeparatorWhitespacesKeycodes.length; k++) {
                if (this.SeparatorWhitespacesKeycodes[k] == charCode || this.SeparatorWhitespacesKeycodes[k] == lowerCaseCharCode) {
                    return true;
                }
            }
        }

        // Ideographs
        if (this.SeparatorListSingleIdeographsString && (this.SeparatorListSingleIdeographsString.indexOf(String.fromCharCode(charCode)) >= 0 || this.SeparatorListSingleIdeographsString.indexOf(String.fromCharCode(lowerCaseCharCode)) >= 0)) {
            return true;
        }

        if (this.SeparatorListSingleIdeographsKeycodes) {
            for (var k = 0; k < this.SeparatorListSingleIdeographsKeycodes.length; k++) {
                if (this.SeparatorListSingleIdeographsKeycodes[k] == charCode || this.SeparatorListSingleIdeographsKeycodes[k] == lowerCaseCharCode) {
                    return true;
                }
            }
        }

        if (this.SeparatorListIdeographsRangesKeycodes) {
            for (var k = 0; k < this.SeparatorListIdeographsRangesKeycodes.length; k++) {
                var range = this.SeparatorListIdeographsRangesKeycodes[k];
                if ((charCode >= range[0] && charCode <= range[1]) || (lowerCaseCharCode >= range[0] && lowerCaseCharCode <= range[1])) {
                    return true;
                }
            }
        }

        return false;
    },

    ValueEditorValidate: function(newValue) {
        var valueEditorContext = this.CurrentValueEditorDisplayed;

        if (this.CurrentValueEditorDisplayed) {
            // closing editor
            try {
                if (this.CurrentValueEditorDisplayed.window) {
                    this.CurrentValueEditorDisplayed.window.close();
                    this.CurrentValueEditorDisplayed = null;
                }
            } catch (ex) { }

            try {
                if (this.CurrentValueEditorDisplayed.inlineLayer) {
                    this.CurrentValueEditorDisplayed.inlineLayer.parentNode.removeChild(this.CurrentValueEditorDisplayed.inlineLayer);
                    this.CurrentValueEditorDisplayed = null;
                }
            } catch (ex) { }

            if (newValue) {
                var bodyNode = this.GetDocumentBodyNode();

                if (valueEditorContext.associatedNode) {
                    var textrangeWholeValue = Ilog.Utilities.FindAdjacentNodes(bodyNode, valueEditorContext.associatedNode, Ilog.Constants.RuleEditorConstants.PatternValueEditorRecognition);
                }
                else if (valueEditorContext.associatedRange) {
                    var textrangeWholeValue = valueEditorContext.associatedRange;
                }
                else {
                    this.CancelGetPredictionsRequest();
                    this.SendRefreshRequest();
                    this.CurrentValueEditorDisplayed = null;
                    return;
                }

                this.Focus();

                this.SelectTextRangeAndChangeValue(textrangeWholeValue, newValue);
                var pos = this.GetTextRange();
                if (pos && pos.end) {
                    this.MoveCaretToPosition(pos.end);
                }
                this.CancelGetPredictionsRequest();
                this.SendRefreshRequest();
            }

            this.CurrentValueEditorDisplayed = null;
        }
    },

    ValueEditorCancel: function() {
        if (this.CurrentValueEditorDisplayed) {
            // closing editor
            try {
                if (this.CurrentValueEditorDisplayed.window) {
                    this.CurrentValueEditorDisplayed.window.close();
                    this.CurrentValueEditorDisplayed = null;
                }
            } catch (ex) { }

            try {
                if (this.CurrentValueEditorDisplayed.inlineLayer) {
                    this.CurrentValueEditorDisplayed.inlineLayer.parentNode.removeChild(this.CurrentValueEditorDisplayed.inlineLayer);
                    this.CurrentValueEditorDisplayed = null;
                }
            } catch (ex) { }
        }
    },

    ClearResources: function() {
        this.OngoingPredictionsRequestTimeStamp = null;

        if (this.PredictionsView) {
            this.PredictionsView.Hide();
        }

        if (this.DatePickerLayer) {
            if (this.DatePickerLayer.TimeField && this.DatePickerLayer.TimeField.isExpanded() === true) {
                this.DatePickerLayer.TimeField.collapse();
            }

            this.DatePickerLayer.parentNode.removeChild(this.DatePickerLayer);
            this.DatePickerLayer = null;
        }
    },

    HideValueEditor: function() {
        // removing currently displayed inline value editor, if any
        try {
            if (this.CurrentValueEditorDisplayed && this.CurrentValueEditorDisplayed.inlineLayer) {
                this.CurrentValueEditorDisplayed.inlineLayer.parentNode.removeChild(this.CurrentValueEditorDisplayed.inlineLayer);
                this.CurrentValueEditorDisplayed = null;
            }
        } catch (ex) { }
    },

    DisplayValueEditor: function(valueEditorClassName, displayType, currentEditedValue, domNode, textrange) {
        this.HideValueEditor();

        var cultureInf = "";

        try {
            if (this.Culture && this.Culture.Culture) {
                cultureInf = this.Culture.Culture;
            }
        }
        catch (ex) {
            Ilog.Debug.RuleEditorDebugger.PrintDebug('[DisplayValueEditor] culture error: ' + ex.message);
        }

        var valueEditorContext = { currentValue: currentEditedValue, culture: cultureInf, actionObject: this };
        try {
            window.eval(" var valueEditorObject = new " + valueEditorClassName + "(valueEditorContext); ");
        }
        catch (ex) {
            Ilog.Debug.RuleEditorDebugger.PrintDebug('[DisplayValueEditor] error on eval (' + valueEditorClassName + '): ' + ex.message);
            return false;
        }

        var containerPanelIdentifier = this.editorFieldId + '_' + valueEditorClassName;
        if (valueEditorObject && valueEditorObject.renderTo) {
            try {
                var modalWindow;
                var valueEditorDiv;

                if (displayType === Ilog.Constants.RuleEditorConstants.DisplayInline) {
                    if (!domNode) {
                        if (!textrange) {
                            return false;
                        }
                        var nodeAndOffset = Ilog.Utilities.GetNodeAndRelativeOffsetForAbsoluteOffset(this.editorDocument, textrange.start);
                        domNode = nodeAndOffset.node;
                    }

                    if (domNode) {
                        while (domNode.nodeType == 3 && ((domNode.tagName && domNode.tagName.toUpperCase() != "BODY") || (domNode.nodeName && domNode.nodeName.toUpperCase() != "BODY"))) {
                            domNode = domNode.parentNode;
                        }

                        if (nodeAndOffset && nodeAndOffset.offset > 0) {
                            if (domNode.nextSibling) {
                                domNode = domNode.nextSibling;
                            }
                        }

                        var extNode = new Ext.Element(domNode);
                        var edArea = this.GetEditionAreaAbsolutePosition();
                        var x = edArea.x + domNode.offsetLeft - this.GetScrollXPosition();
                        var y = edArea.y + domNode.offsetTop + domNode.offsetHeight - this.GetScrollYPosition();
                        valueEditorDiv = document.createElement('div');
                        valueEditorDiv.id = containerPanelIdentifier;
                        valueEditorDiv.style.background = 'white';
                        valueEditorDiv.style.border = '1px grey solid';
                        valueEditorDiv.zindex = 99;
                        valueEditorDiv.style.zindex = 99;
                        valueEditorDiv.style.position = 'absolute';
                        valueEditorDiv.style.top = y + 'px';
                        valueEditorDiv.style.left = x + 'px';
                        document.body.appendChild(valueEditorDiv);

                        var rendered = valueEditorObject.renderTo(containerPanelIdentifier);

                        if (!(rendered === true)) {
                            valueEditorDiv.parentNode.removeChild(valueEditorDiv);
                            return false;
                        }

                        this.CurrentValueEditorDisplayed = { inlineLayer: valueEditorDiv, associatedNode: domNode, associatedRange: textrange };
                        return true;
                    }
                    else {
                        Ilog.Debug.RuleEditorDebugger.PrintDebug('[DisplayValueEditor] no suitable DOM node found for inline display');
                        return false;
                    }
                }
                else {
                    modalWindow = new Ext.Window({
                        modal: true,
                        baseCls: '',
                        shadow: false,
                        expandOnShow: true,
                        plain: false,
                        layout: 'fit',
                        closable: false,
                        draggable: false,
                        resizable: false,
                        bodyBorder: false,
                        hideBorders: true,
                        html: '<div id="' + containerPanelIdentifier + '" style="background-color:white;color:black;"></div>'
                    });

                    modalWindow.show();
                    var rendered = valueEditorObject.renderTo(containerPanelIdentifier);

                    if (!(rendered === true)) {
                        modalWindow.close();
                        return false;
                    }

                    modalWindow.center();
                    this.CurrentValueEditorDisplayed = { window: modalWindow, associatedNode: domNode, associatedRange: textrange };
                    return true;
                }
            } catch (ex) {
                Ilog.Debug.RuleEditorDebugger.PrintDebug('[DisplayValueEditor] display error: ' + ex.message);

                if (modalWindow) {
                    modalWindow.close();
                    return false;
                }
                if (valueEditorDiv) {
                    valueEditorDiv.parentNode.removeChild(valueEditorDiv);
                    return false;
                }
            }
        }
    }

});

// EditorState constructor
Ilog.RuleEditor.EditorState = function(plainText, text, textSelectionStart, textSelectionEnd){
    this.Text = text;
    this.PlainText = plainText;
    this.TextSelectionStart = textSelectionStart;
    this.TextSelectionEnd = textSelectionEnd;
};

// Undo/Redo stack constructor
Ilog.RuleEditor.UndoRedoStack = function(stackSize) {
    this.StackSize = stackSize;
    this.UndoStack = new Array(this.StackSize);
    this.CurrentIndex = -1;
    this.OldestElementIndex = 0;
    this.StateHistory = 0;
};

Ilog.RuleEditor.UndoRedoStack.prototype = {

    AddState: function(editorState) {
        var indexToAdd = this.CurrentIndex + 1;

        if (indexToAdd > this.StackSize - 1) {
            indexToAdd = 0;
        }

        var prevIndex = indexToAdd - 1;
        if (prevIndex < 0) {
            prevIndex = this.StackSize - 1;
        }

        // checking if previous index contains same data
        try {
            if (this.UndoStack[prevIndex] != null) {
                if (this.UndoStack[prevIndex].PlainText == editorState.PlainText) {
                    return false;
                }
            }
        }
        catch (ex) { }

        // Adding state into stack
        this.UndoStack[indexToAdd] = editorState;
        this.UndoStack[indexToAdd].StateHistory = this.StateHistory;
        this.StateHistory++;

        // Registering new current index
        this.CurrentIndex = indexToAdd;

        // Updating oldest element index if the element has just been overwritten
        if (indexToAdd == this.OldestElementIndex) {
            this.OldestElementIndex = this.OldestElementIndex + 1;

            if (this.OldestElementIndex > this.StackSize - 1) {
                this.OldestElementIndex = 0;
            }
        }

        // Potentially erasing elements until oldest one
        for (var p = indexToAdd + 1; p < this.StackSize; p++) {
            if (p == this.OldestElementIndex) {
                break;
            }
            else {
                this.UndoStack[p] = null;
            }
        }
        if (this.OldestElementIndex < indexToAdd) {
            for (var p = 0; p < this.OldestElementIndex; p++) {
                this.UndoStack[p] = null;
            }
        }

        return true;
    },
    
    GetCurrentState: function(){
        if( this.UndoStack[this.CurrentIndex] ){
            return this.UndoStack[this.CurrentIndex];
        }
        
        return null;
    },

    GetPreviousState: function() {
        if (this.CurrentIndex == this.OldestElementIndex) {
            return null;
        }

        var stateIndex = this.CurrentIndex - 1;
        if (stateIndex < 0 && this.UndoStack[this.StackSize - 1] != null) {
            stateIndex = this.StackSize - 1;
        }

        if (this.UndoStack[stateIndex] != undefined && this.UndoStack[stateIndex] != null) {
            return this.UndoStack[stateIndex];
        }

        return null;
    },

    GetFollowingState: function() {
        var stateIndex = this.CurrentIndex + 1;

        if (stateIndex > this.StackSize - 1) {
            stateIndex = 0;
        }

        if (stateIndex == this.OldestElementIndex) {
            return null;
        }

        if (this.UndoStack[stateIndex] != undefined && this.UndoStack[stateIndex] != null) {
            return this.UndoStack[stateIndex];
        }

        return null;
    },

    ValidatePreviousState: function() {
        var index = this.CurrentIndex - 1;

        if (index < 0 && this.UndoStack[this.StackSize - 1] != null) {
            index = this.StackSize - 1;
        }
        this.CurrentIndex = index;

        return true;
    },

    ValidateFollowingState: function() {
        var stateIndex = this.CurrentIndex + 1;

        if (stateIndex > this.StackSize - 1) {
            stateIndex = 0;
        }
        this.CurrentIndex = stateIndex;

        return true;
    }
};